@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2005-10-19
+ *  Updated   : 2010-05-09
  *  Notes     :
  *
- * Copyright (c) 1997-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,6 +57,9 @@
 #ifdef DECL__FLSBUF
 	extern int _flsbuf(int, FILE *);
 #endif /* DECL__FLSBUF */
+#ifdef DECL_ALARM
+	extern unsigned alarm(unsigned);
+#endif /* DECL_ALARM */
 #ifdef DECL_ATOI
 	extern int atoi(const char *);
 #endif /* DECL_ATOI */
@@ -235,12 +238,18 @@
 #ifdef DECL_STRNCASECMP
 	extern int strncasecmp(const char *, const char *, size_t);
 #endif /* DECL_STRNCASECMP */
+#ifdef DECL_STRSEP
+	extern char *strsep(char **, const char *);
+#endif /* DECL_STRSEP */
 #ifdef DECL_STRTOL
 	extern long strtol(const char *, char **, int);
 #endif /* DECL_STRTOL */
 #ifdef DECL_SYSTEM
 	extern int system(const char *);
 #endif /* DECL_SYSTEM */
+#ifdef DECL_TMPFILE
+	extern  FILE *tmpfile(void);
+#endif /* DECL_TMPFILE */
 #ifdef DECL_TGETENT
 	extern int tgetent(char *, char *);
 #endif /* DECL_TGETENT */
@@ -263,7 +272,7 @@
 	extern int tigetnum(char *);
 #endif /* DECL_TGETNUM */
 #ifdef DECL_TIGETSTR
-	extern char *tigetstr(char *, char **);
+	extern char *tigetstr(char *);
 #endif /* DECL_TIGETSTR */
 #ifdef DECL_TIME
 	extern time_t time(time_t *);
@@ -327,8 +336,6 @@ extern char *optarg;
 #	define txt_help_thd_C	txt_help_thd_BIGC
 #endif /* CASE_PROBLEM */
 
-extern char **news_headers_to_display_array;
-extern char **news_headers_to_not_display_array;
 extern char *OPT_CHAR_list[];
 extern char *OPT_STRING_list[];
 extern char *nntp_server;
@@ -337,7 +344,6 @@ extern char article_name[PATH_LEN];
 extern char bug_addr[LEN];
 extern char bug_nntpserver1[PATH_LEN];
 extern char bug_nntpserver2[PATH_LEN];
-extern char cmdline_nntpserver[PATH_LEN];
 extern char cvers[LEN];
 extern char dead_article[PATH_LEN];
 extern char dead_articles[PATH_LEN];
@@ -350,7 +356,6 @@ extern char index_maildir[PATH_LEN];
 extern char index_newsdir[PATH_LEN];
 extern char index_savedir[PATH_LEN];
 extern char inewsdir[PATH_LEN];
-extern char libdir[PATH_LEN];
 extern char local_attributes_file[PATH_LEN];
 extern char local_config_file[PATH_LEN];
 extern char filter_file[PATH_LEN];
@@ -381,6 +386,7 @@ extern char rcdir[PATH_LEN];
 extern char save_active_file[PATH_LEN];
 extern char spooldir[PATH_LEN];
 extern char subscriptions_file[PATH_LEN];
+extern char overviewfmt_file[PATH_LEN];
 extern char txt_help_bug_report[LEN];
 extern char userid[PATH_LEN];
 
@@ -391,10 +397,12 @@ extern const char base64_alphabet[64];
 extern constext *content_encodings[];
 extern constext *content_types[];
 extern constext *txt_attrs[];
+extern constext *txt_auto_cc_bcc_options[];
 #ifdef HAVE_COLOR
 	extern constext *txt_colors[];
 #endif /* HAVE_COLOR */
 extern constext *txt_confirm_choices[];
+extern constext *txt_goto_next_unread_options[];
 extern constext *txt_hide_uue_type[];
 extern constext *txt_interactive_mailers[];
 extern constext *txt_kill_level_type[];
@@ -409,7 +417,7 @@ extern constext *txt_mime_encodings[];
 	extern constext *txt_normalization_forms[];
 #endif /* HAVE_UNICODE_NORMALIZATION */
 extern constext *txt_onoff[];
-extern constext *txt_post_process_type[];
+extern constext *txt_post_process_types[];
 extern constext *txt_quote_style_type[];
 extern constext *txt_show_from[];
 extern constext *txt_show_info_type[];
@@ -418,6 +426,7 @@ extern constext *txt_sort_t_type[];	/* t=threads */
 extern constext *txt_strip_bogus_type[];
 extern constext *txt_threading[];
 extern constext *txt_thread_score_type[];
+extern constext *txt_trim_article_body_options[];
 extern constext *txt_wildcard_type[];
 extern constext txt_1_resp[];
 extern constext txt_7bit[];
@@ -427,7 +436,6 @@ extern constext txt_all[];
 extern constext txt_all_groups[];
 extern constext txt_append_overwrite_quit[];
 extern constext txt_art_cancel[];
-extern constext txt_art_cannot_cancel[];
 extern constext txt_art_newsgroups[];
 extern constext txt_art_not_posted[];
 extern constext txt_art_not_saved[];
@@ -439,6 +447,7 @@ extern constext txt_art_posted[];
 extern constext txt_art_rejected[];
 extern constext txt_art_thread_regex_tag[];
 extern constext txt_art_unavailable[];
+extern constext txt_art_x_of_n[];
 extern constext txt_article_cancelled[];
 extern constext txt_article_plural[];
 extern constext txt_article_reposted[];
@@ -451,16 +460,20 @@ extern constext txt_articles_mailed[];
 extern constext txt_attach[];
 extern constext txt_attach_charset[];
 extern constext txt_attach_description[];
+extern constext txt_attrib_menu_com[];
 extern constext txt_uue[];
 extern constext txt_at_s[];
-extern constext txt_auth_failed[];
-extern constext txt_auth_pass[];
-extern constext txt_auth_user[];
-extern constext txt_auth_needed[];
+#ifdef NNTP_ABLE
+	extern constext txt_auth_failed[];
+	extern constext txt_auth_failed_nopass[];
+	extern constext txt_auth_pass[];
+	extern constext txt_auth_user[];
+	extern constext txt_auth_needed[];
+	extern constext txt_authorization_fail[];
+	extern constext txt_authorization_ok[];
+#endif /* NNTP_ABLE */
 extern constext txt_author_search_backwards[];
 extern constext txt_author_search_forwards[];
-extern constext txt_authorization_fail[];
-extern constext txt_authorization_ok[];
 extern constext txt_autoselecting_articles[];
 extern constext txt_autosubscribed[];
 extern constext txt_autosubscribing_groups[];
@@ -473,14 +486,19 @@ extern constext txt_base_article[];
 extern constext txt_base_article_range[];
 extern constext txt_batch_update_unavail[];
 extern constext txt_begin_of_art[];
-extern constext txt_caching_off[];
-extern constext txt_caching_on[];
+extern constext txt_begin_of_page[];
+#ifdef NNTP_ABLE
+	extern constext txt_caching_off[];
+	extern constext txt_caching_on[];
+#endif /* NNTP_ABLE */
 extern constext txt_cancel_article[];
 extern constext txt_cancelling_art[];
 extern constext txt_cannot_create[];
 extern constext txt_cannot_create_uniq_name[];
 extern constext txt_cannot_find_base_art[];
-extern constext txt_cannot_get_nntp_server_name[];
+#ifdef NNTP_ABLE
+	extern constext txt_cannot_get_nntp_server_name[];
+#endif /* NNTP_ABLE */
 #if !defined(USE_CURSES) && defined(M_UNIX) && !defined(USE_TERMINFO)
 	extern constext txt_cannot_get_term_entry[];
 #endif /* !USE_CURSES && M_UNIX && !USE_TERMINFO */
@@ -488,7 +506,9 @@ extern constext txt_cannot_open[];
 extern constext txt_cannot_open_for_saving[];
 extern constext txt_cannot_post[];
 extern constext txt_cannot_post_group[];
-extern constext txt_cannot_retrieve[];
+#ifdef NNTP_ABLE
+	extern constext txt_cannot_retrieve[];
+#endif /* NNTP_ABLE */
 extern constext txt_cannot_write_to_directory[];
 extern constext txt_catchup[];
 extern constext txt_catchup_group[];
@@ -529,10 +549,13 @@ extern constext txt_deleting[];
 extern constext txt_end_of_art[];
 extern constext txt_end_of_arts[];
 extern constext txt_end_of_groups[];
+extern constext txt_end_of_page[];
+extern constext txt_end_of_scopes[];
 extern constext txt_end_of_thread[];
 extern constext txt_enter_getart_limit[];
 extern constext txt_enter_message_id[];
 extern constext txt_enter_next_thread[];
+extern constext txt_enter_next_unread_art[];
 extern constext txt_enter_next_unread_group[];
 extern constext txt_enter_option_num[];
 extern constext txt_enter_range[];
@@ -542,12 +565,15 @@ extern constext txt_error_bad_approved[];
 extern constext txt_error_bad_from[];
 extern constext txt_error_bad_replyto[];
 extern constext txt_error_bad_msgidfqdn[];
-extern constext txt_error_cant_unlock[];
+#ifndef NO_LOCKING
+	extern constext txt_error_cant_unlock[];
+	extern constext txt_error_couldnt_dotlock[];
+	extern constext txt_error_couldnt_lock[];
+#endif /* NO_LOCKING */
 extern constext txt_error_copy_fp[];
 extern constext txt_error_corrupted_file[];
-extern constext txt_error_couldnt_dotlock[];
-extern constext txt_error_couldnt_lock[];
 extern constext txt_error_fseek[];
+extern constext txt_error_followup_poster[];
 extern constext txt_error_gnksa_internal[];
 extern constext txt_error_gnksa_langle[];
 extern constext txt_error_gnksa_lparen[];
@@ -576,27 +602,35 @@ extern constext txt_error_gnksa_rn_paren[];
 extern constext txt_error_gnksa_rn_invalid[];
 extern constext txt_error_header_and_body_not_separate[];
 extern constext txt_error_header_duplicate[];
+extern constext txt_error_header_format[];
 extern constext txt_error_header_line_bad_charset[];
 extern constext txt_error_header_line_bad_encoding[];
 extern constext txt_error_header_line_blank[];
 extern constext txt_error_header_line_colon[];
 extern constext txt_error_header_line_empty[];
 extern constext txt_error_header_line_missing[];
+extern constext txt_error_header_line_not_7bit[];
 extern constext txt_error_header_line_space[];
 extern constext txt_error_insecure_permissions[];
-extern constext txt_error_invalid_response_to_group[];
-extern constext txt_error_locale[];
+#ifdef NNTP_ABLE
+	extern constext txt_error_invalid_response_to_group[];
+#endif /* NNTP_ABLE */
+#if defined(HAVE_SETLOCALE) && !defined(NO_LOCALE)
+	extern constext txt_error_locale[];
+#endif /* HAVE_SETLOCALE && !NO_LOCALE */
 extern constext txt_error_mime_end[];
 extern constext txt_error_mime_start[];
 extern constext txt_error_no_domain_name[];
 extern constext txt_error_no_enter_permission[];
-extern constext txt_error_no_from[];
+#ifdef NNTP_INEWS
+	extern constext txt_error_no_from[];
+#endif /* NNTP_INEWS */
 extern constext txt_error_no_read_permission[];
 extern constext txt_error_no_such_file[];
 extern constext txt_error_no_write_permission[];
+extern constext txt_error_newsgroups_poster[];
 extern constext txt_error_passwd_missing[];
 extern constext txt_error_plural[];
-extern constext txt_error_sender_in_header_not_allowed[];
 extern constext txt_error_server_has_no_listed_groups[];
 extern constext txt_error_singular[];
 #if defined(NNTP_ABLE) && defined(INET6)
@@ -607,9 +641,9 @@ extern constext txt_error_singular[];
 	extern constext txt_error_topen[];
 #endif /* NNTP_ABLE && TLI && !INET6 */
 extern constext txt_error_unknown_dlevel[];
-#if defined(NNTP_ABLE) && defined(HAVE_GETSERVBYNAME)
+#if defined(NNTP_ABLE) && defined(HAVE_GETSERVBYNAME) && !defined(INET6)
 	extern constext txt_error_unknown_service[];
-#endif /* NNTP_ABLE && HAVE_GETSERVBYNAME */
+#endif /* NNTP_ABLE && HAVE_GETSERVBYNAME && !INET6 */
 #ifdef NNTP_ABLE
 	extern constext txt_error_wrong_newsgroupname_in_group_response[];
 #endif /* NNTP_ABLE */
@@ -637,7 +671,9 @@ extern constext txt_forwarded_end[];
 extern constext txt_from_line_only[];
 extern constext txt_from_line_only_case[];
 extern constext txt_full[];
-extern constext txt_gethostbyname[];
+#ifdef NNTP_ABLE
+	extern constext txt_gethostbyname[];
+#endif /* NNTP_ABLE */
 #if defined(NNTP_ABLE) && !defined(INET6)
 	extern constext txt_giving_up[];
 #endif /* NNTP_ABLE && !INET6 */
@@ -655,13 +691,16 @@ extern constext txt_help_article_autokill[];
 extern constext txt_help_article_autoselect[];
 extern constext txt_help_article_browse_urls[];
 extern constext txt_help_article_by_num[];
-extern constext txt_help_article_cancel[];
+#ifndef NO_POSTING
+	extern constext txt_help_article_cancel[];
+	extern constext txt_help_article_followup[];
+	extern constext txt_help_article_followup_no_quote[];
+	extern constext txt_help_article_followup_with_header[];
+	extern constext txt_help_article_repost[];
+#endif /* !NO_POSTING */
 extern constext txt_help_article_edit[];
 extern constext txt_help_article_first_in_thread[];
 extern constext txt_help_article_first_page[];
-extern constext txt_help_article_followup[];
-extern constext txt_help_article_followup_no_quote[];
-extern constext txt_help_article_followup_with_header[];
 extern constext txt_help_article_last_in_thread[];
 extern constext txt_help_article_last_page[];
 extern constext txt_help_article_mark_thread_read[];
@@ -680,7 +719,6 @@ extern constext txt_help_article_quit_to_select_level[];
 extern constext txt_help_article_reply[];
 extern constext txt_help_article_reply_no_quote[];
 extern constext txt_help_article_reply_with_header[];
-extern constext txt_help_article_repost[];
 extern constext txt_help_article_search_backwards[];
 extern constext txt_help_article_search_forwards[];
 extern constext txt_help_article_show_raw[];
@@ -692,7 +730,23 @@ extern constext txt_help_article_toggle_tabwidth[];
 extern constext txt_help_article_toggle_tex2iso[];
 extern constext txt_help_article_toggle_uue[];
 extern constext txt_help_article_view_attachments[];
+extern constext txt_help_attrib_first_opt[];
+extern constext txt_help_attrib_goto_opt[];
+extern constext txt_help_attrib_last_opt[];
+extern constext txt_help_attrib_reset_attrib[];
+extern constext txt_help_attrib_search_opt_backwards[];
+extern constext txt_help_attrib_search_opt_forwards[];
+extern constext txt_help_attrib_select[];
+extern constext txt_help_attrib_toggle_attrib[];
 extern constext txt_help_bug[];
+extern constext txt_help_config_first_opt[];
+extern constext txt_help_config_goto_opt[];
+extern constext txt_help_config_last_opt[];
+extern constext txt_help_config_scope_menu[];
+extern constext txt_help_config_search_opt_backwards[];
+extern constext txt_help_config_search_opt_forwards[];
+extern constext txt_help_config_select[];
+extern constext txt_help_config_toggle_attrib[];
 extern constext txt_help_filter_comment[];
 extern constext txt_help_filter_from[];
 extern constext txt_help_filter_lines[];
@@ -716,8 +770,10 @@ extern constext txt_help_global_page_up[];
 #ifndef DONT_HAVE_PIPING
 	extern constext txt_help_global_pipe[];
 #endif /* !DONT_HAVE_PIPING */
-extern constext txt_help_global_post[];
-extern constext txt_help_global_post_postponed[];
+#ifndef NO_POSTING
+	extern constext txt_help_global_post[];
+	extern constext txt_help_global_post_postponed[];
+#endif /* !NO_POSTING */
 extern constext txt_help_global_posting_history[];
 extern constext txt_help_global_previous_menu[];
 #ifndef DISABLE_PRINTING
@@ -756,6 +812,8 @@ extern constext txt_help_group_list_thread[];
 extern constext txt_help_group_mark_article_unread[];
 extern constext txt_help_group_mark_thread_read[];
 extern constext txt_help_group_mark_thread_unread[];
+extern constext txt_help_mark_feed_read[];
+extern constext txt_help_mark_feed_unread[];
 extern constext txt_help_group_mark_unsel_art_read[];
 extern constext txt_help_group_next[];
 extern constext txt_help_group_prev[];
@@ -773,6 +831,15 @@ extern constext txt_help_group_toggle_threading[];
 extern constext txt_help_group_undo_thread_selection[];
 extern constext txt_help_group_untag_thread[];
 extern constext txt_help_kill_scope[];
+extern constext txt_help_scope_add[];
+extern constext txt_help_scope_del[];
+extern constext txt_help_scope_edit_attrib_file[];
+extern constext txt_help_scope_first_scope[];
+extern constext txt_help_scope_goto_scope[];
+extern constext txt_help_scope_last_scope[];
+extern constext txt_help_scope_move[];
+extern constext txt_help_scope_rename[];
+extern constext txt_help_scope_select[];
 extern constext txt_help_select_catchup[];
 extern constext txt_help_select_catchup_next_unread[];
 extern constext txt_help_select_first_group[];
@@ -806,11 +873,16 @@ extern constext txt_help_thread_catchup_next_unread[];
 extern constext txt_help_thread_first_article[];
 extern constext txt_help_thread_last_article[];
 extern constext txt_help_thread_mark_article_read[];
+extern constext txt_help_thread_mark_article_unread[];
+extern constext txt_help_thread_mark_thread_unread[];
 extern constext txt_help_thread_read_article[];
 extern constext txt_help_title_disp[];
 extern constext txt_help_title_misc[];
 extern constext txt_help_title_navi[];
 extern constext txt_help_title_ops[];
+extern constext txt_help_title_attrib_ops[];
+extern constext txt_help_title_config_ops[];
+extern constext txt_help_title_scope_ops[];
 extern constext txt_incomplete[];
 extern constext txt_index_page_com[];
 extern constext txt_info_add_kill[];
@@ -827,7 +899,6 @@ extern constext txt_info_no_previous_expression[];
 extern constext txt_info_postponed[];
 extern constext txt_info_x_conversion_note[];
 extern constext txt_invalid_from[];
-extern constext txt_invalid_sender[];
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern constext txt_invalid_multibyte_sequence[];
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -859,13 +930,15 @@ extern constext txt_mail_bug_report_confirm[];
 extern constext txt_mailed[];
 extern constext txt_mailing_to[];
 extern constext txt_mail_save_active_head[];
+extern constext txt_mark[];
 extern constext txt_mark_arts_read[];
-extern constext txt_mark_art_read_tagged_current[];
+extern constext txt_mark_art_read[];
 extern constext txt_mark_group_read[];
 extern constext txt_mark_thread_read[];
-extern constext txt_mark_thread_read_tagged_current[];
+extern constext txt_marked_as_read[];
 extern constext txt_marked_as_unread[];
-extern constext txt_marked_tagged_arts_as_read[];
+extern constext txt_marked_arts_as_read[];
+extern constext txt_marked_arts_as_unread[];
 extern constext txt_matching_cmd_line_groups[];
 extern constext txt_mini_group_1[];
 extern constext txt_mini_group_2[];
@@ -875,6 +948,8 @@ extern constext txt_mini_info_2[];
 extern constext txt_mini_page_1[];
 extern constext txt_mini_page_2[];
 extern constext txt_mini_page_3[];
+extern constext txt_mini_scope_1[];
+extern constext txt_mini_scope_2[];
 extern constext txt_mini_select_1[];
 extern constext txt_mini_select_2[];
 extern constext txt_mini_select_3[];
@@ -882,7 +957,9 @@ extern constext txt_mini_thread_1[];
 extern constext txt_mini_thread_2[];
 extern constext txt_more[];
 extern constext txt_moving[];
+extern constext txt_msgid_line_last[];
 extern constext txt_msgid_line_only[];
+extern constext txt_msgid_refs_line[];
 extern constext txt_name[];
 extern constext txt_newsgroup[];
 extern constext txt_newsgroup_plural[];
@@ -892,7 +969,6 @@ extern constext txt_newsrc_again[];
 extern constext txt_newsrc_nogroups[];
 extern constext txt_newsrc_saved[];
 extern constext txt_next_resp[];
-extern constext txt_nntp_authorization_failed[];
 extern constext txt_no[];
 extern constext txt_no_arts[];
 extern constext txt_no_arts_posted[];
@@ -912,6 +988,7 @@ extern constext txt_no_prev_group[];
 extern constext txt_no_prev_unread_art[];
 extern constext txt_no_responses[];
 extern constext txt_no_resps_in_thread[];
+extern constext txt_no_scopes[];
 extern constext txt_no_search_string[];
 extern constext txt_no_subject[];
 #ifndef USE_CURSES
@@ -922,7 +999,9 @@ extern constext txt_no_subject[];
 	extern constext txt_no_term_set[];
 #endif /* !USE_CURSES */
 extern constext txt_no_viewer_found[];
-extern constext txt_no_xover_support[];
+#ifdef NNTP_ABLE
+	extern constext txt_no_xover_support[];
+#endif /* NNTP_ABLE */
 extern constext txt_not_exist[];
 extern constext txt_not_in_active_file[];
 extern constext txt_nrctbl_create[];
@@ -931,6 +1010,7 @@ extern constext txt_nrctbl_info[];
 extern constext txt_only[];
 extern constext txt_option_not_enabled[];
 extern constext txt_options_menu[];
+extern constext txt_options_menu_com[];
 extern constext txt_out_of_memory[];
 extern constext txt_pcre_error_at[];
 extern constext txt_pcre_error_num[];
@@ -969,9 +1049,10 @@ extern constext txt_quit_edit_xpost[];
 extern constext txt_quit_no_write[];
 extern constext txt_quoted_printable[];
 extern constext txt_range_invalid[];
-extern constext txt_read_abort[];
-extern constext txt_read_exit[];
-extern constext txt_read_resp[];
+#ifdef NNTP_ABLE
+	extern constext txt_read_abort[];
+	extern constext txt_read_exit[];
+#endif /* NNTP_ABLE */
 extern constext txt_reading_article[];
 extern constext txt_reading_arts[];
 extern constext txt_reading_attributes_file[];
@@ -987,6 +1068,7 @@ extern constext txt_reading_newsrc[];
 #ifdef NNTP_ABLE
 	extern constext txt_reconnect_to_news_server[];
 #endif /* NNTP_ABLE */
+extern constext txt_refs_line_only[];
 #ifdef HAVE_GETTIMEOFDAY
 	extern constext txt_remaining[];
 #endif /* HAVE_GETTIMEOFDAY */
@@ -999,7 +1081,6 @@ extern constext txt_repost_group[];
 extern constext txt_reset_newsrc[];
 extern constext txt_resp_redirect[];
 extern constext txt_resp_to_poster[];
-extern constext txt_resp_x_of_n[];
 extern constext txt_return_key[];
 extern constext txt_save[];
 extern constext txt_save_attachment[];
@@ -1018,6 +1099,15 @@ extern constext txt_screen_init_failed[];
 	extern constext txt_screen_too_small[];
 #endif /* !USE_CURSES */
 extern constext txt_screen_too_small_exiting[];
+extern constext txt_scope_delete[];
+extern constext txt_scope_enter[];
+extern constext txt_scope_new_position[];
+extern constext txt_scope_new_position_is_global[];
+extern constext txt_scope_operation_not_allowed[];
+extern constext txt_scope_rename[];
+extern constext txt_scope_select[];
+extern constext txt_scopes_menu[];
+extern constext txt_scopes_menu_com[];
 extern constext txt_search_backwards[];
 extern constext txt_search_body[];
 extern constext txt_search_forwards[];
@@ -1067,7 +1157,6 @@ extern constext txt_thread_marked_as_selected[];
 #if 0 /* unused */
 	extern constext txt_thread_plural[];
 #endif /* 0 */
-extern constext txt_thread_range[];
 extern constext txt_thread_singular[];
 extern constext txt_thread_x_of_n[];
 extern constext txt_threading_arts[];
@@ -1081,11 +1170,13 @@ extern constext txt_toggled_high[];
 extern constext txt_toggled_rot13[];
 extern constext txt_toggled_tex2iso[];
 extern constext txt_toggled_tabwidth[];
-#if defined(NNTP_ABLE) && defined(HAVE_INET_NTOA)
+#if defined(NNTP_ABLE) && defined(HAVE_INET_NTOA) && !defined(INET6)
 	extern constext txt_trying[];
-#endif /* NNTP_ABLE && HAVE_INET_NTOA */
-extern constext txt_trying_dotlock[];
-extern constext txt_trying_lock[];
+#endif /* NNTP_ABLE && HAVE_INET_NTOA && ! INET6 */
+#ifndef NO_LOCKING
+	extern constext txt_trying_dotlock[];
+	extern constext txt_trying_lock[];
+#endif /* NO_LOCKING */
 extern constext txt_type_h_for_help[];
 extern constext txt_unlimited_time[];
 extern constext txt_unread[];
@@ -1107,9 +1198,11 @@ extern constext txt_usage_dont_check_new_newsgroups[];
 extern constext txt_usage_dont_save_files_on_quit[];
 extern constext txt_usage_dont_show_descriptions[];
 #ifdef NNTP_ABLE
+	extern constext txt_unparseable_counts[];
 	extern constext txt_usage_force_authentication[];
 	extern constext txt_usage_newsserver[];
 	extern constext txt_usage_port[];
+	extern constext txt_usage_read_news_remotely[];
 #endif /* NNTP_ABLE */
 extern constext txt_usage_getart_limit[];
 extern constext txt_usage_help_information[];
@@ -1124,7 +1217,6 @@ extern constext txt_usage_newsrc_file[];
 extern constext txt_usage_no_posting[];
 extern constext txt_usage_post_article[];
 extern constext txt_usage_post_postponed_arts[];
-extern constext txt_usage_read_news_remotely[];
 extern constext txt_usage_read_saved_news[];
 extern constext txt_usage_savedir[];
 extern constext txt_usage_save_new_news[];
@@ -1140,6 +1232,7 @@ extern constext txt_usage_verbose[];
 extern constext txt_usage_version[];
 extern constext txt_useful_without_batch_mode[];
 extern constext txt_useful_with_batch_mode[];
+extern constext txt_useless_combination[];
 extern constext txt_use_mime[];
 extern constext txt_value_out_of_range[];
 extern constext txt_view_attachment[];
@@ -1156,6 +1249,7 @@ extern constext txt_warn_encoding_and_external_inews[];
 	extern constext txt_warn_header_line_comma[];
 	extern constext txt_warn_header_line_groups_contd[];
 #endif /* FOLLOW_USEFOR_DRAFT */
+extern constext txt_warn_example_hierarchy[];
 extern constext txt_warn_multiple_sigs[];
 extern constext txt_warn_newsrc[];
 extern constext txt_warn_not_all_arts_saved[];
@@ -1187,6 +1281,7 @@ extern constext txt_you_have_mail[];
 
 #ifndef HAVE_FASCIST_NEWSADMIN
 	extern constext txt_warn_followup_to_several_groups[];
+	extern constext txt_warn_grp_renamed[];
 	extern constext txt_warn_missing_followup_to[];
 	extern constext txt_warn_not_in_newsrc[];
 	extern constext txt_warn_not_valid_newsgroup[];
@@ -1215,10 +1310,12 @@ extern constext txt_uu_success[];
 extern int *my_group;
 extern int NOTESLINES;
 extern int _hp_glitch;
+extern int attrib_file_offset;
 extern int cCOLS;
 extern int cLINES;
 extern int curr_line;
 extern int debug;
+extern int filter_file_offset;
 extern int input_context;
 extern int iso2asc_supported;
 extern int last_resp;
@@ -1226,13 +1323,13 @@ extern int max_active;
 extern int max_art;
 extern int max_newnews;
 extern int max_save;
-extern int num_headers_to_display;
-extern int num_headers_to_not_display;
+extern int max_scope;
 extern int need_resize;
 extern int num_active;
 extern int num_newnews;
 extern int num_of_tagged_arts;
 extern int num_save;
+extern int num_scope;
 extern int signal_context;
 extern int srch_lineno;
 extern int system_status;
@@ -1248,12 +1345,16 @@ extern long *base;
 
 extern signed long int read_newsrc_lines;
 
+extern size_t tabwidth;
+
 extern pid_t process_id;
 
 extern struct regex_cache strip_re_regex;
 extern struct regex_cache strip_was_regex;
 extern struct regex_cache uubegin_regex;
 extern struct regex_cache uubody_regex;
+extern struct regex_cache verbatim_begin_regex;
+extern struct regex_cache verbatim_end_regex;
 extern struct regex_cache url_regex;
 extern struct regex_cache mail_regex;
 extern struct regex_cache news_regex;
@@ -1269,7 +1370,8 @@ extern struct regex_cache strokes_regex;
 #endif /* HAVE_COLOR */
 
 extern struct t_article *arts;
-extern struct t_attribute glob_attributes;
+extern struct t_scope *scopes;
+extern struct t_cmdlineopts cmdline;
 extern struct t_config tinrc;
 extern struct t_filters glob_filter;
 extern struct t_group *active;
@@ -1285,28 +1387,31 @@ extern t_bool check_for_new_newsgroups;
 extern t_bool cmd_line;
 extern t_bool created_rcdir;
 extern t_bool dangerous_signal_exit; /* TRUE if SIGHUP, SIGTERM, SIGUSR1 */
+#ifdef NNTP_ABLE
+	extern t_bool did_reconnect;
+	extern t_bool reconnected_in_last_get_server;
+#endif /* NNTP_ABLE */
 extern t_bool disable_gnksa_domain_check;
 extern t_bool disable_sender;
 extern t_bool force_no_post;
 extern t_bool force_reread_active_file;
-extern t_bool got_sig_pipe;
 extern t_bool have_linescroll;
-extern t_bool filtered_articles;
 extern t_bool list_active;
 extern t_bool newsrc_active;
 extern t_bool no_write;
 extern t_bool post_article_and_exit;
 extern t_bool post_postponed_and_exit;
+extern t_bool range_active;
 extern t_bool read_local_newsgroups_file;
 extern t_bool read_news_via_nntp;
 extern t_bool read_saved_news;
-extern t_bool reconnected_in_last_get_server;
 extern t_bool reread_active_for_posted_arts;
 extern t_bool show_description;
 extern t_bool show_subject;
 extern t_bool batch_mode;
 extern t_bool verbose;
 extern t_bool xref_supported;
+extern t_bool expensive_over_parse;
 
 extern t_function last_search;
 
@@ -1316,6 +1421,8 @@ extern t_menu *currmenu;
 
 extern t_openartinfo pgart;
 
+extern struct t_overview_fmt *ofmt;
+
 enum {
 	HIST_OTHER = 0,
 	HIST_ART_SEARCH,
@@ -1396,6 +1503,12 @@ extern char *input_history[HIST_MAXNUM + 1][HIST_SIZE + 1];
 
 #ifdef FORGERY
 	extern constext txt_warn_cancel_forgery[];
+#else
+	extern constext txt_art_cannot_cancel[];
+	extern constext txt_error_sender_in_header_not_allowed[];
+#	ifdef NNTP_INEWS
+	extern constext txt_invalid_sender[];
+#	endif /* NNTP_INEWS */
 #endif /* FORGERY */
 
 extern t_bool word_highlight;
@@ -1411,6 +1524,7 @@ extern t_bool word_highlight;
 
 #ifdef HAVE_FASCIST_NEWSADMIN
 	extern constext txt_error_followup_to_several_groups[];
+	extern constext txt_error_grp_renamed[];
 	extern constext txt_error_missing_followup_to[];
 	extern constext txt_error_not_valid_newsgroup[];
 #endif /* HAVE_FASCIST_NEWSADMIN */
@@ -1458,6 +1572,7 @@ extern constext txt_processing_saved_arts[];
 	extern t_bool force_auth_on_conn_open;
 #endif /* NNTP_ABLE */
 
+extern struct opttxt txt_abbreviate_groupname;
 extern struct opttxt txt_add_posted_to_filter;
 extern struct opttxt txt_advertising;
 extern struct opttxt txt_alternative_handling;
@@ -1471,11 +1586,12 @@ extern struct opttxt txt_art_marked_return;
 extern struct opttxt txt_art_marked_selected;
 extern struct opttxt txt_art_marked_unread;
 extern struct opttxt txt_ask_for_metamail;
-extern struct opttxt txt_auto_bcc;
-extern struct opttxt txt_auto_cc;
+extern struct opttxt txt_auto_cc_bcc;
 extern struct opttxt txt_auto_list_thread;
 extern struct opttxt txt_auto_reconnect;
 extern struct opttxt txt_auto_save;
+extern struct opttxt txt_auto_select;
+extern struct opttxt txt_delete_tmp_files;
 extern struct opttxt txt_batch_save;
 extern struct opttxt txt_beginner_level;
 extern struct opttxt txt_cache_overview_files;
@@ -1489,17 +1605,24 @@ extern struct opttxt txt_display_options;
 extern struct opttxt txt_draw_arrow;
 extern struct opttxt txt_editor_format;
 extern struct opttxt txt_expert_options;
+extern struct opttxt txt_fcc;
 extern struct opttxt txt_filter_days;
 extern struct opttxt txt_filtering_options;
+extern struct opttxt txt_followup_to;
 extern struct opttxt txt_force_screen_redraw;
+extern struct opttxt txt_from;
 extern struct opttxt txt_getart_limit;
 extern struct opttxt txt_getart_limit_options;
+extern struct opttxt txt_goto_next_unread;
 extern struct opttxt txt_group_catchup_on_exit;
 extern struct opttxt txt_groupname_max_length;
 extern struct opttxt txt_hide_uue;
 extern struct opttxt txt_inews_prog;
 extern struct opttxt txt_interactive_mailer;
 extern struct opttxt txt_inverse_okay;
+#ifdef HAVE_ISPELL
+	extern struct opttxt txt_ispell;
+#endif /* HAVE_ISPELL */
 extern struct opttxt txt_keep_dead_articles;
 extern struct opttxt txt_kill_level;
 extern struct opttxt txt_mail_8bit_header;
@@ -1508,25 +1631,32 @@ extern struct opttxt txt_mail_mime_encoding;
 extern struct opttxt txt_mail_quote_format;
 extern struct opttxt txt_mailbox_format;
 extern struct opttxt txt_maildir;
+extern struct opttxt txt_mailing_list;
 extern struct opttxt txt_mailer_format;
 extern struct opttxt txt_mark_ignore_tags;
 extern struct opttxt txt_mark_saved_read;
+extern struct opttxt txt_mime_forward;
+extern struct opttxt txt_mime_types_to_save;
 extern struct opttxt txt_mono_markstar;
 extern struct opttxt txt_mono_markdash;
 extern struct opttxt txt_mono_markslash;
 extern struct opttxt txt_mono_markstroke;
 #ifndef CHARSET_CONVERSION
 	extern struct opttxt txt_mm_charset;
+#else
+#	ifdef NO_LOCALE
+		extern struct opttxt txt_mm_local_charset;
+#	endif /* NO_LOCALE */
 #endif /* CHARSET_CONVERSION */
 extern struct opttxt txt_metamail_prog;
 extern struct opttxt txt_news_headers_to_display;
 extern struct opttxt txt_news_headers_to_not_display;
 extern struct opttxt txt_news_quote_format;
-extern struct opttxt txt_pgdn_goto_next;
+extern struct opttxt txt_organization;
 extern struct opttxt txt_pos_first_unread;
 extern struct opttxt txt_post_8bit_header;
 extern struct opttxt txt_post_mime_encoding;
-extern struct opttxt txt_post_process;
+extern struct opttxt txt_post_process_type;
 extern struct opttxt txt_post_process_view;
 extern struct opttxt txt_posted_articles_file;
 extern struct opttxt txt_posting_options;
@@ -1536,11 +1666,20 @@ extern struct opttxt txt_posting_options;
 #endif /* !DISABLE_PRINTING */
 extern struct opttxt txt_process_only_unread;
 extern struct opttxt txt_prompt_followupto;
+extern struct opttxt txt_quick_select_scope;
+extern struct opttxt txt_quick_select_header;
+extern struct opttxt txt_quick_select_case;
+extern struct opttxt txt_quick_select_expire;
+extern struct opttxt txt_quick_kill_scope;
+extern struct opttxt txt_quick_kill_header;
+extern struct opttxt txt_quick_kill_case;
+extern struct opttxt txt_quick_kill_expire;
 extern struct opttxt txt_quote_chars;
 extern struct opttxt txt_quote_style;
 extern struct opttxt txt_recent_time;
 extern struct opttxt txt_reread_active_file_secs;
 extern struct opttxt txt_savedir;
+extern struct opttxt txt_savefile;
 extern struct opttxt txt_saving_options;
 extern struct opttxt txt_score_limit_kill;
 extern struct opttxt txt_score_limit_select;
@@ -1559,7 +1698,6 @@ extern struct opttxt txt_signature_repost;
 extern struct opttxt txt_slashes_regex;
 extern struct opttxt txt_sort_article_type;
 extern struct opttxt txt_sort_threads_type;
-extern struct opttxt txt_space_goto_next_unread;
 extern struct opttxt txt_spamtrap_warning_addresses;
 extern struct opttxt txt_stars_regex;
 extern struct opttxt txt_start_editor_offset;
@@ -1569,25 +1707,32 @@ extern struct opttxt txt_strip_newsrc;
 extern struct opttxt txt_strip_re_regex;
 extern struct opttxt txt_strip_was_regex;
 extern struct opttxt txt_strokes_regex;
-extern struct opttxt txt_tab_goto_next_unread;
 extern struct opttxt txt_tex2iso_conv;
 extern struct opttxt txt_thread_articles;
 extern struct opttxt txt_thread_perc;
 extern struct opttxt txt_thread_catchup_on_exit;
 extern struct opttxt txt_thread_score;
+extern struct opttxt txt_trim_article_body;
 extern struct opttxt txt_underscores_regex;
 extern struct opttxt txt_unlink_article;
 extern struct opttxt txt_url_handler;
 extern struct opttxt txt_url_highlight;
 extern struct opttxt txt_use_mouse;
+extern struct opttxt txt_verbatim_begin_regex;
+extern struct opttxt txt_verbatim_end_regex;
+extern struct opttxt txt_verbatim_handling;
 extern struct opttxt txt_wildcard;
 extern struct opttxt txt_word_highlight;
 extern struct opttxt txt_word_h_display_marks;
 extern struct opttxt txt_wrap_column;
 extern struct opttxt txt_wrap_on_next_unread;
+extern struct opttxt txt_x_body;
+extern struct opttxt txt_x_comment_to;
+extern struct opttxt txt_x_headers;
 extern struct opttxt txt_xpost_quote_format;
 #ifdef CHARSET_CONVERSION
 	extern struct opttxt txt_mm_network_charset;
+	extern struct opttxt txt_undeclared_charset;
 #endif /* CHARSET_CONVERSION */
 #ifdef HAVE_COLOR
 	extern struct opttxt txt_quote_regex;
@@ -1613,6 +1758,7 @@ extern struct opttxt txt_xpost_quote_format;
 	extern struct opttxt txt_col_title;
 	extern struct opttxt txt_col_signature;
 	extern struct opttxt txt_col_urls;
+	extern struct opttxt txt_col_verbatim;
 	extern struct opttxt txt_col_markstar;
 	extern struct opttxt txt_col_markdash;
 	extern struct opttxt txt_col_markslash;
