@@ -3,10 +3,10 @@
  *  Module    : keymap.h
  *  Author    : J. Faultless, D. Nimmich
  *  Created   : 1999
- *  Updated   : 2005-10-19
+ *  Updated   : 2010-10-01
  *  Notes     :
  *
- * Copyright (c) 1999-2006 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2010 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -154,16 +154,27 @@ enum defined_functions {
 	DIGIT_7,
 	DIGIT_8,
 	DIGIT_9,
+	ATTACHMENT_PIPE,
+	ATTACHMENT_SAVE,
+	ATTACHMENT_SELECT,
+	ATTACHMENT_TAG,
+	ATTACHMENT_TAG_PATTERN,
+	ATTACHMENT_TOGGLE_TAGGED,
+	ATTACHMENT_UNTAG,
 	SPECIAL_CATCHUP_LEFT,
 	SPECIAL_MOUSE_TOGGLE,
 	CATCHUP,
 	CATCHUP_NEXT_UNREAD,
+	CONFIG_RESET_ATTRIB,
+	CONFIG_SCOPE_MENU,
 	CONFIG_SELECT,
 	CONFIG_NO_SAVE,
+	CONFIG_TOGGLE_ATTRIB,
 	FEED_ARTICLE,
 	FEED_THREAD,
 	FEED_HOT,
 	FEED_PATTERN,
+	FEED_RANGE,
 	FEED_TAGGED,
 	FEED_KEY_REPOST,
 	FEED_SUPERSEDE,
@@ -207,7 +218,7 @@ enum defined_functions {
 	GLOBAL_SET_RANGE,
 #ifndef NO_SHELL_ESCAPE
 	GLOBAL_SHELL_ESCAPE,
-#endif /* NO_SHELL_ESCAPE */
+#endif /* !NO_SHELL_ESCAPE */
 #ifdef HAVE_COLOR
 	GLOBAL_TOGGLE_COLOR,
 #endif /* HAVE_COLOR */
@@ -216,6 +227,7 @@ enum defined_functions {
 	GLOBAL_TOGGLE_INVERSE_VIDEO,
 	GLOBAL_VERSION,
 	GROUP_AUTOSAVE,
+	GROUP_CANCEL,
 	GROUP_DO_AUTOSELECT,
 	GROUP_GOTO,
 	GROUP_LIST_THREAD,
@@ -244,9 +256,9 @@ enum defined_functions {
 	GROUP_UNDO_SELECTIONS,
 	GROUP_UNTAG,
 	MARK_ARTICLE_UNREAD,
-	MARK_READ_CURRENT,
-	MARK_READ_TAGGED,
 	MARK_THREAD_UNREAD,
+	MARK_FEED_READ,
+	MARK_FEED_UNREAD,
 	PAGE_AUTOSAVE,
 	PAGE_BOTTOM_THREAD,
 	PAGE_CANCEL,
@@ -263,7 +275,6 @@ enum defined_functions {
 	PAGE_NEXT_THREAD,
 	PAGE_NEXT_UNREAD,
 	PAGE_NEXT_UNREAD_ARTICLE,
-	PAGE_PAGE_DOWN3,	/* special handling at page.c */
 #ifdef HAVE_PGP_GPG
 	PAGE_PGP_CHECK_ARTICLE,
 #endif /* HAVE_PGP_GPG */
@@ -286,10 +297,12 @@ enum defined_functions {
 	PAGE_TOP_THREAD,
 	PAGE_VIEW_ATTACHMENTS,
 	PAGE_VIEW_URL,
+#ifdef HAVE_PGP_GPG
 	PGP_KEY_ENCRYPT,
 	PGP_KEY_ENCRYPT_SIGN,
 	PGP_INCLUDE_KEY,
 	PGP_KEY_SIGN,
+#endif /* HAVE_PGP_GPG */
 	POST_ABORT,
 	POST_CANCEL,
 	POST_CONTINUE,
@@ -314,6 +327,12 @@ enum defined_functions {
 	PROMPT_YES,
 	SAVE_APPEND_FILE,
 	SAVE_OVERWRITE_FILE,
+	SCOPE_ADD,
+	SCOPE_DELETE,
+	SCOPE_EDIT_ATTRIBUTES_FILE,
+	SCOPE_MOVE,
+	SCOPE_RENAME,
+	SCOPE_SELECT,
 	SELECT_ENTER_GROUP,
 	SELECT_ENTER_NEXT_UNREAD_GROUP,
 	SELECT_GOTO,
@@ -332,6 +351,7 @@ enum defined_functions {
 	SELECT_QUIT_NO_WRITE,
 	SELECT_YANK_ACTIVE,
 	THREAD_AUTOSAVE,
+	THREAD_CANCEL,
 	THREAD_MAIL,
 	THREAD_MARK_ARTICLE_READ,
 	THREAD_READ_NEXT_ARTICLE_OR_THREAD,
@@ -343,7 +363,8 @@ enum defined_functions {
 	THREAD_TOGGLE_ARTICLE_SELECTION,
 	THREAD_TOGGLE_SUBJECT_DISPLAY,
 	THREAD_UNDO_SELECTIONS,
-	THREAD_UNTAG
+	THREAD_UNTAG,
+	URL_SELECT
 };
 typedef enum defined_functions t_function;
 
@@ -365,13 +386,13 @@ struct keylist {
 };
 
 
+extern struct keylist attachment_keys;
 extern struct keylist feed_post_process_keys;
 extern struct keylist feed_supersede_article_keys;
 extern struct keylist feed_type_keys;
 extern struct keylist filter_keys;
 extern struct keylist group_keys;
 extern struct keylist info_keys;
-extern struct keylist mark_read_keys;
 extern struct keylist option_menu_keys;
 extern struct keylist page_keys;
 #ifdef HAVE_PGP_GPG
@@ -390,8 +411,10 @@ extern struct keylist post_postpone_keys;
 extern struct keylist post_send_keys;
 extern struct keylist prompt_keys;
 extern struct keylist save_append_overwrite_keys;
+extern struct keylist scope_keys;
 extern struct keylist select_keys;
 extern struct keylist thread_keys;
+extern struct keylist url_keys;
 
 
 extern t_function global_mouse_action(t_function (*left_action) (void), t_function (*right_action) (void));
