@@ -3,10 +3,10 @@
  *  Module    : policy.h
  *  Author    : Ralf Doeblitz <doeblitz@gmx.de>
  *  Created   : 1999-01-12
- *  Updated   : 2006-08-16
+ *  Updated   : 2010-05-11
  *  Notes     : #defines and static data for policy configuration
  *
- * Copyright (c) 1999-2006 Ralf Doeblitz <doeblitz@gmx.de>
+ * Copyright (c) 1999-2010 Ralf Doeblitz <doeblitz@gmx.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -115,6 +115,7 @@
  * .ke  Kenya                          .kg  Kyrgystan
  * .kh  Cambodia                       .ki  Kiribati
  * .km  Comoros                        .kn  Saint Kitts and Nevis
+ * .kp  Korea, Democratic People's Republic
  * .kr  Korea, Republic of             .kw  Kuwait
  * .ky  Cayman Islands                 .kz  Kazakhstan
  *
@@ -126,16 +127,16 @@
  * .lv  Latvia                         .ly  Libyan Arab Jamahiriya
  *
  * .ma  Morocco                        .mc  Monaco
- * .md  Moldova, Republic of           .mg  Madagascar
- * .mh  Marshall Islands               .mk  Macedonia
- * .ml  Mali                           .mm  Myanmar
- * .mn  Mongolia                       .mo  Macau
- * .mp  Northern Mariana Islands       .mq  Martinique
- * .mr  Mauritania                     .ms  Montserrat
- * .mt  Malta                          .mu  Mauritius
- * .mv  Maldives                       .mw  Malawi
- * .mx  Mexico                         .my  Malaysia
- * .mz  Mozambique
+ * .md  Moldova, Republic of           .me  Montenegro
+ * .mg  Madagascar                     .mh  Marshall Islands
+ * .mk  Macedonia                      .ml  Mali
+ * .mm  Myanmar                        .mn  Mongolia
+ * .mo  Macau                          .mp  Northern Mariana Islands
+ * .mq  Martinique                     .mr  Mauritania
+ * .ms  Montserrat                     .mt  Malta
+ * .mu  Mauritius                      .mv  Maldives
+ * .mw  Malawi                         .mx  Mexico
+ * .my  Malaysia                       .mz  Mozambique
  *
  * .na  Namibia                        .nc  New Caledonia
  * .ne  Niger                          .nf  Norfolk Island
@@ -157,7 +158,8 @@
  * .qa  Qatar
  *
  * .re  Reunion                        .ro  Romania
- * .ru  Russian Federation             .rw  Rwanda
+ * .rs  Republic of Serbia             .ru  Russian Federation
+ * .rw  Rwanda
  *
  * .sa  Saudi Arabia                   .sb  Solomon Islands
  * .sc  Seychelles                     .sd  Sudan
@@ -165,10 +167,9 @@
  * .sh  St. Helena                     .si  Slovenia
  * .sk  Slovakia                       .sl  Sierra Leone
  * .sm  San Marino                     .sn  Senegal
- * .so  Somalia                        .sr  Surinam
- * .st  Sao Tome and Principe          .su  Soviet Union (former)
- * .sv  El Salvador                    .sy  Syrian Arab Republic
- * .sz  Swaziland
+ * .sr  Surinam                        .st  Sao Tome and Principe
+ * .su  Soviet Union (former)          .sv  El Salvador
+ * .sy  Syrian Arab Republic           .sz  Swaziland
  *
  * .tc  The Turks & Caicos Islands     .td  Chad
  * .tf  French Southern Territories    .tg  Togo
@@ -182,7 +183,6 @@
  *
  * .ua  Ukraine                        .ug  Uganda
  * .uk  United Kingdom
- * .um  United States Minor Outlying Islands
  * .us  United States                  .uy  Uruguay
  * .uz  Uzbekistan
  *
@@ -205,6 +205,7 @@
  * .cs  former Czechoslovakia, now .cz and .sk
  * .dd  former German Democratic Republic, now .de
  * .fx  France, Metropolitan
+ * .um  United States Minor Outlying Islands
  * .yd  Democratic Yemen, now .ye
  * .wg  West Bank and Gaza, now .ps
  * .zr  former Zaire, now .cd
@@ -213,18 +214,17 @@
  * .aa, .qm, .qz, .xa, .xz, .zz
  *
  * unused TLDs:
- * .bv  Bouvet Island
+ * .bv  Bouvet Island (Norway)
  * .eh  Western Sahara
- * .kp  Korea, Democratic People's Republic
+ * .gb  United Kingdom, use .uk
  * .nt  Neutral Zone
- * .sj  Svalbard and Jan Mayen Islands
+ * .sj  Svalbard and Jan Mayen Islands (Norway)
+ * .so  Somalia
  *
  * requested new TLDs:
- * .cs  Serbia and Montenegro
  * .ct  Catalonia
  *
  * xccTLDs:
- * .gb  United Kingdom, use .uk
  * .hk  Hong Kong, now .cn
  * .su  former USSR, now .ru
  * .tp  former East Timor, now .tl
@@ -239,21 +239,21 @@ static char gnksa_country_codes[26*26] = {
 /* D */ 0,0,0,0,1, 0,0,0,0,1, 1,0,1,0,1, 0,0,0,0,0, 0,0,0,0,0,1,
 /* E */ 0,0,1,0,1, 0,1,0,0,0, 0,0,0,0,0, 0,0,1,1,1, 1,0,0,0,0,0,
 /* F */ 0,0,0,0,0, 0,0,0,1,1, 1,0,1,0,1, 0,0,1,0,0, 0,0,0,0,0,0,
-/* G */ 1,1,0,1,1, 1,1,1,1,0, 0,1,1,1,0, 1,1,1,1,1, 1,0,1,0,1,0,
+/* G */ 1,0,0,1,1, 1,1,1,1,0, 0,1,1,1,0, 1,1,1,1,1, 1,0,1,0,1,0,
 /* H */ 0,0,0,0,0, 0,0,0,0,0, 1,0,1,1,0, 0,0,1,0,1, 1,0,0,0,0,0,
 /* I */ 0,0,0,1,1, 0,0,0,0,0, 0,1,1,1,1, 0,1,1,1,1, 0,0,0,0,0,0,
 /* J */ 0,0,0,0,1, 0,0,0,0,0, 0,0,1,0,1, 1,0,0,0,0, 0,0,0,0,0,0,
-/* K */ 0,0,0,0,1, 0,1,1,1,0, 0,0,1,1,0, 0,0,1,0,0, 0,0,1,0,1,1,
+/* K */ 0,0,0,0,1, 0,1,1,1,0, 0,0,1,1,0, 1,0,1,0,0, 0,0,1,0,1,1,
 /* L */ 1,1,1,0,0, 0,0,0,1,0, 1,0,0,0,0, 0,0,1,1,1, 1,1,0,0,1,0,
-/* M */ 1,0,1,1,0, 0,1,1,0,0, 1,1,1,1,1, 1,1,1,1,1, 1,1,1,1,1,1,
+/* M */ 1,0,1,1,1, 0,1,1,0,0, 1,1,1,1,1, 1,1,1,1,1, 1,1,1,1,1,1,
 /* N */ 1,0,1,0,1, 1,1,0,1,0, 0,1,0,0,1, 1,0,1,0,0, 1,0,0,0,0,1,
 /* O */ 0,0,0,0,0, 0,0,0,0,0, 0,0,1,0,0, 0,0,0,0,0, 0,0,0,0,0,0,
 /* P */ 1,0,0,0,1, 1,1,1,0,0, 1,1,1,1,0, 0,0,1,1,1, 0,0,1,0,1,0,
 /* Q */ 1,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0,0,
-/* R */ 0,0,0,0,1, 0,0,0,0,0, 0,0,0,0,1, 0,0,0,0,0, 1,0,1,0,0,0,
-/* S */ 1,1,1,1,1, 0,1,1,1,0, 1,1,1,1,1, 0,0,1,0,1, 1,1,0,0,1,1,
+/* R */ 0,0,0,0,1, 0,0,0,0,0, 0,0,0,0,1, 0,0,0,1,0, 1,0,1,0,0,0,
+/* S */ 1,1,1,1,1, 0,1,1,1,0, 1,1,1,1,0, 0,0,1,0,1, 1,1,0,0,1,1,
 /* T */ 0,0,1,1,0, 1,1,1,0,1, 1,1,1,1,1, 1,0,1,0,1, 0,1,1,0,0,1,
-/* U */ 1,0,0,0,0, 0,1,0,0,0, 1,0,1,0,0, 0,0,0,1,0, 0,0,0,0,1,1,
+/* U */ 1,0,0,0,0, 0,1,0,0,0, 1,0,0,0,0, 0,0,0,1,0, 0,0,0,0,1,1,
 /* V */ 1,0,1,0,1, 0,1,0,1,0, 0,0,0,1,0, 0,0,0,0,0, 1,0,0,0,0,0,
 /* W */ 0,0,0,0,0, 1,0,0,0,0, 0,0,0,0,0, 0,0,0,1,0, 0,0,0,0,0,0,
 /* X */ 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0,0,
@@ -277,6 +277,7 @@ static const char *gnksa_domain_list[] = {
 	"net",	/* Networks */
 	"org",	/* Organizations */
 	"pro",	/* Accountants, lawyers, and physicians */
+	"tel",	/* Internet communication */
 	"aero",	/* Air-transport industry */
 	"arpa",	/* Address and Routing Parameter Area */
 	"coop",	/* Non-profit cooperatives */
@@ -284,16 +285,18 @@ static const char *gnksa_domain_list[] = {
 	"jobs",	/* Human resource managers */
 	"mobi",	/* Mobile products and services */
 	"name",	/* For registration by individuals */
+	"asia",	/* Asia and the Pacific */
 	"museum",	/* Museums */
 	"travel",	/* Travel industry */
 	/*
-	 * more gTLDs to come >=2006, proposed are:
-	 *  .asia   .mail    .kids  (.berlin)
-	 *  .post   .tel     .xxx   (.sco)
+	 * more gTLDs to come >=2007, proposed are:
+	 *  .kids     .mail     .post
+	 *  (.berlin) (.sco)    (.bzh)    (.cym)    (.gal)
 	 */
 #	if 0		/* $DEAD */
 	"nato",
 	"uucp",
+	"csnet",
 	"bitnet",
 #	endif /* 0 */
 	/* the next four are defined in RFC 2606 */
@@ -307,6 +310,34 @@ static const char *gnksa_domain_list[] = {
 #	ifdef TINC_DNS
 	"bofh",	/* There Is No Cabal */
 #	endif /* TINC_DNS */
+	/* active IDN ccTLDs */
+	"xn--wgbh1c",		/* Egypt */
+	"xn--mgberp4a5d4ar",	/* Saudi Arabia */
+	"xn--mgbaam7a8h",	/* United Arab Emirates */
+#	if 0
+	/* purposed IDN ccTLDs */
+	"xn--fiqs8S",		/* China, Simplified Chinese */
+	"xn--fiqz9S",		/* China, Traditional Chinese */
+	"xn--j6w193g",		/* Hong Kong, Han */
+	"xn--mgbayh7gpa",	/* Jordan */
+	"xn--ygbi2ammx",	/* Palestinian Territory */
+	"xn--wgbl6a",		/* Qatar */
+	"xn--p1ai",			/* Russian Federation */
+	"xn--fzc2c9e2c",	/* Sri Lanka, Sinhala */
+	"xn--xkc2al3hye2a",	/* Sri Lanka, Tamil */
+	"xn--kpry57d",		/* Taiwan, Simplified Chinese */
+	"xn--kprw13d",		/* Taiwan, Traditional Chinese */
+	"xn--o3cw4h",		/* Thailand */
+	"xn--pgbs0dh",		/* Tunisia */
+	"xn--mgbaam7a8h",	/* United Arab Emirates */	
+#	endif /* 0 */
+#	if 0
+	/* Desired Variant String(s) */
+	"xn--mgberp4a5d4a87g",	/* Saudi Arabia */
+	"xn--mgbqly7c0a67fbc",	/* Saudi Arabia */
+	"xn--mgbqly7cvafr",	/* Saudi Arabia */
+	"xn--nnx388a",		/* Taiwan */
+#	endif /* 0 */
 	/* sentinel */
 	""
 };
