@@ -3,10 +3,10 @@
  *  Module    : tcurses.h
  *  Author    : Thomas Dickey
  *  Created   : 1997-03-02
- *  Updated   : 2004-07-19
+ *  Updated   : 2010-05-10
  *  Notes     : curses #include files, #defines & struct's
  *
- * Copyright (c) 1997-2006 Thomas Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2010 Thomas Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -95,6 +95,7 @@
 #		define HpGlitch(func)			/*nothing*/
 
 extern int cmdReadCh(void);
+extern int cmd_get_arrow_key(int prech);
 extern char *screen_contents(int row, int col, char *buffer);
 extern void MoveCursor(int row, int col);
 extern void my_erase(void);
@@ -118,7 +119,9 @@ extern void my_printf(const char *fmt, ...)
 #		endif /* __GNUC__ */
 	;
 extern void my_retouch(void);
-extern void refresh_color(void);
+#		ifdef HAVE_COLOR
+	extern void refresh_color(void);
+#		endif /* refresh_color */
 extern void write_line(int row, char *buffer);
 
 #	else	/* !USE_CURSES */
