@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-01-10
+ *  Updated   : 2010-03-14
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,9 +47,7 @@
 #		include	<autoconf.h>	/* FIXME: normally we use 'config.h' */
 #	endif /* !TIN_AUTOCONF_H */
 #else
-#	ifndef HAVE_CONFDEFS_H
-#		include	"oldconfig.h"
-#	endif /* !HAVE_CONFDEFS_H */
+#	error "configure run missing"
 #endif /* HAVE_CONFIG_H */
 
 
@@ -99,67 +97,11 @@
 
 #include	<signal.h>
 
-enum context { cMain, cArt, cConfig, cFilter, cGroup, cHelp, cInfopager, cPage, cSelect, cThread };
-enum icontext { cNone, cGetline, cPromptSLK };
+enum context { cMain, cArt, cAttrib, cConfig, cFilter, cGroup, cHelp, cInfopager, cPage, cReconnect, cScope, cSelect, cThread };
+enum icontext { cNone, cGetline, cPromptCONT, cPromptSLK, cPromptYN };
 enum resizer { cNo, cYes, cRedraw };
 enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
-#ifdef VMS
-#	ifdef __DECC
-#		include <unixio.h>
-#	else
-#		ifndef __VMS_VER	/* assume old types.h */
-			typedef unsigned short mode_t;
-#			undef HAVE_STRFTIME
-#		endif /* !__VMS_VER */
-#		include <stdio.h>
-#	endif /* __DECC */
-#	ifdef SOCKETSHR_TCP
-#		include <socketshr.h>
-#		include <unistd.h>
-#		ifndef SOCKETSHR_HAVE_DUP
-#			define dup
-#		endif /* !SOCKETSHR_HAVE_DUP */
-#		ifndef SOCKETSHR_HAVE_FERROR
-#			define ferror(a) (0)
-#		endif /* !SOCKETSHR_HAVE_FERROR */
-#	endif /* SOCKETSHR_TCP */
-#	include <curses.h>
-#	include <stat.h>
-#	undef HAVE_SELECT
-#	define XHDR_XREF	/* enable crosspost support */
-#	define CASE_PROBLEM
-#	define HAVE_ERRNO_H
-#	define NNTP_ONLY
-#	define NNTP_INEWS
-#	define DONT_HAVE_PIPING
-#	define NO_SHELL_ESCAPE
-#	define USE_CLEARSCREEN
-#	ifndef MM_CHARSET
-#		define MM_CHARSET "ISO-8859-1"
-#	endif /* !MM_CHARSET */
-	/*
-	 * Apparently this means fileops=create if not already there - no idea
-	 * why this should be needed. Standard fopen() implies this in arg 2
-	 */
-	extern char *get_uaf_fullname();
-#	ifdef MULTINET
-#		include "MULTINET_ROOT:[MULTINET.INCLUDE.SYS]TYPES.H"
-#		include "MULTINET_ROOT:[MULTINET.INCLUDE.SYS]TIME.H"
-#	else
-#		include "select.h"
-#		ifdef VAXC
-#			include "vmstimval.h"
-#		endif /* VAXC */
-#		include <types.h>
-#	endif /* !MULTINET */
-#	define FOPEN_OPTS		, "fop=cif"
-#	define JOINPATH		joindir
-#else
-#	define FOPEN_OPTS
-#	define JOINPATH		joinpath
-#endif /* VMS */
-
 #include <stdio.h>
 #ifdef HAVE_ERRNO_H
 #	include	<errno.h>
@@ -346,32 +288,21 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 /*
  * Directory handling code
  */
-#ifdef HAVE_CONFIG_H
-#	ifdef HAVE_DIRENT_H
-#		include <dirent.h>
-#		define DIR_BUF	struct dirent
-#	else
-#		ifdef HAVE_SYS_DIR_H
-#			include <sys/dir.h>
-#		endif /* HAVE_SYS_DIR_H */
-#		ifdef HAVE_SYS_NDIR_H
-#			include <sys/ndir.h>
-#		endif /* HAVE_SYS_NDIR_H */
-#		define DIR_BUF	struct direct
-#	endif /* HAVE_DIRENT_H */
+#ifdef HAVE_DIRENT_H
+#	include <dirent.h>
 #else
-#	ifdef M_XENIX
+#	ifdef HAVE_SYS_NDIR_H
 #		include <sys/ndir.h>
-#		define DIR_BUF	struct direct
-#	endif /* M_XENIX */
-#	ifdef VMS
-#		include "ndir.h"
-#		define DIR_BUF	struct direct
-#	endif /* VMS */
-#endif /* !HAVE_CONFIG_H */
+#	endif /* HAVE_SYS_NDIR_H */
+#	ifdef HAVE_SYS_DIR_H
+#		include <sys/dir.h>
+#	endif /* HAVE_SYS_DIR_H */
+#	if HAVE_NDIR_H
+#		include <ndir.h>
+#	endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
 
 #ifndef DIR_BUF
-#	include <dirent.h>
 #	define DIR_BUF	struct dirent
 #endif /* !DIR_BUF */
 
@@ -406,7 +337,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 /*
  * any pgp/gpp support possible and wanted
- * sort out possible conflicts: gpg is prefered over pgp5 over pgp
+ * sort out possible conflicts: gpg is preferred over pgp5 over pgp
  */
 #if defined(HAVE_PGP) || defined(HAVE_PGPK) || defined(HAVE_GPG)
 #	define HAVE_PGP_GPG 1
@@ -421,6 +352,12 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	endif /* HAVE_PGP && HAVE_GPG */
 #endif /* HAVE_PGP || HAVE_PGPK || HAVE_GPG */
 
+/*
+ * slrnface requires some things
+ */
+#if defined(HAVE_SLRNFACE) && defined(HAVE_MKFIFO) && defined(HAVE_FORK) && defined(HAVE_EXECLP) && defined(HAVE_WAITPID) && !defined(DONT_HAVE_PIPING) && !defined(X_DISPLAY_MISSING)
+#	define XFACE_ABLE
+#endif /* HAVE_SLRNFACE && HAVE_MKFIFO && HAVE_FORK && HAVE_EXECLP && HAVE_WAITPID && !DONT_HAVE_PIPING && !X_DISPLAY_MISSING */
 
 /*
  * Setup support for reading from NNTP
@@ -454,39 +391,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  */
 #define VT_ESCAPE_TIMEOUT	1
 
-/*
- * Specify News spool & control directories if not running NNTP_ONLY
- * (on machines who can run configure this is not needed)
- */
-#ifndef HAVE_CONFIG_H
-#	ifndef NNTP_ONLY
-#		ifndef SPOOLDIR
-#			ifdef VMS
-#				define SPOOLDIR	"NEWSSPOOL:[000000]"
-#			else
-#				define SPOOLDIR	"/var/spool/news"
-#			endif /* VMS */
-#		endif /* !SPOOLDIR */
-#		ifndef NEWSLIBDIR
-#			ifdef VMS
-#				define NEWSLIBDIR	"NEWSLIB:[000000]"
-#			else
-#				define NEWSLIBDIR	"/usr/lib/news"
-#			endif /* VMS */
-#		endif /* !NEWSLIBDIR */
-#		ifndef NOVROOTDIR
-#			define NOVROOTDIR	SPOOLDIR
-#		endif /* !NOVROOTDIR */
-#		ifndef INEWSDIR
-#			define INEWSDIR	NEWSLIBDIR
-#		endif /* INEWSDIR */
-#	else
-#		undef	SPOOLDIR
-#		undef	NEWSLIBDIR
-#		undef	NOVROOTDIR
-#	endif /* !NNTP_ONLY */
-#endif /* !HAVE_CONFIG_H */
-
 /*
  * Determine machine configuration for external programs & directories
  */
@@ -534,15 +438,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	ifdef linux
 #		define DEFAULT_PRINTER	"/usr/bin/lpr"
 #	endif /* linux */
-#	ifdef VMS
-#		define DEFAULT_EDITOR	"EDIT/TPU"
-#		define DEFAULT_MAILBOX	"SYS$LOGIN:"
-#		define DEFAULT_MAILER	"MAIL"
-#		define MAILER_FORMAT	"MAIL /SUBJECT=\"%S\" %F MX%%\"%T\""
-#		define DEFAULT_POSTER	"inews %s."
-#		define DEFAULT_PRINTER	"PRINT/DELETE"
-#		define DEFAULT_UNSHAR	"unshar %s."
-#	endif /* VMS */
 #	ifdef QNX42
 #		ifndef DEFAULT_EDITOR
 #			define DEFAULT_EDITOR	"/bin/vedit"
@@ -642,18 +537,13 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* SMALL_MEMORY_MACHINE */
 #define DEFAULT_ACTIVE_NUM	1800
 #define DEFAULT_NEWNEWS_NUM	5
+#define DEFAULT_MAPKEYS_NUM 100	/* ~remappable keys per level (avoid massiv reallocs) */
+#define DEFAULT_SCOPE_NUM	8
 
-#ifdef VMS
-#	define RCDIR	"TIN"
-#	define INDEX_MAILDIR	"MAILIDX"
-#	define INDEX_NEWSDIR	"INDEX"
-#	define INDEX_SAVEDIR	"SAVE"
-#else
-#	define RCDIR	".tin"
-#	define INDEX_MAILDIR	".mail"
-#	define INDEX_NEWSDIR	".news"
-#	define INDEX_SAVEDIR	".save"
-#endif /* VMS */
+#define RCDIR	".tin"
+#define INDEX_MAILDIR	".mail"
+#define INDEX_NEWSDIR	".news"
+#define INDEX_SAVEDIR	".save"
 
 #define ACTIVE_FILE	"active"
 #define ACTIVE_MAIL_FILE	"active.mail"
@@ -664,12 +554,12 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define SERVERCONFIG_FILE	"serverrc"
 #define DEFAULT_MAILDIR	"Mail"
 #define DEFAULT_SAVEDIR	"News"
-#define DEFAULT_URL_HANDLER "url_handler.sh"
+#define DEFAULT_URL_HANDLER "url_handler.pl"
 /* Prefixes saved attachments with no set filename */
 #define SAVEFILE_PREFIX		"unknown"
 
 
-/* MMDF-mailbox seperator */
+/* MMDF-mailbox separator */
 #ifndef MMDFHDRTXT
 #	define MMDFHDRTXT "\01\01\01\01\n"
 #endif /* MMDFHDRTXT */
@@ -699,13 +589,13 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* 0 */
 
 /* case sensitive && ^-anchored */
-#define DEFAULT_STRIP_RE_REGEX	"(?:R[eE](?:\\^\\d+|\\[\\d\\])?|A[wW]|Odp|Sv):\\s"
+#define DEFAULT_STRIP_RE_REGEX	"(?:R[eE](?:\\^\\d+|\\[\\d+\\])?|A[wW]|Odp|Sv):\\s"
 /* case sensitive */
-#define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):.*\\)\\s*$"
-#define DEFAULT_U8_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l\\x{0142}]o):.*\\)\\s*$"
+#define DEFAULT_STRIP_WAS_REGEX	"(?:(?<=\\S)|\\s)\\((?:[Ww]a[rs]|[Bb]y[l\\xb3]o):.*\\)\\s*$"
+#define DEFAULT_U8_STRIP_WAS_REGEX	"(?:(?<=\\S)|\\s)\\((?:[Ww]a[rs]|[Bb]y[l\\x{0142}]o):.*\\)\\s*$"
 /*
  * overkill regexp for balanced '()':
- * #define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):(?:(?:[^)(])*(?:\\([^)(]*\\))*)+\\)\\s*$"
+ * #define DEFAULT_STRIP_WAS_REGEX	"(?:(?<=\\S)|\\s)\\((?:[Ww]a[rs]|[Bb]y[l\xb3]o):(?:(?:[^)(])*(?:\\([^)(]*\\))*)+\\)\\s*$"
  */
 
 /* case sensitive & ^-anchored */
@@ -716,6 +606,9 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 /* case sensitive & ^-anchored */
 #define SHAR_REGEX	"\\#(?:!\\s?(?:/usr)?/bin/sh|\\s?(?i)this\\sis\\sa\\sshell\\sarchive)"
 
+/* slrn verbatim marks, case sensitive & ^-anchored */
+#define DEFAULT_VERBATIM_BEGIN_REGEX	"#v\\+"
+#define DEFAULT_VERBATIM_END_REGEX	"#v-"
 
 /*
  * URL related regexs:
@@ -768,8 +661,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 
 #define FILTER_FILE	"filter"
-/* editor offset for filter-file; TODO: doesn't match for german filter-file */
-#define FILTER_FILE_OFFSET	26
 #define INPUT_HISTORY_FILE	".inputhistory"
 #ifdef HAVE_MH_MAIL_HANDLING
 #	define MAILGROUPS_FILE	"mailgroups"
@@ -789,14 +680,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define NEWSGROUPS_FILE	"newsgroups"
 #define KEYMAP_FILE	"keymap"
 
-
-#ifdef USE_INN_NNTPLIB
-#	include <libinn.h> /* TODO: add configure chek for it */
-#	define _CONF_FROMHOST	"fromhost"
-#	define _CONF_ORGANIZATION	"organization"
-#	define _CONF_SERVER	"server"
-#endif /* USE_INN_NNTPLIB */
-
 #define SIGDASHES "-- \n"
 
 #ifndef BOOL_H
@@ -841,6 +724,12 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #ifdef HAVE_LIBUTF8_H
 #	include <libutf8.h>
 #else
+/*
+ * order is important here:
+ * - Solaris 2.5.1 requires wchar.h before wctype.h
+ * - Tru64 with Desktop Toolkit C requires stdio.h before wchar.h
+ * - BSD/OS 4.0.1 requires stddef.h, stdio.h and time.h before wchar.h
+ */
 #	ifdef HAVE_WCHAR_H
 #		include <wchar.h>
 #	endif /* HAVE_WCHAR_H */
@@ -875,11 +764,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  * HEADER_LEN  = max. size of a news/mail header-line
  * NEWSRC_LINE =
  */
-#if defined(VMS)
-#	define PATH_LEN	256
-#	define NAME_LEN	14
-#	define LEN	512
-#endif /* VMS */
 
 #ifdef M_UNIX
 #	ifdef PATH_MAX
@@ -982,6 +866,28 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define TINRC_CONFIRM_TO_QUIT	(tinrc.confirm_choice == 3 || tinrc.confirm_choice == 4 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 #define TINRC_CONFIRM_SELECT	(tinrc.confirm_choice == 2 || tinrc.confirm_choice == 5 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 
+/*
+ * defines for tinrc.auto_cc_bcc
+ */
+#define AUTO_CC					1
+#define AUTO_BCC				2
+#define AUTO_CC_BCC				3
+
+/*
+ * defines for tinrc.goto_next_unread
+ */
+#define NUM_GOTO_NEXT_UNREAD	4
+#define GOTO_NEXT_UNREAD_PGDN	1
+#define GOTO_NEXT_UNREAD_TAB	2
+
+/*
+ * defines for tinrc.trim_article_body
+ */
+#define NUM_TRIM_ARTICLE_BODY	8
+#define SKIP_LEADING			1
+#define SKIP_TRAILING			2
+#define COMPACT_MULTIPLE		4
+
 /*
  * MIME Encodings
  */
@@ -1038,7 +944,7 @@ enum {
 #define IGNORE_ART_THREAD(i)	(arts[i].thread != ART_UNTHREADED || (tinrc.kill_level == KILL_NOTHREAD && arts[i].killed))
 
 /*
- * Is this part text/plain ?
+ * Is this part text/plain?
  */
 #define IS_PLAINTEXT(x) \
 			(x->type == TYPE_TEXT && strcasecmp("plain", x->subtype) == 0)
@@ -1103,6 +1009,16 @@ enum {
 #define MIN_COLUMNS_ON_TERMINAL		50
 
 
+/*
+ * indicate given cmd-line options
+ */
+#define CMDLINE_GETART_LIMIT	1
+#define CMDLINE_MAILDIR			2
+#define CMDLINE_NNTPSERVER		4
+#define CMDLINE_SAVEDIR			8
+#define CMDLINE_USE_COLOR		16
+
+
 /*
  * used by feed_articles() & show_mini_help() & quick_filter & add_filter_rule
  */
@@ -1111,6 +1027,9 @@ enum {
 #define THREAD_LEVEL	3
 #define PAGE_LEVEL	4
 #define INFO_PAGER	5
+#define SCOPE_LEVEL	6
+#define CONFIG_LEVEL	7
+#define ATTRIB_LEVEL	8
 
 #define MINI_HELP_LINES		5
 
@@ -1120,12 +1039,8 @@ enum {
 #define FEED_SAVE		4
 #define FEED_AUTOSAVE	5
 #define FEED_REPOST		6
-
-#if 0
-#	define DEBUG_IO(x)	fprintf x
-#else
-#	define DEBUG_IO(x)	/* nothing */
-#endif /* 0 */
+#define FEED_MARK_READ		7
+#define FEED_MARK_UNREAD	8
 
 
 /*
@@ -1205,6 +1120,8 @@ enum {
 #define SORT_THREADS_BY_NOTHING			0
 #define SORT_THREADS_BY_SCORE_DESCEND	1
 #define SORT_THREADS_BY_SCORE_ASCEND	2
+#define SORT_THREADS_BY_LAST_POSTING_DATE_DESCEND	3
+#define SORT_THREADS_BY_LAST_POSTING_DATE_ASCEND	4
 
 /*
  * Different values of strip_bogus - the ways to handle bogus groups
@@ -1291,7 +1208,8 @@ enum {
 #	define EXIT_FAILURE	1	/* Failing exit status */
 #endif /* !EXIT_FAILURE */
 
-#define NNTP_ERROR_EXIT	2
+#define NEWS_AVAIL_EXIT 2
+#define NNTP_ERROR_EXIT	3
 
 /*
  * Assertion verifier
@@ -1307,15 +1225,11 @@ enum {
 
 #define ESC	27
 
-
 /*
- * return codes for change_config_file ()
+ * filter entrys expire after DEFAULT_FILTER_DAYS
  */
-#define NO_FILTERING		0
-#define FILTERING		1
 #define DEFAULT_FILTER_DAYS		28
 
-
 /*
  * art.thread
  */
@@ -1323,7 +1237,7 @@ enum {
 #define ART_EXPIRED		-2
 
 /*
- * Where does this belong ?? It is overloaded
+ * Where does this belong?? It is overloaded
  */
 #define ART_NORMAL		-1
 
@@ -1418,7 +1332,6 @@ enum {
 /*
  * used in checking article header before posting
  */
-#define NGLIMIT		20	/* Max. num. of crossposted groups before warning */
 #define MAX_COL		78	/* Max. line length before issuing a warning */
 #define MAX_SIG_LINES	4	/* Max. num. of signature lines before warning */
 
@@ -1435,6 +1348,17 @@ typedef unsigned char	t_bitmap;
  */
 #define MSGID_HASH_SIZE		2609
 
+/*
+ * cmd-line options
+ */
+struct t_cmdlineopts {
+	int getart_limit;			/* getart_limit */
+	char maildir[PATH_LEN];     /* maildir */
+	char nntpserver[PATH_LEN];  /* nntpserver */
+	char savedir[PATH_LEN];     /* savedir */
+	unsigned int args:5;		/* given options */
+};
+
 /*
  * Archive-Name: header
  */
@@ -1445,7 +1369,7 @@ struct t_archive {
 };
 
 /*
- *	struct t_msgid - message id
+ * struct t_msgid - message id
  */
 struct t_msgid {
 	struct t_msgid *next;		/* Next in hash chain */
@@ -1491,11 +1415,20 @@ struct t_article {
 	int score;			/* score article has reached after filtering */
 	unsigned int status:2;	/* 0 = read, 1 = unread, 2 = will return */
 	unsigned int killed:2;	/* 0 = not killed, 1 = killed, 2 = killed unread */
-	unsigned int zombie:1;	/* 1 = was alive (unread) before 'X' command */
-	unsigned int delete_it:1;	/* 1 = delete art when leaving group [mail group] */
+	t_bool zombie:1;	/* 1 = was alive (unread) before 'X' command */
+	t_bool delete_it:1;	/* 1 = delete art when leaving group [mail group] */
 	t_bool selected:1;	/* FALSE = not selected, TRUE = selected */
 	t_bool inrange:1;	/* TRUE = article selected via # range command */
 	t_bool matched:1;	/* TRUE = article matched regex in feed.c */
+	t_bool keep_in_base:1;	/* TRUE = keep (read) article in base[] (show_only_unread_arts) */
+};
+
+/*
+ * struct t_newsheader - holds an array of which news headers to [not] display
+ */
+struct t_newsheader {
+	char **header;	/* array of which headers to [not] display */
+	int num;		/* number of headers in array header */
 };
 
 /*
@@ -1504,62 +1437,196 @@ struct t_article {
 struct t_attribute {
 	char *maildir;				/* mail dir if other than ~/Mail */
 	char *savedir;				/* save dir if other than ~/News */
-	char *savefile;			/* save articles to specified file */
+	char *savefile;				/* save articles to specified file */
 	char *sigfile;				/* sig file if other than ~/.Sig */
+	char *date_format;			/* format string for the date display */
+	char *editor_format;		/* editor + parameters  %E +%N %F */
 	char *organization;			/* organization name */
+	char *fcc;					/* Fcc folder for mail */
 	char *followup_to;			/* where posts should be redirected */
-	char *quick_kill_scope;			/* quick filter kill scope */
-	char *quick_select_scope;		/* quick filter select scope */
+	char *quick_kill_scope;		/* quick filter kill scope */
+	char *quick_select_scope;	/* quick filter select scope */
 	char *mailing_list;			/* mail list email address */
+	char *news_headers_to_display;	/* which headers to display */
+	char *news_headers_to_not_display;	/* which headers to not display */
 	char *x_headers;			/* extra headers for message header */
-	char *x_body;				/* bolierplate text for message body */
-	char *from;				/* from line */
-	char *news_quote_format;		/* another way to begin a posting format */
+	char *x_body;				/* boilerplate text for message body */
+	char *from;					/* from line */
+	char *news_quote_format;	/* another way to begin a posting format */
 	char *quote_chars;			/* string to precede quoted text on each line */
 	char *mime_types_to_save;	/* MIME content major/minors we want to save */
 #ifdef HAVE_ISPELL
-	char *ispell;			/* path to ispell and options */
+	char *ispell;				/* path to ispell and options */
 #endif /* HAVE_ISPELL */
+#ifdef CHARSET_CONVERSION
+	char *undeclared_charset;		/* charset of articles without MIME charset declaration */
+	int mm_network_charset;			/* network charset */
+#endif /* CHARSET_CONVERSION */
+	struct t_newsheader *headers_to_display;	/* array of which headers to display */
+	struct t_newsheader *headers_to_not_display;	/* array of which headers to not display */
 	unsigned global:1;			/* global/group specific */
-	unsigned quick_kill_header:3;		/* quick filter kill header */
-	unsigned quick_kill_expire:1;		/* quick filter kill limited/unlimited time */
+	unsigned quick_kill_header:3;	/* quick filter kill header */
+	unsigned quick_kill_expire:1;	/* quick filter kill limited/unlimited time */
 	unsigned quick_kill_case:1;		/* quick filter kill case sensitive? */
-	unsigned quick_select_header:3;		/* quick filter select header */
-	unsigned quick_select_expire:1;		/* quick filter select limited/unlimited time */
-	unsigned quick_select_case:1;		/* quick filter select case sensitive? */
+	unsigned quick_select_header:3;	/* quick filter select header */
+	unsigned quick_select_expire:1;	/* quick filter select limited/unlimited time */
+	unsigned quick_select_case:1;	/* quick filter select case sensitive? */
+	unsigned add_posted_to_filter:1;	/* add posted articles to filter */
+	unsigned advertising:1;			/* add User-Agent: -header */
+	unsigned alternative_handling:1;	/* skip multipart/alternative parts */
+	unsigned ask_for_metamail:1;	/* ask before using MIME viewer */
+	unsigned auto_cc_bcc:2;			/* add your name to cc/bcc automatically */
+	unsigned auto_list_thread:1;	/* list thread when entering it using right arrow */
 	unsigned auto_select:1;			/* 0=show all unread, 1='X' just hot arts */
 	unsigned auto_save:1;			/* 0=none, 1=save */
 	unsigned batch_save:1;			/* 0=none, 1=save -S/mail -M */
-	unsigned delete_tmp_files:1;		/* 0=leave, 1=delete */
-	unsigned show_only_unread:1;		/* 0=all, 1=only unread */
-	unsigned thread_arts:3;			/* 0=unthread, 1=subject, 2=refs, 3=both, 4=multipart, 5=percentage */
-	unsigned thread_perc:7;			/* percentage thrreading threshold */
+	unsigned delete_tmp_files:1;	/* 0=leave, 1=delete */
+	unsigned group_catchup_on_exit:1;	/* ask if read groups are to be marked read */
+	unsigned mail_8bit_header:1;	/* allow 8bit chars. in header of mail message */
+	unsigned mail_mime_encoding:2;
+	unsigned mark_ignore_tags:1;	/* Ignore tags for GROUP_MARK_THREAD_READ/THREAD_MARK_ARTICLE_READ */
+	unsigned mark_saved_read:1;		/* mark saved article/thread as read */
+	unsigned pos_first_unread:1;	/* position cursor at first/last unread article */
+	unsigned post_8bit_header:1;	/* allow 8bit chars. in header when posting to newsgroup */
+	unsigned post_mime_encoding:2;
+	unsigned post_process_view:1;	/* set TRUE to invoke mailcap viewer app */
+#ifndef DISABLE_PRINTING
+	unsigned print_header:1;		/* print all of mail header or just Subject: & From lines */
+#endif /* !DISABLE_PRINTING */
+	unsigned process_only_unread:1;	/* save/print//mail/pipe unread/all articles */
+	unsigned prompt_followupto:1;	/* display empty Followup-To header in editor */
+	unsigned show_only_unread_arts:1;	/* 0=all, 1=only unread */
+	unsigned sigdashes:1;			/* set TRUE to prepend every signature with dashes */
+	unsigned signature_repost:1;	/* set TRUE to add signature when reposting articles */
+	unsigned start_editor_offset:1;	/* start editor with line offset */
+	unsigned thread_articles:3;			/* 0=unthread, 1=subject, 2=refs, 3=both, 4=multipart, 5=percentage */
+	unsigned thread_catchup_on_exit:1;	/* catchup thread with left arrow key or not */
+	unsigned thread_perc:7;			/* percentage threading threshold */
 	unsigned show_author:2;			/* 0=none, 1=name, 2=addr, 3=both */
 	unsigned show_info:2;			/* 0=none, 1=lines, 2=score, 3=both */
-	unsigned sort_art_type:4;		/* 0=none, 1=subj descend, 2=subj ascend,
+	unsigned show_signatures:1;		/* 0=none, 1=show signatures */
+	unsigned trim_article_body:3;	/* 0=Don't trim article body, 1=Skip leading blank lines,
+						2=Skip trailing blank lines, 3=Skip leading and trailing blank lines,
+						4=Compact multiple blank lines between textblocks,
+						5=Compact multiple blank lines between textblocks and skip leading blank lines,
+						6=Compact multiple blank lines between textblocks and skip trailing blank lines,
+						7=Compact multiple blank lines between textblocks and skip leading and trailing
+						  blank lines */
+	unsigned verbatim_handling:1;	/* 0=none, 1=detect verbatim blocks */
+	unsigned wrap_on_next_unread:1;	/* Wrap around threads when searching next unread article */
+	unsigned sort_article_type:4;		/* 0=none, 1=subj descend, 2=subj ascend,
 						   3=from descend, 4=from ascend,
 						   5=date descend, 6=date ascend,
 						   7=score descend, 8=score ascend */
-	unsigned sort_threads_type:2;	/* 0=none, 1=score descend, 2=score ascend */
-	unsigned int post_proc_type:2;		/* 0=none, 1=shar, 2=uudecode */
-	unsigned int x_comment_to:1;		/* insert X-Comment-To: in Followup */
-	unsigned int tex2iso_conv:1;		/* Convert TeX2ISO */
-	unsigned int mime_forward:1;		/* forward articles as attachment or inline */
-	char *fcc;							/* Fcc folder for mail */
-#ifdef CHARSET_CONVERSION
-	int mm_network_charset;				/* network charset */
-	char *undeclared_charset;			/* charset of articles without MIME charset declaration */
-#endif /* CHARSET_CONVERSION */
+	unsigned sort_threads_type:3;	/* 0=none, 1=score descend, 2=score ascend,
+						   3=last posting date descend, 4=last posting date ascend */
+	unsigned post_process_type:2;	/* 0=none, 1=shar, 2=uudecode */
+	unsigned x_comment_to:1;	/* insert X-Comment-To: in Followup */
+	unsigned tex2iso_conv:1;	/* Convert TeX2ISO */
+	unsigned mime_forward:1;	/* forward articles as attachment or inline */
 };
 
 /*
- * TODO: turn longs to unsigned long long or at least unsigned long
+ * struct t_attribute_state - holds additional information
+ * about numeric attributes within a scope
+ */
+struct t_attribute_state {
+	unsigned add_posted_to_filter:1;
+	unsigned advertising:1;
+	unsigned alternative_handling:1;
+	unsigned ask_for_metamail:1;
+	unsigned auto_cc_bcc:1;
+	unsigned auto_list_thread:1;
+	unsigned auto_save:1;
+	unsigned auto_select:1;
+	unsigned batch_save:1;
+	unsigned date_format:1;
+	unsigned delete_tmp_files:1;
+	unsigned editor_format:1;
+	unsigned fcc:1;
+	unsigned followup_to:1;
+	unsigned from:1;
+	unsigned group_catchup_on_exit:1;
+#ifdef HAVE_ISPELL
+	unsigned ispell:1;
+#endif /* HAVE_ISPELL */
+	unsigned mail_8bit_header:1;
+	unsigned mail_mime_encoding:1;
+	unsigned maildir:1;
+	unsigned mailing_list:1;
+	unsigned mark_ignore_tags:1;
+	unsigned mark_saved_read:1;
+	unsigned mime_forward:1;
+	unsigned mime_types_to_save:1;
+	unsigned news_headers_to_display:1;
+	unsigned news_headers_to_not_display:1;
+	unsigned news_quote_format:1;
+	unsigned organization:1;
+	unsigned pos_first_unread:1;
+	unsigned post_8bit_header:1;
+	unsigned post_mime_encoding:1;
+	unsigned post_process_view:1;
+	unsigned post_process_type:1;
+#ifndef DISABLE_PRINTING
+	unsigned print_header:1;
+#endif /* !DISABLE_PRINTING */
+	unsigned process_only_unread:1;
+	unsigned prompt_followupto:1;
+	unsigned quick_kill_case:1;
+	unsigned quick_kill_expire:1;
+	unsigned quick_kill_header:1;
+	unsigned quick_kill_scope:1;
+	unsigned quick_select_case:1;
+	unsigned quick_select_expire:1;
+	unsigned quick_select_header:1;
+	unsigned quick_select_scope:1;
+	unsigned quote_chars:1;
+	unsigned savedir:1;
+	unsigned savefile:1;
+	unsigned show_author:1;
+	unsigned show_info:1;
+	unsigned show_only_unread_arts:1;
+	unsigned show_signatures:1;
+	unsigned sigdashes:1;
+	unsigned sigfile:1;
+	unsigned signature_repost:1;
+	unsigned sort_article_type:1;
+	unsigned sort_threads_type:1;
+	unsigned start_editor_offset:1;
+	unsigned tex2iso_conv:1;
+	unsigned thread_articles:1;
+	unsigned thread_catchup_on_exit:1;
+	unsigned thread_perc:1;
+	unsigned trim_article_body:1;
+#ifdef CHARSET_CONVERSION
+	unsigned undeclared_charset:1;
+	unsigned mm_network_charset:1;
+#endif /* CHARSET_CONVERSION */
+	unsigned verbatim_handling:1;
+	unsigned wrap_on_next_unread:1;
+	unsigned x_body:1;
+	unsigned x_comment_to:1;
+	unsigned x_headers:1;
+};
+
+/*
+ * struct t_scope
+ */
+struct t_scope {
+	char *scope;				/* scope for these group attributes */
+	struct t_attribute *attribute;	/* the attributes itself */
+	struct t_attribute_state *state;	/* additional information about numeric attributes */
+	unsigned global:1;			/* TRUE for scopes from global_attributes_file */
+};
+
+/*
+ * TODO: turn longs to int_least64_t
  */
 /*
  * struct t_newsrc - newsrc related info.
  */
 struct t_newsrc {
-	t_bool present:1;		/* update newsrc ? */
+	t_bool present:1;		/* update newsrc? */
 	long num_unread;		/* unread articles in group */
 	long xmax;			/* newsrc max */
 	long xmin;			/* newsrc min */
@@ -1568,13 +1635,13 @@ struct t_newsrc {
 };
 
 /*
- * TODO: turn longs to unsigned long long or at least unsigned long
+ * TODO: turn longs to int_least64_t
  */
 /*
  * struct t_group - newsgroup info from active file
  */
 struct t_group {
-	char *name;			/* newsgroup / mailbox name */
+	char *name;			/* newsgroup/mailbox name */
 	char *aliasedto;		/* =new.group in active file, NULL if not */
 	char *description;	/* text from NEWSLIBDIR/newsgroups file */
 	char *spooldir;		/* groups spool directory */
@@ -1582,7 +1649,7 @@ struct t_group {
 	long count;			/* article number count */
 	long xmax;			/* max. article number */
 	long xmin;			/* min. article number */
-	unsigned int type:4;		/* grouptype - newsgroup / mailbox / savebox */
+	unsigned int type:4;		/* grouptype - newsgroup/mailbox/savebox */
 	t_bool inrange:1;		/* TRUE if group selected via # range command */
 	t_bool read_during_session:1;	/* TRUE if group entered during session */
 	t_bool art_was_posted:1;	/* TRUE if art was posted to group */
@@ -1620,7 +1687,7 @@ struct t_hashnode {
  * future directions in adding other retrieval methods to present kill &
  * auto selection.
  *
- * Also seperate kill/select screen to allow ^K=kill ^A=auto-select
+ * Also separate kill/select screen to allow ^K=kill ^A=auto-select
  */
 struct t_filters {
 	int max;
@@ -1653,7 +1720,7 @@ struct t_filter {
 	char *xref;			/* groups in xref line */
 	time_t time;			/* expire time in seconds */
 	struct t_filter *next;		/* next rule valid in group */
-	unsigned int inscope:4;		/* if group matches scope ie. 'comp.os.*' */
+	unsigned int inscope:4;		/* if group matches scope e.g. 'comp.os.*' */
 	unsigned int icase:2;		/* Case sensitive filtering */
 	unsigned int fullref:4;		/* use full references or last entry only */
 };
@@ -1754,7 +1821,7 @@ struct t_option {
 	int *variable;		/* ptr to variable to change */
 	constext **opt_list;	/* ptr to list entries if OPT_LIST */
 	int opt_count;		/* no. of list entries if OPT_LIST */
-	struct opttxt *txt;	/* ptr to information / help on option */
+	struct opttxt *txt;	/* ptr to information/help on option */
 };
 
 /*
@@ -1804,7 +1871,7 @@ typedef struct _TIMEINFO {
 /*
  * mailcap fields
  * the x-token field is missing, we would need something like
- * struct t_xtoken { char *xtoken; t_xtoken *next; } for that...
+ * struct t_xtoken { char *xtoken; t_xtoken *next; } for that ...
  */
 typedef struct {
 	char *type;		/* content-type, mandatory */
@@ -1851,11 +1918,7 @@ typedef int (*t_compfunc)(t_comptype, t_comptype);
 #define _CDECL
 
 /* Seperator between dir part of path & the filename */
-#ifdef VMS
-#	define DIRSEP	']'
-#else
-#	define DIRSEP	'/'
-#endif /* VMS */
+#define DIRSEP	'/'
 
 /*
  * mouse buttons for use in xterm
@@ -1866,20 +1929,6 @@ typedef int (*t_compfunc)(t_comptype, t_comptype);
 
 #define TIN_EDITOR_FMT_OFF		"%E %F"
 
-#ifdef VMS
-#	define REDIRECT_OUTPUT	""
-#	define REDIRECT_PGP_OUTPUT	""
-#	define ENV_VAR_MAILER		"TIN_MAILER"
-/*#	define ENV_VAR_SHELL		"SHELL"*/
-#	define ENV_VAR_POSTER		"TIN_POST"
-#	define TIN_EDITOR_FMT_ON	"%E %F"
-#	define TMPDIR "SYS$SCRATCH:"
-#	ifdef HAVE_KEY_PREFIX
-#		define KEY_PREFIX	0x9b
-#	endif /* HAVE_KEY_PREFIX */
-extern void joindir (char *result, const char *dir, const char *file);
-#endif /* VMS */
-
 #ifdef M_UNIX
 #	define REDIRECT_OUTPUT		"> /dev/null 2>&1"
 #	define REDIRECT_PGP_OUTPUT		"> /dev/null"
@@ -1918,9 +1967,9 @@ extern void joindir (char *result, const char *dir, const char *file);
 #endif /* !TMPDIR */
 
 #if !defined(S_ISDIR)
-#	if defined(M_UNIX) || defined(VMS)
+#	if defined(M_UNIX)
 #		define S_ISDIR(m)	(((m) & S_IFMT) == S_IFDIR)
-#	endif /* M_UNIX || VMS */
+#	endif /* M_UNIX */
 #endif /* !S_ISDIR */
 
 #if !defined(S_ISREG)
@@ -1964,11 +2013,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #endif /* !S_ISVTX */
 
 #ifdef DONT_HAVE_PIPING
-#	ifdef VMS
-#		define TIN_PRINTFILE "Sys$Scratch:TIN_PRINT%d.TMP"
-#	else
-#		define TIN_PRINTFILE "tinprint%d.tmp"
-#	endif /* VMS */
+#	define TIN_PRINTFILE "tinprint%d.tmp"
 #endif /* DONT_HAVE_PIPING */
 
 /*
@@ -1988,7 +2033,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #endif /* !F_OK */
 
 /* Various function redefinitions */
-#ifdef USE_DBMALLOC
+#if defined(USE_DBMALLOC) || defined(USE_DMALLOC)
 #	define my_malloc(size)	malloc(size)
 #	define my_calloc(nmemb, size)	calloc((nmemb), (size))
 #	define my_realloc(ptr, size)	realloc((ptr), (size))
@@ -1996,7 +2041,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #	define my_malloc(size)	my_malloc1(__FILE__, __LINE__, (size))
 #	define my_calloc(nmemb, size)	my_calloc1(__FILE__, __LINE__, (nmemb), (size))
 #	define my_realloc(ptr, size)	my_realloc1(__FILE__, __LINE__, (ptr), (size))
-#endif /* USE_DBMALLOC */
+#endif /* USE_DBMALLOC || USE_DMALLOC */
 
 #define ARRAY_SIZE(array)	((int) (sizeof(array) / sizeof(array[0])))
 
@@ -2005,8 +2050,8 @@ extern void joindir (char *result, const char *dir, const char *file);
 
 #define BlankIfNull(p)	((p) ? (p) : "")
 
-#define my_group_find(x)	add_my_group(x, FALSE)
-#define my_group_add(x)		add_my_group(x, TRUE)
+#define my_group_find(x)	add_my_group(x, FALSE, FALSE)
+#define my_group_add(x, y)		add_my_group(x, TRUE, y)
 #define for_each_group(x)	for (x = 0; x < num_active; x++)
 #define for_each_art(x)		for (x = 0; x < top_art; x++)
 #define for_each_art_in_thread(x, y)	for (x = (int) base[y]; x >= 0; x = arts[x].thread)
@@ -2092,6 +2137,9 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #ifdef USE_DMALLOC
 #	include <dmalloc.h>
 #	define DMALLOC_FUNC_CHECK
+#	ifdef HAVE_STRDUP
+#		define my_strdup(s) strdup((s))
+#	endif /* HAVE_STRDUP */
 #endif /* USE_DMALLOC */
 
 #ifdef DOALLOC
@@ -2114,21 +2162,16 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 	extern void	no_leaks(void);
 #endif /* DOALLOC */
 
-#ifdef __DECC		/* VMS */
-#	ifndef ferror
-#		define ferror(x)		(0)
-#		define EndWin		EndWind
-#	endif /* !ferror */
-#endif /* __DECC */
-
+#if 0 /* unused */
 #ifndef my_tmpfile_only
 /*
- * shortcut if we arn't interrested in the tmpfiles filename/location
+ * shortcut if we aren't interested in the tmpfiles filename/location
  * argument can't be a pointer and if argument is changed on return
- * we must unlik the tmp-file ourself
+ * we must unlink the tmp-file ourself
  */
 #	define my_tmpfile_only(a)	my_tmpfile(a, sizeof(a) - 1, FALSE, (char *) 0)
 #endif /* !my_tmpfile_only */
+#endif /* 0 */
 
 
 /* define some standard places to look for a tin.defaults file */
@@ -2166,7 +2209,7 @@ extern struct tm *localtime(time_t *);
 #ifdef CLOSEDIR_VOID
 #	define CLOSEDIR(DIR)	closedir(DIR)
 #else
-#	define CLOSEDIR(DIR)	if (closedir(DIR)) error_message("closedir() failed: %s %s", __FILE__, __LINE__)
+#	define CLOSEDIR(DIR)	if (closedir(DIR)) error_message(2, "closedir() failed: %s %s", __FILE__, __LINE__)
 #endif /* CLOSEDIR_VOID */
 
 #ifdef HAVE_GETTIMEOFDAY
@@ -2191,9 +2234,14 @@ extern struct tm *localtime(time_t *);
 
 /* libcanlock */
 #ifdef USE_CANLOCK
-#	include "../libcanlock/canlock.h"
+#	include "../libcanlock/include/canlock.h"
 #endif /* USE_CANLOCK */
 
+/* gsasl */
+#ifdef USE_SASL
+#	include <gsasl.h>
+#endif /* USE_SASL */
+
 /* snprintf(), vsnprintf() */
 #ifndef HAVE_SNPRINTF
 #	define snprintf	plp_snprintf
@@ -2228,14 +2276,25 @@ extern struct tm *localtime(time_t *);
 #endif /* !EOF */
 
 /* various filenames used by tin */
-#ifndef VMS	/* TODO: is the VMS case needed? */
-#	define TIN_ARTICLE_NAME	".article"
-#	define TIN_CANCEL_NAME	".cancel"
-#	define TIN_LETTER_NAME	".letter"
+#define TIN_ARTICLE_NAME	".article"
+#define TIN_CANCEL_NAME	".cancel"
+#define TIN_LETTER_NAME	".letter"
+#define TIN_BUGREPORT_NAME	".bugreport"
+
+/* read_news_active_file() / open_newsgroups_fp() */
+#ifndef DISABLE_PIPELINING
+#	define PIPELINE_LIMIT 45
 #else
-#	define TIN_ARTICLE_NAME	"article."
-#	define TIN_CANCEL_NAME	"cancel."
-#	define TIN_LETTER_NAME	"letter."
-#endif /* !VMS */
+#	define PIPELINE_LIMIT 1
+#endif /* DISABLE_PIPELINING */
+
+#ifndef DEBUG_H
+#	include "debug.h"
+#endif /* !DEBUG_H */
+
+struct t_overview_fmt {
+	char *name;
+	enum f_type type;
+};
 
 #endif /* !TIN_H */
