@@ -3,10 +3,10 @@
  *  Module    : tnntp.h
  *  Author    : Thomas Dickey <dickey@invisible-island.net>
  *  Created   : 1997-03-05
- *  Updated   : 2003-09-19
+ *  Updated   : 2006-02-15
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2006 Thomas Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2010 Thomas Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,49 +38,27 @@
 #ifndef TNNTP_H
 #define TNNTP_H 1
 
-#if defined(VMS) && defined(SOCKETSHR_TCP)
-#	ifdef __GNUC__
-#		define __SOCKET_TYPEDEFS
-#	endif /* __GNUC__ */
-#	include <netdb.h>
-#	include <in.h>
-#	include <inet.h>
-#	include <socket.h>
-#	define s_printf	si_fprintf
-#	define s_fdopen	si_fdopen
-#	define s_flush	si_fflush
-#	define s_fclose	si_fclose
-#	define s_gets	si_fgets
-#	define s_close	si_close
-#	define s_puts	si_fputs
-#	define s_dup
-#	define s_init()	(1)
-#	define s_end()
-#else
-#	define s_printf	fprintf
-#	define s_fdopen	fdopen
-#	define s_flush	fflush
-#	define s_fclose	fclose
-#	define s_gets	fgets
-#	if 0 /* __BEOS__ port in progress */
-#		ifdef HAVE_CLOSESOCKET
-#			define s_close closesocket
-#		else
-#			define s_close	close
-#		endif /* HAVE_CLOSESOCKET */
-#		ifdef __BEOS__
-#			define s_puts(s,fd)	write(fileno(fd),s,strlen(s))
-#		else
-#			define s_puts(s,fd)	fputs(s,fd)
-#		endif /* __BEOS__ */
+#define s_fdopen	fdopen
+#define s_flush	fflush
+#define s_fclose	fclose
+#define s_gets	fgets
+#if 0 /* __BEOS__ port in progress */
+#	ifdef HAVE_CLOSESOCKET
+#		define s_close	closesocket
+#	else
+#		define s_close	close
+#	endif /* HAVE_CLOSESOCKET */
+#	ifdef __BEOS__
+#		define s_puts(s,fd)	write(fileno(fd),s,strlen(s))
 #	else
-#		define s_close close
 #		define s_puts	fputs
-#	endif /* 0 */
-#	define s_dup		dup
-#	define s_init()	(1)
-#	define s_end()
-#endif /* VMS && SOCKETSHR_TCP */
+#	endif /* __BEOS__ */
+#else
+#	define s_close	close
+#	define s_puts	fputs
+#endif /* 0 */
+#define s_dup	dup
+#define s_end()
 
 #if defined(NNTP_ABLE) || defined(HAVE_GETHOSTBYNAME)
 #	ifdef HAVE_NETDB_H
@@ -110,47 +88,22 @@
 #			include	<netinet/in.h>
 #		endif /* HAVE_NETINET_IN_H */
 #	else
-#		ifdef VMS
-#			ifdef MULTINET
-#				include "MULTINET_ROOT:[multinet.include]errno.h"
-#				include "MULTINET_ROOT:[multinet.include]netdb.h"
-#				include "MULTINET_ROOT:[multinet.include.vms]inetiodef.h"
-#				include "MULTINET_ROOT:[multinet.include.sys]socket.h"
-#				include "MULTINET_ROOT:[multinet.include.netinet]in.h"
-#				define netopen	socket_open
-#				define netread	socket_read
-#				define netwrite socket_write
-#				define netclose socket_close
-#			else
-#				ifdef UCX
-#					include <errno.h>
-#					include <iodef.h>
-#					include <in.h>
-#					include <socket.h>
-#					define netopen	open
-#					define netread	read
-#					define netwrite	write
-#					define netclose	close
-#				endif /* UCX */
-#			endif /* MULTINET */
+#		ifdef HAVE_SYS_SOCKET_H
+#			include <sys/socket.h>
 #		else
-#			ifdef HAVE_SYS_SOCKET_H
-#				include <sys/socket.h>
-#			else
-#				ifdef HAVE_SOCKET_H
-#					include <socket.h>
-#				endif /* HAVE_SOCKET_H */
-#			endif /* HAVE_SYS_SOCKET_H */
-#			ifdef HAVE_NETINET_IN_H
-#				include <netinet/in.h>
-#			endif /* HAVE_NETINET_IN_H */
-#			ifdef HAVE_NETLIB_H
-#				include <netlib.h>
-#			endif /* HAVE_NETLIB_H */
-#			ifdef HAVE_ARPA_INET_H
-#				include <arpa/inet.h>
-#			endif /* HAVE_ARPA_INET_H */
-#		endif /* VMS */
+#			ifdef HAVE_SOCKET_H
+#				include <socket.h>
+#			endif /* HAVE_SOCKET_H */
+#		endif /* HAVE_SYS_SOCKET_H */
+#		ifdef HAVE_NETINET_IN_H
+#			include <netinet/in.h>
+#		endif /* HAVE_NETINET_IN_H */
+#		ifdef HAVE_NETLIB_H
+#			include <netlib.h>
+#		endif /* HAVE_NETLIB_H */
+#		ifdef HAVE_ARPA_INET_H
+#			include <arpa/inet.h>
+#		endif /* HAVE_ARPA_INET_H */
 #	endif /* TLI */
 
 #	ifdef EXCELAN
