@@ -3,10 +3,10 @@
  *  Module    : version.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2004-10-19
+ *  Updated   : 2008-11-08
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,30 +40,21 @@
 
 #	define PRODUCT		"tin"
 #	ifndef TIN_AUTOCONF_H
-#		define VERSION		"1.8.3"
+#		define VERSION		"1.9.6"
 #	endif /* !TIN_AUTOCONF_H */
-#	define RELEASEDATE	"20070201"
-#	define RELEASENAME	"Scotasay"
+#	define RELEASEDATE	"20101126"
+#	define RELEASENAME	"Burnside"
 /* config-file versions - must by dotted triples */
-#	define TINRC_VERSION	"1.3.7"
-#	define ATTRIBUTES_VERSION	"1.0.4"
+#	define TINRC_VERSION	"1.3.11"
+#	define ATTRIBUTES_VERSION	"1.0.7"
 #	define FILTER_VERSION	"1.0.0"
-#	define KEYMAP_VERSION	"1.0.6"
+#	define KEYMAP_VERSION	"1.0.8"
 #	define SERVERCONFIG_VERSION	"1.0.0"
 
-#	ifdef VMS
-#		define OSNAME	"VMS"
-#	endif /* VMS */
-
 #	ifdef M_UNIX
 #		ifdef __BEOS__
 #			define OSNAME	"BeOS"
 #		endif /* __BEOS__ */
-#		ifdef __CYGWIN__
-#			ifdef __WINNT__ /* also defined on Win95 *sigh* */
-#				define OSNAME	"Windows/NT"
-#			endif /*__WINNT__ */
-#		endif /* __CYGWIN__ */
 #		ifndef OSNAME
 #			define OSNAME	"UNIX"
 #		endif /* !OSNAME */
