@@ -3,10 +3,10 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2004-06-30
+ *  Updated   : 2010-03-15
  *  Notes     :
  *
- * Copyright (c) 1992-2005 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,16 +43,17 @@
 #endif /* !TCURSES_H */
 
 /*
- * List of allowed seperator chars in active file
- * unsed in parse_active_line()
+ * List of allowed separator chars in active file
+ * unused in parse_active_line()
  */
 #define ACTIVE_SEP	" \n"
 
-/*
- * if you are using C-News nntpd set NUM_SIMULTANEOUS_GROUP_COMMAND to 1
- */
 #ifdef NNTP_ABLE
-#	define NUM_SIMULTANEOUS_GROUP_COMMAND 50
+#	ifdef DISABLE_PIPELINING
+#		define NUM_SIMULTANEOUS_GROUP_COMMAND 1
+#	else
+#		define NUM_SIMULTANEOUS_GROUP_COMMAND 50
+#	endif /* DISABLE_PIPELINING */
 #endif /* NNTP_ABLE */
 
 t_bool force_reread_active_file = FALSE;
@@ -64,14 +65,17 @@ static time_t active_timestamp;	/* time active file read (local) */
  */
 static FILE *open_newgroups_fp(int idx);
 static FILE *open_news_active_fp(void);
+static int check_for_any_new_groups(void);
 static void active_add(struct t_group *ptr, long count, long max, long min, const char *moderated);
 static void append_group_line(char *active_file, char *group_path, long art_max, long art_min, char *base_dir);
-static void check_for_any_new_groups(void);
-static void make_group_list(char *active_file, char *base_dir, char *group_path);
-static void make_group_name(char *base_dir, char *group_name, char *group_path);
+static void make_group_list(char *active_file, char *base_dir, char *fixed_base, char *group_path);
 static void read_active_file(void);
 static void read_newsrc_active_file(void);
 static void subscribe_new_group(char *group, char *autosubscribe, char *autounsubscribe);
+#ifdef NNTP_ABLE
+	static t_bool parse_count_line(char *line, long *max, long *min, long *count, char *moderated);
+	static void read_active_counts(void);
+#endif /* NNTP_ABLE */
 
 
 t_bool
@@ -120,7 +124,7 @@ resync_active_file(
 	else
 		toggle_my_groups(old_group);
 
-	FreeAndNull(old_group);
+	FreeIfNeeded(old_group);
 	show_selection_page();
 
 	return TRUE;
@@ -193,7 +197,7 @@ process_bogus(
 	active_add(ptr, 0L, 1L, 0L, "n");
 	ptr->bogus = TRUE;		/* Mark it bogus */
 
-	if (my_group_add(name) < 0)
+	if (my_group_add(name, FALSE) < 0)
 		return TRUE;
 
 	return FALSE;		/* Nothing was printed yet */
@@ -210,18 +214,27 @@ parse_active_line(
 	long *min,
 	char *moderated)
 {
-	char *p, *q, *r;
+	char *p = NULL, *q = NULL, *r = NULL;
+	t_bool lineok = FALSE;
 
 	if (line[0] == '#' || line[0] == '\0')
 		return FALSE;
 
-	(void) strtok(line, ACTIVE_SEP);		/* skip group name */
-	p = strtok(NULL, ACTIVE_SEP);	/* group max count */
-	q = strtok(NULL, ACTIVE_SEP);	/* group min count */
-	r = strtok(NULL, ACTIVE_SEP);	/* mod status or path to mailgroup */
+	if (strtok(line, ACTIVE_SEP)) {		/* skip group name */
+		if ((p = strtok(NULL, ACTIVE_SEP))) {	/* group max count */
+			if ((q = strtok(NULL, ACTIVE_SEP))) {	/* group min count */
+				r = strtok(NULL, ACTIVE_SEP);	/* mod status or path to mailgroup */
+				lineok = TRUE;
+			}
+		}
+	}
 
-	if (!p || !q || !r) {
-		error_message(_(txt_bad_active_file), line);
+	if (!p || !q || !r || !lineok) {
+#ifdef DEBUG
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", txt_bad_active_file, line);
+#endif /* DEBUG */
+		error_message(2, _(txt_bad_active_file), line);
 		return FALSE;
 	}
 
@@ -233,6 +246,56 @@ parse_active_line(
 }
 
 
+#ifdef NNTP_ABLE
+/*
+ * Parse line from "LIST COUNTS"
+ * group high low count status, i.e.:
+ * trigofacile.test 326 6 297 y
+ */
+static t_bool
+parse_count_line(
+	char *line,
+	long *max,
+	long *min,
+	long *count,
+	char *moderated)
+{
+	char *p = NULL, *q = NULL, *r = NULL, *s = NULL;
+	t_bool lineok = FALSE;
+
+	if (line[0] == '#' || line[0] == '\0')
+		return FALSE;
+
+	if (strtok(line, ACTIVE_SEP)) {		/* skip group name */
+		if ((p = strtok(NULL, ACTIVE_SEP))) {	/* group max */
+			if ((q = strtok(NULL, ACTIVE_SEP))) {	/* group min */
+				if ((r = strtok(NULL, ACTIVE_SEP))) { /* group count */
+					s = strtok(NULL, ACTIVE_SEP);	/* mod status or path to mailgroup */
+					lineok = TRUE;
+				}
+			}
+		}
+	}
+
+	if (!p || !q || !r || !s || !lineok) {
+#	ifdef DEBUG
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", _(txt_unparseable_counts), line);
+#	endif /* DEBUG */
+		error_message(2, _(txt_unparseable_counts), line);
+		return FALSE;
+	}
+
+	*max = atol(p);
+	*min = atol(q);
+	*count = atol(r);
+	strcpy(moderated, s);
+
+	return TRUE;
+}
+#endif /* NNTP_ABLE */
+
+
 /*
  * Load the active information into active[] by counting the min/max/count
  * for each news group.
@@ -256,6 +319,7 @@ read_newsrc_active_file(
 	static char ngname[NNTP_STRLEN];
 	struct t_group *grpptr;
 #ifdef NNTP_ABLE
+	t_bool need_auth = FALSE;
 	char *ngnames[NUM_SIMULTANEOUS_GROUP_COMMAND];
 	int index_i = 0;
 	int index_o = 0;
@@ -273,7 +337,7 @@ read_newsrc_active_file(
 		return;
 	}
 
-	if (!batch_mode)
+	if (!batch_mode || verbose)
 		wait_message(0, _(txt_reading_news_newsrc_file));
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL || window != 0) {
@@ -293,22 +357,23 @@ read_newsrc_active_file(
 			ptr = ngname;
 		}
 
-		if (read_news_via_nntp) {
+		if (read_news_via_nntp && !read_saved_news) {
 #ifdef NNTP_ABLE
 			char buf[NNTP_STRLEN];
 			char line[NNTP_STRLEN];
-			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr) {
+			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr && (!list_active || (newsrc_active && list_active && group_find(ptr, FALSE)))) {
 				ngnames[index_i] = my_strdup(ptr);
 				snprintf(buf, sizeof(buf), "GROUP %s", ngnames[index_i]);
 #	ifdef DEBUG
-				debug_nntp("read_newsrc_active_file", buf);
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "read_newsrc_active_file() %s", buf);
 #	endif /* DEBUG */
 				put_server(buf);
 				index_i = (index_i + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
 				window++;
 			}
 			if (window == NUM_SIMULTANEOUS_GROUP_COMMAND || ptr == NULL) {
-				int respcode = get_respcode(line, sizeof(line));
+				int respcode = get_only_respcode(line, sizeof(line));
 
 				if (reconnected_in_last_get_server) {
 					/*
@@ -322,7 +387,8 @@ read_newsrc_active_file(
 					for (i = 0; i < window - 1; i++) {
 						snprintf(buf, sizeof(buf), "GROUP %s", ngnames[j]);
 #	ifdef DEBUG
-						debug_nntp("read_newsrc_active_file", buf);
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "read_newsrc_active_file() %s", buf);
 #	endif /* DEBUG */
 						put_server(buf);
 						j = (j + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -342,10 +408,20 @@ read_newsrc_active_file(
 							char fmt[20];
 
 							snprintf(fmt, sizeof(fmt), "%%ld %%ld %%ld %%%ds", NNTP_STRLEN);
-							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4)
-								error_message(_(txt_error_invalid_response_to_group), line);
-							if (strcmp(ngname, ngnames[index_o]) != 0)
-								error_message(_(txt_error_wrong_newsgroupname_in_group_response), line, ngnames[index_o]);
+							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4) {
+								error_message(2, _(txt_error_invalid_response_to_group), line);
+#	ifdef DEBUG
+								if (debug & DEBUG_NNTP) /* TODO: -> lang.c */
+									debug_print_file("NNTP", "Invalid response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
+#	endif /* DEBUG */
+							}
+							if (strcmp(ngname, ngnames[index_o]) != 0) {
+								error_message(2, _(txt_error_wrong_newsgroupname_in_group_response), ngname, ngnames[index_o], line);
+#	ifdef DEBUG
+								if (debug & DEBUG_NNTP) /* TODO: -> lang.c */
+									debug_print_file("NNTP", "Groupname mismatch in response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
+#	endif /* DEBUG */
+							}
 							ptr = ngname;
 							free(ngnames[index_o]);
 							index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -353,6 +429,12 @@ read_newsrc_active_file(
 							break;
 						}
 
+					case ERR_NOAUTH:
+					case NEED_AUTHINFO:
+						need_auth = TRUE; /* delay auth till end of loop */
+						/* keep lint quiet: */
+						/* FALLTHROUGH */
+
 					case ERR_NOGROUP:
 						free(ngnames[index_o]);
 						index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -360,14 +442,15 @@ read_newsrc_active_file(
 						continue;
 
 					case ERR_ACCESS:
-						error_message("%s%s", cCRLF, line);
+						error_message(2, "%s%s", cCRLF, line);
 						tin_done(NNTP_ERROR_EXIT);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
 
 					default:
 #	ifdef DEBUG
-						debug_nntp("NOT_OK", line);
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "NOT_OK %s", line);
 #	endif /* DEBUG */
 						free(ngnames[index_o]);
 						index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -395,7 +478,7 @@ read_newsrc_active_file(
 		if ((grpptr = group_add(ptr)) == NULL) {
 			t_bool changed = FALSE;
 
-			if ((grpptr = group_find(ptr)) == NULL)
+			if ((grpptr = group_find(ptr, FALSE)) == NULL)
 				continue;
 
 			if (max > grpptr->xmax) {
@@ -419,6 +502,15 @@ read_newsrc_active_file(
 		active_add(grpptr, count, max, min, moderated);
 	}
 
+#ifdef NNTP_ABLE
+	if (need_auth) { /* delayed auth */
+		if (!authenticate(nntp_server, userid, FALSE)) {
+			error_message(2, _(txt_auth_failed), ERR_ACCESS);
+			tin_done(EXIT_FAILURE);
+		}
+	}
+#endif /* NNTP_ABLE */
+
 	fclose(fp);
 
 	/*
@@ -426,14 +518,14 @@ read_newsrc_active_file(
 	 */
 	if (tin_errno || !num_active) {
 		if (newsrc_active && !num_active)
-			error_message(_(txt_error_server_has_no_listed_groups), newsrc);
+			error_message(2, _(txt_error_server_has_no_listed_groups), newsrc);
 		else
-			error_message(_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
+			error_message(2, _(txt_active_file_is_empty), (read_news_via_nntp ? (read_saved_news ? news_active_file : _(txt_servers_active)) : news_active_file));
 		tin_done(EXIT_FAILURE);
 	}
 
-	if (!batch_mode)
-		my_fputs("\n", stdout);
+	if (!batch_mode || verbose)
+		my_fputc('\n', stdout);
 }
 
 
@@ -467,7 +559,7 @@ read_active_file(
 	long processed = 0L;
 	struct t_group *grpptr;
 
-	if (!batch_mode)
+	if (!batch_mode || verbose)
 		wait_message(0, _(txt_reading_news_active_file));
 
 	if ((fp = open_news_active_fp()) == NULL) {
@@ -476,13 +568,13 @@ read_active_file(
 
 #ifdef NNTP_ABLE
 		if (read_news_via_nntp)
-			error_message(_(txt_cannot_retrieve), ACTIVE_FILE);
+			error_message(2, _(txt_cannot_retrieve), ACTIVE_FILE);
 #	ifndef NNTP_ONLY
 		else
-			error_message(_(txt_cannot_open_active_file), news_active_file, tin_progname);
+			error_message(2, _(txt_cannot_open_active_file), news_active_file, tin_progname);
 #	endif /* !NNTP_ONLY */
 #else
-		error_message(_(txt_cannot_open), news_active_file);
+		error_message(2, _(txt_cannot_open), news_active_file);
 #endif /* NNTP_ABLE */
 
 		tin_done(EXIT_FAILURE);
@@ -501,7 +593,7 @@ read_active_file(
 		 * This call may implicitly ++num_active
 		 */
 		if ((grpptr = group_add(ptr)) == NULL) {
-			if ((grpptr = group_find(ptr)) == NULL)
+			if ((grpptr = group_find(ptr, FALSE)) == NULL)
 				continue;
 
 			if (max > grpptr->xmax) {
@@ -529,24 +621,101 @@ read_active_file(
 	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
-		error_message(_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
+		error_message(2, _(txt_active_file_is_empty), (read_news_via_nntp ? (read_saved_news ? news_active_file : _(txt_servers_active)) : news_active_file));
 		tin_done(EXIT_FAILURE);
 	}
 
-	if (!batch_mode)
-		my_fputs("\n", stdout);
+	if (!batch_mode || verbose)
+		my_fputc('\n', stdout);
 }
 
 
+#ifdef NNTP_ABLE
+/*
+ * Load the active file into active[] via LIST COUNTS
+ */
+static void
+read_active_counts(
+	void)
+{
+	FILE *fp;
+	char *ptr;
+	char moderated[PATH_LEN];
+	long count = -1L, min = 1L, max = 0L;
+	long processed = 0L;
+	struct t_group *grpptr;
+
+	if (!batch_mode || verbose)
+		wait_message(0, _(txt_reading_news_active_file));
+
+	if ((fp = nntp_command("LIST COUNTS", OK_GROUPS, NULL, 0)) == NULL) {
+		if (cmd_line && !batch_mode)
+			my_fputc('\n', stderr);
+
+		error_message(2, _(txt_cannot_retrieve), ACTIVE_FILE);
+		tin_done(EXIT_FAILURE);
+	}
+
+	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
+		if (!parse_count_line(ptr, &max, &min, &count, moderated))
+			continue;
+
+		if (++processed % MODULO_COUNT_NUM == 0)
+			spin_cursor();
+
+		/*
+		 * Load group into group hash table
+		 * NULL means group already present, so we just fixup the counters
+		 * This call may implicitly ++num_active
+		 */
+		if ((grpptr = group_add(ptr)) == NULL) {
+			if ((grpptr = group_find(ptr, FALSE)) == NULL)
+				continue;
+
+			if (max > grpptr->xmax) {
+				grpptr->xmax = max;
+				grpptr->count = count;
+			}
+
+			if (min > grpptr->xmin) {
+				grpptr->xmin = min;
+				grpptr->count = count;
+			}
+
+			continue;
+		}
+
+		/*
+		 * Load the new group in active[]
+		 */
+		active_add(grpptr, count, max, min, moderated);
+	}
+
+	/*
+	 * Exit if active file wasn't read correctly or is empty
+	 */
+	if (tin_errno || !num_active) {
+		error_message(2, _(txt_active_file_is_empty), _(txt_servers_active));
+		tin_done(EXIT_FAILURE);
+	}
+
+	if (!batch_mode || verbose)
+		my_fputc('\n', stdout);
+}
+#endif /* NNTP_ABLE*/
+
+
 /*
  * Load the active file into active[]
  * Check and preload any new newgroups into my_group[]
  */
-void
+int
 read_news_active_file(
 	void)
 {
 	FILE *fp;
+	int newgrps = 0;
+	t_bool do_group_cmds = !nntp_caps.list_counts;
 
 	/*
 	 * Ignore -n if no .newsrc can be found or .newsrc is empty
@@ -565,12 +734,133 @@ read_news_active_file(
 	}
 
 	/* Read an active file if it is allowed */
-	if (list_active)
-		read_active_file();
+	if (list_active) {
+#ifdef NNTP_ABLE
+		if (read_news_via_nntp && nntp_caps.type == CAPABILITIES && nntp_caps.list_counts)
+			read_active_counts();
+		else
+#endif /* NNTP_ABLE */
+			read_active_file();
+	}
 
 	/* Read .newsrc and check each group */
-	if (newsrc_active)
-		read_newsrc_active_file();
+	if (newsrc_active) {
+#ifdef NNTP_ABLE
+#	ifndef DISABLE_PIPELINING
+		/*
+		 * use "LIST ACTIVE grp" (or even LIST ACTIVE grp,...) if we have
+		 * less than PIPELINE_LIMIT groups and we use -n but not -Q
+		 *
+		 * TODO: test me. do we want this overhead? add a DISABLE_PIPELINING
+		 *       code-path? we don't have list_active set but we use some
+		 *       sort of LIST ACTIVE -> our documentation is a bit incorrect
+		 *       now.
+		 */
+		if (read_news_via_nntp && !list_active && ((nntp_caps.type == CAPABILITIES && nntp_caps.list_active) || nntp_caps.type != CAPABILITIES) && (show_description || check_for_new_newsgroups)) {
+			char buff[NNTP_STRLEN];
+			char *ptr, *q;
+			char moderated[PATH_LEN];
+			int r = 0, i = 0, j = 0;
+			long count = -1L, min = 1L, max = 0L;
+			struct t_group *grpptr;
+			t_bool need_auth = FALSE;
+
+			*buff='\0';
+			/* we can't use for_each_group(i) yet, so we have to prase the newsrc */
+			if ((fp = fopen(newsrc, "r")) != NULL) {
+				while (tin_fgets(fp, FALSE) != NULL)
+					j++;
+				rewind(fp);
+				if (j < PIPELINE_LIMIT) {
+					while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
+						if (!(q = strpbrk(ptr, ":!")))
+							continue;
+						*q = '\0';
+						if (nntp_caps.type == CAPABILITIES && (nntp_caps.list_active || nntp_caps.list_counts)) {
+							/* LIST ACTIVE or LIST COUNTS takes wildmats */
+							if (!i) { /* new wildmat list */
+								i++;
+								snprintf(buff, sizeof(buff), "LIST %s %s", nntp_caps.list_counts ? "COUNTS" : "ACTIVE", ptr);
+								continue;
+							}
+							if (strlen(buff) + strlen(ptr) + 1 < NNTP_STRLEN) { /* append group name */
+								snprintf(buff + strlen(buff), sizeof(buff) - strlen(buff), ",%s", ptr);
+								continue;
+							}
+							i = 0; /* new wildmatlist required */
+						} else
+							snprintf(buff, sizeof(buff), "LIST ACTIVE %s", ptr);
+						put_server(buff);
+						r++;
+						*buff='\0';
+					}
+					if (*buff) {
+						put_server(buff);
+						r++;
+					}
+				} else {
+					do_group_cmds = TRUE;
+				}
+				fclose(fp);
+
+				if (j < PIPELINE_LIMIT) {
+					for (i = 0; i < r && !did_reconnect; i++) {
+						if ((j = get_only_respcode(buff, sizeof(buff))) != OK_GROUPS) {
+							/* TODO: add 483 (RFC 3977) code */
+							if (j == ERR_NOAUTH || r == NEED_AUTHINFO)
+								need_auth = TRUE;
+#if 0 /* do we need something like this? */
+							if (j == ERR_CMDSYN)
+								list_active = TRUE;
+#endif /* 0 */
+							continue;
+						} else {
+							while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
+#		ifdef DEBUG
+								if (debug & DEBUG_NNTP)
+									debug_print_file("NNTP", "<<< %s", ptr);
+#		endif /* DEBUG */
+								if (nntp_caps.type == CAPABILITIES && nntp_caps.list_counts) {
+									if (!parse_count_line(ptr, &max, &min, &count, moderated))
+										continue;
+								} else {
+									if (!parse_active_line(ptr, &max, &min, moderated))
+										continue;
+								}
+
+								if ((grpptr = group_add(ptr)) == NULL) {
+									if ((grpptr = group_find(ptr, FALSE)) == NULL)
+										continue;
+
+									if (max > grpptr->xmax) {
+										grpptr->xmax = max;
+										grpptr->count = count;
+									}
+									if (min > grpptr->xmin) {
+										grpptr->xmin = min;
+										grpptr->count = count;
+									}
+									continue;
+								}
+								active_add(grpptr, count, max, min, moderated);
+							}
+						}
+					}
+					if (need_auth) { /* retry after auth is overkill here, so just auth */
+						if (!authenticate(nntp_server, userid, FALSE)) {
+							error_message(2, _(txt_auth_failed), nntp_caps.type == CAPABILITIES ? ERR_AUTHFAIL : ERR_ACCESS);
+							tin_done(EXIT_FAILURE);
+						}
+					}
+				}
+				did_reconnect = FALSE;
+			}
+		}
+#	endif /* !DISABLE_PIPELINING */
+#endif /* NNTP_ABLE */
+		if (!nntp_caps.list_counts || do_group_cmds)
+			read_newsrc_active_file();
+	}
 
 	(void) time(&active_timestamp);
 	force_reread_active_file = FALSE;
@@ -579,12 +869,14 @@ read_news_active_file(
 	 * check_for_any_new_groups() also does $AUTOSUBSCRIBE
 	 */
 	if (check_for_new_newsgroups)
-		check_for_any_new_groups();
+		newgrps = check_for_any_new_groups();
 
 	/*
 	 * finally we have a list of all groups an can set the attributes
 	 */
-	read_attributes_files();
+	assign_attributes_to_groups();
+
+	return newgrps;
 }
 
 
@@ -605,7 +897,8 @@ open_newgroups_fp(
 		if (idx == -1)
 			return (FILE *) 0;
 
-		ngtm = localtime(&newnews[idx].time);
+		if ((ngtm = localtime(&newnews[idx].time)) == NULL)
+			return (FILE *) 0;
 		/*
 		 * in the current draft, NEWGROUPS is allowed to take a 4 digit year
 		 * component - but even with a 2 digit year component it is y2k
@@ -632,7 +925,7 @@ open_newgroups_fp(
  * If reading news via NNTP issue a NEWGROUPS command.
  * Format:   (as active file) Groupname Maxart Minart moderated
  */
-static void
+static int
 check_for_any_new_groups(
 	void)
 {
@@ -641,6 +934,7 @@ check_for_any_new_groups(
 	char *ptr, *line, buf[NNTP_STRLEN];
 	char old_newnews_host[PATH_LEN];
 	int newnews_index;
+	int newgrps = 0;
 	time_t old_newnews_time;
 	time_t new_newnews_time;
 
@@ -661,10 +955,8 @@ check_for_any_new_groups(
 	}
 
 #ifdef DEBUG
-	if (debug == 2) {
-		error_message("Newnews old=[%lu]  new=[%lu]", (unsigned long int) old_newnews_time, (unsigned long int) new_newnews_time);
-		(void) sleep(2);
-	}
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "Newnews old=[%lu]  new=[%lu]", (unsigned long int) old_newnews_time, (unsigned long int) new_newnews_time);
 #endif /* DEBUG */
 
 	if ((fp = open_newgroups_fp(newnews_index)) != NULL) {
@@ -687,11 +979,12 @@ check_for_any_new_groups(
 				*ptr = '\0';
 			}
 			subscribe_new_group(line, autosubscribe, autounsubscribe);
+			newgrps++;
 		}
 		TIN_FCLOSE(fp);
 
 		if (tin_errno)
-			return;				/* Don't update the time if we quit */
+			return 0;				/* Don't update the time if we quit */
 	}
 
 	/*
@@ -705,6 +998,11 @@ check_for_any_new_groups(
 		snprintf(buf, sizeof(buf), "%s %lu", nntp_server, (unsigned long int) new_newnews_time);
 		load_newnews_info(buf);
 	}
+
+	if (!batch_mode)
+		my_fputc('\n', stdout);
+
+	return newgrps;
 }
 
 
@@ -738,7 +1036,7 @@ subscribe_new_group(
 	 * be properly updated when we enter the group. Otherwise there is some
 	 * mismatch in the active.times data and we ignore the newgroup.
 	 */
-	if ((idx = my_group_add(group)) < 0) {
+	if ((idx = my_group_add(group, FALSE)) < 0) {
 		if (list_active) {
 /*			my_fprintf(stderr, "subscribe_new_group: %s not in active[] && list_active\n", group); */
 			return;
@@ -747,7 +1045,7 @@ subscribe_new_group(
 		if ((ptr = group_add(group)) != NULL)
 			active_add(ptr, 0L, 1L, 0L, "y");
 
-		if ((idx = my_group_add(group)) < 0)
+		if ((idx = my_group_add(group, FALSE)) < 0)
 			return;
 	}
 
@@ -879,8 +1177,8 @@ load_newnews_info(
 	newnews[i].time = new_time;
 
 #ifdef DEBUG
-	if (debug == 2)
-		error_message("ACTIVE host=[%s] time=[%lu]", newnews[i].host, (unsigned long int) newnews[i].time);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "ACTIVE host=[%s] time=[%lu]", newnews[i].host, (unsigned long int) newnews[i].time);
 #endif /* DEBUG */
 }
 
@@ -906,9 +1204,6 @@ find_newnews_index(
 /*
  * Get a single status char from the moderated field. Used on selection screen
  * and in header of group screen
- *
- * TODO: what about 'j' groups? active(5) says:
- *       "local postings to that group should not be generated"
  */
 char
 group_flag(
@@ -920,6 +1215,7 @@ group_flag(
 
 		case 'x':
 		case 'n':
+		case 'j':
 			return 'X';
 
 		case '=':
@@ -936,19 +1232,24 @@ void
 create_save_active_file(
 	void)
 {
+	char *fb;
 	char group_path[PATH_LEN];
 	char local_save_active_file[PATH_LEN];
 
-	joinpath(local_save_active_file, rcdir, ACTIVE_SAVE_FILE);
+	joinpath(local_save_active_file, sizeof(local_save_active_file), rcdir, ACTIVE_SAVE_FILE);
 
 	if (no_write && file_size(local_save_active_file) != -1L)
 		return;
 
-	my_printf(_(txt_creating_active));
+	wait_message(0, _(txt_creating_active));
 
 	print_active_head(local_save_active_file);
-	strcpy(group_path, tinrc.savedir);
-	make_group_list(local_save_active_file, tinrc.savedir, group_path);
+	strfpath(cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : tinrc.savedir, group_path, sizeof(group_path), NULL, FALSE);
+	while (strlen(group_path) && group_path[strlen(group_path) - 1] == '/')
+		group_path[strlen(group_path) - 1] = '\0';
+	fb = my_strdup(group_path);
+	make_group_list(local_save_active_file, cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : tinrc.savedir, fb, group_path);
+	free(fb);
 }
 
 
@@ -956,6 +1257,7 @@ static void
 make_group_list(
 	char *active_file,
 	char *base_dir,
+	char *fixed_base,
 	char *group_path)
 {
 	DIR *dir;
@@ -972,8 +1274,7 @@ make_group_list(
 		is_dir = FALSE;
 		while ((direntry = readdir(dir)) != NULL) {
 			STRCPY(filename, direntry->d_name);
-			joinpath(path, group_path, filename);
-
+			joinpath(path, sizeof(path), group_path, filename);
 			if (!(filename[0] == '.' && filename[1] == '\0') &&
 				!(filename[0] == '.' && filename[1] == '.' && filename[2] == '\0')) {
 				if (stat(path, &stat_info) != -1) {
@@ -985,9 +1286,9 @@ make_group_list(
 				is_dir = FALSE;
 				strcpy(group_path, path);
 
-				make_group_list(active_file, base_dir, group_path);
+				make_group_list(active_file, base_dir, fixed_base, group_path);
 				find_art_max_min(group_path, &art_max, &art_min);
-				append_group_line(active_file, group_path, art_max, art_min, base_dir);
+				append_group_line(active_file, group_path + strlen(fixed_base) + 1, art_max, art_min, fixed_base);
 				if ((ptr = strrchr(group_path, '/')) != NULL) /* TODO: Unix'ism */
 					*ptr = '\0';
 			}
@@ -1007,7 +1308,6 @@ append_group_line(
 {
 	FILE *fp;
 	char *file_tmp;
-	char group_name[PATH_LEN];
 
 	if (art_max == 0 && art_min == 1)
 		return;
@@ -1020,43 +1320,26 @@ append_group_line(
 	}
 
 	if ((fp = fopen(active_file, "a+")) != NULL) {
-		make_group_name(base_dir, group_name, group_path);
-		my_printf("Appending=[%s %ld %ld %s]\n", group_name, art_max, art_min, base_dir);
+		char *ptr;
+		char *group_name;
+		int err;
+
+		ptr = group_name = my_strdup(group_path);
+		ptr++;
+		while ((ptr = strchr(ptr, '/')) != NULL)
+			*ptr = '.';
+
+		wait_message(0, "Appending=[%s %ld %ld %s]\n", group_name, art_max, art_min, base_dir);
 		print_group_line(fp, group_name, art_max, art_min, base_dir);
-		if (ferror(fp) || fclose(fp)) /* TODO: issue warning? */
+		if ((err = ferror(fp)) || fclose(fp)) { /* TODO: issue warning? */
 			rename(file_tmp, active_file);
+			if (err) {
+				clearerr(fp);
+				fclose(fp);
+			}
+		}
+		free(group_name);
 	}
 	unlink(file_tmp);
 	free(file_tmp);
 }
-
-
-/*
- * Given an absolute pathname & a base pathname build a newsgroup name
- * base = /usr/spool/news
- * absolute path = /usr/spool/news/alt/sources
- * newsgroup = alt.sources
- */
-static void
-make_group_name(
-	char *base_dir,
-	char *group_name,
-	char *group_path)
-{
-	char *base_ptr;
-	char *name_ptr;
-	char *path_ptr;
-
-	base_ptr = base_dir;
-	path_ptr = group_path;
-
-	while (*base_ptr && (*base_ptr == *path_ptr)) {
-		base_ptr++;
-		path_ptr++;
-	}
-	strcpy(group_name, ++path_ptr);
-
-	name_ptr = group_name;
-	while ((name_ptr = strchr(name_ptr, '/')) != NULL)
-		*name_ptr = '.';
-}
