@@ -3,11 +3,11 @@
  *  Module    : auth.c
  *  Author    : Dirk Nimmich <nimmich@muenster.de>
  *  Created   : 1997-04-05
- *  Updated   : 2005-08-16
+ *  Updated   : 2009-06-27
  *  Notes     : Routines to authenticate to a news server via NNTP.
  *              DON'T USE get_respcode() THROUGHOUT THIS CODE.
  *
- * Copyright (c) 1997-2006 Dirk Nimmich <nimmich@muenster.de>
+ * Copyright (c) 1997-2010 Dirk Nimmich <nimmich@muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,91 +48,16 @@
  * we don't need authentication stuff at all if we don't have NNTP support
  */
 #ifdef NNTP_ABLE
-
 /*
  * local prototypes
  */
-static int do_authinfo_original(char *server, char *authuser, char *authpass);
-static t_bool authinfo_generic(void);
+static int do_authinfo_user(char *server, char *authuser, char *authpass);
 static t_bool read_newsauth_file(char *server, char *authuser, char *authpass);
-static t_bool authinfo_original(char *server, char *authuser, t_bool startup);
-
-
-/*
- * Process AUTHINFO GENERIC method.
- * TRUE means succeeded.
- * FALSE means failed
- */
-static t_bool
-authinfo_generic(
-	void)
-{
-	char *authcmd;
-	char authval[NNTP_STRLEN];
-	char tmpbuf[NNTP_STRLEN];
-	static int cookiefd = -1;
-	t_bool builtinauth = FALSE;
-#if !defined(HAVE_SETENV) && defined(HAVE_PUTENV)
-	char *new_env;
-	static char *old_env = NULL;
-#endif /* !HAVE_SETENV && HAVE_PUTENV */
-
-#ifdef DEBUG
-	debug_nntp("authorization", "authinfo generic");
-#endif /* DEBUG */
-
-	/*
-	 * If we have authenticated before, NNTP_AUTH_FDS already
-	 * exists, pull out the cookiefd. Just in case we've nested.
-	 */
-	if (cookiefd == -1 && (authcmd = getenv("NNTP_AUTH_FDS")))
-		(void) sscanf(authcmd, "%*d.%*d.%d", &cookiefd);
-
-	if (cookiefd == -1) {
-		char tempfile[PATH_LEN];
-		if ((cookiefd = my_tmpfile_only(tempfile)) == -1) {
-#	ifdef DEBUG
-			debug_nntp("authorization", txt_cannot_create_uniq_name);
-#	endif /* DEBUG */
-			return FALSE;
-		} else {
-			close(cookiefd);
-			if (tempfile[0] != '\0')
-				(void) unlink(tempfile);
-		}
-	}
-
-	strcpy(tmpbuf, "AUTHINFO GENERIC ");
-	STRCPY(authval, get_val("NNTPAUTH", ""));
-	if (strlen(authval))
-		strcat(tmpbuf, authval);
-	else {
-		strcat(tmpbuf, "ANY ");
-		strcat(tmpbuf, userid);
-		builtinauth = TRUE;
-	}
-	put_server(tmpbuf);
-
-#ifdef HAVE_SETENV
-	snprintf(tmpbuf, sizeof(tmpbuf), "%d.%d.%d",
-			fileno(get_nntp_fp(FAKE_NNTP_FP)),
-			fileno(get_nntp_wr_fp(FAKE_NNTP_FP)), cookiefd);
-	setenv("NNTP_AUTH_FDS", tmpbuf, 1);
-#else
-#	ifdef HAVE_PUTENV
-	snprintf(tmpbuf, sizeof(tmpbuf), "NNTP_AUTH_FDS=%d.%d.%d",
-			fileno(get_nntp_fp(FAKE_NNTP_FP)),
-			fileno(get_nntp_wr_fp(FAKE_NNTP_FP)), cookiefd);
-	new_env = my_strdup(tmpbuf);
-	(void) putenv(new_env);
-	FreeIfNeeded(old_env);
-	old_env = new_env;	/* we are 'leaking' the last malloced mem at exit here */
-#	endif /* HAVE_PUTENV */
-#endif /* HAVE_SETENV */
-
-	/* TODO: is it possible that we should have drained server here? */
-	return (builtinauth ? (get_only_respcode(NULL, 0) == OK_AUTH) : (invoke_cmd(authval) ? TRUE : FALSE));
-}
+static t_bool authinfo_plain(char *server, char *authuser, t_bool startup);
+#	ifdef USE_SASL
+	static char *sasl_auth_plain(char *user, char *pass);
+	static int do_authinfo_sasl_plain(char *authuser, char *authpass);
+#	endif /* USE_SASL */
 
 
 /*
@@ -150,14 +75,15 @@ read_newsauth_file(
 	FILE *fp;
 	char *_authpass;
 	char *ptr;
+	char filename[PATH_LEN];
 	char line[PATH_LEN];
 	int found = 0;
 	int fd;
 	struct stat statbuf;
 
-	joinpath(line, homedir, ".newsauth");
+	joinpath(filename, sizeof(filename), homedir, ".newsauth");
 
-	if ((fp = fopen(line, "r"))) {
+	if ((fp = fopen(filename, "r"))) {
 		if ((fd = fileno(fp)) == -1) {
 			fclose(fp);
 			return FALSE;
@@ -167,33 +93,27 @@ read_newsauth_file(
 			return FALSE;
 		}
 
+#	ifndef FILE_MODE_BROKEN
 		if (S_ISREG(statbuf.st_mode) && (statbuf.st_mode|S_IRUSR|S_IWUSR) != (S_IRUSR|S_IWUSR|S_IFREG)) {
-			error_message(_(txt_error_insecure_permissions), line, statbuf.st_mode);
-			sleep(2);
+			error_message(4, _(txt_error_insecure_permissions), filename, statbuf.st_mode);
 			/*
 			 * TODO: fix permssions?
 			 * fchmod(fd, S_IRUSR|S_IWUSR);
 			 */
 		}
+#	endif /* !FILE_MODE_BROKEN */
 
 		/*
 		 * Search through authorization file for correct NNTP server
 		 * File has format:  'nntp-server' 'password' ['username']
 		 */
-		while (fgets(line, PATH_LEN, fp) != NULL) {
-
-			/*
-			 * strip trailing newline character
-			 */
-
+		while (fgets(line, sizeof(line), fp) != NULL) {
+			/* strip trailing newline character */
 			ptr = strchr(line, '\n');
 			if (ptr != NULL)
 				*ptr = '\0';
 
-			/*
-			 * Get server from 1st part of the line
-			 */
-
+			/* Get server from 1st part of the line */
 			ptr = strpbrk(line, " \t");
 
 			if (ptr == NULL)		/* no passwd, no auth, skip */
@@ -204,10 +124,7 @@ read_newsauth_file(
 			if ((strcasecmp(line, server)))
 				continue;		/* wrong server, keep on */
 
-			/*
-			 * Get password from 2nd part of the line
-			 */
-
+			/* Get password from 2nd part of the line */
 			while (*ptr == ' ' || *ptr == '\t')
 				ptr++;	/* skip any blanks */
 
@@ -222,10 +139,7 @@ read_newsauth_file(
 					ptr = _authpass;
 			}
 
-			/*
-			 * Get user from 3rd part of the line
-			 */
-
+			/* Get user from 3rd part of the line */
 			ptr = strpbrk(ptr, " \t");	/* find next separating blank */
 
 			if (ptr != NULL) {	/* a 3rd argument follows */
@@ -246,11 +160,13 @@ read_newsauth_file(
 
 
 /*
- * Perform authentication with ORIGINAL AUTHINFO method. Return response
+ * Perform authentication with AUTHINFO USER method. Return response
  * code from server.
+ *
+ * we don't handle ERR_ENCRYPT right now
  */
 static int
-do_authinfo_original(
+do_authinfo_user(
 	char *server,
 	char *authuser,
 	char *authpass)
@@ -259,25 +175,28 @@ do_authinfo_original(
 	int ret;
 
 	snprintf(line, sizeof(line), "AUTHINFO USER %s", authuser);
-#ifdef DEBUG
-	debug_nntp("authorization", line);
-#endif /* DEBUG */
+#	ifdef DEBUG
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "authorization %s", line);
+#	endif /* DEBUG */
 	put_server(line);
 	if ((ret = get_only_respcode(NULL, 0)) != NEED_AUTHDATA)
 		return ret;
 
 	if ((authpass == NULL) || (*authpass == '\0')) {
-#ifdef DEBUG
-		debug_nntp("authorization", "failed: no password");
-#endif /* DEBUG */
-		error_message(_(txt_nntp_authorization_failed), server);
+#	ifdef DEBUG
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "authorization failed: no password");
+#	endif /* DEBUG */
+		error_message(2, _(txt_auth_failed_nopass), server);
 		return ERR_AUTHBAD;
 	}
 
 	snprintf(line, sizeof(line), "AUTHINFO PASS %s", authpass);
-#ifdef DEBUG
-	debug_nntp("authorization", line);
-#endif /* DEBUG */
+#	ifdef DEBUG
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "authorization %s", line);
+#	endif /* DEBUG */
 	put_server(line);
 	ret = get_only_respcode(line, sizeof(line));
 	if (!batch_mode || verbose || ret != OK_AUTH)
@@ -289,6 +208,10 @@ do_authinfo_original(
 /*
  * NNTP user authorization. Returns TRUE if authorization succeeded,
  * FALSE if not.
+ *
+ * tries AUTHINFO SASL PLAIN (if available) fist and if not succcessfull
+ * AUTHINFO USER/PASS
+ *
  * If username/passwd already given, and server wasn't changed, retry those.
  * Otherwise, read password from ~/.newsauth or, if not present or no matching
  * server found, from console.
@@ -299,7 +222,7 @@ do_authinfo_original(
  *   etc.
  */
 static t_bool
-authinfo_original(
+authinfo_plain(
 	char *server,
 	char *authuser,
 	t_bool startup)
@@ -312,11 +235,6 @@ authinfo_original(
 	static t_bool already_failed = FALSE;
 	static t_bool initialized = FALSE;
 
-#ifdef DEBUG
-	debug_nntp("authorization", "original authinfo");
-#endif /* DEBUG */
-
-
 	changed = strcmp(server, last_server);	/* do we need new auth values? */
 	strncpy(last_server, server, PATH_LEN - 1);
 	last_server[PATH_LEN - 1] = '\0';
@@ -325,8 +243,18 @@ authinfo_original(
 	 * Let's try the previous auth pair first, if applicable.
 	 * Else, proceed to the other mechanisms.
 	 */
-	if (initialized && !changed && !already_failed && do_authinfo_original(server, authusername, authpassword))
-		return TRUE;
+	if (initialized && !changed && !already_failed) {
+#	ifdef USE_SASL
+		if (nntp_caps.sasl & SASL_PLAIN)
+			ret = do_authinfo_sasl_plain(authusername, authpassword);
+		if (ret != OK_AUTH)
+#	endif /* USE_SASL */
+		{
+			if (nntp_caps.type != CAPABILITIES || (nntp_caps.type == CAPABILITIES && nntp_caps.authinfo_user))
+				ret = do_authinfo_user(server, authusername, authpassword);
+		}
+		return (ret == OK_AUTH);
+	}
 
 	authpassword[0] = '\0';
 	authuser = strncpy(authusername, authuser, sizeof(authusername) - 1);
@@ -346,14 +274,25 @@ authinfo_original(
 	 * and restart tin or change to another server and back in order to get
 	 * it read again.
 	 */
-	if (changed || (!changed && !already_failed)) {
-		already_failed = FALSE;
+	if ((changed || !initialized) && !already_failed) {
 		if (read_newsauth_file(server, authuser, authpass)) {
-			ret = do_authinfo_original(server, authuser, authpass);
-			if (!(already_failed = (ret != OK_AUTH))) {
-#ifdef DEBUG
-				debug_nntp("authorization", "succeeded");
-#endif /* DEBUG */
+#	ifdef USE_SASL
+			if (nntp_caps.sasl & SASL_PLAIN)
+				ret = do_authinfo_sasl_plain(authuser, authpass);
+
+			if (ret != OK_AUTH)
+#	endif /* USE_SASL */
+			{
+				if (force_auth_on_conn_open || nntp_caps.type != CAPABILITIES || (nntp_caps.type == CAPABILITIES && nntp_caps.authinfo_user))
+					ret = do_authinfo_user(server, authuser, authpass);
+			}
+			already_failed = (ret != OK_AUTH);
+
+			if (ret == OK_AUTH) {
+#	ifdef DEBUG
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "authorization succeeded");
+#	endif /* DEBUG */
 				initialized = TRUE;
 				return TRUE;
 			}
@@ -370,52 +309,94 @@ authinfo_original(
 	 * messages are annoying (and even wrong).
 	 * UNSURE: Maybe we want to make this decision configurable in the
 	 * options menu, too, so that the user doesn't need -A.
-	 * TODO: Put questions into do_authinfo_original because it is possible
+	 * TODO: Put questions into do_authinfo_user() because it is possible
 	 * that the server doesn't want a password; so only ask for it if needed.
 	 */
-	if (force_auth_on_conn_open || !startup) {
-#ifdef USE_CURSES
-		int state = RawState();
-#endif /* USE_CURSES */
+	 if (force_auth_on_conn_open || !startup) {
+		if (nntp_caps.type != CAPABILITIES || (nntp_caps.type == CAPABILITIES && !nntp_caps.authinfo_state && ((nntp_caps.sasl & SASL_PLAIN) || nntp_caps.authinfo_user || (!nntp_caps.authinfo_user && !(nntp_caps.sasl & SASL_PLAIN))))) {
+#	ifdef USE_CURSES
+			int state = RawState();
+#	endif /* USE_CURSES */
 
-		wait_message(0, _(txt_auth_needed));
-#ifdef USE_CURSES
-		Raw(TRUE);
-#endif /* USE_CURSES */
+			wait_message(0, _(txt_auth_needed));
+#	ifdef USE_CURSES
+			Raw(TRUE);
+#	endif /* USE_CURSES */
+			if (!prompt_default_string(_(txt_auth_user), authuser, PATH_LEN, authusername, HIST_NONE)) {
+#	ifdef DEBUG
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "authorization failed: no username");
+#	endif /* DEBUG */
+				return FALSE;
+			}
 
-		if (!prompt_default_string(_(txt_auth_user), authuser, PATH_LEN, authusername, HIST_NONE)) {
-#ifdef DEBUG
-			debug_nntp("authorization", "failed: no username");
-#endif /* DEBUG */
-			return FALSE;
-		}
-
-#ifdef USE_CURSES
-		Raw(state);
-		my_printf("%s", _(txt_auth_pass));
-		wgetnstr(stdscr, authpassword, sizeof(authpassword));
-		Raw(TRUE);
-#else
-#	if 0
-		/*
-		 * on some systems (i.e. Solaris) getpass(3) is limited to 8 chars ->
-		 * we use tin_getline() till we have a config check
-		 * for getpass() or our own getpass()
-		 */
-		authpass = strncpy(authpassword, getpass(_(txt_auth_pass)), sizeof(authpassword) - 1);
+#	ifdef USE_CURSES
+			Raw(state);
+			my_printf("%s", _(txt_auth_pass));
+			wgetnstr(stdscr, authpassword, sizeof(authpassword));
+			Raw(TRUE);
 #	else
-		authpass = strncpy(authpassword, tin_getline(_(txt_auth_pass), FALSE, NULL, PATH_LEN, TRUE, HIST_NONE), sizeof(authpassword) - 1);
-#	endif /* 0 */
-#endif /* USE_CURSES */
+			/*
+			 * on some systems (i.e. Solaris) getpass(3) is limited to 8 chars ->
+			 * we use tin_getline()
+			 */
+			authpass = strncpy(authpassword, tin_getline(_(txt_auth_pass), FALSE, NULL, PATH_LEN, TRUE, HIST_NONE), sizeof(authpassword) - 1);
+#	endif /* USE_CURSES */
 
-		ret = do_authinfo_original(server, authuser, authpass);
-		initialized = TRUE;
-		my_retouch();			/* Get rid of the chaff */
+#	ifdef USE_SASL
+			if (nntp_caps.sasl & SASL_PLAIN)
+				ret = do_authinfo_sasl_plain(authuser, authpass);
+			if (ret != OK_AUTH)
+#	endif /* USE_SASL */
+			{
+				if (nntp_caps.type != CAPABILITIES || (nntp_caps.type == CAPABILITIES && (nntp_caps.authinfo_user || (!nntp_caps.authinfo_user && !nntp_caps.authinfo_sasl)))) {
+#	ifdef DEBUG
+					if (debug & DEBUG_NNTP) {
+						if (nntp_caps.type == CAPABILITIES && !nntp_caps.authinfo_sasl && !nntp_caps.authinfo_user)
+							debug_print_file("NNTP", "!!! No supported authmethod available, trying AUTHINFO USER/PASS");
+					}
+#	endif /* DEBUG */
+					ret = do_authinfo_user(server, authuser, authpass);
+					if (ret != OK_AUTH)
+						already_failed = TRUE;
+					/*
+					 * giganews once responded to CAPABILITIES with just
+					 * "VERSION 2", no mode-switching indication, no reader
+					 * indication, no post indication, no authentication
+					 * indication, ... so in case AUTHINFO USER/PASS succeeds
+					 * if not advertized we simply go on but fully ignore
+					 * CAPABILITIES
+					 */
+					if (nntp_caps.type == CAPABILITIES && !nntp_caps.authinfo_user && !nntp_caps.authinfo_sasl && ret == OK_AUTH)
+						nntp_caps.type = BROKEN;
+				}
+			}
+			initialized = TRUE;
+			my_retouch();			/* Get rid of the chaff */
+		} else {
+			/*
+			 * TODO:
+			 * nntp_caps.type == CAPABILITIES && nntp_caps.authinfo_state
+			 * can we change the sate here? and if so how? SARTTLS? MODE
+			 * READER?
+			 */
+#	ifdef DEBUG
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "authorization not allowed in current sate");
+#	endif /* DEBUG */
+			/*
+			 * we return OK_AUTH here once so tin doesn't exit just because a
+			 * single command requested auth ...
+			 */
+			if (!already_failed)
+				ret = OK_AUTH;
+		}
 	}
 
-#ifdef DEBUG
-	debug_nntp("authorization", (ret == OK_AUTH ? "succeeded" : "failed"));
-#endif /* DEBUG */
+#	ifdef DEBUG
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "authorization %s", (ret == OK_AUTH ? "succeeded" : "failed"));
+#	endif /* DEBUG */
 
 	return (ret == OK_AUTH);
 }
@@ -425,8 +406,8 @@ authinfo_original(
  * Do authentication stuff. Return TRUE if authentication was successful,
  * FALSE otherwise.
  *
- * First try AUTHINFO GENERIC method, then, if that failed, ORIGINAL
- * AUTHINFO method. Other authentication methods can easily be added.
+ * try ORIGINAL AUTHINFO method.
+ * Other authentication methods can easily be added.
  */
 t_bool
 authenticate(
@@ -434,9 +415,67 @@ authenticate(
 	char *user,
 	t_bool startup)
 {
-	return (authinfo_generic() || authinfo_original(server, user, startup));
+	return authinfo_plain(server, user, startup);
 }
 
+
+#	ifdef USE_SASL
+static int
+do_authinfo_sasl_plain(
+	char *authuser,
+	char *authpass)
+{
+	char line[PATH_LEN];
+	char *foo;
+	int ret;
+
+#		ifdef DEBUG
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "do_authinfo_sasl_plain(%s, %s)", BlankIfNull(authuser), BlankIfNull(authpass));
+#		endif /* DEBUG */
+
+	if ((foo = sasl_auth_plain(authuser, authpass)) == NULL)
+		return ERR_AUTHBAD;
+
+	snprintf(line, sizeof(line), "AUTHINFO SASL PLAIN %s", foo);
+	free(foo);
+#		ifdef DEBUG
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "authorization %s", line);
+#		endif /* DEBUG */
+	put_server(line);
+	ret = get_only_respcode(line, sizeof(line));
+	if (!batch_mode || verbose || ret != OK_AUTH)
+		wait_message(2, (ret == OK_AUTH ? _(txt_authorization_ok) : _(txt_authorization_fail)), authuser);
+	return ret;
+}
+
+
+static char *sasl_auth_plain(
+	char *user,
+	char *pass)
+{
+	Gsasl *ctx = NULL;
+	Gsasl_session *session;
+	char *p = NULL;
+	const char *mech = "PLAIN";
+
+	if (gsasl_init(&ctx) != GSASL_OK) /* TODO: do this only once at startup */
+		return p;
+	if (gsasl_client_start(ctx, mech, &session) != GSASL_OK) {
+		gsasl_done(ctx);
+		return p;
+	}
+	gsasl_property_set(session, GSASL_AUTHID, user);
+	gsasl_property_set(session, GSASL_PASSWORD, pass);
+	if (gsasl_step64(session, NULL, &p) != GSASL_OK)
+		FreeAndNull(p);
+	gsasl_finish(session);
+	gsasl_done(ctx);
+	return p;
+}
+#	endif /* USE_SASL */
+
 #else
 static void no_authenticate(void);			/* proto-type */
 static void
