@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   : 2005-07-20
+ *  Updated   : 2010-09-27
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2005 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2010 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,17 +48,21 @@
 #define CHUNK		50
 
 #define STRIP_ALTERNATIVE(x) \
-			(tinrc.alternative_handling && \
+			(curr_group->attribute->alternative_handling && \
 			(x)->hdr.ext->type == TYPE_MULTIPART && \
 			strcasecmp("alternative", (x)->hdr.ext->subtype) == 0)
 
 #define MATCH_REGEX(x,y,z)	(pcre_exec(x.re, x.extra, y, z, 0, 0, NULL, 0) >= 0)
 
 
+static t_bool charset_unsupported(const char *charset);
 static t_bool header_wanted(const char *line);
 static t_part *new_uue(t_part **part, char *name);
-static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part, int hide_uue, int tabs);
+static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part, int hide_uue);
 static void put_cooked(size_t buf_len, t_bool wrap_lines, int flags, const char *fmt, ...);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	static t_bool wexpand_ctrl_chars(wchar_t **wline, size_t *length, size_t lcook_width);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #ifdef DEBUG_ART
 	static void dump_cooked(void);
 #endif /* DEBUG_ART */
@@ -79,63 +83,115 @@ static t_openartinfo *art;
 t_bool
 expand_ctrl_chars(
 	char **line,
-	int *length,
+	size_t *length,
 	size_t lcook_width)
 {
-	int curr_len = LEN;
-	int i = 0, j;
-	char *buf = my_malloc(curr_len);
-	char *c;
-	t_bool ctrl_L = FALSE, resize = FALSE;
+	t_bool ctrl_L = FALSE;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wline = char2wchar_t(*line);
+	size_t wlen;
 
-	c = *line;
+	/*
+	 * remove the assert() before release
+	 * it should help us find problems with wide-char strings
+	 * in the development branch
+	 */
+	assert (wline != NULL);
+	wlen = wcslen(wline);
+	ctrl_L = wexpand_ctrl_chars(&wline, &wlen, lcook_width);
+	free(*line);
+	*line = wchar_t2char(wline);
+	free(wline);
+	assert (line != NULL);
+	*length = strlen(*line);
+#else
+	int curr_len = LEN;
+	int i = 0, j, ln = 0;
+	char *buf = my_malloc(curr_len);
+	unsigned char *c;
+
+	c = (unsigned char *) *line;
 	while (*c) {
-		if (resize) {
+		if (i > curr_len - (lcook_width + 1)) {
 			curr_len <<= 1;
 			buf = my_realloc(buf, curr_len);
-			resize = FALSE;
 		}
+		if (*c == '\n')
+			ln = i + 1;
 		if (*c == '\t') { /* expand tabs */
-/*			j = ((i + lcook_width) / lcook_width) * lcook_width; */
-			j = i + lcook_width - (i % lcook_width);
-			if (j > curr_len - 2) {
-				resize = TRUE;
-				continue;
-			}
+			j = i + lcook_width - ((i - ln) % lcook_width);
 			for (; i < j; i++)
 				buf[i] = ' ';
+		} else if (((*c) & 0xFF) < ' ' && *c != '\n' && (!IS_LOCAL_CHARSET("Big5") || *c != 27)) {	/* literal ctrl chars */
+			buf[i++] = '^';
+			buf[i++] = ((*c) & 0xFF) + '@';
+			if (*c == '\f')		/* ^L detected */
+				ctrl_L = TRUE;
 		} else {
-			if (((*c) & 0xFF) < ' ' && *c != '\n' && (!IS_LOCAL_CHARSET("Big5") || *c != 27)) {	/* literal ctrl chars */
-				if (i > curr_len - 4) {
-					resize = TRUE;
-					continue;
-				}
-				buf[i++] = '^';
-				buf[i++] = ((*c) & 0xFF) + '@';
-				if (*c == '\f')	/* ^L detected */
-					ctrl_L = TRUE;
-			} else {
-				if (i > curr_len - 3) {
-					resize = TRUE;
-					continue;
-				}
+			if (!my_isprint(*c) && *c != '\n')
+				buf[i++] = '?';
+			else
 				buf[i++] = *c;
-			}
 		}
 		c++;
 	}
-	/* put_cooked() requires a newline at the end of the line */
-	if (buf[i - 1] != '\n')
-		buf[i++] = '\n';	/* Force last char of string to be \n */
 	buf[i] = '\0';
 	*length = i + 1;
 	*line = my_realloc(*line, *length);
 	strcpy(*line, buf);
 	free(buf);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	return ctrl_L;
 }
 
 
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+static t_bool
+wexpand_ctrl_chars(
+	wchar_t **wline,
+	size_t *length,
+	size_t lcook_width)
+{
+	size_t cur_len = LEN, i = 0, j, ln = 0;
+	wchar_t *wbuf = my_malloc(cur_len * sizeof(wchar_t));
+	wchar_t *wc;
+	t_bool ctrl_L = FALSE;
+
+	wc = *wline;
+	while (*wc) {
+		if (i > cur_len - (lcook_width + 1)) {
+			cur_len <<= 1;
+			wbuf = my_realloc(wbuf, cur_len * sizeof(wchar_t));
+		}
+		if (*wc == '\n')
+			ln = i + 1;
+		if (*wc == '\t') {		/* expand_tabs */
+			j = i + lcook_width - ((i - ln) % lcook_width);
+			for (; i < j; i++)
+				wbuf[i] = ' ';
+		} else if (*wc < ' ' && *wc != '\n' && (!IS_LOCAL_CHARSET("Big5") || *wc != 27)) {	/* literal ctrl chars */
+			wbuf[i++] = '^';
+			wbuf[i++] = *wc + '@';
+			if (*wc == '\f')	/* ^L detected */
+				ctrl_L = TRUE;
+		} else {
+			if (!iswprint((wint_t) *wc) && *wc != '\n')
+				wbuf[i++] = '?';
+			else
+				wbuf[i++] = *wc;
+		}
+		wc++;
+	}
+	wbuf[i] = '\0';
+	*length = i + 1;
+	*wline = my_realloc(*wline, *length * sizeof(wchar_t));
+	wcscpy(*wline, wbuf);
+	free(wbuf);
+	return ctrl_L;
+}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+
 /*
  * Output text to the cooked stream. Wrap lines as necessary.
  * Update the line count and the array of line offsets
@@ -170,7 +226,11 @@ put_cooked(
 	if (tinrc.wrap_column < 0)
 		wrap_column = ((tinrc.wrap_column > -cCOLS) ? cCOLS + tinrc.wrap_column : cCOLS);
 	else
+#if 1
 		wrap_column = ((tinrc.wrap_column > 0) ? tinrc.wrap_column : cCOLS);
+#else	/* never cut off long lines */
+		wrap_column = (((tinrc.wrap_column > 0) && (tinrc.wrap_column < cCOLS)) ? tinrc.wrap_column : cCOLS);
+#endif /* 1 */
 
 	p = bufp = buf;
 
@@ -310,27 +370,28 @@ get_filename(
 }
 
 
-#define PUT_UUE(part, qualifier_text)					\
-	put_cooked(LEN, wrap_lines, C_UUE, _(txt_uue),			\
-		part->depth * 4, "",							\
-		content_types[part->type], part->subtype,		\
+#define PUT_UUE(part, qualifier_text) \
+	put_cooked(LEN, wrap_lines, C_UUE, _(txt_uue), \
+		part->depth ? (part->depth - 1) * 4 : 0, "", \
+		content_types[part->type], part->subtype, \
 		qualifier_text, part->line_count, get_filename(part->params))
 
 #define PUT_ATTACH(part, depth, name, charset) \
-	put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_attach),	\
-		depth, "",	\
-		content_types[part->type], part->subtype,	\
-		content_encodings[part->encoding],	\
-		charset ? _(txt_attach_charset) : "", BlankIfNull(charset),	\
-		part->line_count,	\
-		name ? _(txt_name) : "", BlankIfNull(name));	\
+	put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_attach), \
+		depth, "", \
+		content_types[part->type], part->subtype, \
+		content_encodings[part->encoding], \
+		charset ? _(txt_attach_charset) : "", BlankIfNull(charset), \
+		part->line_count, \
+		name ? _(txt_name) : "", BlankIfNull(name)); \
 		\
-	if (part->description)	\
-		put_cooked(LEN, wrap_lines, C_ATTACH,	\
-			_(txt_attach_description),	\
-			depth, "",	\
-			part->description);	\
-	put_cooked(1, wrap_lines, C_ATTACH, "\n")
+	if (part->description) \
+		put_cooked(LEN, wrap_lines, C_ATTACH, \
+			_(txt_attach_description), \
+			depth, "", \
+			part->description); \
+	if (part->next != NULL || IS_PLAINTEXT(part)) \
+		put_cooked(1, wrap_lines, C_ATTACH, "\n")
 
 /*
  * Decodes text bodies, remove sig's, detects uuencoded sections
@@ -340,18 +401,22 @@ process_text_body_part(
 	t_bool wrap_lines,
 	FILE *in,
 	t_part *part,
-	int hide_uue,
-	int tabs)
+	int hide_uue)
 {
 	char *rest = NULL;
-	char *line = NULL, *buf;
-	int max_line_len = 0;
-	int flags, len, lines_left;
+	char *line = NULL, *buf, *tmpline;
+	size_t max_line_len = 0;
+	int flags, len, lines_left, len_blank;
 	int offsets[6];
 	int size_offsets = ARRAY_SIZE(offsets);
+	unsigned int lines_skipped = 0;
 	t_bool in_sig = FALSE;			/* Set when in sig portion */
 	t_bool in_uue = FALSE;			/* Set when in uuencoded section */
+	t_bool in_verbatim = FALSE;		/* Set when in verbatim section */
+	t_bool verbatim_begin = FALSE;	/* Set when verbatim_begin_regex matches */
 	t_bool is_uubody;				/* Set when current line looks like a uuencoded line */
+	t_bool first_line_blank = TRUE;	/* Unset when first non-blank line is reached */
+	t_bool put_blank_lines = FALSE;	/* Set when previously skipped lines needs to put */
 	t_part *curruue = NULL;
 
 	if (part->uue) {				/* These are redone each time we recook/resize etc.. */
@@ -387,10 +452,10 @@ process_text_body_part(
 				 * especially if we must resize it.
 				 * So copy buf to line (and resize line if necessary).
 				 */
-				if (max_line_len < (int) strlen(buf) + 2) {
+				if (max_line_len < strlen(buf) + 2) {
 					max_line_len = strlen(buf) + 2;
 					line = my_realloc(line, max_line_len);
-				};
+				}
 				strcpy(line, buf);
 
 				/*
@@ -417,104 +482,174 @@ process_text_body_part(
 		len = (int) strlen(line);
 
 		/*
-		 * Detect and skip signatures if necessary
+		 * trim article body and sig (not verbatim blocks):
+		 * - skip leading blank lines
+		 * - replace multiple blank lines with one empty line
+		 * - skip tailing blank lines, keep one if an
+		 *   attachement follows
 		 */
-		if (!in_sig) {
-			if (strcmp(line, SIGDASHES) == 0) {
-				in_sig = TRUE;
+		if (curr_group->attribute->trim_article_body && !in_uue && !in_verbatim && !verbatim_begin) {
+			len_blank = 1;
+			tmpline = line;
+			/* check if line contains only whitespace */
+			while ((' ' == *tmpline) || ('\t' == *tmpline)) {
+				len_blank++;
+				tmpline++;
+			}
+			if (len_blank == len) {		/* line is blank */
+				if (lines_left == 0 && (curr_group->attribute->trim_article_body & SKIP_TRAILING)) {
+					if (!(part->next == NULL || (STRIP_ALTERNATIVE(art) && !IS_PLAINTEXT(part->next))))
+						put_cooked(1, TRUE, in_sig ? C_SIG : C_BODY, "\n");
+					continue;
+				}
+				if (first_line_blank) {
+					if (curr_group->attribute->trim_article_body & SKIP_LEADING)
+						continue;
+				} else if ((curr_group->attribute->trim_article_body & (COMPACT_MULTIPLE | SKIP_TRAILING)) && (!in_sig || curr_group->attribute->show_signatures)) {
+					lines_skipped++;
+					if (lines_left == 0 && !(curr_group->attribute->trim_article_body & SKIP_TRAILING)) {
+						for (; lines_skipped > 0; lines_skipped--)
+							put_cooked(1, TRUE, in_sig ? C_SIG : C_BODY, "\n");
+					}
+					continue;
+				}
+			} else {	/* line is not blank */
+				if (first_line_blank)
+					first_line_blank = FALSE;
+				if (lines_skipped && (!in_sig || curr_group->attribute->show_signatures)) {
+					if (strcmp(line, SIGDASHES) != 0 || curr_group->attribute->show_signatures) {
+						if (curr_group->attribute->trim_article_body & COMPACT_MULTIPLE)
+							put_cooked(1, TRUE, in_sig ? C_SIG : C_BODY, "\n");
+						else
+							put_blank_lines = TRUE;
+					} else if (!(curr_group->attribute->trim_article_body & SKIP_TRAILING))
+						put_blank_lines = TRUE;
+					if (put_blank_lines) {
+						for (; lines_skipped > 0; lines_skipped--)
+							put_cooked(1, TRUE, in_sig ? C_SIG : C_BODY, "\n");
+					}
+					put_blank_lines = FALSE;
+					lines_skipped = 0;
+				}
+			}
+		} /* if (tinrc.trim_article_body...) */
+
+		/* look for verbatim marks, set in_verbatim only for lines in between */
+		if (curr_group->attribute->verbatim_handling) {
+			if (verbatim_begin) {
+				in_verbatim = TRUE;
+				verbatim_begin = FALSE;
+			} else if (!in_sig && !in_uue && !in_verbatim && MATCH_REGEX(verbatim_begin_regex, line, len))
+				verbatim_begin = TRUE;
+			if (in_verbatim && MATCH_REGEX(verbatim_end_regex, line, len))
+				in_verbatim = FALSE;
+		}
+
+		if (!in_verbatim) {
+			/*
+			 * Detect and skip signatures if necessary
+			 */
+			if (!in_sig) {
+				if (strcmp(line, SIGDASHES) == 0) {
+					in_sig = TRUE;
+					if (in_uue) {
+						in_uue = FALSE;
+						if (hide_uue)
+							PUT_UUE(curruue, _(txt_incomplete));
+					}
+				}
+			}
+
+			if (in_sig && !(curr_group->attribute->show_signatures))
+				continue;					/* No further processing needed */
+
+			/*
+			 * Detect and process uuencoded sections
+			 * Look for the start or the end of a uuencoded section
+			 *
+			 * TODO: look for a tailing size line after end (non standard
+			 *       extension)?
+			 */
+			if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
+				in_uue = TRUE;
+				curruue = new_uue(&part, line + offsets[1]);
+				if (hide_uue)
+					continue;				/* Don't cook the 'begin' line */
+			} else if (strncmp(line, "end\n", 4) == 0) {
 				if (in_uue) {
 					in_uue = FALSE;
-					if (hide_uue)
-						PUT_UUE(curruue, _(txt_incomplete));
+					if (hide_uue) {
+						PUT_UUE(curruue, "");
+						continue;			/* Don't cook the 'end' line */
+					}
 				}
 			}
-		}
 
-		if (in_sig && !tinrc.show_signatures)
-			continue;					/* No further processing needed */
-
-		/*
-		 * Detect and process uuencoded sections
-		 * Look for the start or the end of a uuencoded section
-		 *
-		 * TODO: look for a tailing size line after end (non standard
-		 *       extension)?
-		 */
-		if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
-			in_uue = TRUE;
-			curruue = new_uue(&part, line + offsets[1]);
-			if (hide_uue)
-				continue;				/* Don't cook the 'begin' line */
-		} else if (strncmp(line, "end\n", 4) == 0) {
-			if (in_uue) {
-				in_uue = FALSE;
-				if (hide_uue) {
-					PUT_UUE(curruue, "");
-					continue;			/* Don't cook the 'end' line */
-				}
-			}
-		}
-
-		/*
-		 * See if this line looks like a uuencoded 'body' line
-		 */
-		is_uubody = FALSE;
-
-		if (MATCH_REGEX (uubody_regex, line, len)) {
-			int sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
-
-			/* sum = 0 in a uubody only on the last line, a single ` */
-			if (sum == 0 && len == 1 + 1)			/* +1 for the \n */
-				is_uubody = TRUE;
-			else if (len == sum + 1 + 1)
-				is_uubody = TRUE;
-#ifdef DEBUG_ART
-			if (debug == 2)
-				fprintf(stderr, "%s sum=%d len=%d (%s)\n", bool_unparse(is_uubody), sum, len, line);
-#endif /* DEBUG_ART */
-		}
-
-		if (in_uue) {
-			if (is_uubody)
-				curruue->line_count++;
-			else {
-				if (line[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
-					/* fprintf(stderr, "not a uue line while reading a uue body?\n"); */
-					in_uue = FALSE;
-					if (hide_uue)
-						/* don't continue here, so we see the line that 'broke' in_uue */
-						PUT_UUE(curruue, _(txt_incomplete));
-				}
-			}
-		} else {
 			/*
-			 * UUE_ALL = 'Try harder' - we never saw a begin line, but useful
-			 * when uue sections are split across > 1 article
+			 * See if this line looks like a uuencoded 'body' line
 			 */
-			if (is_uubody && hide_uue == UUE_ALL) {
-				char name[] = N_("(unknown)");
+			is_uubody = FALSE;
 
-				curruue = new_uue(&part, name);
-				curruue->line_count++;
-				in_uue = TRUE;
-				continue;
+			if (MATCH_REGEX(uubody_regex, line, len)) {
+				int sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
+
+				/* sum = 0 in a uubody only on the last line, a single ` */
+				if (sum == 0 && len == 1 + 1)			/* +1 for the \n */
+					is_uubody = TRUE;
+				else if (len == sum + 1 + 1)
+					is_uubody = TRUE;
+#ifdef DEBUG_ART
+				if (debug & DEBUG_MISC)
+					fprintf(stderr, "%s sum=%d len=%d (%s)\n", bool_unparse(is_uubody), sum, len, line);
+#endif /* DEBUG_ART */
 			}
+
+			if (in_uue) {
+				if (is_uubody)
+					curruue->line_count++;
+				else {
+					if (line[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
+						/* fprintf(stderr, "not a uue line while reading a uue body?\n"); */
+						in_uue = FALSE;
+						if (hide_uue)
+							/* don't continue here, so we see the line that 'broke' in_uue */
+							PUT_UUE(curruue, _(txt_incomplete));
+					}
+				}
+			} else {
+				/*
+				 * UUE_ALL = 'Try harder' - we never saw a begin line, but useful
+				 * when uue sections are split across > 1 article
+				 */
+				if (is_uubody && hide_uue == UUE_ALL) {
+					char name[] = N_("(unknown)");
+
+					curruue = new_uue(&part, name);
+					curruue->line_count++;
+					in_uue = TRUE;
+					continue;
+				}
+			}
+
+			/*
+			 * Skip output if we're hiding uue or the sig
+			 */
+			if (in_uue && hide_uue)
+				continue;	/* No further processing needed */
 		}
 
-		/*
-		 * Skip output if we're hiding uue or the sig
-		 */
-		if (in_uue && hide_uue)
-			continue;	/* No further processing needed */
-
-		flags = in_sig ? C_SIG : C_BODY;
+		flags = in_verbatim ? C_VERBATIM : in_sig ? C_SIG : C_BODY;
 
 		/*
-		 * Don't do any further handling of uue lines - the data is binary after all
+		 * Don't do any further handling of uue || verbatim lines
 		 */
 		if (in_uue) {
 			put_cooked(max_line_len, wrap_lines, flags, "%s", line);
 			continue;
+		} else if (in_verbatim) {
+			expand_ctrl_chars(&line, &max_line_len, 8);
+			put_cooked(max_line_len, wrap_lines, flags, "%s", line);
+			continue;
 		}
 
 #ifdef HAVE_COLOR
@@ -539,14 +674,7 @@ process_text_body_part(
 		if (MATCH_REGEX(news_regex, line, len))
 			flags |= C_NEWS;
 
-		/*
-		 * Basically, c_b2p() does: if (!(my_isprint(*c) || *c==8 || *c==9 || *c==12))
-		 * It is only used here
-		 * How about if !isprint() && !isctrl() - expand_ctrl_chars is done at display time.
-		 * TODO: integrate into expand_ctrl_chars
-		 */
-		convert_body2printable(line);
-		if (expand_ctrl_chars(&line, &max_line_len, tabs))
+		if (expand_ctrl_chars(&line, &max_line_len, tabwidth))
 			flags |= C_CTRLL;				/* Line contains form-feed */
 		put_cooked(max_line_len, wrap_lines && (!IS_LOCAL_CHARSET("Big5")), flags, "%s", line);
 	} /* while */
@@ -572,22 +700,22 @@ header_wanted(
 	int i;
 	t_bool ret = FALSE;
 
-	if (num_headers_to_display && (news_headers_to_display_array[0][0] == '*'))
+	if (curr_group->attribute->headers_to_display->num && (curr_group->attribute->headers_to_display->header[0][0] == '*'))
 		ret = TRUE; /* wild do */
 	else {
-		for (i = 0; i < num_headers_to_display; i++) {
-			if (!strncasecmp(line, news_headers_to_display_array[i], strlen(news_headers_to_display_array[i]))) {
+		for (i = 0; i < curr_group->attribute->headers_to_display->num; i++) {
+			if (!strncasecmp(line, curr_group->attribute->headers_to_display->header[i], strlen(curr_group->attribute->headers_to_display->header[i]))) {
 				ret = TRUE;
 				break;
 			}
 		}
 	}
 
-	if (num_headers_to_not_display && (news_headers_to_not_display_array[0][0] == '*'))
+	if (curr_group->attribute->headers_to_not_display->num && (curr_group->attribute->headers_to_not_display->header[0][0] == '*'))
 		ret = FALSE; /* wild don't: doesn't make sense! */
 	else {
-		for (i = 0; i < num_headers_to_not_display; i++) {
-			if (!strncasecmp(line, news_headers_to_not_display_array[i], strlen(news_headers_to_not_display_array[i]))) {
+		for (i = 0; i < curr_group->attribute->headers_to_not_display->num; i++) {
+			if (!strncasecmp(line, curr_group->attribute->headers_to_not_display->header[i], strlen(curr_group->attribute->headers_to_not_display->header[i]))) {
 				ret = FALSE;
 				break;
 			}
@@ -617,6 +745,39 @@ dump_cooked(
 #endif /* DEBUG_ART */
 
 
+/*
+ * Check for charsets which may contain NULL bytes and thus break string
+ * functions. Possibly incomplete.
+ *
+ * TODO: fix the other code to handle those charsets properly.
+ */
+static t_bool
+charset_unsupported(
+	const char *charset)
+{
+	static const char *charsets[] = {
+		"csUnicode",	/* alias for ISO-10646-UCS-2 */
+		"csUCS4",		/* alias for ISO-10646-UCS-4 */
+		"ISO-10646-UCS-2",
+		"ISO-10646-UCS-4",
+		"UTF-16",		/* covers also BE/LE */
+		"UTF-32",		/* covers also BE/LE */
+		NULL };
+	const char **charsetptr = charsets;
+	t_bool ret = FALSE;
+
+	if (!charset)
+		return ret;
+
+	do {
+		if (!strncasecmp(charset, *charsetptr, strlen(*charsetptr)))
+			ret = TRUE;
+	} while (!ret && *(++charsetptr) != NULL);
+
+	return ret;
+}
+
+
 /*
  * 'cooks' an article, ie, prepare what will actually appear on the screen
  * It is not easy to do this in the same pass as the initial read since
@@ -638,12 +799,14 @@ dump_cooked(
  *		stripping of sigs if !show_signatures
  * Returns:
  *		TRUE on success
+ *
+ * TODO:
+ *      give an error-message on at least disk-full
  */
 t_bool
 cook_article(
 	t_bool wrap_lines,
 	t_openartinfo *artinfo,
-	int tabs,
 	int hide_uue)
 {
 	const char *charset;
@@ -651,6 +814,10 @@ cook_article(
 	char *line;
 	struct t_header *hdr = &artinfo->hdr;
 	t_bool header_put = FALSE;
+	static const char *struct_header[] = {
+		"Approved: ", "From: ", "Originator: ",
+		"Reply-To: ", "Sender: ", "X-Cancelled-By: ", "X-Comment-To: ",
+		"X-Submissions-To: ", "To: ", "Cc: ", "Bcc: ", "X-Originator: ", 0 };
 
 	art = artinfo;				/* Global saves lots of passing artinfo around */
 
@@ -678,12 +845,44 @@ cook_article(
 		}
 
 		if (header_wanted(line)) {	/* Put cooked data */
-			int i = LEN;
-			char *l = my_strdup(convert_body2printable(rfc1522_decode(line)));	/* FIXME: don't decode addr-part of From:/Cc:/ etc.pp. */
+			const char **strptr = struct_header;
+			char *l = NULL, *ptr, *foo, *bar;
+			size_t i = LEN;
+			t_bool found = FALSE;
 
+			/* structured headers */
+			do {
+				if (!strncasecmp(line, *strptr, strlen(*strptr))) {
+					foo = my_strdup(*strptr);
+					if ((ptr = strchr(foo, ':'))) {
+						*ptr = '\0';
+						if ((ptr = parse_header(line, foo, TRUE, TRUE))) {
+							bar = idna_decode(ptr);	/* do we wan't idna_decode() here? */
+							l = my_calloc(1, strlen(bar) + strlen(*strptr) + 1);
+							strncpy(l, line, strlen(*strptr));
+							strcat(l, bar);
+							free(bar);
+						}
+					}
+					free(foo);
+					found = TRUE;
+				}
+			} while (!found && *(++strptr) != 0);
+
+			/* unstructured but must not be decoded */
+			if (l == NULL && (!strncasecmp(line, "References: ", 12) || !strncasecmp(line, "Message-ID: ", 12) || !strncasecmp(line, "Date: ", 6) || !strncasecmp(line, "Newsgroups: ", 12) || !strncasecmp(line, "Distribution: ", 14) || !strncasecmp(line, "Followup-To: ", 13) || !strncasecmp(line, "X-Face: ", 8) || !strncasecmp(line, "Cancel-Lock: ", 13) || !strncasecmp(line, "Cancel-Key: ", 12)))
+				l = my_strdup(line);
+
+			if (l == NULL)
+				l = my_strdup(rfc1522_decode(line));
+
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			if (IS_LOCAL_CHARSET("UTF-8"))
+				utf8_valid(l);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			header_put = TRUE;
-			expand_ctrl_chars(&l, &i, tabs);
-			put_cooked(LEN, wrap_lines, C_HEADER, "%s", l);
+			expand_ctrl_chars(&l, &i, tabwidth);
+			put_cooked(i, wrap_lines, C_HEADER, "%s", l);
 			free(l);
 		}
 	}
@@ -710,27 +909,36 @@ cook_article(
 				charset = get_param(ptr->params, "charset");
 			else
 				charset = NULL;
-			PUT_ATTACH(ptr, ptr->depth * 4, name, charset);
+			PUT_ATTACH(ptr, (ptr->depth - 1) * 4, name, charset);
 
 			/* Try to view anything of type text, may need to review this */
-			if (IS_PLAINTEXT(ptr))
-				process_text_body_part(wrap_lines, artinfo->raw, ptr, hide_uue, tabs);
+			if (IS_PLAINTEXT(ptr)) {
+				if (charset_unsupported(charset)) {
+					put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_attach_unsup_charset), (ptr->depth - 1) * 4, "", charset);
+					if (ptr->next)
+						put_cooked(1, wrap_lines, C_ATTACH, "\n");
+				} else
+					process_text_body_part(wrap_lines, artinfo->raw, ptr, hide_uue);
+			}
 		}
 	} else {
+		if (!strcmp(content_types[hdr->ext->type], "text"))
+			charset = get_param(hdr->ext->params, "charset");
+		else
+			charset = NULL;
 		/*
 		 * A regular single-body article
 		 */
-		if (IS_PLAINTEXT(hdr->ext))
-			process_text_body_part(wrap_lines, artinfo->raw, hdr->ext, hide_uue, tabs);
-		else {
+		if (IS_PLAINTEXT(hdr->ext)) {
+			if (charset_unsupported(charset))
+				put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_attach_unsup_charset), 0, "", charset);
+			else
+				process_text_body_part(wrap_lines, artinfo->raw, hdr->ext, hide_uue);
+		} else {
 			/*
 			 * Non-textual main body
 			 */
 			name = get_filename(hdr->ext->params);
-			if (!strcmp(content_types[hdr->ext->type], "text"))
-				charset = get_param(hdr->ext->params, "charset");
-			else
-				charset = NULL;
 			PUT_ATTACH(hdr->ext, 0, name, charset);
 		}
 	}
