@@ -3,7 +3,7 @@
  *  Module    : curses.c
  *  Author    : D. Taylor & I. Lea
  *  Created   : 1986-01-01
- *  Updated   : 2005-04-18
+ *  Updated   : 2010-10-31
  *  Notes     : This is a screen management library borrowed with permission
  *              from the Elm mail system. This library was hacked to provide
  *              what tin needs.
@@ -32,6 +32,7 @@
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && defined(M_UNIX)
 #	define ReadWch cmdReadWch
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE && M_UNIX */
+#define get_arrow_key cmd_get_arrow_key
 
 void my_dummy(void) { }	/* ANSI C requires non-empty file */
 t_bool have_linescroll = TRUE;	/* USE_CURSES always allows line scrolling */
@@ -42,17 +43,6 @@ t_bool have_linescroll = TRUE;	/* USE_CURSES always allows line scrolling */
 #	undef	sinix
 #endif /* !ns32000 */
 
-#ifdef VMS
-#	include <descrip.h>
-#	include <iodef.h>
-#	include <ssdef.h>
-#	include <dvidef.h>
-#	ifdef __GNUC__ /* M.St. 22.01.98 */
-#		include <vms/sys$routines.h>
-#		include <vms/lib$routines.h>
-#	endif /* __GNUC__ */
-#endif /* VMS */
-
 #define DEFAULT_LINES_ON_TERMINAL	24
 #define DEFAULT_COLUMNS_ON_TERMINAL	80
 
@@ -65,89 +55,39 @@ t_bool have_linescroll = FALSE;
 
 #define TTYIN	0
 
-#ifdef HAVE_CONFIG_H
-
-#	if defined(HAVE_TERMIOS_H) && defined(HAVE_TCGETATTR) && defined(HAVE_TCSETATTR)
-#		ifdef HAVE_IOCTL_H
-#			include <ioctl.h>
-#		else
-#			ifdef HAVE_SYS_IOCTL_H
-#				include <sys/ioctl.h>
-#			endif /* HAVE_SYS_IOCTL_H */
-#		endif /* HAVE_IOCTL_H */
-#		if !defined(sun) || !defined(NL0)
-#			include <termios.h>
-#		endif /* !sun || !NL0 */
-#		define USE_POSIX_TERMIOS 1
-#		define TTY struct termios
+#if defined(HAVE_TERMIOS_H) && defined(HAVE_TCGETATTR) && defined(HAVE_TCSETATTR)
+#	ifdef HAVE_IOCTL_H
+#		include <ioctl.h>
 #	else
-#		ifdef HAVE_TERMIO_H
-#			include <termio.h>
-#			define USE_TERMIO 1
-#			define TTY struct termio
-#		else
-#			ifdef HAVE_SGTTY_H
-#				include <sgtty.h>
-#				define USE_SGTTY 1
-#				define TTY struct sgttyb
+#		ifdef HAVE_SYS_IOCTL_H
+#			include <sys/ioctl.h>
+#		endif /* HAVE_SYS_IOCTL_H */
+#	endif /* HAVE_IOCTL_H */
+#	if !defined(sun) || !defined(NL0)
+#		include <termios.h>
+#	endif /* !sun || !NL0 */
+#	define USE_POSIX_TERMIOS 1
+#	define TTY struct termios
+#else
+#	ifdef HAVE_TERMIO_H
+#		include <termio.h>
+#		define USE_TERMIO 1
+#		define TTY struct termio
+#	else
+#		ifdef HAVE_SGTTY_H
+#			include <sgtty.h>
+#			define USE_SGTTY 1
+#			define TTY struct sgttyb
 /*
 	#			else
 	#				error "No termios.h, termio.h or sgtty.h found"
 */
-#			endif /* HAVE_SGTTY_H */
-#		endif /* HAVE_TERMIO_H */
-#	endif /* HAVE_TERMIOS_H && HAVE_TCGETATTR && HAVE_TCSETATTR */
+#		endif /* HAVE_SGTTY_H */
+#	endif /* HAVE_TERMIO_H */
+#endif /* HAVE_TERMIOS_H && HAVE_TCGETATTR && HAVE_TCSETATTR */
 
 static TTY _raw_tty, _original_tty;
 
-#else	/* FIXME: prune the non-autoconf'd stuff */
-
-#	if defined(COHERENT) || defined(BSD)
-#		ifdef BSD4_1
-#			include <termio.h>
-#			define USE_TERMIO 1
-#		else
-#			include <sgtty.h>
-#			define USE_SGTTY 1
-#		endif /* BSD4_1 */
-#	else
-#		if !defined(SYSV)
-#			ifdef MINIX
-#				include <sgtty.h>
-#				define USE_SGTTY 1
-#			else
-#				ifndef QNX42
-#					ifdef sinix
-#						include <termios.h>
-#						define USE_POSIX_TERMIOS 1
-#					else
-#						ifdef VMS
-#							include <curses.h>
-#						else
-#							include <termio.h>
-#							define USE_TERMIO 1
-#						endif /* VMS */
-#					endif /* sinix */
-#				endif /* !QNX42 */
-#			endif /* MINIX */
-#		endif /* !SYSV */
-#	endif /* COHERENT || BSD */
-
-#	ifndef VMS
-#		if defined(BSD) || defined(MINIX)
-#			define USE_SGTTY 1
-struct sgttyb _raw_tty, _original_tty;
-#		else
-#			if defined(HAVE_TERMIOS_H) || defined(sinix)
-#				define USE_POSIX_TERMIOS 1
-struct termios _raw_tty, _original_tty;
-#			else
-#				define USE_TERMIO 1
-struct termio _raw_tty, _original_tty;
-#			endif /* HAVE_TERMIOS_H || sinix */
-#		endif /* BSD || MINIX */
-#	endif /* !VMS */
-#endif /* HAVE_CONFIG_H */
 
 #ifndef USE_SGTTY
 #	define USE_SGTTY 0
@@ -427,11 +367,6 @@ InitScreen(
 	_clearunderline = "\033[0m";
 	_keypadlocal = "";
 	_keypadxmit = "";
-#	ifdef VMS
-	_cleartoeos = "\033[J";
-	_terminalinit = NULL;
-	_terminalend = "";
-#	endif /* VMS */
 	/* needed for word highlighting */
 	_reset = "\033[0m";
 	_reversevideo = "\033[7m";
@@ -450,80 +385,6 @@ InitScreen(
 	/*
 	 * If that failed, try get a response from the console itself
 	 */
-#	ifdef VMS /* moved from below InitWin() M.St. 22.01.98 */
-	{
-		int input_chan, status;
-		int item_code, eightbit;
-		struct sensemode {
-			short status;
-			unsigned char xmit_baud;
-			unsigned char rcv_baud;
-			unsigned char crfill;
-			unsigned char lffill;
-			unsigned char parity;
-			unsigned char unused;
-			char class;
-			char type;
-			short scr_wid;
-			unsigned long tt_char : 24, scr_len : 8;
-			unsigned long tt2_char;
-		} tty;
-		$DESCRIPTOR(input_dsc, "TT");
-
-		status = SYS$ASSIGN(&input_dsc, &input_chan, 0, 0);
-		if (!(status & 1))
-			LIB$STOP(status);
-		SYS$QIOW(0, input_chan, IO$_SENSEMODE, &tty, 0, 0, &tty.class, 12, 0, 0, 0, 0);
-		item_code = DVI$_TT_EIGHTBIT;
-		status = LIB$GETDVI(&item_code, &input_chan, 0, &eightbit, 0, 0);
-		_columns = tty.scr_wid;
-		_lines = tty.scr_len;
-
-		if (eightbit) { /* if using eightbit then use CSI (octal 233) rather than ESC "[" */
-			_clearscreen = "\2331;1H\233J";
-			_moveto = "\233%d;%dH"; /* not a termcap string! */
-			_cleartoeoln = "\233K";
-			_cleartoeos = "\233J";
-			_setinverse = "\2337m";
-			_clearinverse = "\2330m";
-			_setunderline = "\2334m";
-			_clearunderline = "\2330m";
-			_keypadlocal = "";
-			_keypadxmit = "";
-		} else {
-			_clearscreen = "\033[1;1H\033[J";
-			_moveto = "\033[%d;%dH";	/* not a termcap string! */
-			_cleartoeoln = "\033[K";
-			_cleartoeos = "\033[J";
-			_setinverse = "\033[7m";
-			_clearinverse = "\033[0m";
-			_setunderline = "\033[4m";
-			_clearunderline = "\033[0m";
-			_keypadlocal = "";
-			_keypadxmit = "";
-		}
-#		ifdef HAVE_IS_XTERM
-		if (is_xterm()) {
-			xclicks = TRUE;
-			if (!eightbit) {
-				_xclickinit = "\033[?9h";
-				_xclickend = "\033[?9l";
-			}
-#			if 0
-			else {
-				/*
-				 * These are the settings for a DECterm but the reply can't easily be parsed
-				 * Reply is of the form - CSI Pe ; Pb ; Pr ; Pc & w
-				 * Where Pe is the event, Pb the button, Pr and Pc the row and column
-				 */
-				_xclickinit = "\2331;2'z";
-				_xclickend = "\2330;0'z";
-			}
-#			endif /* 0 */
-		}
-#		endif /* HAVE_IS_XTERM */
-	}
-#	endif /* VMS */
 
 	if (_lines < MIN_LINES_ON_TERMINAL || _columns < MIN_COLUMNS_ON_TERMINAL) {
 		my_fprintf(stderr, _(txt_screen_too_small), tin_progname);
@@ -606,6 +467,19 @@ ClearScreen(
 }
 
 
+#ifdef HAVE_COLOR
+void
+reset_screen_attr(
+	void)
+{
+	if (!NO_CAP(_reset)) {
+		tputs(_reset, 1, outchar);
+		my_flush();
+	}
+}
+#endif /* HAVE_COLOR */
+
+
 /*
  *  move cursor to the specified row column on the screen.
  *  0,0 is the top left!
@@ -807,38 +681,28 @@ void
 Raw(
 	int state)
 {
-#ifdef VMS
-	if (!state && _inraw) {
-/*		vmsnoraw(); */
-		_inraw = 0;
-	} else if (state && !_inraw) {
-/*		vmsraw(); */
-		_inraw = 1;
-	}
-#else
 	if (!state && _inraw) {
 		SET_TTY(&_original_tty);
 		_inraw = 0;
 	} else if (state && !_inraw) {
 		GET_TTY(&_original_tty);
 		GET_TTY(&_raw_tty);
-#		if USE_SGTTY
+#if USE_SGTTY
 		_raw_tty.sg_flags &= ~(ECHO | CRMOD);	/* echo off */
 		_raw_tty.sg_flags |= CBREAK;		/* raw on */
-#		else
-#			ifdef __FreeBSD__
+#else
+#	ifdef __FreeBSD__
 		cfmakeraw(&_raw_tty);
 		_raw_tty.c_lflag |= ISIG;		/* for ^Z */
-#			else
+#	else
 		_raw_tty.c_lflag &= ~(ICANON | ECHO);	/* noecho raw mode */
 		_raw_tty.c_cc[VMIN] = '\01';	/* minimum # of chars to queue */
 		_raw_tty.c_cc[VTIME] = '\0';	/* minimum time to wait for input */
-#			endif /* __FreeBSD__ */
-#		endif /* USE_SGTTY */
+#	endif /* __FreeBSD__ */
+#endif /* USE_SGTTY */
 		SET_TTY(&_raw_tty);
 		_inraw = 1;
 	}
-#endif /* !VMS */
 }
 
 
@@ -953,8 +817,8 @@ highlight_string(
 	MoveCursor(row, col);
 	StartInverse();
 	my_fputs(output, stdout);
-	my_flush();
 	EndInverse();
+	my_flush();
 
 	stow_cursor();
 }
@@ -1070,6 +934,7 @@ word_highlight_string(
 			tputs(_reset, 1, outchar);
 	stow_cursor();
 }
+#endif /* USE_CURSES */
 
 
 /*
@@ -1083,6 +948,7 @@ input_pending(
 {
 #if 0
 	int ch;
+
 	nodelay(stdscr, TRUE);
 	if ((ch = getch()) != ERR)
 		ungetch(ch);
@@ -1153,17 +1019,16 @@ get_arrow_key(
 	while (!input_pending(0) \
 		&& i < ((VT_ESCAPE_TIMEOUT * 1000) / SECOND_CHARACTER_DELAY))
 
-#	ifndef VMS
 	if (!input_pending(0)) {
-#			ifdef HAVE_USLEEP
+#	ifdef HAVE_USLEEP
 		int i = 0;
 
 		wait_a_while(i) {
 			usleep((unsigned long) (SECOND_CHARACTER_DELAY * 1000));
 			i++;
 		}
-#			else	/* !HAVE_USLEEP */
-#				ifdef HAVE_SELECT
+#	else	/* !HAVE_USLEEP */
+#		ifdef HAVE_SELECT
 		struct timeval tvptr;
 		int i = 0;
 
@@ -1173,8 +1038,8 @@ get_arrow_key(
 			select(0, NULL, NULL, NULL, &tvptr);
 			i++;
 		}
-#				else /* !HAVE_SELECT */
-#					ifdef HAVE_POLL
+#		else /* !HAVE_SELECT */
+#			ifdef HAVE_POLL
 		struct pollfd fds[1];
 		int i = 0;
 
@@ -1182,15 +1047,14 @@ get_arrow_key(
 			poll(fds, 0, SECOND_CHARACTER_DELAY);
 			i++;
 		}
-#					else /* !HAVE_POLL */
+#			else /* !HAVE_POLL */
 		(void) sleep(1);
-#					endif /* HAVE_POLL */
-#				endif /* HAVE_SELECT */
-#			endif /* HAVE_USLEEP */
+#			endif /* HAVE_POLL */
+#		endif /* HAVE_SELECT */
+#	endif /* HAVE_USLEEP */
 		if (!input_pending(0))
 			return prech;
 	}
-#	endif /* !VMS */
 	ch = ReadCh();
 	if (ch == '[' || ch == 'O')
 		ch = ReadCh();
@@ -1293,7 +1157,6 @@ get_arrow_key(
 			return KEYMAP_UNKNOWN;
 	}
 }
-#endif /* USE_CURSES */
 
 
 /*
@@ -1388,7 +1251,7 @@ ReadWch(
 	allow_resize(TRUE);
 	while ((result = read(0, mbs, 1)) < 0 && errno == EINTR) { /* spin on signal interrupts */
 		if (need_resize) {
-			handle_resize((need_resize == cRedraw) ? TRUE :  FALSE);
+			handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
 			need_resize = cNo;
 		}
 	}
@@ -1455,7 +1318,7 @@ ReadWch(
 			allow_resize(TRUE);
 			while ((result = read(0, mbs + 1, to_read)) < 0 && errno == EINTR) { /* spin on signal interrupts */
 				if (need_resize) {
-					handle_resize((need_resize == cRedraw) ? TRUE :  FALSE);
+					handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
 					need_resize = cNo;
 				}
 			}
@@ -1479,6 +1342,7 @@ ReadWch(
 
 	/* FIXME: add support for other multi-byte charsets */
 
+	free(mbs);
 	return WEOF;
 }
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
