@@ -3,10 +3,10 @@
  *  Module    : filter.c
  *  Author    : I. Lea
  *  Created   : 1992-12-28
- *  Updated   : 2006-06-19
+ *  Updated   : 2010-11-13
  *  Notes     : Filter articles. Kill & auto selection are supported.
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,6 +48,7 @@
 
 #define IS_READ(i)	(arts[i].status == ART_READ)
 #define IS_KILLED(i)	(arts[i].killed)
+#define IS_KILLED_UNREAD(i)	(arts[i].killed == ART_KILLED_UNREAD)
 #define IS_SELECTED(i)	(arts[i].selected)
 
 /*
@@ -83,6 +84,12 @@
 struct t_filters glob_filter = { 0, 0, (struct t_filter *) 0 };
 
 
+/*
+ * Global filter file offset
+ */
+int filter_file_offset;
+
+
 /*
  * Local prototypes
  */
@@ -209,7 +216,7 @@ test_regex(
 			if (regex_errpos >= 0)
 				return 1;
 			else if (regex_errpos != PCRE_ERROR_NOMATCH) {
-				error_message(_(txt_pcre_error_num), regex_errpos);
+				error_message(2, _(txt_pcre_error_num), regex_errpos);
 				return -1;
 			}
 		}
@@ -310,7 +317,7 @@ read_filter_file(
 	t_bool expired_time = FALSE;
 	time_t current_secs = (time_t) 0;
 	static t_bool first_read = TRUE;
-	int upgrade = RC_CHECK;
+	enum rc_state upgrade = RC_CHECK;
 
 	if ((fp = fopen(file, "r")) == NULL)
 		return FALSE;
@@ -326,14 +333,19 @@ read_filter_file(
 	if (!first_read)
 		free_filter_array(&glob_filter);
 
+	filter_file_offset = 1;
+	scope[0] = '\0';
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
-		if (*buf == '#' || *buf == '\n') {
-			if (upgrade == RC_CHECK && first_read) {
+		if (*buf == '\n')
+			continue;
+		if (*buf == '#') {
+			if (scope[0] == '\0')
+				filter_file_offset++;
+			if (upgrade == RC_CHECK && first_read && match_string(buf, "# Filter file V", NULL, 0)) {
 				first_read = FALSE;
 				upgrade = check_upgrade(buf, "# Filter file V", FILTER_VERSION);
 				if (upgrade != RC_IGNORE)
-					upgrade_prompt_quit(upgrade, FILTER_FILE);
-				/* TODO: do something (more) useful here */
+					upgrade_prompt_quit(upgrade, FILTER_FILE); /* TODO: do something (more) useful here */
 			}
 			continue;
 		}
@@ -346,8 +358,10 @@ read_filter_file(
 
 					break;
 				}
-				if (match_string(buf + 1, "omment=", comment_line, sizeof(comment_line)))
+				if (match_string(buf + 1, "omment=", comment_line, sizeof(comment_line))) {
+					str_trim(comment_line);
 					comment = add_filter_comment(comment, comment_line);
+				}
 				break;
 
 			case 'f':
@@ -368,11 +382,10 @@ read_filter_file(
 
 			case 'g':
 				if (match_string(buf + 1, "roup=", scope, sizeof(scope))) {
+					str_trim(scope);
 #ifdef DEBUG
-					if (debug) {
-						my_printf("group=[%s] num=[%d]\n", scope, glob_filter.num);
-						my_flush();
-					}
+					if (debug & DEBUG_FILTER)
+						debug_print_file("FILTER", "\nnum=[%d] group=[%s]", glob_filter.num, scope);
 #endif /* DEBUG */
 					if (glob_filter.num >= glob_filter.max)
 						expand_filter_array(&glob_filter);
@@ -504,14 +517,11 @@ read_filter_file(
 							FreeIfNeeded(ptr[i].subj);
 							ptr[i].subj = my_strdup(subj);
 						}
-					}
-
 #ifdef DEBUG
-					if (debug) {
-						my_printf("6. buf=[%s]  Gsubj=[%s]\n", ptr[i].subj, glob_filter.filter[i].subj);
-						my_flush();
-					}
+						if (debug & DEBUG_FILTER)
+							debug_print_file("FILTER","buf=[%s]  Gsubj=[%s]", ptr[i].subj, glob_filter.filter[i].subj);
 #endif /* DEBUG */
+					}
 					break;
 				}
 
@@ -521,10 +531,8 @@ read_filter_file(
 				if (match_string(buf + 1, "core=", scbuf, PATH_LEN)) {
 					score = atoi(scbuf);
 #ifdef DEBUG
-					if (debug) {
-						my_printf("score=[%d]\n", score);
-						my_flush();
-					}
+					if (debug & DEBUG_FILTER)
+						debug_print_file("FILTER","score=[%d]", score);
 #endif /* DEBUG */
 					if (ptr && !expired_time) {
 						if (score > SCORE_MAX)
@@ -555,10 +563,8 @@ read_filter_file(
 						/* rule expired? */
 						if (secs && current_secs > (time_t) secs) {
 #ifdef DEBUG
-							if (debug) {
-								my_printf("EXPIRED  secs=[%lu]  current_secs=[%lu]\n", (unsigned long int) secs, (unsigned long int) current_secs);
-								my_flush();
-							}
+							if (debug & DEBUG_FILTER)
+								debug_print_file("FILTER", "EXPIRED  secs=[%lu]  current_secs=[%lu]", (unsigned long int) secs, (unsigned long int) current_secs);
 #endif /* DEBUG */
 							glob_filter.num--;
 							expired_time = TRUE;
@@ -574,6 +580,7 @@ read_filter_file(
 				 *       should we comment out older xref rules like below?
 				 */
 				if (match_string(buf + 1, "ref=", xref, sizeof(xref))) {
+					str_trim(xref);
 					if (ptr && !expired_time) {
 						if (tinrc.wildcard && ptr[i].xref != NULL) {
 							/* merge with already read value */
@@ -592,17 +599,17 @@ read_filter_file(
 
 					if (match_string(buf + 1, "ref_max=", foo, LEN - 1)) {
 						/*
-						 * TODO: add to the right rule, give better explanation, -> lang.c
+						 * TODO: add to the right rule, give better explanation.
 						 */
-						snprintf(foo, HEADER_LEN, "%s%s", _("Removed from the previous rule: "), str_trim(buf));
+						snprintf(foo, HEADER_LEN, "%s%s", _(txt_removed_rule), str_trim(buf));
 						comment = add_filter_comment(comment, foo);
 						break;
 					}
 					if (match_string(buf + 1, "ref_score=", foo, LEN - 1)) {
 						/*
-						 * TODO: add to the right rule, give better explanation, -> lang.c
+						 * TODO: add to the right rule, give better explanation.
 						 */
-						snprintf(foo, HEADER_LEN, "%s%s", _("Removed from the previous rule: "), str_trim(buf));
+						snprintf(foo, HEADER_LEN, "%s%s", _(txt_removed_rule), str_trim(buf));
 						comment = add_filter_comment(comment, foo);
 					}
 				}
@@ -617,10 +624,7 @@ read_filter_file(
 	if (expired || upgrade == RC_UPGRADE)
 		write_filter_file(file);
 
-	if (cmd_line && !batch_mode)
-		printf("\r\n");
-
-	if (!batch_mode)
+	if (!cmd_line && !batch_mode)
 		clear_message();
 
 	return TRUE;
@@ -636,6 +640,8 @@ write_filter_file(
 {
 	FILE *fp;
 	char *file_tmp;
+	int i;
+	long fpos;
 
 	if (no_write)
 		return;
@@ -644,17 +650,30 @@ write_filter_file(
 	file_tmp = get_tmpfilename(filename);
 
 	if (!backup_file(filename, file_tmp)) {
-		error_message(_(txt_filesystem_full_backup), filename);
+		error_message(2, _(txt_filesystem_full_backup), filename);
 		free(file_tmp);
 		return;
 	}
 
-	if ((fp = fopen(filename, "w")) == NULL)
+	if ((fp = fopen(filename, "w+")) == NULL)
 		return;
 
 	/* TODO: -> lang.c */
 	fprintf(fp, "# Filter file V%s for the TIN newsreader\n#\n", FILTER_VERSION);
 	fprintf(fp, _(txt_filter_file));
+
+	/* determine the file offset */
+	if (!batch_mode) {
+		fpos = ftell(fp);
+		rewind(fp);
+		filter_file_offset = 1;
+		while((i = fgetc(fp)) != EOF) {
+			if (i == '\n')
+				filter_file_offset++;
+		}
+		fseek(fp, fpos, SEEK_SET);
+	}
+
 	fflush(fp);
 
 	/*
@@ -662,9 +681,13 @@ write_filter_file(
 	 */
 	write_filter_array(fp, &glob_filter);
 
-	if (ferror(fp) || fclose(fp)) {
-		error_message(_(txt_filesystem_full), filename);
+	if ((i = ferror(fp)) || fclose(fp)) {
+		error_message(2, _(txt_filesystem_full), filename);
 		rename_file(file_tmp, filename);
+		if (i) {
+			clearerr(fp);
+			fclose(fp);
+		}
 	} else
 		unlink(file_tmp);
 
@@ -685,14 +708,18 @@ write_filter_array(
 		return;
 
 	for (i = 0; i < ptr->num; i++) {
-/* my_printf("WRITE i=[%d] subj=[%s] from=[%s]\n", i, BlankIfNull(ptr->filter[i].subj), BlankIfNull(ptr->filter[i].from)); */
+#ifdef DEBUG
+		if (debug & DEBUG_FILTER)
+			debug_print_file("FILTER", "WRITE i=[%d] subj=[%s] from=[%s]\n", i, BlankIfNull(ptr->filter[i].subj), BlankIfNull(ptr->filter[i].from));
+#endif /* DEBUG */
 
 		if (ptr->filter[i].time && theTime > ptr->filter[i].time)
 				continue;
-/*
-my_printf("Scope=[%s]" cCRLF, (ptr->filter[i].scope != NULL ? ptr->filter[i].scope : "*"));
-my_flush();
-*/
+#ifdef DEBUG
+		if (debug & DEBUG_FILTER)
+			debug_print_file("FILTER", "Scope=[%s]" cCRLF, (ptr->filter[i].scope != NULL ? ptr->filter[i].scope : "*"));
+#endif /* DEBUG */
+
 		fprintf(fp, "\n");		/* makes filter file more readable */
 
 		/* comments appear always first, if there are any... */
@@ -812,9 +839,6 @@ get_choice(
 	int list_size)
 {
 	int ch, y, i = 0;
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *wbuf;
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	if (help)
 		show_menu_help(help);
@@ -822,16 +846,7 @@ get_choice(
 	if (list == NULL || list_size < 1)
 		return -1;
 
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	if ((wbuf = char2wchar_t(prompt)) != NULL) {
-		wconvert_to_printable(wbuf);
-		if ((y = wcswidth(wbuf, wcslen(wbuf) + 1)) == -1) /* something went wrong, use wcslen() as fallback */
-			y = wcslen(wbuf);
-
-		free(wbuf);
-	} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		y = (int) strlen(prompt);
+	y = strwidth(prompt);
 
 	do {
 		MoveCursor(x, y);
@@ -999,14 +1014,11 @@ filter_menu(
 	char text_time[PATH_LEN];
 	char double_time[PATH_LEN];
 	char quat_time[PATH_LEN];
-	int i, len, clen = 0, flen = 0;
+	int i, len, clen, flen;
 	struct t_filter_rule rule;
 	t_bool proceed;
 	t_bool ret;
 	t_function func, default_func = FILTER_SAVE;
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *wbuf;
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	signal_context = cFilter;
 
@@ -1062,49 +1074,16 @@ filter_menu(
 	ptr_filter_comment = _(txt_filter_comment);
 	ptr_filter_groupname = group->name;
 
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	if ((wbuf = char2wchar_t(_(txt_no))) != NULL) {
-		clen = MAX(clen, wcswidth(wbuf, wcslen(wbuf)));
-		free(wbuf);
-	}
-	if ((wbuf = char2wchar_t(_(txt_yes))) != NULL) {
-		clen = MAX(clen, wcswidth(wbuf, wcslen(wbuf)));
-		free(wbuf);
-	}
-	if ((wbuf = char2wchar_t(_(txt_full))) != NULL) {
-		clen = MAX(clen, wcswidth(wbuf, wcslen(wbuf)));
-		free(wbuf);
-	}
-	if ((wbuf = char2wchar_t(_(txt_last))) != NULL) {
-		clen = MAX(clen, wcswidth(wbuf, wcslen(wbuf)));
-		free(wbuf);
-	}
-	if ((wbuf = char2wchar_t(_(txt_only))) != NULL) {
-		clen = MAX(clen, wcswidth(wbuf, wcslen(wbuf)));
-		free(wbuf);
-	}
-	if ((wbuf = char2wchar_t(ptr_filter_subj)) != NULL) {
-		flen = MAX(flen, wcswidth(wbuf, wcslen(wbuf)) - 2);
-		free(wbuf);
-	}
-	if ((wbuf = char2wchar_t(ptr_filter_from)) != NULL) {
-		flen = MAX(flen, wcswidth(wbuf, wcslen(wbuf)) - 2);
-		free(wbuf);
-	}
-	if ((wbuf = char2wchar_t(ptr_filter_msgid)) != NULL) {
-		flen = MAX(flen, wcswidth(wbuf, wcslen(wbuf)) - 2);
-		free(wbuf);
-	}
-#else
-	clen = MAX(clen, (int) strlen(_(txt_no)));
-	clen = MAX(clen, (int) strlen(_(txt_yes)));
-	clen = MAX(clen, (int) strlen(_(txt_full)));
-	clen = MAX(clen, (int) strlen(_(txt_last)));
-	clen = MAX(clen, (int) strlen(_(txt_only)));
-	flen = MAX(flen, (int) strlen(ptr_filter_subj) - 2);
-	flen = MAX(flen, (int) strlen(ptr_filter_from) - 2);
-	flen = MAX(flen, (int) strlen(ptr_filter_msgid) - 2);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	clen = strwidth(_(txt_no));
+	clen = MAX(clen, strwidth(_(txt_yes)));
+	clen = MAX(clen, strwidth(_(txt_full)));
+	clen = MAX(clen, strwidth(_(txt_last)));
+	clen = MAX(clen, strwidth(_(txt_only)));
+
+	flen = strwidth(ptr_filter_subj) - 2;
+	flen = MAX(flen, strwidth(ptr_filter_from) - 2);
+	flen = MAX(flen, strwidth(ptr_filter_msgid) - 2);
+
 	len = cCOLS - flen - clen - 1 + 4;
 
 	snprintf(text_time, sizeof(text_time), _(txt_time_default_days), tinrc.filter_days);
@@ -1140,14 +1119,17 @@ filter_menu(
 	}
 
 	if (*rule.text) {
-		list = my_malloc(sizeof(char *) * 5);
+		list = my_malloc(sizeof(char *) * 8);
 		list[0] = (char *) _(txt_subj_line_only_case);
 		list[1] = (char *) _(txt_subj_line_only);
 		list[2] = (char *) _(txt_from_line_only_case);
 		list[3] = (char *) _(txt_from_line_only);
-		list[4] = (char *) _(txt_msgid_line_only);
+		list[4] = (char *) _(txt_msgid_refs_line);
+		list[5] = (char *) _(txt_msgid_line_last);
+		list[6] = (char *) _(txt_msgid_line_only);
+		list[7] = (char *) _(txt_refs_line_only);
 
-		i = get_choice(INDEX_TOP + 3, _(txt_help_filter_text_type), _(txt_filter_text_type), list, 5);
+		i = get_choice(INDEX_TOP + 3, _(txt_help_filter_text_type), _(txt_filter_text_type), list, 8);
 		free(list);
 
 		if (i == -1) {
@@ -1165,10 +1147,9 @@ filter_menu(
 			case FILTER_SUBJ_CASE_SENSITIVE:
 			case FILTER_FROM_CASE_SENSITIVE:
 			case FILTER_MSGID:
-				rule.counter = FILTER_MSGID_ONLY;
-/*			case FILTER_MSGID_LAST:
-			case FILTER_MSGID_ONLY: */
-				/* rule.icase is FALSE already, no assignment necessary */
+			case FILTER_MSGID_LAST:
+			case FILTER_MSGID_ONLY:
+			case FILTER_REFS_ONLY:
 				break;
 
 			default: /* should not happen */
@@ -1407,9 +1388,9 @@ filter_menu(
 		case FILTER_EDIT:
 			add_filter_rule(group, art, &rule, FALSE); /* save the rule */
 			rule.comment = free_filter_comment(rule.comment);
-			if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
+			if (!invoke_editor(filter_file, filter_file_offset, NULL))
 				return FALSE;
-			unfilter_articles();
+			unfilter_articles(group);
 			(void) read_filter_file(filter_file);
 			return TRUE;
 			/* keep lint quiet: */
@@ -1469,10 +1450,8 @@ quick_filter(
 	}
 
 #ifdef DEBUG
-	if (debug)
-		error_message("%s header=[%d] scope=[%s] expire=[%s] case=[%d]",
-			(type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header,
-			BlankIfNull(scope), txt_onoff[expire != FALSE ? 1 : 0], icase);
+	if (debug & DEBUG_FILTER)
+		error_message(2, "%s header=[%d] scope=[%s] expire=[%s] case=[%d]", (type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header, BlankIfNull(scope), txt_onoff[expire != FALSE ? 1 : 0], icase);
 #endif /* DEBUG */
 
 	/*
@@ -1722,7 +1701,7 @@ add_filter_rule(
 			 * So the thread remains open (in group level). To overcome this,
 			 * the first msgid from references field is taken in this case.
 			 */
-			if (quick_filter_rule && group->attribute->thread_arts == THREAD_REFS &&
+			if (quick_filter_rule && group->attribute->thread_articles == THREAD_REFS &&
 				(group->attribute->quick_kill_header == FILTER_MSGID ||
 				 group->attribute->quick_kill_header == FILTER_REFS_ONLY) &&
 				 art->refptr->parent != NULL)
@@ -1748,18 +1727,9 @@ add_filter_rule(
 
 	if (filtered) {
 #ifdef DEBUG
-		if (debug)
-			wait_message(2, "inscope=[%s] scope=[%s]  case=[%d] subj=[%s] from=[%s] msgid=[%s] fullref=[%d] line=[%d %d] time=[%lu]",
-				bool_unparse(ptr[i].inscope),
-				BlankIfNull(rule->scope),
-				ptr[i].icase,
-				BlankIfNull(ptr[i].subj),
-				BlankIfNull(ptr[i].from),
-				BlankIfNull(ptr[i].msgid),
-				ptr[i].fullref, ptr[i].lines_cmp,
-				ptr[i].lines_num, (unsigned long int) ptr[i].time);
+		if (debug & DEBUG_FILTER)
+			wait_message(2, "inscope=[%s] scope=[%s] case=[%d] subj=[%s] from=[%s] msgid=[%s] fullref=[%d] line=[%d %d] time=[%lu]", bool_unparse(ptr[i].inscope), BlankIfNull(rule->scope), ptr[i].icase, BlankIfNull(ptr[i].subj), BlankIfNull(ptr[i].from), BlankIfNull(ptr[i].msgid), ptr[i].fullref, ptr[i].lines_cmp, ptr[i].lines_num, (unsigned long int) ptr[i].time);
 #endif /* DEBUG */
-
 		write_filter_file(filter_file);
 	}
 	return filtered;
@@ -1768,20 +1738,22 @@ add_filter_rule(
 
 /*
  * We assume that any articles which are tagged as killed are also
- * tagged as being read BECAUSE they were killed. So, we retag
- * them as being unread. Selected articles will be un"select"ed.
+ * tagged as being read BECAUSE they were killed. We retag them as
+ * being unread if they were unread before killing (ART_KILLED_UNREAD).
+ * Selected articles will be un"select"ed.
  */
 void
 unfilter_articles(
-	void)
+	struct t_group *group)
 {
 	int i;
 
 	for_each_art(i) {
 		arts[i].score = 0;
 		if (IS_KILLED(i)) {
+			if (IS_KILLED_UNREAD(i))
+				art_mark(group, &arts[i], ART_UNREAD);
 			arts[i].killed = ART_NOTKILLED;
-			arts[i].status = ART_UNREAD;
 		}
 		if (IS_SELECTED(i))
 			arts[i].selected = FALSE;
@@ -1854,7 +1826,7 @@ filter_articles(
 	}
 
 	/*
-	 * loop thru all arts applying global & local filtering rules
+	 * loop through all arts applying global & local filtering rules
 	 */
 	for (i = 0; (i < top_art) && !error; i++) {
 		arts[i].score = 0;
@@ -1960,6 +1932,10 @@ filter_articles(
 					switch (x) {
 						case 1:
 							SET_FILTER(group, i, j);
+#ifdef DEBUG
+							if (debug & DEBUG_FILTER)
+								debug_print_file("FILTER", "Group[%s] Rule[%d][%s] ArtMSGID[%s] Artnum[%d]", group->name, j, ptr[j].msgid, mymsgid, i);
+#endif /* DEBUG */
 							break;
 
 						case -1:
@@ -1978,27 +1954,18 @@ filter_articles(
 					switch (ptr[j].lines_cmp) {
 						case FILTER_LINES_EQ:
 							if (arts[i].line_count == ptr[j].lines_num) {
-/*
-wait_message(1, "FILTERED Lines arts[%d] == [%d]", arts[i].line_count, ptr[j].lines_num);
-*/
 								SET_FILTER(group, i, j);
 							}
 							break;
 
 						case FILTER_LINES_LT:
 							if (arts[i].line_count < ptr[j].lines_num) {
-/*
-wait_message(1, "FILTERED Lines arts[%d] < [%d]", arts[i].line_count, ptr[j].lines_num);
-*/
 								SET_FILTER(group, i, j);
 							}
 							break;
 
 						case FILTER_LINES_GT:
 							if (arts[i].line_count > ptr[j].lines_num) {
-/*
-wait_message(1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lines_num);
-*/
 								SET_FILTER(group, i, j);
 							}
 							break;
@@ -2049,7 +2016,7 @@ wait_message(1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lin
 						t_bool skip = FALSE;
 
 						s = arts[i].xref;
-						while (*s && !isspace((int) *s))
+						while (*s && !isspace((int) *s))	/* skip server name */
 							s++;
 						while (*s && isspace((int) *s))
 							s++;
@@ -2123,11 +2090,16 @@ wait_message(1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lin
 					arts[i].killed = ART_KILLED;
 				filtered = TRUE;
 				art_mark(group, &arts[i], ART_READ);
+				if (group->attribute->show_only_unread_arts)
+					arts[i].keep_in_base = FALSE;
 			} else if (arts[i].score >= tinrc.score_limit_select) {
 				arts[i].selected = TRUE;
 			}
 		}
 	}
+	if (!cmd_line && !batch_mode)
+		clear_message();
+
 	return filtered;
 }
 
