@@ -3,10 +3,10 @@
  *  Module    : group.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2005-07-02
+ *  Updated   : 2010-03-14
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,15 +57,15 @@ struct t_group *curr_group;
 static int do_search(t_function func, t_bool repeat);
 static int enter_pager(int art, t_bool ignore_unavail);
 static int enter_thread(int depth, t_pagerinfo *page);
+static int find_new_pos(long old_artnum, int cur_pos);
 static int group_catchup(t_function func);
-static int prompt_getart_limit(void);
 static int tab_pressed(void);
+static t_bool prompt_getart_limit(void);
 static t_function group_left(void);
 static t_function group_right(void);
 static void build_sline(int i);
 static void build_multipart_header(char *dest, int maxlen, const char *src, int cmplen, int have, int total);
 static void draw_subject_arrow(void);
-static void mark_thd_read(struct t_group *group, t_bool range_active);
 static void show_group_title(t_bool clear_title);
 static void show_tagged_lines(void);
 static void toggle_read_unread(t_bool force);
@@ -98,7 +98,7 @@ static t_function
 group_left(
 	void)
 {
-	if (tinrc.group_catchup_on_exit)
+	if (curr_group->attribute->group_catchup_on_exit)
 		return SPECIAL_CATCHUP_LEFT;		/* ie, not via 'c' or 'C' */
 	else
 		return GLOBAL_QUIT;
@@ -110,7 +110,7 @@ group_right(
 	void)
 {
 	if (grpmenu.curr >= 0 && HAS_FOLLOWUPS(grpmenu.curr)) {
-		if (tinrc.auto_list_thread)
+		if (curr_group->attribute->auto_list_thread)
 			return GROUP_LIST_THREAD;
 		else {
 			int n = next_unread((int) base[grpmenu.curr]);
@@ -140,12 +140,10 @@ group_page(
 {
 	char key[MAXKEYLEN];
 	int i, n, ii;
-	int old_top = 0;
 	int thread_depth;	/* Starting depth in threads we enter */
 	long old_artnum = 0L;
 	struct t_art_stat sbuf;
 	t_bool flag;
-	t_bool range_active = FALSE;		/* Set if a range is defined */
 	t_bool xflag = FALSE;	/* 'X'-flag */
 	t_bool repeat_search;
 	t_function func;
@@ -157,6 +155,7 @@ group_page(
 
 	curr_group = group;					/* For global access to the current group */
 	num_of_tagged_arts = 0;
+	range_active = FALSE;
 
 	last_resp = -1;
 	this_resp = -1;
@@ -179,16 +178,17 @@ group_page(
 	clear_note_area();
 
 	if (group->attribute->auto_select) {
-		error_message(_(txt_autoselecting_articles), printascii(key, func_to_key(GROUP_MARK_UNSELECTED_ARTICLES_READ, group_keys)));
+		error_message(2, _(txt_autoselecting_articles), printascii(key, func_to_key(GROUP_MARK_UNSELECTED_ARTICLES_READ, group_keys)));
 		do_auto_select_arts();						/* 'X' command */
 		xflag = TRUE;
 	}
 
 	show_group_page();
 
-#	ifdef DEBUG_NEWSRC
-	debug_print_bitmap(group, NULL);
-#	endif /* DEBUG_NEWSRC */
+#	ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_bitmap(group, NULL);
+#	endif /* DEBUG */
 
 	/* reset ret_code */
 	ret_code = 0;
@@ -243,38 +243,41 @@ group_page(
 
 			case GLOBAL_PIPE:		/* pipe article/thread/tagged arts to command */
 				if (grpmenu.curr >= 0)
-					feed_articles(FEED_PIPE, GROUP_LEVEL, group, (int) base[grpmenu.curr]);
+					feed_articles(FEED_PIPE, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
 				break;
 
 			case GROUP_MAIL:	/* mail article to somebody */
 				if (grpmenu.curr >= 0)
-					feed_articles(FEED_MAIL, GROUP_LEVEL, group, (int) base[grpmenu.curr]);
+					feed_articles(FEED_MAIL, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
 				break;
 
 #ifndef DISABLE_PRINTING
 			case GLOBAL_PRINT:	/* output art/thread/tagged arts to printer */
 				if (grpmenu.curr >= 0)
-					feed_articles(FEED_PRINT, GROUP_LEVEL, group, (int) base[grpmenu.curr]);
+					feed_articles(FEED_PRINT, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
 				break;
 #endif /* !DISABLE_PRINTING */
 
 			case GROUP_REPOST:	/* repost current article */
-				if (grpmenu.curr >= 0)
-					feed_articles(FEED_REPOST, GROUP_LEVEL, group, (int) base[grpmenu.curr]);
+				if (can_post) {
+					if (grpmenu.curr >= 0)
+						feed_articles(FEED_REPOST, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
+				} else
+					info_message(_(txt_cannot_post));
 				break;
 
 			case GROUP_SAVE:	/* save articles with prompting */
 				if (grpmenu.curr >= 0)
-					feed_articles(FEED_SAVE, GROUP_LEVEL, group, (int) base[grpmenu.curr]);
+					feed_articles(FEED_SAVE, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
 				break;
 
 			case GROUP_AUTOSAVE:	/* Auto-save articles without prompting */
 				if (grpmenu.curr >= 0)
-					feed_articles(FEED_AUTOSAVE, GROUP_LEVEL, group, (int) base[grpmenu.curr]);
+					feed_articles(FEED_AUTOSAVE, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
 				break;
 
 			case GLOBAL_SET_RANGE:	/* set range */
-				if (set_range(GROUP_LEVEL, 1, grpmenu.max, grpmenu.curr + 1)) {
+				if (grpmenu.curr >= 0 && set_range(GROUP_LEVEL, 1, grpmenu.max, grpmenu.curr + 1)) {
 					range_active = TRUE;
 					show_group_page();
 				}
@@ -303,6 +306,24 @@ group_page(
 					info_message(_(txt_no_arts));
 				break;
 
+			case GROUP_CANCEL:	/* cancel current basenote */
+				if (grpmenu.curr >= 0) {
+					if (can_post || group->attribute->mailing_list != NULL) {
+						char *progress_msg = my_strdup(_(txt_reading_article));
+						int ret;
+
+						n = (int) base[grpmenu.curr];
+						ret = art_open(TRUE, &arts[n], group, &pgart, TRUE, progress_msg);
+						free(progress_msg);
+						if (ret != ART_UNAVAILABLE && ret != ART_ABORT && cancel_article(group, &arts[n], n))
+							show_group_page();
+						art_close(&pgart);
+					} else
+						info_message(_(txt_cannot_post));
+				} else
+					info_message(_(txt_no_arts));
+				break;
+
 			case GROUP_NEXT_UNREAD_ARTICLE_OR_GROUP:	/* goto next unread article/group */
 				ret_code = tab_pressed();
 				break;
@@ -317,26 +338,25 @@ group_page(
 					info_message(_(txt_no_arts));
 					break;
 				}
-				old_top = top_art;
 				n = (int) base[grpmenu.curr];
-				old_artnum = arts[n].artnum;
 				if (filter_menu(func, group, &arts[n])) {
-					if (filter_articles(group)) {
-						make_threads(group, FALSE);
-						grpmenu.curr = find_new_pos(old_top, old_artnum, grpmenu.curr);
-					}
+					old_artnum = arts[n].artnum;
+					unfilter_articles(group);
+					filter_articles(group);
+					make_threads(group, FALSE);
+					grpmenu.curr = find_new_pos(old_artnum, grpmenu.curr);
 				}
 				show_group_page();
 				break;
 
 			case GLOBAL_EDIT_FILTER:
-				if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
-					break;
-				unfilter_articles();
-				(void) read_filter_file(filter_file);
-				if (filter_articles(group)) {
+				if (invoke_editor(filter_file, filter_file_offset, NULL)) {
+					old_artnum = grpmenu.max > 0 ? arts[(int) base[grpmenu.curr]].artnum : -1L;
+					unfilter_articles(group);
+					(void) read_filter_file(filter_file);
+					filter_articles(group);
 					make_threads(group, FALSE);
-					grpmenu.curr = find_new_pos(old_top, old_artnum, grpmenu.curr);
+					grpmenu.curr = old_artnum >= 0L ? find_new_pos(old_artnum, grpmenu.curr) : grpmenu.max - 1;
 				}
 				show_group_page();
 				break;
@@ -348,18 +368,16 @@ group_page(
 					break;
 				}
 				if ((!TINRC_CONFIRM_ACTION) || prompt_yn((func == GLOBAL_QUICK_FILTER_KILL) ? _(txt_quick_filter_kill) : _(txt_quick_filter_select), TRUE) == 1) {
-					old_top = top_art;
-					n = (int) base[grpmenu.curr]; /* should this depend on show_only_unread? */
-					old_artnum = arts[n].artnum;
+					n = (int) base[grpmenu.curr]; /* should this depend on show_only_unread_arts? */
 					if (quick_filter(func, group, &arts[n])) {
+						old_artnum = arts[n].artnum;
+						unfilter_articles(group);
+						filter_articles(group);
+						make_threads(group, FALSE);
+						grpmenu.curr = find_new_pos(old_artnum, grpmenu.curr);
+						show_group_page();
 						info_message((func == GLOBAL_QUICK_FILTER_KILL) ? _(txt_info_add_kill) : _(txt_info_add_select));
-						if (filter_articles(group)) {
-							make_threads(group, FALSE);
-							/* TODO: position is wrong after quick killing a thread */
-							grpmenu.curr = find_new_pos(old_top, old_artnum, grpmenu.curr);
-						}
 					}
-					show_group_page();
 				}
 				break;
 
@@ -435,9 +453,16 @@ group_page(
 #	endif /* HAVE_COLOR */
 
 			case GROUP_MARK_THREAD_READ:			/* mark current thread/range/tagged threads as read */
-				mark_thd_read(group, range_active);
-				if (range_active)
-					range_active = FALSE; /* Range has gone now */
+			case MARK_THREAD_UNREAD:				/* or unread */
+				if (grpmenu.curr < 0)
+					info_message(_(txt_no_arts));
+				else {
+					t_function function, type;
+
+					function = func == GROUP_MARK_THREAD_READ ? (t_function) FEED_MARK_READ : (t_function) FEED_MARK_UNREAD;
+					type = range_active ? FEED_RANGE : (num_of_tagged_arts && !group->attribute->mark_ignore_tags) ? NOT_ASSIGNED : FEED_THREAD;
+					feed_articles(function, GROUP_LEVEL, type, group, (int) base[grpmenu.curr]);
+				}
 				break;
 
 			case GROUP_LIST_THREAD:				/* list articles within current thread */
@@ -450,14 +475,9 @@ group_page(
 				break;
 
 			case GLOBAL_OPTION_MENU:			/* option menu */
-				if (grpmenu.max > 0) {
-					old_top = top_art;
-					old_artnum = arts[(int) base[grpmenu.curr]].artnum;
-				}
-				n = tinrc.sort_article_type;
-				if ((change_config_file(group) == NO_FILTERING) && n != tinrc.sort_article_type)
-					make_threads(group, TRUE);
-				grpmenu.curr = find_new_pos(old_top, old_artnum, grpmenu.curr);
+				old_artnum = grpmenu.max > 0 ? arts[(int) base[grpmenu.curr]].artnum : -1L;
+				config_page(group->name);
+				grpmenu.curr = old_artnum >= 0L ? find_new_pos(old_artnum, grpmenu.curr) : grpmenu.max - 1;
 				show_group_page();
 				break;
 
@@ -542,8 +562,16 @@ group_page(
 				break;
 
 			case GROUP_TOGGLE_GET_ARTICLES_LIMIT:
-				tinrc.getart_limit = prompt_getart_limit();
-				ret_code = GRP_NEXTUNREAD;
+				if (prompt_getart_limit()) {
+					/*
+					 * if getart limit was given via cmd-line
+					 * make it inactive now in order to use
+					 * tinrc.getart_limit
+					 */
+					if (cmdline.args & CMDLINE_GETART_LIMIT)
+						cmdline.args &= ~CMDLINE_GETART_LIMIT;
+					ret_code = GRP_NEXTUNREAD;
+				}
 				break;
 
 			case GLOBAL_BUGREPORT:
@@ -635,7 +663,7 @@ group_page(
 				break;
 
 			case GROUP_TOGGLE_THREADING:		/* Cycle through the threading types */
-				group->attribute->thread_arts = (group->attribute->thread_arts + 1) % (THREAD_MAX + 1);
+				group->attribute->thread_articles = (group->attribute->thread_articles + 1) % (THREAD_MAX + 1);
 				if (grpmenu.curr >= 0) {
 					i = base[grpmenu.curr];								/* Save a copy of current thread */
 					make_threads(group, TRUE);
@@ -709,36 +737,14 @@ group_page(
 				}
 				break;
 
-			case MARK_THREAD_UNREAD:		/* mark whole thread as unread */
-				if (grpmenu.curr < 0)
-					info_message(_(txt_no_arts));
-				else {
-					const char *ptr;
+			case MARK_FEED_READ:	/* mark selected articles as read */
+				if (grpmenu.curr >= 0)
+					feed_articles(FEED_MARK_READ, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
+				break;
 
-					/*
-					 * We process all articles in case the threading changed since
-					 * the range was created
-					 */
-					if (range_active) {
-						for_each_art(ii) {
-							if (arts[ii].inrange) {
-								arts[ii].inrange = FALSE;
-								art_mark(group, &arts[ii], ART_WILL_RETURN);
-							}
-						}
-						range_active = FALSE;
-						show_group_page();
-						ptr = _(txt_thread_range);
-					} else {
-						thd_mark_unread(group, base[grpmenu.curr]);
-						ptr = _(txt_thread_upper);
-					}
-
-					show_group_title(TRUE);
-					build_sline(grpmenu.curr);
-					draw_subject_arrow();
-					info_message(_(txt_marked_as_unread), ptr);
-				}
+			case MARK_FEED_UNREAD:	/* mark selected articles as unread */
+				if (grpmenu.curr >= 0)
+					feed_articles(FEED_MARK_UNREAD, GROUP_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
 				break;
 
 			case GROUP_SELECT_THREAD:	/* mark thread as selected */
@@ -768,14 +774,14 @@ group_page(
 					mark_screen(grpmenu.curr, MARK_OFFSET, mark);
 				}
 
-				info_message(flag ? _(txt_thread_marked_as_selected) : _(txt_thread_marked_as_deselected));
 				show_group_title(TRUE);
 
-				if (grpmenu.curr + 1 < grpmenu.max) {
+				if (grpmenu.curr + 1 < grpmenu.max)
 					move_down();
-					break;
-				}
-				draw_subject_arrow();
+				else
+					draw_subject_arrow();
+
+				info_message(flag ? _(txt_thread_marked_as_selected) : _(txt_thread_marked_as_deselected));
 				break;
 
 			case GROUP_REVERSE_SELECTIONS:	/* reverse selections */
@@ -793,7 +799,7 @@ group_page(
 				break;
 
 			case GROUP_SELECT_PATTERN:	/* select matching patterns */
-				{
+				if (grpmenu.curr >= 0) {
 					char pat[128];
 					char *prompt;
 					struct regex_cache cache = { NULL, NULL };
@@ -809,7 +815,7 @@ group_page(
 						if (tinrc.wildcard)
 							STRCPY(pat, ".*");
 						else
-							strncpy(pat, tinrc.default_select_pattern, sizeof(pat));
+							STRCPY(pat, tinrc.default_select_pattern);
 					} else
 						snprintf(pat, sizeof(pat), REGEX_FMT, tinrc.default_select_pattern);
 
@@ -834,8 +840,8 @@ group_page(
 						FreeIfNeeded(cache.re);
 						FreeIfNeeded(cache.extra);
 					}
-					break;
 				}
+				break;
 
 			case GROUP_SELECT_THREAD_IF_UNREAD_SELECTED:	/* select all unread arts in thread hot if 1 is hot */
 				for (n = 0; n < grpmenu.max; n++) {
@@ -973,7 +979,7 @@ clear_note_area(
 
 
 /*
- * If in show_only_unread mode or there are unread articles we know this
+ * If in show_only_unread_arts mode or there are unread articles we know this
  * thread will exist after toggle. Otherwise we find the next closest to
  * return to. 'force' can be set to force tin to show all messages
  */
@@ -983,14 +989,23 @@ toggle_read_unread(
 {
 	int n, i = -1;
 
+	/*
+	 * Clear art->keep_in_base if switching to !show_only_unread_arts
+	 */
+	if (curr_group->attribute->show_only_unread_arts) {
+		for_each_art(n)
+			arts[n].keep_in_base = FALSE;
+	}
+
+	/* force currently is always false */
 	if (force)
-		curr_group->attribute->show_only_unread = TRUE;	/* Yes - really, we change it in a bit */
+		curr_group->attribute->show_only_unread_arts = TRUE;	/* Yes - really, we change it in a bit */
 
 	wait_message(0, _(txt_reading_arts),
-		(curr_group->attribute->show_only_unread) ? _(txt_all) : _(txt_unread));
+		(curr_group->attribute->show_only_unread_arts) ? _(txt_all) : _(txt_unread));
 
 	if (grpmenu.curr >= 0) {
-		if (curr_group->attribute->show_only_unread || new_responses(grpmenu.curr))
+		if (curr_group->attribute->show_only_unread_arts || new_responses(grpmenu.curr))
 			i = base[grpmenu.curr];
 		else if ((n = prev_unread((int) base[grpmenu.curr])) >= 0)
 			i = n;
@@ -998,13 +1013,15 @@ toggle_read_unread(
 			i = n;
 	}
 
-	curr_group->attribute->show_only_unread = bool_not(curr_group->attribute->show_only_unread);
+	if (!force)
+		curr_group->attribute->show_only_unread_arts = bool_not(curr_group->attribute->show_only_unread_arts);
 
 	find_base(curr_group);
 	if (i >= 0 && (n = which_thread(i)) >= 0)
 		grpmenu.curr = n;
 	else if (grpmenu.max > 0)
 		grpmenu.curr = grpmenu.max - 1;
+	clear_message();
 }
 
 
@@ -1013,24 +1030,16 @@ toggle_read_unread(
  * author it is impossible to know which, if any, articles will be left
  * afterwards. So we make a "best attempt" to find a new index point.
  */
-int
+static int
 find_new_pos(
-	int old_top,
 	long old_artnum,
 	int cur_pos)
 {
 	int i, pos;
 
-	if (top_art == old_top)
-		return cur_pos;
+	if ((i = find_artnum(old_artnum)) >= 0 && (pos = which_thread(i)) >= 0)
+		return pos;
 
-	for_each_art(i) {
-		if (arts[i].artnum == old_artnum) {
-			pos = which_thread(arts[i].artnum);
-			if (pos >= 0)
-				return pos;
-		}
-	}
 	return ((cur_pos < grpmenu.max) ? cur_pos : (grpmenu.max - 1));
 }
 
@@ -1045,7 +1054,7 @@ pos_first_unread_thread(
 {
 	int i;
 
-	if (tinrc.pos_first_unread) {
+	if (curr_group->attribute->pos_first_unread) {
 		for (i = 0; i < grpmenu.max; i++) {
 			if (new_responses(i))
 				break;
@@ -1205,7 +1214,7 @@ build_sline(
 	/*
 	 * n is number of articles in this thread
 	 */
-	n = ((curr_group->attribute->show_only_unread) ? (sbuf.unread + sbuf.seen) : sbuf.total);
+	n = ((curr_group->attribute->show_only_unread_arts) ? (sbuf.unread + sbuf.seen) : sbuf.total);
 	/*
 	 * if you like to see the number of responses excluding the first
 	 *	art in thread - add the following:
@@ -1255,7 +1264,7 @@ build_sline(
 	if (sbuf.multipart_have > 1) /* We have a multipart msg so lets built our new header info. */
 		build_multipart_header(arts_sub, len_subj, arts[j].subject, sbuf.multipart_compare_len, sbuf.multipart_have, sbuf.multipart_total);
 	else
-		strncpy(arts_sub, arts[j].subject, sizeof(arts_sub) - 1);
+		STRCPY(arts_sub, arts[j].subject);
 
 #ifndef USE_CURSES
 	buffer = screen[INDEX2SNUM(i)].col;
@@ -1281,7 +1290,7 @@ build_sline(
 		tmp_from = wcspart(wc, len_from, TRUE);
 	}
 
-#	if 0 /* use additional space if !draw_arrow - usefull? */
+#	if 0 /* use additional space if !draw_arrow - useful? */
 	if (!tinrc.draw_arrow) {
 		if (curr_group->attribute->show_info == SHOW_INFO_SCORE || curr_group->attribute->show_info == SHOW_INFO_BOTH) {
 			mbstowcs(format, "%s %s %s%6d %-ls%s%-ls", ARRAY_SIZE(format) - 1);
@@ -1330,7 +1339,7 @@ build_sline(
 #else
 	arts_sub[len_subj - 12 + 1] = '\0';
 
-#	if 0 /* use additional space if !draw_arrow - usefull? */
+#	if 0 /* use additional space if !draw_arrow - useful? */
 	if (!tinrc.draw_arrow) {
 		if (curr_group->attribute->show_info == SHOW_INFO_SCORE || curr_group->attribute->show_info == SHOW_INFO_BOTH)
 			snprintf(buffer, cCOLS + 1, "%s %s %s%6d %-*.*s%s%-*.*s",
@@ -1386,24 +1395,24 @@ static void
 show_group_title(
 	t_bool clear_title)
 {
-	char buf[LEN], tmp[LEN];
+	char buf[LEN], tmp[LEN], flag;
 	int i, art_cnt = 0, recent_art_cnt = 0, selected_art_cnt = 0, read_selected_art_cnt = 0, killed_art_cnt = 0;
 
 	for_each_art(i) {
 		if (arts[i].thread == ART_EXPIRED)
 			continue;
 
-		if (curr_group->attribute->show_only_unread) {
+		if (curr_group->attribute->show_only_unread_arts) {
 			if (arts[i].status != ART_READ) {
 				art_cnt++;
-				if (tinrc.recent_time && ((time((time_t) 0) - arts[i].date) < (tinrc.recent_time * DAY)))
+				if (tinrc.recent_time && ((time((time_t *) 0) - arts[i].date) < (tinrc.recent_time * DAY)))
 					recent_art_cnt++;
 			}
 			if (arts[i].killed == ART_KILLED_UNREAD)
 				killed_art_cnt++;
 		} else {
 			art_cnt++;
-			if (tinrc.recent_time && ((time((time_t) 0) - arts[i].date) < (tinrc.recent_time * DAY)))
+			if (tinrc.recent_time && ((time((time_t *) 0) - arts[i].date) < (tinrc.recent_time * DAY)))
 				recent_art_cnt++;
 
 			if (arts[i].killed)
@@ -1423,22 +1432,22 @@ show_group_title(
 	/* group name and thread count */
 	snprintf(buf, sizeof(buf), "%s (%d%c",
 		curr_group->name, grpmenu.max,
-		*txt_threading[curr_group->attribute->thread_arts]);
+		*txt_threading[curr_group->attribute->thread_articles]);
 
 	/* article count */
-	if (tinrc.getart_limit)
+	if (cmdline.args & CMDLINE_GETART_LIMIT ? cmdline.getart_limit : tinrc.getart_limit)
 		snprintf(tmp, sizeof(tmp), " %d/%d%c",
-			tinrc.getart_limit, art_cnt,
-			(curr_group->attribute->show_only_unread ? tinrc.art_marked_unread : tinrc.art_marked_read));
+			cmdline.args & CMDLINE_GETART_LIMIT ? cmdline.getart_limit : tinrc.getart_limit, art_cnt,
+			(curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
 	else
 		snprintf(tmp, sizeof(tmp), " %d%c",
 			art_cnt,
-			(curr_group->attribute->show_only_unread ? tinrc.art_marked_unread : tinrc.art_marked_read));
+			(curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
 
 	/* selected articles */
-	if (curr_group->attribute->show_only_unread)
+	if (curr_group->attribute->show_only_unread_arts)
 		snprintf(tmp, sizeof(tmp), " %d%c",
 			selected_art_cnt, tinrc.art_marked_selected);
 	else
@@ -1464,8 +1473,10 @@ show_group_title(
 		strcat(buf, tmp);
 
 	/* group flag */
-	snprintf(tmp, sizeof(tmp), ") %c",
-		group_flag(curr_group->moderated));
+	if ((flag = group_flag(curr_group->moderated)) == ' ')
+		snprintf(tmp, sizeof(tmp), ")");
+	else
+		snprintf(tmp, sizeof(tmp), ") %c", flag);
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
 
@@ -1567,14 +1578,19 @@ enter_thread(
 
 			case GRP_NEXTUNREAD:				/* 'C'atchup */
 				if ((n = next_unread((int) base[grpmenu.curr])) >= 0) {
+					if (page)
+						page->art = n;
 					if ((n = which_thread(n)) >= 0) {
 						grpmenu.curr = n;
 						depth = 0;
 						break;		/* Drop into next thread with unread */
 					}
 				}
-				/* No more unread threads in this group */
-				/* FALLTHROUGH */
+				/* No more unread threads in this group, enter next group */
+				grpmenu.curr = 0;
+				return GRP_NEXTUNREAD;
+				/* NOTREACHED */
+				break;
 
 			case GRP_KILLED:
 				grpmenu.curr = 0;
@@ -1668,116 +1684,64 @@ group_catchup(
 }
 
 
-static int
+static t_bool
 prompt_getart_limit(
 	void)
 {
 	char *p;
-	int num = 0;
+	t_bool ret = FALSE;
 
 	clear_message();
-	if ((p = tin_getline(_(txt_enter_getart_limit), 2, 0, 0, FALSE, HIST_OTHER)) != NULL)
-		num = atoi(p);
-
+	if ((p = tin_getline(_(txt_enter_getart_limit), 2, 0, 0, FALSE, HIST_OTHER)) != NULL) {
+		tinrc.getart_limit = atoi(p);
+		ret = TRUE;
+	}
 	clear_message();
-	return num;
+	return ret;
 }
 
 
 /*
- * If there's an active range, use that one.
- * If there are any tagged and unread articles, prompt user to mark either
- * all tagged arts/threads as read, or only current thread, or cancel operation.
- * Otherwise, use current thread.
- * Finally move to next unread thread.
+ * Redraw all necessary parts of the screen after FEED_MARK_(UN)READ
+ * Move cursor to next unread item if needed
+ *
+ * Returns TRUE when no next unread art, FALSE otherwise
  */
-static void
-mark_thd_read(
-	struct t_group *group,
-	t_bool range_active)
+t_bool
+group_mark_postprocess(
+	int function,
+	t_function feed_type,
+	int respnum)
 {
-	char keytagged[MAXKEYLEN], keycurrent[MAXKEYLEN], keyquit[MAXKEYLEN];
-	t_function func = MARK_READ_CURRENT;
-	int n, cnt = 0;
-	int tmp_num_of_tagged_arts = num_of_tagged_arts;
+	int n;
 
-	if (grpmenu.curr < 0) {
-		info_message(_(txt_no_next_unread_art));
-		return;
-	}
-
-	/* Don't prompt if there's an active range or if prompting is turned off */
-	if (!range_active && !tinrc.mark_ignore_tags && got_tagged_unread_arts()) {
-		func = prompt_slk_response(MARK_READ_TAGGED, mark_read_keys,
-				_(txt_mark_thread_read_tagged_current),
-				printascii(keytagged, func_to_key(MARK_READ_TAGGED, mark_read_keys)),
-				printascii(keycurrent, func_to_key(MARK_READ_CURRENT, mark_read_keys)),
-				printascii(keyquit, func_to_key(GLOBAL_QUIT, mark_read_keys)));
-	}
-
-	switch (func) {
-		case MARK_READ_TAGGED: /* mark tagged unread articles/threads as read */
-			cnt = mark_tagged_read(group);
-			break;
-
-		case MARK_READ_CURRENT: /* mark current thread/range as read */
-			/*
-			 * If a range is active, use it.
-			 */
-			if (range_active) {
-				/*
-				 * We check all arts, in case the user did something clever like
-				 * change the threading mode on us since the range was created
-				 */
-				for_each_art(n) {
-					if (arts[n].inrange) {
-						arts[n].inrange = FALSE;	/* Clear the range */
-						art_mark(group, &arts[n], ART_READ);
-					}
-				}
-			}
-			else
-				thd_mark_read(group, base[grpmenu.curr]);
-			break;
-
-		case GLOBAL_QUIT: /* cancel operation */
-		case GLOBAL_ABORT:
-		default:
-			return;
-			/* NOTREACHED */
-			break;
-	}
-
-	/*
-	 * update the header
-	 */
 	show_group_title(TRUE);
+	switch (function) {
+		case (FEED_MARK_READ):
+			if (feed_type == FEED_THREAD || feed_type == FEED_ARTICLE)
+				build_sline(grpmenu.curr);
+			else
+				show_group_page();
 
-	/*
-	 * Refresh current line or, if necessary, whole group page
-	 */
-	if (range_active || func == MARK_READ_TAGGED)
-		show_group_page();
-	else
-		build_sline(grpmenu.curr);
+			if ((n = next_unread(next_response(respnum))) == -1) {
+				draw_subject_arrow();
+				return TRUE;
+			}
 
-	/*
-	 * Move cursor to next unread
-	 */
-	if ((n = next_unread(next_response((int) base[grpmenu.curr]))) < 0) {
-		draw_subject_arrow();
-		info_message(_(txt_no_next_unread_art));
-		return;
+			move_to_item(which_thread(n));
+			break;
+
+		case (FEED_MARK_UNREAD):
+			if (feed_type == FEED_THREAD || feed_type == FEED_ARTICLE)
+				build_sline(grpmenu.curr);
+			else
+				show_group_page();
+
+			draw_subject_arrow();
+			break;
+
+		default:
+			break;
 	}
-
-	if ((n = which_thread(n)) < 0) {
-		/* TODO: -> lang.c */
-		error_message("Internal error: which_thread(%d) < 0", n);
-		return;
-	}
-
-	move_to_item(n);
-
-	if (func == MARK_READ_TAGGED)
-		info_message(_(txt_marked_tagged_arts_as_read), cnt, tmp_num_of_tagged_arts, PLURAL(tmp_num_of_tagged_arts, txt_article));
+	return FALSE;
 }
