@@ -3,9 +3,9 @@
  *  Module    : header.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
- *  Updated   : 2006-01-10
+ *  Updated   : 2008-11-22
  *
- * Copyright (c) 1997-2006 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2010 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,7 @@ get_host_name(
 #endif /* HAVE_GETHOSTNAME */
 #ifdef HAVE_SYS_UTSNAME_H
 	if (!*hostname)
-		strncpy(hostname, system_info.nodename, sizeof(hostname) - 1);
+		my_strncpy(hostname, system_info.nodename, sizeof(hostname) - 1);
 #endif /* HAVE_SYS_UTSNAME_H */
 	if (!*hostname) {
 		if ((ptr = getenv("HOST")) != NULL)
@@ -80,8 +80,6 @@ get_host_name(
 /*
  * find domainname - check DOMAIN_NAME
  * TODO: check /etc/defaultdomain as a last resort?
- *       there is M_UNIX specific code in here, but no VMS
- *       alternative
  */
 const char *
 get_domain_name(
@@ -135,7 +133,7 @@ get_fqdn(
 	char name[MAXHOSTNAMELEN + 1];
 	static char fqdn[1024];
 	struct hostent *hp;
-	struct in_addr in;
+	struct in_addr in = {0};
 
 	*fqdn = '\0';
 	domain = NULL;
@@ -144,7 +142,7 @@ get_fqdn(
 	if (host) {
 		if (strchr(host, '.'))
 			return host;
-		strncpy(name, host, sizeof(name) - 1);
+		my_strncpy(name, host, sizeof(name) - 1);
 	} else {
 #	ifdef HAVE_GETHOSTNAME
 		if (gethostname(name, sizeof(name) - 1))
@@ -231,31 +229,17 @@ get_user_name(
 {
 	static char username[128];
 	struct passwd *pw;
-#ifdef VMS
-	char *p;
-#endif /* VMS */
 
 	username[0] = '\0';
 
-#ifndef VMS
 	if ((pw = getpwuid(getuid())) != NULL)
 		strcpy(username, pw->pw_name);
-	else
-#else
-	if (((p = getlogin()) != NULL) && strlen(p)) {
-		if ((pw = getpwnam(p)) != NULL)
-			strcpy(username, pw->pw_name);
+	else {
+		if (!*username) {
+			error_message(2, _(txt_error_passwd_missing));
+			tin_done(EXIT_FAILURE);
+		}
 	}
-#endif /* !VMS */
-
-	if (!*username) {
-		error_message(_(txt_error_passwd_missing));
-		tin_done(EXIT_FAILURE);
-	}
-
-#	ifdef VMS
-	lower(username);
-#	endif /* VMS */
 
 	return username;
 }
@@ -267,11 +251,11 @@ get_full_name(
 {
 	char *p;
 	static char fullname[128];
-#	if !defined(VMS) && !defined(DONT_HAVE_PW_GECOS)
+#	ifndef DONT_HAVE_PW_GECOS
 	char buf[128];
 	char tmp[128];
 	struct passwd *pw;
-#	endif /* !VMS && !DONT_HAVE_PW_GECOS */
+#	endif /* !DONT_HAVE_PW_GECOS */
 
 	fullname[0] = '\0';
 
@@ -284,10 +268,7 @@ get_full_name(
 		return fullname;
 	}
 
-#	ifdef VMS
-	STRCPY(fullname, fix_fullname(get_uaf_fullname()));
-#	else
-#		ifndef DONT_HAVE_PW_GECOS
+#	ifndef DONT_HAVE_PW_GECOS
 	if ((pw = getpwuid(getuid())) != NULL) {
 		STRCPY(buf, pw->pw_gecos);
 		if ((p = strchr(buf, ',')))
@@ -301,8 +282,7 @@ get_full_name(
 		} else
 			STRCPY(fullname, buf);
 	}
-#		endif /* !DONT_HAVE_PW_GECOS */
-#	endif /* VMS */
+#	endif /* !DONT_HAVE_PW_GECOS */
 	return fullname;
 }
 
@@ -319,14 +299,7 @@ get_from_name(
 	char *from_name,
 	struct t_group *thisgrp)
 {
-#	ifdef USE_INN_NNTPLIB
-	char *fromhost = GetConfigValue(_CONF_FROMHOST);
-#	else
-	char *fromhost = NULL;
-#	endif /* USE_INN_NNTPLIB */
-
-	if (!(fromhost && *fromhost))
-		fromhost = domain_name;
+	char *fromhost = domain_name;
 
 	if (thisgrp && *thisgrp->attribute->from != '\0') {
 		strcpy(from_name, thisgrp->attribute->from);
@@ -336,8 +309,8 @@ get_from_name(
 	sprintf(from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), get_full_name(), get_user_name(), fromhost);
 
 #	ifdef DEBUG
-	if (debug == 2)
-		error_message("FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, get_user_name(), domain_name, get_full_name());
+	if (debug & DEBUG_MISC)
+		error_message(2, "FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, get_user_name(), domain_name, get_full_name());
 #	endif /* DEBUG */
 
 }
