@@ -3,10 +3,10 @@
  *  Module    : help.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2005-07-02
+ *  Updated   : 2010-10-07
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,154 @@ static void make_help_page(FILE *fp, const t_help_page *helppage, const struct k
 
 static constext txt_help_empty_line[] = "";
 
+static t_help_page attachment_help_page[] = {
+	{ txt_help_title_navi, NOT_ASSIGNED },
+	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
+	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
+	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
+	{ txt_help_global_line_up, GLOBAL_LINE_UP },
+	{ txt_help_global_scroll_down, GLOBAL_SCROLL_DOWN },
+	{ txt_help_global_scroll_up, GLOBAL_SCROLL_UP },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_attachment_first, GLOBAL_FIRST_PAGE },
+	{ txt_help_attachment_last, GLOBAL_LAST_PAGE },
+	{ txt_help_attachment_goto, NOT_ASSIGNED },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_disp, NOT_ASSIGNED },
+	{ txt_help_attachment_toggle_info_line, GLOBAL_TOGGLE_INFO_LAST_LINE },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_attachment_ops, NOT_ASSIGNED },
+	{ txt_help_attachment_select, ATTACHMENT_SELECT },
+#ifndef DONT_HAVE_PIPING
+	{ txt_help_attachment_pipe, ATTACHMENT_PIPE },
+	{ txt_help_attachment_pipe_raw, GLOBAL_PIPE },
+#endif /* !DONT_HAVE_PIPING */
+	{ txt_help_attachment_save, ATTACHMENT_SAVE },
+	{ txt_help_attachment_tag, ATTACHMENT_TAG },
+	{ txt_help_attachment_tag_pattern, ATTACHMENT_TAG_PATTERN },
+	{ txt_help_attachment_toggle_tagged, ATTACHMENT_TOGGLE_TAGGED },
+	{ txt_help_attachment_untag, ATTACHMENT_UNTAG },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_attachment_search_forwards, GLOBAL_SEARCH_SUBJECT_FORWARD },
+	{ txt_help_attachment_search_backwards, GLOBAL_SEARCH_SUBJECT_BACKWARD },
+	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_misc, NOT_ASSIGNED },
+	{ txt_help_select_quit, GLOBAL_QUIT },
+	{ txt_help_global_help, GLOBAL_HELP },
+	{ txt_help_global_toggle_mini_help, GLOBAL_TOGGLE_HELP_DISPLAY },
+	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
+	{ NULL, NOT_ASSIGNED }
+};
+
+static t_help_page attrib_help_page[] = {
+	{ txt_help_title_navi, NOT_ASSIGNED },
+	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
+	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
+	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
+	{ txt_help_global_line_up, GLOBAL_LINE_UP },
+	{ txt_help_global_scroll_down, GLOBAL_SCROLL_DOWN },
+	{ txt_help_global_scroll_up, GLOBAL_SCROLL_UP },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_attrib_first_opt, GLOBAL_FIRST_PAGE },
+	{ txt_help_attrib_last_opt, GLOBAL_LAST_PAGE },
+	{ txt_help_attrib_goto_opt, NOT_ASSIGNED },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_attrib_search_opt_forwards, GLOBAL_SEARCH_SUBJECT_FORWARD },
+	{ txt_help_attrib_search_opt_backwards, GLOBAL_SEARCH_SUBJECT_BACKWARD },
+	{ txt_help_select_search_group_comment, NOT_ASSIGNED },
+	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_attrib_ops, NOT_ASSIGNED },
+	{ txt_help_attrib_reset_attrib, CONFIG_RESET_ATTRIB },
+	{ txt_help_attrib_select, CONFIG_SELECT },
+	{ txt_help_attrib_toggle_attrib, CONFIG_TOGGLE_ATTRIB },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_misc, NOT_ASSIGNED },
+	{ txt_help_select_quit, GLOBAL_QUIT },
+	{ txt_help_select_quit_no_write, CONFIG_NO_SAVE },
+	{ txt_help_global_help, GLOBAL_HELP },
+	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_global_version, GLOBAL_VERSION },
+	{ NULL, NOT_ASSIGNED }
+};
+
+static t_help_page config_help_page[] = {
+	{ txt_help_title_navi, NOT_ASSIGNED },
+	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
+	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
+	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
+	{ txt_help_global_line_up, GLOBAL_LINE_UP },
+	{ txt_help_global_scroll_down, GLOBAL_SCROLL_DOWN },
+	{ txt_help_global_scroll_up, GLOBAL_SCROLL_UP },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_config_first_opt, GLOBAL_FIRST_PAGE },
+	{ txt_help_config_last_opt, GLOBAL_LAST_PAGE },
+	{ txt_help_config_goto_opt, NOT_ASSIGNED },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_config_search_opt_forwards, GLOBAL_SEARCH_SUBJECT_FORWARD },
+	{ txt_help_config_search_opt_backwards, GLOBAL_SEARCH_SUBJECT_BACKWARD },
+	{ txt_help_select_search_group_comment, NOT_ASSIGNED },
+	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_config_ops, NOT_ASSIGNED },
+	{ txt_help_config_select, CONFIG_SELECT },
+	{ txt_help_config_toggle_attrib, CONFIG_TOGGLE_ATTRIB },
+	{ txt_help_config_scope_menu, CONFIG_SCOPE_MENU },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_misc, NOT_ASSIGNED },
+	{ txt_help_select_quit, GLOBAL_QUIT },
+	{ txt_help_select_quit_no_write, CONFIG_NO_SAVE },
+	{ txt_help_global_help, GLOBAL_HELP },
+	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_global_version, GLOBAL_VERSION },
+	{ NULL, NOT_ASSIGNED }
+};
+
+static t_help_page scope_help_page[] = {
+	{ txt_help_title_navi, NOT_ASSIGNED },
+	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
+	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
+	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
+	{ txt_help_global_line_up, GLOBAL_LINE_UP },
+	{ txt_help_global_scroll_down, GLOBAL_SCROLL_DOWN },
+	{ txt_help_global_scroll_up, GLOBAL_SCROLL_UP },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_scope_first_scope, GLOBAL_FIRST_PAGE },
+	{ txt_help_scope_last_scope, GLOBAL_LAST_PAGE },
+	{ txt_help_scope_goto_scope, NOT_ASSIGNED },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_scope_ops, NOT_ASSIGNED },
+	{ txt_help_scope_add, SCOPE_ADD },
+	{ txt_help_scope_move, SCOPE_MOVE },
+	{ txt_help_scope_rename, SCOPE_RENAME },
+	{ txt_help_scope_del, SCOPE_DELETE },
+	{ txt_help_scope_select, SCOPE_SELECT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_scope_edit_attrib_file, SCOPE_EDIT_ATTRIBUTES_FILE },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_misc, NOT_ASSIGNED },
+	{ txt_help_select_quit, GLOBAL_QUIT },
+	{ txt_help_global_help, GLOBAL_HELP },
+	{ txt_help_global_toggle_mini_help, GLOBAL_TOGGLE_HELP_DISPLAY },
+	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
+	{ NULL, NOT_ASSIGNED }
+};
+
 static t_help_page select_help_page[] = {
 	{ txt_help_title_navi, NOT_ASSIGNED },
 	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
@@ -180,6 +328,7 @@ static t_help_page group_help_page[] = {
 	{ txt_help_global_post, GLOBAL_POST },
 	{ txt_help_global_post_postponed, GLOBAL_POSTPONED },
 	{ txt_help_article_repost, GROUP_REPOST },
+	{ txt_help_article_cancel, GROUP_CANCEL },
 #endif /* NO_POSTING */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_global_article_range, GLOBAL_SET_RANGE },
@@ -203,6 +352,8 @@ static t_help_page group_help_page[] = {
 	{ txt_help_group_catchup_next, CATCHUP_NEXT_UNREAD },
 	{ txt_help_group_mark_article_unread, MARK_ARTICLE_UNREAD },
 	{ txt_help_group_mark_thread_unread, MARK_THREAD_UNREAD },
+	{ txt_help_mark_feed_read, MARK_FEED_READ },
+	{ txt_help_mark_feed_unread, MARK_FEED_UNREAD },
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_group_select_all, GROUP_DO_AUTOSELECT },
 	{ txt_help_group_select_thread, GROUP_SELECT_THREAD },
@@ -273,6 +424,7 @@ static t_help_page thread_help_page[] = {
 #ifndef NO_POSTING
 	{ txt_help_global_post, GLOBAL_POST },
 	{ txt_help_global_post_postponed, GLOBAL_POSTPONED },
+	{ txt_help_article_cancel, THREAD_CANCEL },
 #endif /* NO_POSTING */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_global_article_range, GLOBAL_SET_RANGE },
@@ -293,8 +445,10 @@ static t_help_page thread_help_page[] = {
 	{ txt_help_thread_mark_article_read, THREAD_MARK_ARTICLE_READ },
 	{ txt_help_thread_catchup, CATCHUP },
 	{ txt_help_thread_catchup_next_unread, CATCHUP_NEXT_UNREAD },
-	{ txt_help_group_mark_article_unread, MARK_ARTICLE_UNREAD },
-	{ txt_help_group_mark_thread_unread, MARK_THREAD_UNREAD },
+	{ txt_help_thread_mark_article_unread, MARK_ARTICLE_UNREAD },
+	{ txt_help_thread_mark_thread_unread, MARK_THREAD_UNREAD },
+	{ txt_help_mark_feed_read, MARK_FEED_READ },
+	{ txt_help_mark_feed_unread, MARK_FEED_UNREAD },
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_group_select_thread, THREAD_SELECT_ARTICLE },
 	{ txt_help_group_toggle_thread_selection, THREAD_TOGGLE_ARTICLE_SELECTION },
@@ -326,7 +480,6 @@ static t_help_page thread_help_page[] = {
 static t_help_page page_help_page[] = {
 	{ txt_help_title_navi, NOT_ASSIGNED },
 	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
-	{ txt_help_global_page_down, PAGE_PAGE_DOWN3 },
 	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
 	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
 	{ txt_help_global_line_up, GLOBAL_LINE_UP },
@@ -403,7 +556,7 @@ static t_help_page page_help_page[] = {
 	{ txt_help_thread_catchup, CATCHUP },
 	{ txt_help_thread_catchup_next_unread, CATCHUP_NEXT_UNREAD },
 	{ txt_help_group_mark_article_unread, MARK_ARTICLE_UNREAD },
-	{ txt_help_group_mark_thread_unread, MARK_THREAD_UNREAD },
+	{ txt_help_thread_mark_thread_unread, MARK_THREAD_UNREAD },
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_article_autoselect, GLOBAL_MENU_FILTER_SELECT },
 	{ txt_help_article_autokill, GLOBAL_MENU_FILTER_KILL },
@@ -433,6 +586,41 @@ static t_help_page page_help_page[] = {
 	{ NULL, NOT_ASSIGNED }
 };
 
+static t_help_page url_help_page[] = {
+	{ txt_help_title_navi, NOT_ASSIGNED },
+	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
+	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
+	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
+	{ txt_help_global_line_up, GLOBAL_LINE_UP },
+	{ txt_help_global_scroll_down, GLOBAL_SCROLL_DOWN },
+	{ txt_help_global_scroll_up, GLOBAL_SCROLL_UP },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_url_first_url, GLOBAL_FIRST_PAGE },
+	{ txt_help_url_last_url, GLOBAL_LAST_PAGE },
+	{ txt_help_url_goto_url, NOT_ASSIGNED },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_url_ops, NOT_ASSIGNED },
+	{ txt_help_url_select, URL_SELECT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_url_search_forwards, GLOBAL_SEARCH_SUBJECT_FORWARD },
+	{ txt_help_url_search_backwards, GLOBAL_SEARCH_SUBJECT_BACKWARD },
+	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_disp, NOT_ASSIGNED },
+	{ txt_help_url_toggle_info_line, GLOBAL_TOGGLE_INFO_LAST_LINE },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_misc, NOT_ASSIGNED },
+	{ txt_help_select_quit, GLOBAL_QUIT },
+	{ txt_help_global_help, GLOBAL_HELP },
+	{ txt_help_global_toggle_mini_help, GLOBAL_TOGGLE_HELP_DISPLAY },
+	{ txt_help_global_esc, GLOBAL_ABORT },
+	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
+	{ NULL, NOT_ASSIGNED }
+};
+
 
 static void
 make_help_page(
@@ -447,7 +635,7 @@ make_help_page(
 	 * length is only needed to pass it to expand_ctrl_chars()
 	 * we have no need for the value
 	 */
-	int length;
+	size_t length;
 	size_t i;
 
 	if (!helppage)
@@ -458,18 +646,18 @@ make_help_page(
 	while (helppage->helptext) {
 		if (helppage->func == NOT_ASSIGNED) {
 			/*
-			 * as expand_ctrl_chars() may has shrinked buf
+			 * as expand_ctrl_chars() may has shrunk buf
 			 * make sure buf is large enough to contain the helpline
 			 */
 			buf = my_realloc(buf, LEN);
 
 			if (!strlen(helppage->helptext))	/* avoid translation of empty strings */
-				snprintf(buf, LEN, "\n");
+				strcpy(buf, "\n");
 			else
-				snprintf(buf, LEN, "%s", _(helppage->helptext));
+				strncpy(buf, _(helppage->helptext), LEN);
 			buf[LEN - 1] = '\0';
 			expand_ctrl_chars(&buf, &length, 8);
-			fprintf(fp, "%s", buf);
+			fprintf(fp, "%s\n", buf);
 		} else {
 			for (i = 0; i < keys.used; i++) {
 				if (keys.list[i].function == helppage->func && keys.list[i].key) {
@@ -478,7 +666,7 @@ make_help_page(
 					buf[LEN - 1] = '\0';
 					expand_ctrl_chars(&buf, &length, 8);
 					if (strcmp(last, buf)) {
-						fprintf(fp, "%s", buf);
+						fprintf(fp, "%s\n", buf);
 						strncpy(last, buf, LEN);
 					}
 				}
@@ -503,6 +691,22 @@ show_help_page(
 		return;
 
 	switch (level) {
+		case ATTACHMENT_LEVEL:
+			make_help_page(fp, attachment_help_page, attachment_keys);
+			break;
+
+		case ATTRIB_LEVEL:
+			make_help_page(fp, attrib_help_page, option_menu_keys);
+			break;
+
+		case CONFIG_LEVEL:
+			make_help_page(fp, config_help_page, option_menu_keys);
+			break;
+
+		case SCOPE_LEVEL:
+			make_help_page(fp, scope_help_page, scope_keys);
+			break;
+
 		case SELECT_LEVEL:
 			make_help_page(fp, select_help_page, select_keys);
 			break;
@@ -519,15 +723,20 @@ show_help_page(
 			make_help_page(fp, page_help_page, page_keys);
 			break;
 
+		case URL_LEVEL:
+			make_help_page(fp, url_help_page, url_keys);
+			break;
+
 		case INFO_PAGER:
 		default: /* should not happen */
-			error_message(_(txt_error_unknown_dlevel));
+			error_message(2, _(txt_error_unknown_dlevel));
 			fclose(fp);
 			return;
 	}
 
 	info_pager(fp, title, TRUE);
 	fclose(fp);
+	info_pager(NULL, NULL, TRUE); /* free mem */
 	return;
 }
 
@@ -545,13 +754,54 @@ show_mini_help(
 		return;
 
 	line = NOTESLINES + MINI_HELP_LINES - 2;
-	bufs = (size_t) MIN((unsigned) cCOLS, (sizeof(buf) - 1));
+	bufs = sizeof(buf) - 1;
 
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_minihelp);
 #endif /* HAVE_COLOR */
 
 	switch (level) {
+		case ATTACHMENT_LEVEL:
+			snprintf(buf, bufs, _(txt_mini_attachment_1),
+				printascii(key[0], func_to_key(GLOBAL_LINE_DOWN, attachment_keys)),
+				printascii(key[1], func_to_key(GLOBAL_LINE_UP, attachment_keys)),
+				printascii(key[2], func_to_key(GLOBAL_HELP, attachment_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, attachment_keys)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_attachment_2),
+				printascii(key[0], func_to_key(ATTACHMENT_SELECT, attachment_keys)),
+#ifndef DONT_HAVE_PIPING
+				printascii(key[5], func_to_key(ATTACHMENT_PIPE, attachment_keys)),
+				printascii(key[6], func_to_key(GLOBAL_PIPE, attachment_keys)),
+#endif /* !DONT_HAVE_PIPING */
+				printascii(key[1], func_to_key(ATTACHMENT_SAVE, attachment_keys)),
+				printascii(key[2], func_to_key(ATTACHMENT_TAG, attachment_keys)),
+				printascii(key[3], func_to_key(ATTACHMENT_TAG_PATTERN, attachment_keys)),
+				printascii(key[4], func_to_key(ATTACHMENT_UNTAG, attachment_keys)));
+			center_line(line + 1, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_attachment_3),
+				printascii(key[1], func_to_key(ATTACHMENT_TOGGLE_TAGGED, attachment_keys)),
+				printascii(key[2], func_to_key(GLOBAL_SEARCH_SUBJECT_FORWARD, attachment_keys)),
+				printascii(key[3], func_to_key(GLOBAL_SEARCH_SUBJECT_BACKWARD, attachment_keys)),
+				printascii(key[4], func_to_key(GLOBAL_SEARCH_REPEAT, attachment_keys)));
+			center_line(line + 2, FALSE, buf);
+			break;
+
+		case SCOPE_LEVEL:
+			snprintf(buf, bufs, _(txt_mini_scope_1),
+				printascii(key[0], func_to_key(SCOPE_ADD, scope_keys)),
+				printascii(key[1], func_to_key(SCOPE_MOVE, scope_keys)),
+				printascii(key[2], func_to_key(SCOPE_RENAME, scope_keys)),
+				printascii(key[3], func_to_key(SCOPE_DELETE, scope_keys)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_scope_2),
+				printascii(key[0], func_to_key(GLOBAL_LINE_DOWN, scope_keys)),
+				printascii(key[1], func_to_key(GLOBAL_LINE_UP, scope_keys)),
+				printascii(key[2], func_to_key(GLOBAL_HELP, scope_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, scope_keys)));
+			center_line(line + 1, FALSE, buf);
+			break;
+
 		case SELECT_LEVEL:
 			snprintf(buf, bufs, _(txt_mini_select_1),
 				printascii(key[0], func_to_key(SELECT_ENTER_NEXT_UNREAD_GROUP, select_keys)),
@@ -651,6 +901,20 @@ show_mini_help(
 			center_line(line + 2, FALSE, buf);
 			break;
 
+		case URL_LEVEL:
+			snprintf(buf, bufs, _(txt_mini_url_1),
+				printascii(key[0], func_to_key(GLOBAL_LINE_DOWN, url_keys)),
+				printascii(key[1], func_to_key(GLOBAL_LINE_UP, url_keys)),
+				printascii(key[2], func_to_key(GLOBAL_HELP, url_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, url_keys)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_url_2),
+				printascii(key[2], func_to_key(GLOBAL_SEARCH_SUBJECT_FORWARD, url_keys)),
+				printascii(key[3], func_to_key(GLOBAL_SEARCH_SUBJECT_BACKWARD, url_keys)),
+				printascii(key[4], func_to_key(GLOBAL_SEARCH_REPEAT, url_keys)));
+			center_line(line + 1, FALSE, buf);
+			break;
+
 		case INFO_PAGER:
 			snprintf(buf, bufs, _(txt_mini_info_1),
 				printascii(key[0], func_to_key(GLOBAL_LINE_UP, info_keys)),
@@ -668,7 +932,7 @@ show_mini_help(
 			break;
 
 		default: /* should not happen */
-			error_message(_(txt_error_unknown_dlevel));
+			error_message(2, _(txt_error_unknown_dlevel));
 			break;
 	}
 #ifdef HAVE_COLOR
