@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2005-08-14
+ *  Updated   : 2010-10-07
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,21 +55,15 @@
  * local prototypes
  */
 static int read_site_config(void);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static t_bool utf8_pcre(void);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #ifdef HAVE_COLOR
 	static void preinit_colors(void);
 #endif /* HAVE_COLOR */
 
 
-char **news_headers_to_display_array;	/* array of which headers to display */
-char **news_headers_to_not_display_array;	/* array of which headers to not display */
 char active_times_file[PATH_LEN];
 char article_name[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
 char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
 char bug_nntpserver2[PATH_LEN];		/* welcome message of NNTP server used */
-char cmdline_nntpserver[PATH_LEN];
 char cvers[LEN];
 char dead_article[PATH_LEN];		/* ~/dead.article file */
 char dead_articles[PATH_LEN];		/* ~/dead.articles file */
@@ -83,7 +77,6 @@ char index_maildir[PATH_LEN];
 char index_newsdir[PATH_LEN];	/* directory for private overview data */
 char index_savedir[PATH_LEN];
 char inewsdir[PATH_LEN];
-char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
 char local_attributes_file[PATH_LEN];
 char local_config_file[PATH_LEN];
 char local_input_history_file[PATH_LEN];
@@ -104,7 +97,8 @@ char postponed_articles_file[PATH_LEN];	/* ~/.tin/postponed.articles file */
 char rcdir[PATH_LEN];
 char save_active_file[PATH_LEN];
 char spooldir[PATH_LEN];		/* directory where news is */
-char subscriptions_file[PATH_LEN];
+char overviewfmt_file[PATH_LEN];	/* full path to overview.fmt */
+char subscriptions_file[PATH_LEN];	/* full path to subscriptions */
 char tin_progname[PATH_LEN];		/* program name */
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
@@ -117,16 +111,10 @@ char userid[PATH_LEN];
 	char novrootdir[PATH_LEN];		/* root directory of nov index files */
 #endif /* !NNTP_ONLY */
 
-#ifdef VMS
-	char rcdir_asfile[PATH_LEN];	/* rcdir expressed as dev:[dir]tin.dir, for stat() */
-#endif /* VMS */
-
-t_function last_search;	/* for repeated search */
+t_function last_search = GLOBAL_SEARCH_REPEAT;	/* for repeated search */
 int hist_last[HIST_MAXNUM + 1];
 int hist_pos[HIST_MAXNUM + 1];
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
-int num_headers_to_display;		/* num headers to display -- swp */
-int num_headers_to_not_display;		/* num headers to not display -- swp */
 int system_status;
 int xmouse, xrow, xcol;			/* xterm button pressing information */
 
@@ -139,18 +127,17 @@ t_bool created_rcdir;			/* checks if first time tin is started */
 t_bool dangerous_signal_exit;		/* no get_respcode() in nntp_command when dangerous signal exit */
 t_bool disable_gnksa_domain_check;	/* disable checking TLD in From: etc. */
 t_bool disable_sender;			/* disable generation of Sender: header */
-t_bool filtered_articles;		/* locally killed / auto-selected articles */
 #ifdef NO_POSTING
 	t_bool force_no_post = TRUE;		/* force no posting mode */
 #else
 	t_bool force_no_post = FALSE;	/* don't force no posting mode */
 #endif /* NO_POSTING */
-t_bool got_sig_pipe = FALSE;
 t_bool list_active;
 t_bool newsrc_active;
 t_bool no_write = FALSE;		/* do not write newsrc on quit (-X cmd-line flag) */
 t_bool post_article_and_exit;		/* quick post of an article then exit(elm like) */
 t_bool post_postponed_and_exit;		/* post postponed articles and exit */
+t_bool range_active;		/* Set if a range is defined */
 t_bool reread_active_for_posted_arts;
 t_bool read_local_newsgroups_file;	/* read newsgroups file locally or via NNTP */
 t_bool read_news_via_nntp = FALSE;	/* read news locally or via NNTP */
@@ -180,6 +167,7 @@ char *input_history[HIST_MAXNUM + 1][HIST_SIZE + 1];
 struct regex_cache
 		strip_re_regex, strip_was_regex,
 		uubegin_regex, uubody_regex,
+		verbatim_begin_regex, verbatim_end_regex,
 		url_regex, mail_regex, news_regex,
 		shar_regex,
 		slashes_regex, stars_regex, underscores_regex, strokes_regex
@@ -191,6 +179,8 @@ struct regex_cache
 		NULL
 	};
 
+struct t_cmdlineopts cmdline;
+
 struct t_config tinrc = {
 	ART_MARK_DELETED,		/* art_marked_deleted */
 	MARK_INRANGE,			/* art_marked_inrange */
@@ -244,7 +234,7 @@ struct t_config tinrc = {
 #else
 	-1,		/* mm_network_charset, defaults to $MM_CHARSET */
 #endif /* !CHARSET_CONVERSION */
-	"",		/* mm_local_charset, display charset */
+	"US-ASCII",		/* mm_local_charset, display charset */
 #ifdef HAVE_ICONV_OPEN_TRANSLIT
 	FALSE,	/* translit */
 #endif /* HAVE_ICONV_OPEN_TRANSLIT */
@@ -264,6 +254,8 @@ struct t_config tinrc = {
 	"",		/* sigfile */
 	"",		/* strip_re_regex */
 	"",		/* strip_was_regex */
+	"",		/* verbatim_begin_regex */
+	"",		/* verbatim_end_regex */
 	"",		/* savedir */
 	"",		/* spamtrap_warning_addresses */
 	DEFAULT_URL_HANDLER,	/* url_handler */
@@ -275,12 +267,13 @@ struct t_config tinrc = {
 	'a',		/* default_save_mode */
 	0,		/* getart_limit */
 	2,		/* recent_time */
+	GOTO_NEXT_UNREAD_TAB,		/* goto_next_unread */
 	32,		/* groupname_max_length */
 	UUE_NO,	/* hide_uue */
 	KILL_UNREAD,		/* kill_level */
-	MIME_ENCODING_7BIT,		/* mail_mime_encoding */
-	MIME_ENCODING_7BIT,		/* post_mime_encoding */
-	POST_PROC_NO,			/* post_process */
+	MIME_ENCODING_QP,		/* mail_mime_encoding */
+	MIME_ENCODING_8BIT,		/* post_mime_encoding */
+	POST_PROC_NO,			/* post_process_type */
 	REREAD_ACTIVE_FILE_SECS,	/* reread_active_file_secs */
 	1,		/* scroll_lines */
 	SHOW_FROM_NAME,				/* show_author */
@@ -295,6 +288,7 @@ struct t_config tinrc = {
 	50,		/* score_limit_select */
 	-100,		/* score_kill */
 	100,		/* score_select */
+	0,		/* trim_article_body */
 #ifdef HAVE_COLOR
 	0,		/* col_back (initialised later) */
 	0,		/* col_from (initialised later) */
@@ -316,6 +310,7 @@ struct t_config tinrc = {
 	0,		/* col_response (initialised later) */
 	0,		/* col_signature (initialised later) */
 	0,		/* col_urls (initialised later) */
+	0,		/* col_verbatim (initialised later) */
 	0,		/* col_subject (initialised later) */
 	0,		/* col_text (initialised later) */
 	0,		/* col_title (initialised later) */
@@ -331,11 +326,11 @@ struct t_config tinrc = {
 #ifdef HAVE_COLOR
 	FALSE,		/* use_color */
 #endif /* HAVE_COLOR */
+	FALSE,		/* abbreviate_groupname */
 	TRUE,		/* add_posted_to_filter */
 	TRUE,		/* advertising */
 	TRUE,		/* alternative_handling */
-	FALSE,		/* auto_bcc */
-	FALSE,		/* auto_cc */
+	0,			/* auto_cc_bcc */
 	TRUE,		/* auto_list_thread */
 	FALSE,		/* auto_reconnect */
 	FALSE,		/* auto_save */
@@ -362,7 +357,6 @@ struct t_config tinrc = {
 	FALSE,		/* mail_8bit_header */
 	FALSE,		/* mark_ignore_tags */
 	TRUE,		/* mark_saved_read */
-	TRUE,		/* pgdn_goto_next */
 	TRUE,		/* pos_first_unread */
 	FALSE,		/* post_8bit_header */
 	TRUE,		/* post_process_view */
@@ -379,7 +373,6 @@ struct t_config tinrc = {
 	TRUE,		/* show_signatures */
 	TRUE,		/* sigdashes */
 	TRUE,		/* signature_repost */
-	FALSE,		/* space_goto_next_unread */
 #ifdef M_UNIX
 	TRUE,		/* start_editor_offset */
 #else
@@ -387,10 +380,10 @@ struct t_config tinrc = {
 #endif /* M_UNIX */
 	TRUE,		/* strip_blanks */
 	FALSE,		/* strip_newsrc */
-	TRUE,		/* tab_goto_next_unread */
 	FALSE,		/* tex2iso_conv */
 	TRUE,		/* thread_catchup_on_exit */
 	TRUE,		/* unlink_article */
+	TRUE,		/* verbatim_handling */
 	"",		/* inews_prog */
 	INTERACTIVE_NONE,		/* interactive_mailer */
 	FALSE,		/* use_mouse */
@@ -413,12 +406,92 @@ struct t_config tinrc = {
 	NORMALIZE_NFKC,		/* normalization form */
 #endif /* HAVE_UNICODE_NORMALIZATION */
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
-	FALSE		/* render_bidi */
+	FALSE,		/* render_bidi */
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
+#ifdef CHARSET_CONVERSION
+	-1,		/* attrib_mm_network_charset, defaults to $MM_CHARSET */
+	"",		/* attrib_undeclared_charset */
+#endif /* !CHARSET_CONVERSION */
+	"",		/* attrib_editor_format */
+	"",		/* attrib_fcc */
+	"",		/* attrib_maildir */
+	"",		/* attrib_from */
+	"",		/* attrib_mailing_list */
+	"",		/* attrib_organization */
+	"",		/* attrib_followup_to */
+	"",		/* attrib_mime_types_to_save */
+	"",		/* attrib_news_headers_to_display */
+	"",		/* attrib_news_headers_to_not_display */
+	"",		/* attrib_news_quote_format */
+	"",		/* attrib_quote_chars */
+	"",		/* attrib_sigfile */
+	"",		/* attrib_savedir */
+	"",		/* attrib_savefile */
+	"",		/* attrib_x_body */
+	"",		/* attrib_x_headers */
+#ifdef HAVE_ISPELL
+	"",		/* attrib_ispell */
+#endif /* HAVE_ISPELL */
+	"",		/* attrib_quick_kill_scope */
+	"",		/* attrib_quick_select_scope */
+	"",		/* attrib_date_format */
+	0,		/* attrib_trim_article_body */
+	0,		/* attrib_auto_cc_bcc */
+	SHOW_INFO_LINES,		/* attrib_show_info */
+	FILTER_SUBJ_CASE_SENSITIVE,		/* attrib_quick_kill_header */
+	FILTER_SUBJ_CASE_SENSITIVE,		/* attrib_quick_select_header */
+	MIME_ENCODING_QP,		/* attrib_mail_mime_encoding */
+	MIME_ENCODING_8BIT,		/* attrib_post_mime_encoding */
+	POST_PROC_NO,			/* attrib_post_process_type */
+	SHOW_FROM_NAME,			/* attrib_show_author */
+	SORT_ARTICLES_BY_DATE_ASCEND,		/* attrib_sort_article_type */
+	SORT_THREADS_BY_SCORE_DESCEND,		/* attrib_sort_threads_type */
+	THREAD_BOTH,		/* attrib_thread_articles */
+	THREAD_PERC_DEFAULT,	/* attrib_thread_perc */
+	TRUE,		/* attrib_add_posted_to_filter */
+	TRUE,		/* attrib_advertising */
+	TRUE,		/* attrib_alternative_handling */
+	TRUE,		/* attrib_auto_list_thread */
+	FALSE,		/* attrib_auto_select */
+	FALSE,		/* attrib_auto_save */
+	TRUE,		/* attrib_batch_save */
+	TRUE,		/* attrib_delete_tmp_files */
+	TRUE,		/* attrib_group_catchup_on_exit */
+	FALSE,		/* attrib_mail_8bit_header */
+	FALSE,		/* attrib_mime_forward */
+	FALSE,		/* attrib_mark_ignore_tags */
+	TRUE,		/* attrib_mark_saved_read */
+	TRUE,		/* attrib_pos_first_unread */
+	FALSE,		/* attrib_post_8bit_header */
+	TRUE,		/* attrib_post_process_view */
+#ifndef DISABLE_PRINTING
+	FALSE,		/* attrib_print_header */
+#endif /* !DISABLE_PRINTING */
+	FALSE,		/* attrib_process_only_unread */
+	FALSE,		/* attrib_prompt_followupto */
+	TRUE,		/* attrib_show_only_unread_arts */
+	TRUE,		/* attrib_show_signatures */
+	TRUE,		/* attrib_sigdashes */
+	TRUE,		/* attrib_signature_repost */
+#ifdef M_UNIX
+	TRUE,		/* attrib_start_editor_offset */
+#else
+	FALSE,		/* attrib_start_editor_offset */
+#endif /* M_UNIX */
+	FALSE,		/* attrib_tex2iso_conv */
+	TRUE,		/* attrib_thread_catchup_on_exit */
+	TRUE,		/* attrib_verbatim_handling */
+	FALSE,		/* attrib_x_comment_to */
+	TRUE,		/* attrib_wrap_on_next_unread */
+	FALSE,		/* attrib_ask_for_metamail */
+	FALSE,		/* attrib_quick_kill_case */
+	FALSE,		/* attrib_quick_kill_expire */
+	FALSE,		/* attrib_quick_select_case */
+	FALSE		/* attrib_quick_select_expire */
 };
 
 struct t_capabilities nntp_caps = {
-	0, /* type (none, LIST EXTENSIONS, CAPABILITIES) */
+	NONE, /* type (NONE, CAPABILITIES, BROKEN) */
 	0, /* CAPABILITIES version */
 	FALSE, /* MODE-READER: "MODE READER" */
 	FALSE, /* READER: "ARTICLE", "BODY" */
@@ -433,6 +506,7 @@ struct t_capabilities nntp_caps = {
 	FALSE, /* LIST: "LIST SUBSCRIPTIONS" */
 	FALSE, /* LIST: "LIST DISTRIBUTIONS" */
 	FALSE, /* LIST: "LIST MODERATORS" */
+	FALSE, /* LIST: "LIST COUNTS" */
 	FALSE, /* XPAT */
 	FALSE, /* HDR: "HDR", "LIST HEADERS" */
 	NULL, /* [X]HDR */
@@ -444,17 +518,22 @@ struct t_capabilities nntp_caps = {
 	FALSE, /* STARTTLS */
 	FALSE, /* AUTHINFO USER/PASS */
 	FALSE, /* AUTHINFO SASL */
+	FALSE, /* AUTHINFO available but not in current state */
+	SASL_NONE, /* SASL CRAM-MD5 DIGEST-MD5 PLAIN GSSAPI EXTERNAL OTP NTLM LOGIN */
+	FALSE, /* COMPRESS */
+	COMPRESS_NONE, /* COMPRESS_NONE, COMPRESS_DEFLATE */
 #if 0
-	FALSE, /* SASL CRAM-MD5 */
-	FALSE, /* SASL DIGEST-MD5 */
-	FALSE, /* SASL PLAIN */
-	FALSE, /* SASL GSSAPI */
-	FALSE, /* SASL EXTERNAL */
 	FALSE, /* STREAMING: "MODE STREAM", "CHECK", "TAKETHIS" */
 	FALSE /* IHAVE */
 #endif /* 0 */
+#ifndef BROKEN_LISTGROUP
+	FALSE /* LISTGROUP doesn't select group */
+#else
+	TRUE
+#endif /* !BROKEN_LISTGROUP */
 };
 
+static char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
 static mode_t real_umask;
 
 #ifdef HAVE_COLOR
@@ -487,11 +566,13 @@ static const struct {
 	{ &tinrc.col_response,    2 },
 	{ &tinrc.col_signature,   4 },
 	{ &tinrc.col_urls,       -1 },
+	{ &tinrc.col_verbatim,    5 },
 	{ &tinrc.col_subject,     6 },
 	{ &tinrc.col_text,       DFT_FORE },
 	{ &tinrc.col_title,       4 },
 };
 
+
 static void
 preinit_colors(
 	void)
@@ -548,9 +629,7 @@ init_selfinfo(
 
 #ifdef HAVE_SYS_UTSNAME_H
 #	ifdef HAVE_UNAME
-	if (uname(&system_info) != -1)
-		;
-	else
+	if (uname(&system_info) == -1)
 #	endif /* HAVE_UNAME */
 	{
 		strcpy(system_info.sysname, "unknown");
@@ -578,26 +657,13 @@ init_selfinfo(
 	real_umask = umask(0);
 	(void) umask(real_umask);
 
-#ifndef VMS
 	if ((myentry = getpwuid(getuid())) == NULL) {
-		error_message(_(txt_error_passwd_missing));
+		error_message(2, _(txt_error_passwd_missing));
 		giveup();
 	}
-#else
-	if (((ptr = getlogin()) != NULL) && strlen(ptr))
-		myentry = getpwnam(ptr);
-	else {
-		error_message(_(txt_error_passwd_missing));
-		giveup();
-	}
-#endif /* !VMS */
 
 	strcpy(userid, myentry->pw_name);
 
-#ifdef VMS
-	lower(userid);
-#endif /* VMS */
-
 	if (((ptr = getenv("TIN_HOMEDIR")) != NULL) && strlen(ptr)) {
 		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
 	} else if (((ptr = getenv("HOME")) != NULL) && strlen(ptr)) {
@@ -607,27 +673,15 @@ init_selfinfo(
 	} else
 		strncpy(homedir, TMPDIR, sizeof(homedir) - 1);
 
-	cmdline_nntpserver[0] = '\0';
 	created_rcdir = FALSE;
 	dangerous_signal_exit = FALSE;
 	disable_gnksa_domain_check = FALSE;
-#ifdef MAC_OS_X	/* usualy they don't have a valid FQDN */
-	/*
-	 * TODO: check if gnksa_check_domain(get_fqdn()) returns ok
-	 *       and if it does, don't disable Sender
-	 */
-	disable_sender = TRUE;
-#else
-	disable_sender = FALSE;
-#endif /* MAC_OS_X */
-	filtered_articles = FALSE;
+	disable_sender = FALSE;	/* we set force_no_post=TRUE later on if we don't have a valid FQDN */
 	iso2asc_supported = atoi(get_val("ISO2ASC", DEFAULT_ISO2ASC));
 	if (iso2asc_supported > NUM_ISO_TABLES || iso2asc_supported < 0) /* TODO: issue a warning here? */
 		iso2asc_supported = -1;
 	list_active = FALSE;
 	newsrc_active = FALSE;
-	num_headers_to_display = 0;
-	num_headers_to_not_display = 0;
 	post_article_and_exit = FALSE;
 	post_postponed_and_exit = FALSE;
 	read_local_newsgroups_file = FALSE;
@@ -654,8 +708,6 @@ init_selfinfo(
 	snprintf(cvers, sizeof(cvers), txt_copyright_notice, page_header);
 
 	default_organization[0] = '\0';
-	news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
-	news_headers_to_not_display_array = NULL;
 
 	strncpy(bug_addr, BUG_REPORT_ADDRESS, sizeof(bug_addr) - 1);
 
@@ -674,11 +726,6 @@ init_selfinfo(
 	my_strncpy(default_organization, get_val("ORGANIZATION", ""), sizeof(default_organization) - 1);
 #endif /* apollo */
 
-#ifdef USE_INN_NNTPLIB
-	if ((ptr = GetConfigValue(_CONF_ORGANIZATION)) != NULL)
-		my_strncpy(default_organization, ptr, sizeof(default_organization) - 1);
-#endif /* USE_INN_NNTPLIB */
-
 #ifndef NNTP_ONLY
 	my_strncpy(libdir, get_val("TIN_LIBDIR", NEWSLIBDIR), sizeof(libdir) - 1);
 	my_strncpy(novrootdir, get_val("TIN_NOVROOTDIR", NOVROOTDIR), sizeof(novrootdir) - 1);
@@ -689,6 +736,7 @@ init_selfinfo(
 	news_active_file[0] = '\0';
 	active_times_file[0] = '\0';
 	newsgroups_file[0] = '\0';
+	overviewfmt_file[0] = '\0';
 	subscriptions_file[0] = '\0';
 
 	/*
@@ -702,8 +750,7 @@ init_selfinfo(
 	 * if it's still unset fall into no posting mode.
 	 */
 	if (domain_name[0] == '\0') {
-		error_message(_(txt_error_no_domain_name));
-		sleep(2);
+		error_message(4, _(txt_error_no_domain_name));
 		force_no_post = TRUE;
 	}
 
@@ -715,18 +762,20 @@ init_selfinfo(
 	 * values given in env-vars? ($MM_CHARSET, $TIN_ACTIVEFILE)
 	 */
 	if (!*news_active_file) /* TODO: really prepend libdir here in case of $TIN_ACTIVEFILE is set? */
-		joinpath(news_active_file, libdir, get_val("TIN_ACTIVEFILE", ACTIVE_FILE));
+		joinpath(news_active_file, sizeof(news_active_file), libdir, get_val("TIN_ACTIVEFILE", ACTIVE_FILE));
 	if (!*active_times_file)
-		joinpath(active_times_file, libdir, ACTIVE_TIMES_FILE);
+		joinpath(active_times_file, sizeof(active_times_file), libdir, ACTIVE_TIMES_FILE);
 	if (!*newsgroups_file)
-		joinpath(newsgroups_file, libdir, NEWSGROUPS_FILE);
+		joinpath(newsgroups_file, sizeof(newsgroups_file), libdir, NEWSGROUPS_FILE);
 	if (!*subscriptions_file)
-		joinpath(subscriptions_file, libdir, SUBSCRIPTIONS_FILE);
+		joinpath(subscriptions_file, sizeof(subscriptions_file), libdir, SUBSCRIPTIONS_FILE);
+	if (!*overviewfmt_file)
+		joinpath(overviewfmt_file, sizeof(overviewfmt_file), libdir, OVERVIEW_FMT);
 	if (!*default_organization) {
-		char buf[LEN];
+		char buf[LEN], filename[PATH_LEN];
 
-		joinpath(buf, libdir, "organization");
-		if ((fp = fopen(buf, "r")) != NULL) {
+		joinpath(filename, sizeof(filename), libdir, "organization");
+		if ((fp = fopen(filename, "r")) != NULL) {
 			if (fgets(buf, (int) sizeof(buf), fp) != NULL) {
 				ptr = strrchr(buf, '\n');
 				if (ptr != NULL)
@@ -760,24 +809,16 @@ init_selfinfo(
 #endif /* !CHARSET_CONVERSION */
 
 #ifdef TIN_DEFAULTS_DIR
-	joinpath(global_attributes_file, TIN_DEFAULTS_DIR, ATTRIBUTES_FILE);
-	joinpath(global_config_file, TIN_DEFAULTS_DIR, CONFIG_FILE);
+	joinpath(global_attributes_file, sizeof(global_attributes_file), TIN_DEFAULTS_DIR, ATTRIBUTES_FILE);
+	joinpath(global_config_file, sizeof(global_config_file), TIN_DEFAULTS_DIR, CONFIG_FILE);
 #else
 	/* read_site_config() might have changed the value of libdir */
-	joinpath(global_attributes_file, libdir, ATTRIBUTES_FILE);
-	joinpath(global_config_file, libdir, CONFIG_FILE);
+	joinpath(global_attributes_file, sizeof(global_attributes_file), libdir, ATTRIBUTES_FILE);
+	joinpath(global_config_file, sizeof(global_config_file), libdir, CONFIG_FILE);
 #endif /* TIN_DEFAULTS_DIR */
 
-#ifdef VMS
-	joindir(rcdir, homedir, RCDIR); /* we're naming a directory here */
-	joinpath(rcdir_asfile, homedir, RCDIR);	/* for stat() */
-	strcat(rcdir_asfile, ".DIR");
-	if (stat(rcdir_asfile, &sb) == -1)
-#else
-	joinpath(rcdir, homedir, RCDIR);
-	if (stat(rcdir, &sb) == -1)
-#endif /* VMS */
-	{
+	joinpath(rcdir, sizeof(rcdir), homedir, RCDIR);
+	if (stat(rcdir, &sb) == -1) {
 		created_rcdir = TRUE;
 		my_mkdir(rcdir, (mode_t) (S_IRWXU));
 	}
@@ -787,49 +828,45 @@ init_selfinfo(
 	strcpy(tinrc.printer, DEFAULT_PRINTER);
 #endif /* !DISABLE_PRINTING */
 	strcpy(tinrc.inews_prog, PATH_INEWS);
-	joinpath(article_name, homedir, TIN_ARTICLE_NAME);
+	joinpath(article_name, sizeof(article_name), homedir, TIN_ARTICLE_NAME);
 #ifdef APPEND_PID
-	snprintf(article_name + strlen(article_name), sizeof(article_name) - strlen(article_name), ".%d", (int) process_id);
+	snprintf(article_name + strlen(article_name), sizeof(article_name) - strlen(article_name), ".%ld", (long) process_id);
 #endif /* APPEND_PID */
-	joinpath(dead_article, homedir, "dead.article");
-	joinpath(dead_articles, homedir, "dead.articles");
-	JOINPATH(tinrc.maildir, homedir, DEFAULT_MAILDIR);
-	JOINPATH(tinrc.savedir, homedir, DEFAULT_SAVEDIR);
-	joinpath(tinrc.sigfile, homedir, ".Sig");
-	joinpath(default_signature, homedir, ".signature");
+	joinpath(dead_article, sizeof(dead_article), homedir, "dead.article");
+	joinpath(dead_articles, sizeof(dead_articles), homedir, "dead.articles");
+	joinpath(tinrc.maildir, sizeof(tinrc.maildir), homedir, DEFAULT_MAILDIR);
+	joinpath(tinrc.savedir, sizeof(tinrc.savedir), homedir, DEFAULT_SAVEDIR);
+	joinpath(tinrc.sigfile, sizeof(tinrc.sigfile), homedir, ".Sig");
+	joinpath(default_signature, sizeof(default_signature), homedir, ".signature");
 
 	if (!index_newsdir[0])
-		JOINPATH(index_newsdir, get_val("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
-	JOINPATH(index_maildir, get_val("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
+		joinpath(index_newsdir, sizeof(index_newsdir), get_val("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
+	joinpath(index_maildir, sizeof(index_maildir), get_val("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
 	if (stat(index_maildir, &sb) == -1)
 		my_mkdir(index_maildir, (mode_t) S_IRWXU);
-	JOINPATH(index_savedir, get_val("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
+	joinpath(index_savedir, sizeof(index_savedir), get_val("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
 	if (stat(index_savedir, &sb) == -1)
 		my_mkdir(index_savedir, (mode_t) S_IRWXU);
-	joinpath(local_attributes_file, rcdir, ATTRIBUTES_FILE);
-	joinpath(local_config_file, rcdir, CONFIG_FILE);
-	joinpath(filter_file, rcdir, FILTER_FILE);
-	joinpath(local_input_history_file, rcdir, INPUT_HISTORY_FILE);
-	joinpath(local_newsrctable_file, rcdir, NEWSRCTABLE_FILE);
+	joinpath(local_attributes_file, sizeof(local_attributes_file), rcdir, ATTRIBUTES_FILE);
+	joinpath(local_config_file, sizeof(local_config_file), rcdir, CONFIG_FILE);
+	joinpath(filter_file, sizeof(filter_file), rcdir, FILTER_FILE);
+	joinpath(local_input_history_file, sizeof(local_input_history_file), rcdir, INPUT_HISTORY_FILE);
+	joinpath(local_newsrctable_file, sizeof(local_newsrctable_file), rcdir, NEWSRCTABLE_FILE);
 #ifdef HAVE_MH_MAIL_HANDLING
-	joinpath(mail_active_file, rcdir, ACTIVE_MAIL_FILE);
+	joinpath(mail_active_file, sizeof(mail_active_file), rcdir, ACTIVE_MAIL_FILE);
 #endif /* HAVE_MH_MAIL_HANDLING */
-#ifdef VMS
-	joinpath(mailbox, DEFAULT_MAILBOX, "MAIL.TXT");
-#else
-	joinpath(mailbox, DEFAULT_MAILBOX, userid);
-#endif /* VMS */
+	joinpath(mailbox, sizeof(mailbox), DEFAULT_MAILBOX, userid);
 #ifdef HAVE_MH_MAIL_HANDLING
-	joinpath(mailgroups_file, rcdir, MAILGROUPS_FILE);
+	joinpath(mailgroups_file, sizeof(mailgroups_file), rcdir, MAILGROUPS_FILE);
 #endif /* HAVE_MH_MAIL_HANDLING */
-	joinpath(newsrc, homedir, NEWSRC_FILE);
-	joinpath(newnewsrc, homedir, NEWNEWSRC_FILE);
+	joinpath(newsrc, sizeof(newsrc), homedir, NEWSRC_FILE);
+	joinpath(newnewsrc, sizeof(newnewsrc), homedir, NEWNEWSRC_FILE);
 #ifdef APPEND_PID
 	snprintf(newnewsrc + strlen(newnewsrc), sizeof(newnewsrc) - strlen(newnewsrc), ".%d", (int) process_id);
 #endif /* APPEND_PID */
-	joinpath(posted_info_file, rcdir, POSTED_FILE);
-	joinpath(postponed_articles_file, rcdir, POSTPONED_FILE);
-	joinpath(save_active_file, rcdir, ACTIVE_SAVE_FILE);
+	joinpath(posted_info_file, sizeof(posted_info_file), rcdir, POSTED_FILE);
+	joinpath(postponed_articles_file, sizeof(postponed_articles_file), rcdir, POSTPONED_FILE);
+	joinpath(save_active_file, sizeof(save_active_file), rcdir, ACTIVE_SAVE_FILE);
 
 	snprintf(lock_file, sizeof(lock_file), INDEX_LOCK, TMPDIR, userid);
 
@@ -845,9 +882,6 @@ init_selfinfo(
 		}
 	}
 
-	if (stat(local_attributes_file, &sb) == -1)
-		write_attributes_file(local_attributes_file);
-
 	init_postinfo();
 	snprintf(txt_help_bug_report, sizeof(txt_help_bug_report), _(txt_help_bug), bug_addr);
 
@@ -871,7 +905,7 @@ read_site_config(
 	void)
 {
 	FILE *fp = (FILE *) 0;
-	char buf[LEN];
+	char buf[LEN], filename[PATH_LEN];
 	static const char *tin_defaults[] = { TIN_DEFAULTS };
 	int i = 0;
 
@@ -879,8 +913,8 @@ read_site_config(
 	 * try to find tin.defaults in some different locations
 	 */
 	while (tin_defaults[i] != NULL) {
-		joinpath(buf, tin_defaults[i++], "tin.defaults");
-		if ((fp = fopen(buf, "r")) != NULL)
+		joinpath(filename, sizeof(filename), tin_defaults[i++], "tin.defaults");
+		if ((fp = fopen(filename, "r")) != NULL)
 			break;
 	}
 
@@ -909,6 +943,8 @@ read_site_config(
 			continue;
 		if (match_string(buf, "subscriptionsfile=", subscriptions_file, sizeof(subscriptions_file)))
 			continue;
+		if (match_string(buf, "overviewfmtfile=", overviewfmt_file, sizeof(overviewfmt_file)))
+			continue;
 		if (match_string(buf, "domainname=", domain_name, sizeof(domain_name)))
 			continue;
 		if (match_string(buf, "inewsdir=", inewsdir, sizeof(inewsdir)))
@@ -1004,6 +1040,13 @@ postinit_regexp(
 		STRCPY(tinrc.underscores_regex, DEFAULT_UNDERSCORES_REGEX);
 	compile_regex(tinrc.underscores_regex, &underscores_regex, PCRE_CASELESS);
 
+	if (!strlen(tinrc.verbatim_begin_regex))
+		STRCPY(tinrc.verbatim_begin_regex, DEFAULT_VERBATIM_BEGIN_REGEX);
+	compile_regex(tinrc.verbatim_begin_regex, &verbatim_begin_regex, PCRE_ANCHORED);
+	if (!strlen(tinrc.verbatim_end_regex))
+		STRCPY(tinrc.verbatim_end_regex, DEFAULT_VERBATIM_END_REGEX);
+	compile_regex(tinrc.verbatim_end_regex, &verbatim_end_regex, PCRE_ANCHORED);
+
 	compile_regex(UUBEGIN_REGEX, &uubegin_regex, PCRE_ANCHORED);
 	compile_regex(UUBODY_REGEX, &uubody_regex, PCRE_ANCHORED);
 
@@ -1016,7 +1059,7 @@ postinit_regexp(
 
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-static t_bool
+t_bool
 utf8_pcre(
 	void)
 {
