@@ -3,10 +3,10 @@
  *  Module    : keymap.c
  *  Author    : D. Nimmich, J. Faultless
  *  Created   : 2000-05-25
- *  Updated   : 2005-10-19
+ *  Updated   : 2010-04-06
  *  Notes     : This file contains key mapping routines and variables.
  *
- * Copyright (c) 2000-2006 Dirk Nimmich <nimmich@muenster.de>
+ * Copyright (c) 2000-2010 Dirk Nimmich <nimmich@muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,6 @@ struct keylist feed_type_keys = { NULL, 0, 0 };
 struct keylist filter_keys = { NULL, 0, 0 };
 struct keylist group_keys = { NULL, 0, 0 };
 struct keylist info_keys = { NULL, 0, 0 };
-struct keylist mark_read_keys = { NULL, 0, 0 };
 struct keylist option_menu_keys = { NULL, 0, 0 };
 struct keylist page_keys = { NULL, 0, 0 };
 #ifdef HAVE_PGP_GPG
@@ -81,6 +80,7 @@ struct keylist post_postpone_keys = { NULL, 0, 0 };
 struct keylist post_send_keys = { NULL, 0, 0 };
 struct keylist prompt_keys = { NULL, 0, 0 };
 struct keylist save_append_overwrite_keys = { NULL, 0, 0 };
+struct keylist scope_keys = { NULL, 0, 0 };
 struct keylist select_keys = { NULL, 0, 0 };
 struct keylist thread_keys = { NULL, 0, 0 };
 
@@ -137,7 +137,7 @@ func_to_key(
 /*
  * adds a key to a keylist
  * default_key: TRUE if a default key should be added
- * returns TRUE if the key was succesfully added else FALSE
+ * returns TRUE if the key was successfully added else FALSE
  */
 static t_bool
 add_key(
@@ -154,7 +154,7 @@ add_key(
 	struct keynode *entry = NULL;
 
 	/* is a function already associated with this key */
-	for (i = 0; i < keys->used; i++) {
+	for (i = 0; key != '\0' && i < keys->used; i++) {
 		if (keys->list[i].key == key)
 			entry = &keys->list[i];
 	}
@@ -169,7 +169,7 @@ add_key(
 		/* add a new entry */
 		if (keys->used >= keys->max) {
 			if (keys->list == NULL) {
-				keys->max = 1;
+				keys->max = DEFAULT_MAPKEYS_NUM;
 				keys->list = my_malloc(keys->max * sizeof(struct keynode));
 			} else {
 				keys->max++;
@@ -240,12 +240,12 @@ free_keymaps(
 	free_keylist(&post_delete_keys);
 	free_keylist(&post_cancel_keys);
 	free_keylist(&filter_keys);
-	free_keylist(&mark_read_keys);
 #ifdef HAVE_PGP_GPG
 	free_keylist(&pgp_mail_keys);
 	free_keylist(&pgp_news_keys);
 #endif /* HAVE_PGP_GPG */
 	free_keylist(&save_append_overwrite_keys);
+	free_keylist(&scope_keys);
 	free_keylist(&feed_type_keys);
 	free_keylist(&feed_post_process_keys);
 	free_keylist(&feed_supersede_article_keys);
@@ -319,8 +319,8 @@ read_keymap_file(
 	FILE *fp = (FILE *) 0;
 	char *line, *keydef, *kname;
 	char *map, *ptr;
-	char buf[LEN], buff[NAME_LEN + 1];
-	int upgrade = RC_CHECK;
+	char buf[LEN], buff[NAME_LEN + 1], filename[PATH_LEN];
+	enum rc_state upgrade = RC_CHECK;
 	t_bool ret = TRUE;
 
 	/*
@@ -329,7 +329,7 @@ read_keymap_file(
 	 *
 	 * locale is first match from LC_ALL, LC_CTYPE, LC_MESSAGES, LANG
 	 *
-	 * TODO: fix possible buf-overflows
+	 * TODO: LC_CTYPE has higgher priority than LC_MESSAGES, does this make sense?
 	 */
 	/* get locale suffix */
 	map = my_strdup(get_val("LC_ALL", get_val("LC_CTYPE", get_val("LC_MESSAGES", get_val("LANG", "")))));
@@ -337,21 +337,21 @@ read_keymap_file(
 		if ((ptr = strchr(map, '.')))
 			*ptr = '\0';
 		snprintf(buff, sizeof(buff), "%s.%s", KEYMAP_FILE, map);
-		joinpath(buf, rcdir, buff);
-		fp = fopen(buf, "r");
+		joinpath(filename, sizeof(filename), rcdir, buff);
+		fp = fopen(filename, "r");
 	}
 	if (!fp) {
-		joinpath(buf, rcdir, KEYMAP_FILE);
-		fp = fopen(buf, "r");
+		joinpath(filename, sizeof(filename), rcdir, KEYMAP_FILE);
+		fp = fopen(filename, "r");
 	}
 #ifdef TIN_DEFAULTS_DIR
 	if (strlen(map) && !fp) {
-		joinpath(buf, TIN_DEFAULTS_DIR, buff);
-		fp = fopen(buf, "r");
+		joinpath(filename, sizeof(filename), TIN_DEFAULTS_DIR, buff);
+		fp = fopen(filename, "r");
 	}
 	if (!fp) {
-		joinpath(buf, TIN_DEFAULTS_DIR, KEYMAP_FILE);
-		fp = fopen(buf, "r");
+		joinpath(filename, sizeof(filename), TIN_DEFAULTS_DIR, KEYMAP_FILE);
+		fp = fopen(filename, "r");
 	}
 #endif /* TIN_DEFAULTS_DIR */
 
@@ -360,20 +360,25 @@ read_keymap_file(
 	if (!fp)
 		return TRUE; /* no keymap file is not an error */
 
-	map = my_strdup(buf); /* remember keymap-name */
+	map = my_strdup(filename); /* remember keymap-name */
 
 	/* check if keymap file is uptodate */
 	while ((line = fgets(buf, sizeof(buf), fp)) != NULL) {
-		if (line[0] == '#' && upgrade == RC_CHECK) {
-			if ((upgrade = check_upgrade(line, "# Keymap file V", KEYMAP_VERSION)) == RC_UPGRADE) {
-				fclose(fp);
-				upgrade_keymap_file(map);
-				upgrade = RC_IGNORE;
-				fp = fopen(map, "r");
+		if (line[0] == '#') {
+			if (upgrade == RC_CHECK && match_string(buf, "# Keymap file V", NULL, 0)) {
+				/* TODO: keymap downgrade */
+				if ((upgrade = check_upgrade(line, "# Keymap file V", KEYMAP_VERSION)) == RC_UPGRADE) {
+					fclose(fp);
+					upgrade_keymap_file(map);
+					upgrade = RC_IGNORE;
+					if (!(fp = fopen(map, "r"))) /* TODO: issue error message? */
+						return TRUE;
+				}
+				break;
 			}
-			break;
 		}
 	}
+	rewind(fp);
 
 	free_keymaps();
 	while ((line = fgets(buf, sizeof(buf), fp)) != NULL) {
@@ -389,7 +394,7 @@ read_keymap_file(
 			 * Warn about basic syntax errors
 			 */
 			if (keydef == NULL || !strlen(keydef)) {
-				wait_message(0, _(txt_keymap_missing_key), kname);
+				error_message(0, _(txt_keymap_missing_key), kname);
 				ret = FALSE;
 				continue;
 			}
@@ -398,10 +403,10 @@ read_keymap_file(
 
 		/*
 		 * TODO: useful? shared keymaps (NFS-Home) may differ
-		 * depending on the OS (i.e. on tin has color the other has not)
+		 * depending on the OS (i.e. one tin has color the other has not)
 		 */
 		if (!process_mapping(kname, keydef)) {
-			wait_message(0, _(txt_keymap_invalid_name), kname);
+			error_message(0, _(txt_keymap_invalid_name), kname);
 			prompt_continue();
 			ret = FALSE;
 			continue;
@@ -444,7 +449,7 @@ process_keys(
 		error = FALSE;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if ((wkeydef = char2wchar_t(keydef)) == NULL) {
-			wait_message(1, _(txt_invalid_multibyte_sequence));
+			error_message(1, _(txt_invalid_multibyte_sequence));
 			ret = FALSE;
 
 			keydef = strtok(NULL, KEYSEPS);
@@ -494,7 +499,7 @@ process_keys(
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 					/* FALLTHROUGH */
 				default:
-					wait_message(0, _(txt_keymap_invalid_key), keydef);
+					error_message(0, _(txt_keymap_invalid_key), keydef);
 					ret = FALSE;
 					error = TRUE;
 					break;
@@ -506,7 +511,7 @@ process_keys(
 			if (isdigit(key = keydef[0]))
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			{
-				wait_message(0, _(txt_keymap_invalid_key), keydef);
+				error_message(0, _(txt_keymap_invalid_key), keydef);
 				ret = FALSE;
 				error = TRUE;
 			}
@@ -578,11 +583,26 @@ process_mapping(
 
 				return TRUE;
 			}
+			if (strcmp(keyname, "ConfigResetAttrib") == 0) {
+				process_keys(CONFIG_RESET_ATTRIB, keys, &option_menu_keys);
+
+				return TRUE;
+			}
+			if (strcmp(keyname, "ConfigScopeMenu") == 0) {
+				process_keys(CONFIG_SCOPE_MENU, keys, &option_menu_keys);
+
+				return TRUE;
+			}
 			if (strcmp(keyname, "ConfigSelect") == 0) {
 				process_keys(CONFIG_SELECT, keys, &option_menu_keys);
 
 				return TRUE;
 			}
+			if (strcmp(keyname, "ConfigToggleAttrib") == 0) {
+				process_keys(CONFIG_TOGGLE_ATTRIB, keys, &option_menu_keys);
+
+				return TRUE;
+			}
 			break;
 
 		case 'D':
@@ -599,6 +619,7 @@ process_mapping(
 				process_keys(GLOBAL_LINE_DOWN, keys, &info_keys);
 				process_keys(GLOBAL_LINE_DOWN, keys, &option_menu_keys);
 				process_keys(GLOBAL_LINE_DOWN, keys, &page_keys);
+				process_keys(GLOBAL_LINE_DOWN, keys, &scope_keys);
 				process_keys(GLOBAL_LINE_DOWN, keys, &select_keys);
 				process_keys(GLOBAL_LINE_DOWN, keys, &thread_keys);
 
@@ -633,6 +654,11 @@ process_mapping(
 
 				return TRUE;
 			}
+			if (strcmp(keyname, "FeedRange") == 0) {
+				process_keys(FEED_RANGE, keys, &feed_type_keys);
+
+				return TRUE;
+			}
 			if (strcmp(keyname, "FeedRepost") == 0) {
 				process_keys(FEED_KEY_REPOST, keys, &feed_supersede_article_keys);
 
@@ -668,6 +694,7 @@ process_mapping(
 				process_keys(GLOBAL_FIRST_PAGE, keys, &info_keys);
 				process_keys(GLOBAL_FIRST_PAGE, keys, &option_menu_keys);
 				process_keys(GLOBAL_FIRST_PAGE, keys, &page_keys);
+				process_keys(GLOBAL_FIRST_PAGE, keys, &scope_keys);
 				process_keys(GLOBAL_FIRST_PAGE, keys, &select_keys);
 				process_keys(GLOBAL_FIRST_PAGE, keys, &thread_keys);
 
@@ -681,6 +708,11 @@ process_mapping(
 
 				return TRUE;
 			}
+			if (strcmp(keyname, "GroupCancel") == 0) {
+				process_keys(GROUP_CANCEL, keys, &group_keys);
+
+				return TRUE;
+			}
 			if (strcmp(keyname, "GroupDoAutoSel") == 0) {
 				process_keys(GROUP_DO_AUTOSELECT, keys, &group_keys);
 
@@ -821,7 +853,9 @@ process_mapping(
 		case 'H':
 			if (strcmp(keyname, "Help") == 0) {
 				process_keys(GLOBAL_HELP, keys, &group_keys);
+				process_keys(GLOBAL_HELP, keys, &option_menu_keys);
 				process_keys(GLOBAL_HELP, keys, &page_keys);
+				process_keys(GLOBAL_HELP, keys, &scope_keys);
 				process_keys(GLOBAL_HELP, keys, &select_keys);
 				process_keys(GLOBAL_HELP, keys, &thread_keys);
 
@@ -845,6 +879,7 @@ process_mapping(
 				process_keys(GLOBAL_LAST_PAGE, keys, &info_keys);
 				process_keys(GLOBAL_LAST_PAGE, keys, &option_menu_keys);
 				process_keys(GLOBAL_LAST_PAGE, keys, &page_keys);
+				process_keys(GLOBAL_LAST_PAGE, keys, &scope_keys);
 				process_keys(GLOBAL_LAST_PAGE, keys, &select_keys);
 				process_keys(GLOBAL_LAST_PAGE, keys, &thread_keys);
 
@@ -867,16 +902,6 @@ process_mapping(
 			break;
 
 		case 'M':
-			if (strcmp(keyname, "MarkReadCur") == 0) {
-				process_keys(MARK_READ_CURRENT, keys, &mark_read_keys);
-
-				return TRUE;
-			}
-			if (strcmp(keyname, "MarkReadTag") == 0) {
-				process_keys(MARK_READ_TAGGED, keys, &mark_read_keys);
-
-				return TRUE;
-			}
 			if (strcmp(keyname, "MarkArticleUnread") == 0) {
 				process_keys(MARK_ARTICLE_UNREAD, keys, &group_keys);
 				process_keys(MARK_ARTICLE_UNREAD, keys, &page_keys);
@@ -891,6 +916,18 @@ process_mapping(
 
 				return TRUE;
 			}
+			if (strcmp(keyname, "MarkFeedRead") == 0) {
+				process_keys(MARK_FEED_READ, keys, &group_keys);
+				process_keys(MARK_FEED_READ, keys, &thread_keys);
+
+				return TRUE;
+			}
+			if (strcmp(keyname, "MarkFeedUnread") == 0) {
+				process_keys(MARK_FEED_UNREAD, keys, &group_keys);
+				process_keys(MARK_FEED_UNREAD, keys, &thread_keys);
+
+				return TRUE;
+			}
 			if (strcmp(keyname, "MenuFilterKill") == 0) {
 				process_keys(GLOBAL_MENU_FILTER_KILL, keys, &group_keys);
 				process_keys(GLOBAL_MENU_FILTER_KILL, keys, &page_keys);
@@ -941,21 +978,12 @@ process_mapping(
 				process_keys(GLOBAL_PAGE_DOWN, keys, &info_keys);
 				process_keys(GLOBAL_PAGE_DOWN, keys, &option_menu_keys);
 				process_keys(GLOBAL_PAGE_DOWN, keys, &page_keys);
+				process_keys(GLOBAL_PAGE_DOWN, keys, &scope_keys);
 				process_keys(GLOBAL_PAGE_DOWN, keys, &select_keys);
 				process_keys(GLOBAL_PAGE_DOWN, keys, &thread_keys);
 
 				return TRUE;
 			}
-			if (strcmp(keyname, "PageDown3") == 0) {
-				process_keys(GLOBAL_PAGE_DOWN, keys, &group_keys);
-				process_keys(GLOBAL_PAGE_DOWN, keys, &info_keys);
-				process_keys(GLOBAL_PAGE_DOWN, keys, &option_menu_keys);
-				process_keys(GLOBAL_PAGE_DOWN, keys, &select_keys);
-				process_keys(GLOBAL_PAGE_DOWN, keys, &thread_keys);
-				process_keys(PAGE_PAGE_DOWN3, keys, &page_keys);
-
-				return TRUE;
-			}
 			if (strcmp(keyname, "PageEditArticle") == 0) {
 				process_keys(PAGE_EDIT_ARTICLE, keys, &page_keys);
 
@@ -1128,6 +1156,7 @@ process_mapping(
 				process_keys(GLOBAL_PAGE_UP, keys, &info_keys);
 				process_keys(GLOBAL_PAGE_UP, keys, &option_menu_keys);
 				process_keys(GLOBAL_PAGE_UP, keys, &page_keys);
+				process_keys(GLOBAL_PAGE_UP, keys, &scope_keys);
 				process_keys(GLOBAL_PAGE_UP, keys, &select_keys);
 				process_keys(GLOBAL_PAGE_UP, keys, &thread_keys);
 
@@ -1340,7 +1369,6 @@ process_mapping(
 				process_keys(GLOBAL_QUIT, keys, &filter_keys);
 				process_keys(GLOBAL_QUIT, keys, &group_keys);
 				process_keys(GLOBAL_QUIT, keys, &info_keys);
-				process_keys(GLOBAL_QUIT, keys, &mark_read_keys);
 				process_keys(GLOBAL_QUIT, keys, &option_menu_keys);
 				process_keys(GLOBAL_QUIT, keys, &page_keys);
 #ifdef HAVE_PGP_GPG
@@ -1359,6 +1387,7 @@ process_mapping(
 				process_keys(GLOBAL_QUIT, keys, &post_send_keys);
 				process_keys(GLOBAL_QUIT, keys, &prompt_keys);
 				process_keys(GLOBAL_QUIT, keys, &save_append_overwrite_keys);
+				process_keys(GLOBAL_QUIT, keys, &scope_keys);
 				process_keys(GLOBAL_QUIT, keys, &select_keys);
 				process_keys(GLOBAL_QUIT, keys, &thread_keys);
 
@@ -1379,6 +1408,7 @@ process_mapping(
 				process_keys(GLOBAL_REDRAW_SCREEN, keys, &group_keys);
 				process_keys(GLOBAL_REDRAW_SCREEN, keys, &option_menu_keys);
 				process_keys(GLOBAL_REDRAW_SCREEN, keys, &page_keys);
+				process_keys(GLOBAL_REDRAW_SCREEN, keys, &scope_keys);
 				process_keys(GLOBAL_REDRAW_SCREEN, keys, &select_keys);
 				process_keys(GLOBAL_REDRAW_SCREEN, keys, &thread_keys);
 
@@ -1397,9 +1427,40 @@ process_mapping(
 
 				return TRUE;
 			}
+			if (strcmp(keyname, "ScopeAdd") == 0) {
+				process_keys(SCOPE_ADD, keys, &scope_keys);
+
+				return TRUE;
+			}
+			if (strcmp(keyname, "ScopeDelete") == 0) {
+				process_keys(SCOPE_DELETE, keys, &scope_keys);
+
+				return TRUE;
+			}
+			if (strcmp(keyname, "ScopeEditAttributesFile") == 0) {
+				process_keys(SCOPE_EDIT_ATTRIBUTES_FILE, keys, &scope_keys);
+
+				return TRUE;
+			}
+			if (strcmp(keyname, "ScopeMove") == 0) {
+				process_keys(SCOPE_MOVE, keys, &scope_keys);
+
+				return TRUE;
+			}
+			if (strcmp(keyname, "ScopeRename") == 0) {
+				process_keys(SCOPE_RENAME, keys, &scope_keys);
+
+				return TRUE;
+			}
+			if (strcmp(keyname, "ScopeSelect") == 0) {
+				process_keys(SCOPE_SELECT, keys, &scope_keys);
+
+				return TRUE;
+			}
 			if (strcmp(keyname, "ScrollDown") == 0) {
 				process_keys(GLOBAL_SCROLL_DOWN, keys, &group_keys);
 				process_keys(GLOBAL_SCROLL_DOWN, keys, &option_menu_keys);
+				process_keys(GLOBAL_SCROLL_DOWN, keys, &scope_keys);
 				process_keys(GLOBAL_SCROLL_DOWN, keys, &select_keys);
 				process_keys(GLOBAL_SCROLL_DOWN, keys, &thread_keys);
 
@@ -1408,6 +1469,7 @@ process_mapping(
 			if (strcmp(keyname, "ScrollUp") == 0) {
 				process_keys(GLOBAL_SCROLL_UP, keys, &group_keys);
 				process_keys(GLOBAL_SCROLL_UP, keys, &option_menu_keys);
+				process_keys(GLOBAL_SCROLL_UP, keys, &scope_keys);
 				process_keys(GLOBAL_SCROLL_UP, keys, &select_keys);
 				process_keys(GLOBAL_SCROLL_UP, keys, &thread_keys);
 
@@ -1559,7 +1621,9 @@ process_mapping(
 			if (strcmp(keyname, "ShellEscape") == 0) {
 #ifndef NO_SHELL_ESCAPE
 				process_keys(GLOBAL_SHELL_ESCAPE, keys, &group_keys);
+				process_keys(GLOBAL_SHELL_ESCAPE, keys, &option_menu_keys);
 				process_keys(GLOBAL_SHELL_ESCAPE, keys, &page_keys);
+				process_keys(GLOBAL_SHELL_ESCAPE, keys, &scope_keys);
 				process_keys(GLOBAL_SHELL_ESCAPE, keys, &select_keys);
 				process_keys(GLOBAL_SHELL_ESCAPE, keys, &thread_keys);
 #endif /* !NO_SHELL_ESCAPE */
@@ -1574,6 +1638,11 @@ process_mapping(
 
 				return TRUE;
 			}
+			if (strcmp(keyname, "ThreadCancel") == 0) {
+				process_keys(THREAD_CANCEL, keys, &thread_keys);
+
+				return TRUE;
+			}
 			if (strcmp(keyname, "ThreadMail") == 0) {
 				process_keys(THREAD_MAIL, keys, &thread_keys);
 
@@ -1648,6 +1717,7 @@ process_mapping(
 				process_keys(GLOBAL_TOGGLE_HELP_DISPLAY, keys, &group_keys);
 				process_keys(GLOBAL_TOGGLE_HELP_DISPLAY, keys, &info_keys);
 				process_keys(GLOBAL_TOGGLE_HELP_DISPLAY, keys, &page_keys);
+				process_keys(GLOBAL_TOGGLE_HELP_DISPLAY, keys, &scope_keys);
 				process_keys(GLOBAL_TOGGLE_HELP_DISPLAY, keys, &select_keys);
 				process_keys(GLOBAL_TOGGLE_HELP_DISPLAY, keys, &thread_keys);
 
@@ -1677,6 +1747,7 @@ process_mapping(
 				process_keys(GLOBAL_LINE_UP, keys, &info_keys);
 				process_keys(GLOBAL_LINE_UP, keys, &option_menu_keys);
 				process_keys(GLOBAL_LINE_UP, keys, &page_keys);
+				process_keys(GLOBAL_LINE_UP, keys, &scope_keys);
 				process_keys(GLOBAL_LINE_UP, keys, &select_keys);
 				process_keys(GLOBAL_LINE_UP, keys, &thread_keys);
 
@@ -1713,7 +1784,7 @@ upgrade_keymap_file(
 	FILE *oldfp, *newfp;
 	char *line, *backup;
 	const char *keyname, *keydef;
-	char new[NAME_LEN + 1], buf[LEN];
+	char newk[NAME_LEN + 1], buf[LEN];
 	char *bugreport[3] = { NULL, NULL, NULL };
 	char *catchup[4] = { NULL, NULL, NULL, NULL };
 	char *catchup_next_unread[4] = { NULL, NULL, NULL, NULL };
@@ -1725,7 +1796,7 @@ upgrade_keymap_file(
 	char *mark_thread_unread[3] = { NULL, NULL, NULL };
 	char *menu_filter_kill[3] = { NULL, NULL, NULL };
 	char *menu_filter_select[3] = { NULL, NULL, NULL };
-	char *pagedown[2] = { NULL, NULL };
+	char *pagedown[3] = { NULL, NULL, NULL };
 	char *pagenextthd[2] = { NULL, NULL };
 	char *pageup[3] = { NULL, NULL, NULL };
 	char *postponed[2] = { NULL, NULL };
@@ -1742,8 +1813,8 @@ upgrade_keymap_file(
 	if ((oldfp = fopen(old, "r")) == NULL)
 		return;
 
-	snprintf(new, sizeof(new), "%s.%d", old, (int) process_id);
-	if ((newfp = fopen(new, "w")) == NULL) {
+	snprintf(newk, sizeof(newk), "%s.%ld", old, (long) process_id);
+	if ((newfp = fopen(newk, "w")) == NULL) {
 		fclose(oldfp);
 		return;
 	}
@@ -1787,15 +1858,18 @@ upgrade_keymap_file(
 				FreeAndNull(groupreadbasenote[0]);
 				FreeAndNull(groupreadbasenote[1]);
 			}
-			if (pagedown[0] || pagedown[1]) {
+			if (pagedown[0] || pagedown[1] || pagedown[2]) {
 				fprintf(newfp, "PageDown\t\t");
 				if (pagedown[0])
 					fprintf(newfp, "\t%s", pagedown[0]);
 				if (pagedown[1])
 					fprintf(newfp, "\t%s", pagedown[1]);
+				if (pagedown[2])
+					fprintf(newfp, "\t%s", pagedown[2]);
 				fprintf(newfp, "\n");
 				FreeAndNull(pagedown[0]);
 				FreeAndNull(pagedown[1]);
+				FreeAndNull(pagedown[2]);
 			}
 			if (pagenextthd[0] || pagenextthd[1]) {
 				fprintf(newfp, "PageNextThd\t\t");
@@ -2014,6 +2088,8 @@ upgrade_keymap_file(
 					pagedown[0] = my_strdup(keydef);
 				else if (strcmp(keyname, "PageDown2") == 0)
 					pagedown[1] = my_strdup(keydef);
+				else if (strcmp(keyname, "PageDown3") == 0)
+					pagedown[2] = my_strdup(keydef);
 				else if (strcmp(keyname, "PageEditFilter") == 0)
 					edit_filter[1] = my_strdup(keydef);
 				else if (strcmp(keyname, "PageNextThd") == 0)
@@ -2049,8 +2125,8 @@ upgrade_keymap_file(
 				else if (strcmp(keyname, "PageCatchupNextUnread") == 0)
 					catchup_next_unread[1] = my_strdup(keydef);
 				else if (strcmp(keyname, "PromptNo") == 0 || strcmp(keyname, "PromptYes") == 0) {
-					if (strlen(keydef) == 1 && islower(keydef[0]))
-						fprintf(newfp, "%s\t\t\t%c\t%c\n", keyname, keydef[0], toupper(keydef[0]));
+					if (strlen(keydef) == 1 && islower((int)(unsigned char) keydef[0]))
+						fprintf(newfp, "%s\t\t\t%c\t%c\n", keyname, keydef[0], toupper((int)(unsigned char) keydef[0]));
 					else
 						fprintf(newfp, "%s", backup);
 				} else
@@ -2307,7 +2383,7 @@ upgrade_keymap_file(
 
 	fclose(oldfp);
 	fclose(newfp);
-	rename(new, old);
+	rename(newk, old);
 	wait_message(0, _(txt_keymap_upgraded), KEYMAP_VERSION);
 	prompt_continue();
 
@@ -2322,9 +2398,40 @@ void
 setup_default_keys(
 	void)
 {
+	/* scope level */
+	add_default_key(&scope_keys, "1", DIGIT_1);
+	add_default_key(&scope_keys, "2", DIGIT_2);
+	add_default_key(&scope_keys, "3", DIGIT_3);
+	add_default_key(&scope_keys, "4", DIGIT_4);
+	add_default_key(&scope_keys, "5", DIGIT_5);
+	add_default_key(&scope_keys, "6", DIGIT_6);
+	add_default_key(&scope_keys, "7", DIGIT_7);
+	add_default_key(&scope_keys, "8", DIGIT_8);
+	add_default_key(&scope_keys, "9", DIGIT_9);
+	add_default_key(&scope_keys, "a", SCOPE_ADD);
+	add_default_key(&scope_keys, "b", GLOBAL_PAGE_UP);
+	add_default_key(&scope_keys, " ", GLOBAL_PAGE_DOWN);
+	add_default_key(&scope_keys, "d", SCOPE_DELETE);
+	add_default_key(&scope_keys, "h", GLOBAL_HELP);
+	add_default_key(&scope_keys, "\n\r", SCOPE_SELECT);
+	add_default_key(&scope_keys, "E", SCOPE_EDIT_ATTRIBUTES_FILE);
+	add_default_key(&scope_keys, "H", GLOBAL_TOGGLE_HELP_DISPLAY);
+	add_default_key(&scope_keys, "", GLOBAL_REDRAW_SCREEN);
+	add_default_key(&scope_keys, "m", SCOPE_MOVE);
+	add_default_key(&scope_keys, "j", GLOBAL_LINE_DOWN);
+	add_default_key(&scope_keys, "k", GLOBAL_LINE_UP);
+	add_default_key(&scope_keys, "g^", GLOBAL_FIRST_PAGE);
+	add_default_key(&scope_keys, "G$", GLOBAL_LAST_PAGE);
+	add_default_key(&scope_keys, "q", GLOBAL_QUIT);
+	add_default_key(&scope_keys, "r", SCOPE_RENAME);
+	add_default_key(&scope_keys, ">", GLOBAL_SCROLL_DOWN);
+	add_default_key(&scope_keys, "<", GLOBAL_SCROLL_UP);
+#ifndef NO_SHELL_ESCAPE
+	add_default_key(&scope_keys, "!", GLOBAL_SHELL_ESCAPE);
+#endif /* !NO_SHELL_ESCAPE */
+
 	/* select level */
 	add_global_keys(&select_keys);
-	add_default_key(&select_keys, " ", GLOBAL_PAGE_DOWN);
 	add_default_key(&select_keys, "\n\r", SELECT_ENTER_GROUP);
 	add_default_key(&select_keys, "", SELECT_RESET_NEWSRC);
 	add_default_key(&select_keys, "c", CATCHUP);
@@ -2351,10 +2458,11 @@ setup_default_keys(
 
 	/* group level */
 	add_global_keys(&group_keys);
-	add_default_key(&group_keys, " ", GLOBAL_PAGE_DOWN);
 	add_default_key(&group_keys, "", GLOBAL_MENU_FILTER_SELECT);
 	add_default_key(&group_keys, "\n\r", GROUP_READ_BASENOTE);
 	add_default_key(&group_keys, "", GLOBAL_MENU_FILTER_KILL);
+	add_default_key(&group_keys, "", MARK_FEED_READ);
+	add_default_key(&group_keys, "", MARK_FEED_UNREAD);
 	add_default_key(&group_keys, "a", GLOBAL_SEARCH_AUTHOR_FORWARD);
 	add_default_key(&group_keys, "c", CATCHUP);
 	add_default_key(&group_keys, "d", GROUP_TOGGLE_SUBJECT_DISPLAY);
@@ -2375,6 +2483,7 @@ setup_default_keys(
 	add_default_key(&group_keys, "A", GLOBAL_SEARCH_AUTHOR_BACKWARD);
 	add_default_key(&group_keys, "B", GLOBAL_SEARCH_BODY);
 	add_default_key(&group_keys, "C", CATCHUP_NEXT_UNREAD);
+	add_default_key(&group_keys, "D", GROUP_CANCEL);
 	add_default_key(&group_keys, "E", GLOBAL_EDIT_FILTER);
 	add_default_key(&group_keys, "G", GROUP_TOGGLE_GET_ARTICLES_LIMIT);
 	add_default_key(&group_keys, "K", GROUP_MARK_THREAD_READ);
@@ -2403,9 +2512,10 @@ setup_default_keys(
 
 	/* thread keys */
 	add_global_keys(&thread_keys);
-	add_default_key(&thread_keys, " ", GLOBAL_PAGE_DOWN);
 	add_default_key(&thread_keys, "", GLOBAL_MENU_FILTER_SELECT);
 	add_default_key(&thread_keys, "", GLOBAL_MENU_FILTER_KILL);
+	add_default_key(&thread_keys, "", MARK_FEED_READ);
+	add_default_key(&thread_keys, "", MARK_FEED_UNREAD);
 	add_default_key(&thread_keys, "\n\r", THREAD_READ_ARTICLE);
 	add_default_key(&thread_keys, "a", GLOBAL_SEARCH_AUTHOR_FORWARD);
 	add_default_key(&thread_keys, "c", CATCHUP);
@@ -2420,6 +2530,7 @@ setup_default_keys(
 	add_default_key(&thread_keys, "A", GLOBAL_SEARCH_AUTHOR_BACKWARD);
 	add_default_key(&thread_keys, "B", GLOBAL_SEARCH_BODY);
 	add_default_key(&thread_keys, "C", CATCHUP_NEXT_UNREAD);
+	add_default_key(&thread_keys, "D", THREAD_CANCEL);
 	add_default_key(&thread_keys, "E", GLOBAL_EDIT_FILTER);
 	add_default_key(&thread_keys, "K", THREAD_MARK_ARTICLE_READ);
 	add_default_key(&thread_keys, "L", GLOBAL_LOOKUP_MESSAGEID);
@@ -2438,7 +2549,6 @@ setup_default_keys(
 
 	/* page level */
 	add_global_keys(&page_keys);
-	add_default_key(&page_keys, "", GLOBAL_PAGE_DOWN);
 	add_default_key(&page_keys, "", GLOBAL_MENU_FILTER_SELECT);
 	add_default_key(&page_keys, "", PAGE_REPLY_QUOTE_HEADERS);
 #ifdef HAVE_PGP_GPG
@@ -2485,7 +2595,6 @@ setup_default_keys(
 	add_default_key(&page_keys, "V", PAGE_VIEW_ATTACHMENTS);
 	add_default_key(&page_keys, "Z", MARK_THREAD_UNREAD);
 	add_default_key(&page_keys, "\t", PAGE_NEXT_UNREAD);
-	add_default_key(&page_keys, " ", PAGE_PAGE_DOWN3);
 	add_default_key(&page_keys, "-", GLOBAL_LAST_VIEWED);
 	add_default_key(&page_keys, "|", GLOBAL_PIPE);
 	add_default_key(&page_keys, "<", PAGE_TOP_THREAD);
@@ -2526,19 +2635,26 @@ setup_default_keys(
 	add_default_key(&option_menu_keys, "b", GLOBAL_PAGE_UP);
 	add_default_key(&option_menu_keys, " ", GLOBAL_PAGE_DOWN);
 	add_default_key(&option_menu_keys, "\n\r", CONFIG_SELECT);
+	add_default_key(&option_menu_keys, "\t", CONFIG_TOGGLE_ATTRIB);
 	add_default_key(&option_menu_keys, "", GLOBAL_REDRAW_SCREEN);
 	add_default_key(&option_menu_keys, "j", GLOBAL_LINE_DOWN);
 	add_default_key(&option_menu_keys, "k", GLOBAL_LINE_UP);
 	add_default_key(&option_menu_keys, "g^", GLOBAL_FIRST_PAGE);
 	add_default_key(&option_menu_keys, "G$", GLOBAL_LAST_PAGE);
+	add_default_key(&option_menu_keys, "h", GLOBAL_HELP);
 	add_default_key(&option_menu_keys, "q", GLOBAL_QUIT);
+	add_default_key(&option_menu_keys, "r", CONFIG_RESET_ATTRIB);
 	add_default_key(&option_menu_keys, "v", GLOBAL_VERSION);
 	add_default_key(&option_menu_keys, "Q", CONFIG_NO_SAVE);
+	add_default_key(&option_menu_keys, "S", CONFIG_SCOPE_MENU);
 	add_default_key(&option_menu_keys, ">", GLOBAL_SCROLL_DOWN);
 	add_default_key(&option_menu_keys, "<", GLOBAL_SCROLL_UP);
 	add_default_key(&option_menu_keys, "/", GLOBAL_SEARCH_SUBJECT_FORWARD);
 	add_default_key(&option_menu_keys, "?", GLOBAL_SEARCH_SUBJECT_BACKWARD);
 	add_default_key(&option_menu_keys, "\\", GLOBAL_SEARCH_REPEAT);
+#ifndef NO_SHELL_ESCAPE
+	add_default_key(&option_menu_keys, "!", GLOBAL_SHELL_ESCAPE);
+#endif /* !NO_SHELL_ESCAPE */
 
 	/* prompt keys */
 	add_default_key(&prompt_keys, "", GLOBAL_ABORT);
@@ -2624,6 +2740,7 @@ setup_default_keys(
 	add_default_key(&feed_type_keys, "a", FEED_ARTICLE);
 	add_default_key(&feed_type_keys, "h", FEED_HOT);
 	add_default_key(&feed_type_keys, "p", FEED_PATTERN);
+	add_default_key(&feed_type_keys, "r", FEED_RANGE);
 	add_default_key(&feed_type_keys, "q", GLOBAL_QUIT);
 	add_default_key(&feed_type_keys, "t", FEED_THREAD);
 	add_default_key(&feed_type_keys, "T", FEED_TAGGED);
@@ -2639,12 +2756,6 @@ setup_default_keys(
 	add_default_key(&filter_keys, "q", GLOBAL_QUIT);
 	add_default_key(&filter_keys, "s", FILTER_SAVE);
 
-	/* mark read */
-	add_default_key(&mark_read_keys, "", GLOBAL_ABORT);
-	add_default_key(&mark_read_keys, "c", MARK_READ_CURRENT);
-	add_default_key(&mark_read_keys, "q", GLOBAL_QUIT);
-	add_default_key(&mark_read_keys, "t", MARK_READ_TAGGED);
-
 #ifdef HAVE_PGP_GPG
 	/* pgp mail */
 	add_default_key(&pgp_mail_keys, "", GLOBAL_ABORT);
@@ -2688,6 +2799,7 @@ add_global_keys(
 	add_default_key(keys, "8", DIGIT_8);
 	add_default_key(keys, "9", DIGIT_9);
 	add_default_key(keys, "b", GLOBAL_PAGE_UP);
+	add_default_key(keys, " ", GLOBAL_PAGE_DOWN);
 	add_default_key(keys, "", GLOBAL_REDRAW_SCREEN);
 	add_default_key(keys, "j", GLOBAL_LINE_DOWN);
 	add_default_key(keys, "k", GLOBAL_LINE_UP);
@@ -2711,7 +2823,7 @@ add_global_keys(
 	add_default_key(keys, "#", GLOBAL_SET_RANGE);
 #ifndef NO_SHELL_ESCAPE
 	add_default_key(keys, "!", GLOBAL_SHELL_ESCAPE);
-#endif /* NO_SHELL_ESCAPE */
+#endif /* !NO_SHELL_ESCAPE */
 #ifdef HAVE_COLOR
 	add_default_key(keys, "&", GLOBAL_TOGGLE_COLOR);
 #endif /* HAVE COLOR */
