@@ -3,10 +3,10 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2005-08-14
+ *  Updated   : 2010-03-07
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,7 @@ static t_bool start_any_unread = FALSE;	/* only start if unread news */
 /*
  * Local prototypes
  */
-static t_bool create_mail_save_dirs(void);
+static void create_mail_save_dirs(void);
 static void read_cmd_line_options(int argc, char *argv[]);
 static void show_intro_page(void);
 static void update_index_files(void);
@@ -78,7 +78,7 @@ main(
 	char *argv[])
 {
 	int count;
-	int num_cmd_line_groups;
+	int num_cmd_line_groups = 0;
 	int start_groupnum = 0;
 	t_bool tmp_no_write;
 
@@ -91,20 +91,34 @@ main(
 		bindtextdomain(PACKAGE, LOCALEDIR);
 		textdomain(PACKAGE);
 #	endif /* ENABLE_NLS */
-	} else {
-		error_message(txt_error_locale);
-		sleep(2);
-	}
+	} else
+		error_message(4, txt_error_locale);
 #endif /* HAVE_SETLOCALE && !NO_LOCALE */
 
+	/*
+	 * determine local charset
+	 */
+#ifndef NO_LOCALE
+	{
+		const char *p;
+
+		if ((p = tin_nl_langinfo(CODESET)) != NULL) {
+			if (!strcasecmp(p, "ANSI_X3.4-1968"))
+				STRCPY(tinrc.mm_local_charset, "US-ASCII");
+			else
+				STRCPY(tinrc.mm_local_charset, p);
+		}
+	}
+#endif /* !NO_LOCALE */
+	/* always set a default value */
+	if (!*tinrc.mm_local_charset)
+		STRCPY(tinrc.mm_local_charset, "US-ASCII");
+
 	set_signal_handlers();
 
 	debug = 0;	/* debug OFF */
 
 	base_name(argv[0], tin_progname);
-#ifdef VMS
-	argv[0] = tin_progname;
-#endif /* VMS */
 
 #ifdef NNTP_ONLY
 	read_news_via_nntp = TRUE;
@@ -116,7 +130,7 @@ main(
 #	ifdef NNTP_ABLE
 		read_news_via_nntp = TRUE;
 #	else
-		error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
+		error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
 		giveup();
 #	endif /* NNTP_ABLE */
 	}
@@ -131,19 +145,21 @@ main(
 	init_group_hash();
 	setup_default_keys(); /* preinit keybindings */
 
-	/*
-	 * Read user local & global config files
-	 * These override the compiled in defaults
-	 */
-	read_config_file(global_config_file, TRUE);
-	read_config_file(local_config_file, FALSE);
-
 	/*
 	 * Process envargs & command line options
 	 * These override the configured in values
 	 */
 	read_cmd_line_options(argc, argv);
 
+	/*
+	 * Read user local & global config files
+	 * These override the compiled in defaults
+	 *
+	 * must be called before setup_screen()
+	 */
+	read_config_file(global_config_file, TRUE);
+	read_config_file(local_config_file, FALSE);
+
 	tmp_no_write = no_write; /* keep no_write */
 	no_write = TRUE;		/* don't allow any writing back during startup */
 
@@ -151,7 +167,7 @@ main(
 #ifdef M_UNIX
 #	ifndef USE_CURSES
 		if (!get_termcaps()) {
-			error_message(_(txt_screen_init_failed), tin_progname);
+			error_message(2, _(txt_screen_init_failed), tin_progname);
 			giveup();
 		}
 #	endif /* !USE_CURSES */
@@ -161,7 +177,7 @@ main(
 		 * Init curses emulation
 		 */
 		if (!InitScreen()) {
-			error_message(_(txt_screen_init_failed), tin_progname);
+			error_message(2, _(txt_screen_init_failed), tin_progname);
 			giveup();
 		}
 
@@ -188,30 +204,24 @@ main(
 	/*
 	 * exit early - unfortunately we can't do that in read_cmd_line_options()
 	 * as nntp_caps.over_cmd is set in nntp_open()
+	 *
+	 * TODO: does the logic make sense? what
+	 * if (update_index && !nntp_caps.over_cmd && !tinrc.cache_overview_files)
+	 * no error message? why?
 	 */
 	if (update_index && nntp_caps.over_cmd && !tinrc.cache_overview_files) {
-		error_message(_(txt_batch_update_unavail), tin_progname);
+		error_message(2, _(txt_batch_update_unavail), tin_progname);
 		giveup();
 	}
 
 	/*
 	 * Check if overview indexes contain Xref: lines
 	 */
-	if (nntp_caps.over_cmd)
+#ifdef NNTP_ABLE
+	if ((read_news_via_nntp && nntp_caps.over_cmd) || !read_news_via_nntp)
+#endif /* NNTP_ABLE */
 		xref_supported = overview_xref_support();
 
-#ifdef DEBUG_NEWSRC
-	{
-		char file[PATH_LEN];
-
-		joinpath(file, TMPDIR, "BITMAP");
-		unlink(file);
-#	if 0
-		newsrc_test_harness();
-#	endif /* 0 */
-	}
-#endif /* DEBUG_NEWSRC */
-
 	/*
 	 * avoid empty regexp, we also need to do this in batch_mode
 	 * as read_overview() calls eat_re() which uses a regexp to
@@ -246,7 +256,7 @@ main(
 
 	/*
 	 * Initialise active[] and add new newsgroups to start of my_group[]
-	 * also reads attributes
+	 * also reads global/local attributes
 	 */
 	selmenu.max = 0;
 	/*
@@ -254,7 +264,9 @@ main(
 	 * $AUTOSUBSCRIBE groups
 	 */
 	no_write = tmp_no_write;
-	read_news_active_file();
+	read_attributes_file(TRUE);
+	read_attributes_file(FALSE);
+	start_groupnum = read_news_active_file();
 #ifdef DEBUG
 	debug_print_active();
 #endif /* DEBUG */
@@ -263,13 +275,20 @@ main(
 	 * Read in users filter preferences file. This has to be done before
 	 * quick post because the filters might be updated.
 	 */
-	filtered_articles = read_filter_file(filter_file);
+	read_filter_file(filter_file);
 
 	no_write = TRUE;
 #ifdef DEBUG
 	debug_print_filters();
 #endif /* DEBUG */
 
+	/*
+	 * Preloads active[] with command line groups. They will follow any
+	 * new newsgroups
+	 */
+	if (!post_postponed_and_exit)
+		num_cmd_line_groups = read_cmd_line_groups();
+
 	/*
 	 * Quick post an article and exit if -w or -o specified
 	 */
@@ -294,19 +313,13 @@ main(
 		no_write = TRUE; /* disable newsrc updates */
 	}
 
-	/*
-	 * TODO: what has write_config_file() to do with create_mail_save_dirs ()
-	 */
-	if (create_mail_save_dirs())
+	/* what about "if (!no_write)" here? */
+	create_mail_save_dirs();
+	if (created_rcdir) /* first start */
 		write_config_file(local_config_file);
 
-	/*
-	 * Preloads active[] with command line groups. They will follow any
-	 * new newsgroups
-	 */
-	num_cmd_line_groups = read_cmd_line_groups();
-
-	backup_newsrc();
+	if (!tmp_no_write)	/* do not (over)write oldnewsrc with -X */
+		backup_newsrc();
 
 	/*
 	 * Load my_groups[] from the .newsrc file. We append these groups to any
@@ -350,7 +363,7 @@ main(
 	 * Save any new articles to savedir structure for later reading
 	 *
 	 * TODO: should we temporarely set
-	 *       getart_limit=-1,thread_arts=0,sort_art_type=0
+	 *       getart_limit=-1,thread_articles=0,sort_article_type=0
 	 *       for speed reasons?
 	 */
 	if (mail_news || save_news) {
@@ -420,9 +433,9 @@ read_cmd_line_options(
 		switch (ch) {
 			case 'a':
 #ifdef HAVE_COLOR
-				use_color = bool_not(use_color);
+				cmdline.args |= CMDLINE_USE_COLOR;
 #else
-				error_message(_(txt_option_not_enabled), "-DHAVE_COLOR");
+				error_message(2, _(txt_option_not_enabled), "-DHAVE_COLOR");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -433,7 +446,7 @@ read_cmd_line_options(
 #ifdef NNTP_ABLE
 				force_auth_on_conn_open = TRUE;
 #else
-				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -449,12 +462,12 @@ read_cmd_line_options(
 				show_description = FALSE;
 				break;
 
-			case 'D':		/* debug mode 1=NNTP, 2=ALL, 3=newsrc, 4=malloc */
+			case 'D':		/* debug mode */
 #ifdef DEBUG
 				debug = atoi(optarg);
 				debug_delete_files();
 #else
-				error_message(_(txt_option_not_enabled), "-DDEBUG");
+				error_message(2, _(txt_option_not_enabled), "-DDEBUG");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -467,15 +480,17 @@ read_cmd_line_options(
 				break;
 
 			case 'G':
-				tinrc.getart_limit = atoi(optarg);
+				cmdline.getart_limit = atoi(optarg);
+				cmdline.args |= CMDLINE_GETART_LIMIT;
 				break;
 
 			case 'g':	/* select alternative NNTP-server, implies -r */
 #ifdef NNTP_ABLE
-				my_strncpy(cmdline_nntpserver, optarg, sizeof(cmdline_nntpserver) - 1);
+				my_strncpy(cmdline.nntpserver, optarg, sizeof(cmdline.nntpserver) - 1);
+				cmdline.args |= CMDLINE_NNTPSERVER;
 				read_news_via_nntp = TRUE;
 #else
-				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -497,7 +512,8 @@ read_cmd_line_options(
 				break;
 
 			case 'm':
-				my_strncpy(tinrc.maildir, optarg, sizeof(tinrc.maildir) - 1);
+				my_strncpy(cmdline.maildir, optarg, sizeof(cmdline.maildir) - 1);
+				cmdline.args |= CMDLINE_MAILDIR;
 				break;
 
 			case 'M':	/* mail new news to specified user */
@@ -525,7 +541,7 @@ read_cmd_line_options(
 				post_postponed_and_exit = TRUE;
 				check_for_new_newsgroups = FALSE;
 #else
-				error_message(_(txt_option_not_enabled), "-UNO_POSTING");
+				error_message(2, _(txt_option_not_enabled), "-UNO_POSTING");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -538,7 +554,7 @@ read_cmd_line_options(
 				if (atoi(optarg) != 0)
 					nntp_tcp_port = (unsigned short) atoi(optarg);
 #else
-				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -559,7 +575,7 @@ read_cmd_line_options(
 #ifdef NNTP_ABLE
 				read_news_via_nntp = TRUE;
 #else
-				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -575,7 +591,8 @@ read_cmd_line_options(
 				break;
 
 			case 's':
-				my_strncpy(tinrc.savedir, optarg, sizeof(tinrc.savedir) - 1);
+				my_strncpy(cmdline.savedir, optarg, sizeof(cmdline.savedir) - 1);
+				cmdline.args |= CMDLINE_SAVEDIR;
 				break;
 
 			case 'S':	/* save new news to dir structure */
@@ -603,7 +620,7 @@ read_cmd_line_options(
 				post_article_and_exit = TRUE;
 				check_for_new_newsgroups = FALSE;
 #else
-				error_message(_(txt_option_not_enabled), "-UNO_POSTING");
+				error_message(2, _(txt_option_not_enabled), "-UNO_POSTING");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -644,24 +661,24 @@ read_cmd_line_options(
 	max_cmdargs = argc;
 	if (!newsrc_set) {
 		if (read_news_via_nntp)
-			get_newsrcname(newsrc, getserverbyfile(NNTP_SERVER_FILE));
+			get_newsrcname(newsrc, sizeof(newsrc), getserverbyfile(NNTP_SERVER_FILE));
 		else {
 #if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
 			struct utsname uts;
 			(void) uname(&uts);
-			get_newsrcname(newsrc, uts.nodename);
+			get_newsrcname(newsrc, sizeof(newsrc), uts.nodename);
 #else
 			char nodenamebuf[256]; /* SUSv2 limit; better use HOST_NAME_MAX */
 #ifdef HAVE_GETHOSTNAME
 			(void) gethostname(nodenamebuf, sizeof(nodenamebuf));
 #endif /* HAVE_GETHOSTNAME */
-			get_newsrcname(newsrc, nodenamebuf);
+			get_newsrcname(newsrc, sizeof(newsrc), nodenamebuf);
 #endif /* HAVE_SYS_UTSNAME_H && HAVE_UNAME */
 		}
 	}
 
 	/*
-	 * Sort out conflicts of options....
+	 * Sort out option conflicts
 	 */
 	if (!batch_mode) {
 		if (verbose) {
@@ -678,6 +695,58 @@ read_cmd_line_options(
 			read_saved_news = FALSE;
 		}
 	}
+	if (post_postponed_and_exit && force_no_post) {
+		wait_message(2, _(txt_useless_combination), "-o", "-x", "-x");
+		force_no_post = FALSE;
+	}
+	if (post_article_and_exit && force_no_post) {
+		wait_message(2, _(txt_useless_combination), "-w", "-x", "-x");
+		force_no_post = FALSE;
+	}
+	if (catchup && start_any_unread) {
+		wait_message(2, _(txt_useless_combination), "-c", "-z", "-c");
+		catchup = FALSE;
+	}
+	if (catchup && no_write) {
+		wait_message(2, _(txt_useless_combination), "-c", "-X", "-c");
+		catchup = FALSE;
+	}
+	if (catchup && check_any_unread) {
+		wait_message(2, _(txt_useless_combination), "-c", "-Z", "-c");
+		catchup = FALSE;
+	}
+	if (newsrc_active && read_saved_news) {
+		wait_message(2, _(txt_useless_combination), "-n", "-R", "-n");
+		newsrc_active = read_news_via_nntp = FALSE;
+	}
+	if (start_any_unread && save_news) {
+		wait_message(2, _(txt_useless_combination), "-z", "-S", "-z");
+		start_any_unread = FALSE;
+	}
+	if (save_news && check_any_unread) {
+		wait_message(2, _(txt_useless_combination), "-S", "-Z", "-S");
+		save_news = FALSE;
+	}
+	if (start_any_unread && check_any_unread) {
+		wait_message(2, _(txt_useless_combination), "-Z", "-z", "-Z");
+		check_any_unread = FALSE;
+	}
+	if (mail_news || save_news || update_index || check_any_unread || catchup)
+		batch_mode = TRUE;
+	else
+		batch_mode = FALSE;
+	if (batch_mode && (post_article_and_exit || post_postponed_and_exit))
+		batch_mode = FALSE;
+
+	/*
+	 * When updating index files set getart_limit to 0 in order to get overview
+	 * information for all article; this overwrites '-G limit' and disables
+	 * tinrc.getart_limit temporary
+	 */
+	if (update_index) {
+		cmdline.getart_limit = 0;
+		cmdline.args |= CMDLINE_GETART_LIMIT;
+	}
 #ifdef NNTP_ABLE
 	/*
 	 * If we're reading from an NNTP server and we've been asked not to look
@@ -702,64 +771,68 @@ static void
 usage(
 	char *theProgname)
 {
-	error_message(_(txt_usage_tin), theProgname);
+	error_message(2, _(txt_usage_tin), theProgname);
 
 #ifdef HAVE_COLOR
-		error_message(_(txt_usage_toggle_color));
+		error_message(2, _(txt_usage_toggle_color));
 #endif /* HAVE_COLOR */
 #ifdef NNTP_ABLE
-		error_message(_(txt_usage_force_authentication));
+		error_message(2, _(txt_usage_force_authentication));
 #endif /* NNTP_ABLE */
 
-	error_message(_(txt_usage_catchup));
-	error_message(_(txt_usage_dont_show_descriptions));
+	error_message(2, _(txt_usage_catchup));
+	error_message(2, _(txt_usage_dont_show_descriptions));
 
 #ifdef DEBUG
-	error_message(_(txt_usage_debug));
+	error_message(2, _(txt_usage_debug));
 #endif /* DEBUG */
 
-	error_message(_(txt_usage_newsrc_file), newsrc);
-	error_message(_(txt_usage_getart_limit));
+	error_message(2, _(txt_usage_newsrc_file), newsrc);
+	error_message(2, _(txt_usage_getart_limit));
 
 #ifdef NNTP_ABLE
-	error_message(_(txt_usage_newsserver), get_val("NNTPSERVER", NNTP_DEFAULT_SERVER));
+#	ifdef NNTP_DEFAULT_SERVER
+	error_message(2, _(txt_usage_newsserver), get_val("NNTPSERVER", NNTP_DEFAULT_SERVER));
+#	else
+	error_message(2, _(txt_usage_newsserver), get_val("NNTPSERVER", "news"));
+#	endif /* NNTP_DEFAULT_SERVER */
 #endif /* NNTP_ABLE */
 
-	error_message(_(txt_usage_help_message));
-	error_message(_(txt_usage_help_information), theProgname);
-	error_message(_(txt_usage_index_newsdir), index_newsdir);
-	error_message(_(txt_usage_read_only_active));
-	error_message(_(txt_usage_maildir), tinrc.maildir);
-	error_message(_(txt_usage_mail_new_news_to_user));
-	error_message(_(txt_usage_read_only_subscribed));
-	error_message(_(txt_usage_mail_new_news));
-	error_message(_(txt_usage_post_postponed_arts));
+	error_message(2, _(txt_usage_help_message));
+	error_message(2, _(txt_usage_help_information), theProgname);
+	error_message(2, _(txt_usage_index_newsdir), index_newsdir);
+	error_message(2, _(txt_usage_read_only_active));
+	error_message(2, _(txt_usage_maildir), tinrc.maildir);
+	error_message(2, _(txt_usage_mail_new_news_to_user));
+	error_message(2, _(txt_usage_read_only_subscribed));
+	error_message(2, _(txt_usage_mail_new_news));
+	error_message(2, _(txt_usage_post_postponed_arts));
 
 #ifdef NNTP_ABLE
-	error_message(_(txt_usage_port), nntp_tcp_port);
+	error_message(2, _(txt_usage_port), nntp_tcp_port);
 #endif /* NNTP_ABLE */
 
-	error_message(_(txt_usage_dont_check_new_newsgroups));
-	error_message(_(txt_usage_quickstart));
+	error_message(2, _(txt_usage_dont_check_new_newsgroups));
+	error_message(2, _(txt_usage_quickstart));
 
 #ifdef NNTP_ABLE
 	if (!read_news_via_nntp)
-		error_message(_(txt_usage_read_news_remotely));
+		error_message(2, _(txt_usage_read_news_remotely));
 #endif /* NNTP_ABLE */
 
-	error_message(_(txt_usage_read_saved_news));
-	error_message(_(txt_usage_savedir), tinrc.savedir);
-	error_message(_(txt_usage_save_new_news));
-	error_message(_(txt_usage_update_index_files));
-	error_message(_(txt_usage_verbose));
-	error_message(_(txt_usage_version));
-	error_message(_(txt_usage_post_article));
-	error_message(_(txt_usage_no_posting));
-	error_message(_(txt_usage_dont_save_files_on_quit));
-	error_message(_(txt_usage_start_if_unread_news));
-	error_message(_(txt_usage_check_for_unread_news));
+	error_message(2, _(txt_usage_read_saved_news));
+	error_message(2, _(txt_usage_savedir), tinrc.savedir);
+	error_message(2, _(txt_usage_save_new_news));
+	error_message(2, _(txt_usage_update_index_files));
+	error_message(2, _(txt_usage_verbose));
+	error_message(2, _(txt_usage_version));
+	error_message(2, _(txt_usage_post_article));
+	error_message(2, _(txt_usage_no_posting));
+	error_message(2, _(txt_usage_dont_save_files_on_quit));
+	error_message(2, _(txt_usage_start_if_unread_news));
+	error_message(2, _(txt_usage_check_for_unread_news));
 
-	error_message(_(txt_usage_mail_bugreport), bug_addr);
+	error_message(2, _(txt_usage_mail_bugreport), bug_addr);
 }
 
 
@@ -827,8 +900,14 @@ read_cmd_line_groups(
 
 			for_each_group(i) {
 				if (match_group_list(active[i].name, cmdargs[num])) {
-					if (my_group_add(active[i].name) != -1)
+					if (my_group_add(active[i].name, TRUE) != -1) {
+						if (post_article_and_exit) {
+							my_strncpy(tinrc.default_post_newsgroups, active[i].name, sizeof(tinrc.default_post_newsgroups) - 1);
+							matched++;
+							break;
+						}
 						matched++;
+					}
 				}
 			}
 		}
@@ -839,33 +918,25 @@ read_cmd_line_groups(
 
 /*
  * Create default mail & save directories if they do not exist
- * TODO: return code not meaningful ?
  */
-static t_bool
+static void
 create_mail_save_dirs(
 	void)
 {
-	t_bool created = FALSE;
 	char path[PATH_LEN];
 	struct stat sb;
 
-	if (!strfpath(tinrc.maildir, path, sizeof(path), NULL))
-		joinpath(path, homedir, DEFAULT_MAILDIR);
+	if (!strfpath(tinrc.maildir, path, sizeof(path), NULL, FALSE))
+		joinpath(path, sizeof(path), homedir, DEFAULT_MAILDIR);
 
-	if (stat(path, &sb) == -1) {
+	if (stat(path, &sb) == -1)
 		my_mkdir(path, (mode_t) (S_IRWXU));
-		created = TRUE;
-	}
 
-	if (!strfpath(tinrc.savedir, path, sizeof(path), NULL))
-		joinpath(path, homedir, DEFAULT_SAVEDIR);
+	if (!strfpath(tinrc.savedir, path, sizeof(path), NULL, FALSE))
+		joinpath(path, sizeof(path), homedir, DEFAULT_SAVEDIR);
 
-	if (stat(path, &sb) == -1) {
+	if (stat(path, &sb) == -1)
 		my_mkdir(path, (mode_t) (S_IRWXU));
-		created = TRUE;
-	}
-
-	return created;
 }
 
 
