@@ -3,10 +3,10 @@
  *  Module    : memory.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2005-07-02
+ *  Updated   : 2010-05-16
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,8 @@ int num_newnews = 0;
 int max_art;
 int max_save;
 int num_save = 0;
+int max_scope;
+int num_scope = -1;
 
 /*
  * Dynamic arrays
@@ -58,6 +60,7 @@ int num_save = 0;
 int *my_group;				/* .newsrc --> active[] */
 long *base;				/* base articles for each thread */
 struct t_group *active;			/* active newsgroups */
+struct t_scope *scopes = NULL;	/* attributes stores in .tin/attributes */
 struct t_newnews *newnews;		/* active file sizes on differnet servers */
 struct t_article *arts;			/* articles headers in current group */
 struct t_save *save;			/* sorts articles before saving them */
@@ -66,9 +69,11 @@ struct t_save *save;			/* sorts articles before saving them */
  * Local prototypes
  */
 static void free_active_arrays(void);
+static void free_attributes(struct t_attribute *attributes);
+static void free_scopes_arrays(void);
 static void free_newnews_array(void);
+static void free_if_not_default(char **attrib, char *deflt);
 static void free_input_history(void);
-static void free_global_arrays(void);
 
 
 /*
@@ -97,9 +102,11 @@ init_alloc(
 	 */
 	max_art = DEFAULT_ARTICLE_NUM;
 
-	arts = my_malloc(sizeof(*arts) * max_art);
+	arts = my_calloc(1, sizeof(*arts) * max_art);
 	base = my_malloc(sizeof(long) * max_art);
 
+	ofmt = my_calloc(1, sizeof(*ofmt) * 9);	/* initial number of overview fields */
+
 	/*
 	 * save file array
 	 */
@@ -107,6 +114,12 @@ init_alloc(
 
 	save = my_malloc(sizeof(*save) * max_save);
 
+	/*
+	 * scope array
+	 */
+	max_scope = DEFAULT_SCOPE_NUM;
+	expand_scope();
+
 #ifndef USE_CURSES
 	screen = (struct t_screen *) 0;
 #endif /* !USE_CURSES */
@@ -117,9 +130,13 @@ void
 expand_art(
 	void)
 {
+	int i = max_art;
+
 	max_art += max_art >> 1;		/* increase by 50% */
 	arts = my_realloc(arts, sizeof(*arts) * max_art);
 	base = my_realloc(base, sizeof(long) * max_art);
+	for (; i < max_art; i++)		/* use memset() instead? */
+		arts[i].subject =  arts[i].from = arts[i].xref =  arts[i].refs = arts[i].msgid = NULL;
 }
 
 
@@ -147,6 +164,21 @@ expand_save(
 }
 
 
+void
+expand_scope(
+	void)
+{
+	if ((scopes == NULL) || (num_scope < 0)) {
+		if (scopes == NULL)
+			scopes = my_malloc(sizeof(*scopes) * max_scope);
+		num_scope = 0;
+	} else {
+		max_scope += max_scope >> 1;	/* increase by 50 % */
+		scopes = my_realloc(scopes, sizeof(*scopes) * max_scope);
+	}
+}
+
+
 void
 expand_newnews(
 	void)
@@ -200,7 +232,7 @@ free_all_arrays(
 	FreeAndNull(arts);
 	free_filter_array(&glob_filter);
 	free_active_arrays();
-	free_global_arrays();
+	free_scopes_arrays();
 
 #ifdef HAVE_COLOR
 	FreeIfNeeded(quote_regex.re);
@@ -226,6 +258,10 @@ free_all_arrays(
 	FreeIfNeeded(uubegin_regex.extra);
 	FreeIfNeeded(uubody_regex.re);
 	FreeIfNeeded(uubody_regex.extra);
+	FreeIfNeeded(verbatim_begin_regex.re);
+	FreeIfNeeded(verbatim_begin_regex.extra);
+	FreeIfNeeded(verbatim_end_regex.re);
+	FreeIfNeeded(verbatim_end_regex.extra);
 	FreeIfNeeded(url_regex.re);
 	FreeIfNeeded(url_regex.extra);
 	FreeIfNeeded(mail_regex.re);
@@ -254,6 +290,13 @@ free_all_arrays(
 
 	FreeAndNull(nntp_caps.implementation);
 
+	if (ofmt) { /* ofmt might not be allocated yet on early abort */
+		int i;
+		for (i = 0; ofmt[i].name; i++)
+			free(ofmt[i].name);
+		free(ofmt);
+	}
+
 	tin_fgets(NULL, FALSE);
 }
 
@@ -292,7 +335,7 @@ free_art_array(
  * Use this only for attributes that have a fixed default of a static string
  * in tinrc
  */
-void
+static void
 free_if_not_default(
 	char **attrib,
 	char *deflt)
@@ -303,103 +346,79 @@ free_if_not_default(
 
 
 /*
- * TODO: fix the leaks in the global case
+ * Free memory of one attributes struct only
  */
-void
-free_attributes_array(
-	void)
+static void
+free_attributes(
+	struct t_attribute *attributes)
 {
-	int i;
-	struct t_group *group;
-
-	for_each_group(i) {
-		group = &active[i];
-		if (!group->bogus && group->attribute && !group->attribute->global) {
-			free_if_not_default(&group->attribute->maildir, tinrc.maildir);
-			free_if_not_default(&group->attribute->savedir, tinrc.savedir);
-
-			FreeAndNull(group->attribute->savefile);
-
-			free_if_not_default(&group->attribute->sigfile, tinrc.sigfile);
-			free_if_not_default(&group->attribute->organization, default_organization);
-
-			FreeAndNull(group->attribute->followup_to);
-
-			FreeAndNull(group->attribute->fcc);
-
-			FreeAndNull(group->attribute->mailing_list);
-			FreeAndNull(group->attribute->x_headers);
-			FreeAndNull(group->attribute->x_body);
-
-			free_if_not_default(&group->attribute->from, tinrc.mail_address);
-			free_if_not_default(&group->attribute->news_quote_format, tinrc.news_quote_format);
-			free_if_not_default(&group->attribute->quote_chars, tinrc.quote_chars);
-
-			FreeAndNull(group->attribute->mime_types_to_save);
-
+	free_if_not_default(&attributes->date_format, tinrc.date_format);
+	free_if_not_default(&attributes->editor_format, tinrc.editor_format);
+	FreeAndNull(attributes->fcc);
+	free_if_not_default(&attributes->from, tinrc.mail_address);
+	FreeAndNull(attributes->followup_to);
 #ifdef HAVE_ISPELL
-			FreeAndNull(group->attribute->ispell);
+	FreeAndNull(attributes->ispell);
 #endif /* HAVE_ISPELL */
-
-			FreeAndNull(group->attribute->quick_kill_scope);
-			FreeAndNull(group->attribute->quick_select_scope);
-
-#ifdef CHARSET_CONVERSION
-			FreeAndNull(group->attribute->undeclared_charset);
-#endif /* CHARSET_CONVERSION */
-
-			free(group->attribute);
-		}
-		group->attribute = (struct t_attribute *) 0;
+	free_if_not_default(&attributes->maildir, tinrc.maildir);
+	FreeAndNull(attributes->mailing_list);
+	FreeAndNull(attributes->mime_types_to_save);
+	free_if_not_default(&attributes->news_headers_to_display, tinrc.news_headers_to_display);
+	free_if_not_default(&attributes->news_headers_to_not_display, tinrc.news_headers_to_not_display);
+	if (attributes->headers_to_display) {
+		if (attributes->headers_to_display->header)
+			FreeIfNeeded(*attributes->headers_to_display->header);
+		FreeAndNull(attributes->headers_to_display->header);
+		free(attributes->headers_to_display);
+		attributes->headers_to_display = (struct t_newsheader *) 0;
 	}
+	if (attributes->headers_to_not_display) {
+		if (attributes->headers_to_not_display->header)
+			FreeIfNeeded(*attributes->headers_to_not_display->header);
+		FreeAndNull(attributes->headers_to_not_display->header);
+		free(attributes->headers_to_not_display);
+		attributes->headers_to_not_display = (struct t_newsheader *) 0;
+	}
+	free_if_not_default(&attributes->news_quote_format, tinrc.news_quote_format);
+	free_if_not_default(&attributes->organization, default_organization);
+	FreeAndNull(attributes->quick_kill_scope);
+	FreeAndNull(attributes->quick_select_scope);
+	free_if_not_default(&attributes->quote_chars, tinrc.quote_chars);
+	free_if_not_default(&attributes->savedir, tinrc.savedir);
+	FreeAndNull(attributes->savefile);
+	free_if_not_default(&attributes->sigfile, tinrc.sigfile);
+#ifdef CHARSET_CONVERSION
+	FreeAndNull(attributes->undeclared_charset);
+#endif /* CHARSET_CONVERSION */
+	FreeAndNull(attributes->x_headers);
+	FreeAndNull(attributes->x_body);
+}
 
-	/* free the global attributes array */
-	free_if_not_default(&glob_attributes.maildir, tinrc.maildir);
-	free_if_not_default(&glob_attributes.savedir, tinrc.savedir);
 
-	FreeAndNull(glob_attributes.savefile);
+void
+free_scope(
+	int num)
+{
+	struct t_scope *scope;
 
-	free_if_not_default(&glob_attributes.sigfile, tinrc.sigfile);
-	free_if_not_default(&glob_attributes.organization, default_organization);
-
-	FreeAndNull(glob_attributes.followup_to);
-
-	FreeAndNull(glob_attributes.fcc);
-
-	FreeAndNull(glob_attributes.mailing_list);
-	FreeAndNull(glob_attributes.x_headers);
-	FreeAndNull(glob_attributes.x_body);
-
-	free_if_not_default(&glob_attributes.from, tinrc.mail_address);
-	free_if_not_default(&glob_attributes.news_quote_format, tinrc.news_quote_format);
-	free_if_not_default(&glob_attributes.quote_chars, tinrc.quote_chars);
-
-	FreeAndNull(glob_attributes.mime_types_to_save);
-
-#ifdef HAVE_ISPELL
-	FreeAndNull(glob_attributes.ispell);
-#endif /* HAVE_ISPELL */
-
-	FreeAndNull(glob_attributes.quick_kill_scope);
-	FreeAndNull(glob_attributes.quick_select_scope);
-
-#ifdef CHARSET_CONVERSATION
-	FreeAndNull(glob_attributes.undeclared_charset);
-#endif /* CHARSET_CONVERSATION */
+	scope = &scopes[num];
+	FreeAndNull(scope->scope);
+	free_attributes(scope->attribute);
+	free(scope->attribute);
+	scope->attribute = (struct t_attribute *) 0;
+	free(scope->state);
+	scope->state = (struct t_attribute_state *) 0;
 }
 
 
 static void
-free_global_arrays(
+free_scopes_arrays(
 	void)
 {
-	if (news_headers_to_display_array)
-		FreeIfNeeded(*news_headers_to_display_array);
-	FreeAndNull(news_headers_to_display_array);
-
-	if (news_headers_to_not_display_array)
-		FreeIfNeeded(*news_headers_to_not_display_array);
-	FreeAndNull(news_headers_to_not_display_array);
+	while (num_scope > 0)
+		free_scope(--num_scope);
+	FreeAndNull(scopes);
+	num_scope = -1;
 }
 
 
@@ -407,21 +426,24 @@ static void
 free_active_arrays(
 	void)
 {
-	int i;
-
 	FreeAndNull(my_group);	/* my_group[] */
 
 	if (active != NULL) {	/* active[] */
+		int i;
+
 		for_each_group(i) {
 			FreeAndNull(active[i].name);
 			FreeAndNull(active[i].description);
 			FreeAndNull(active[i].aliasedto);
-			if (active[i].type == GROUP_TYPE_MAIL) {
+			if (active[i].type == GROUP_TYPE_MAIL || active[i].type == GROUP_TYPE_SAVE) {
 				FreeAndNull(active[i].spooldir);
 			}
 			FreeAndNull(active[i].newsrc.xbitmap);
+			if (active[i].attribute && !active[i].attribute->global) {
+				free(active[i].attribute);
+				active[i].attribute = (struct t_attribute *) 0;
+			}
 		}
-		free_attributes_array();
 		FreeAndNull(active);
 	}
 	num_active = -1;
@@ -484,7 +506,7 @@ my_malloc1(
 #endif /* DEBUG */
 
 	if ((p = malloc(size)) == NULL) {
-		error_message(txt_out_of_memory, tin_progname, size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, size, file, line);
 		giveup();
 	}
 	return p;
@@ -508,7 +530,7 @@ my_calloc1(
 #endif /* DEBUG */
 
 	if ((p = calloc(nmemb, size)) == NULL) {
-		error_message(txt_out_of_memory, tin_progname, nmemb * size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, nmemb * size, file, line);
 		giveup();
 	}
 	return p;
@@ -526,10 +548,17 @@ my_realloc1(
 	debug_print_malloc(FALSE, file, line, size);
 #endif /* DEBUG */
 
-	p = ((!p) ? (calloc(1, size)) : realloc(p, size));
+	if (!size) {
+		if (p)
+			free(p);
+
+		return NULL;
+	}
+
+	p = ((!p) ? (malloc(size)) : realloc(p, size));
 
 	if (p == NULL) {
-		error_message(txt_out_of_memory, tin_progname, size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, size, file, line);
 		giveup();
 	}
 	return p;
