@@ -3,10 +3,10 @@
  *  Module    : misc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2006-04-06
+ *  Updated   : 2010-11-03
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -72,7 +72,7 @@
  * Local prototypes
  */
 static char *strfpath_cp(char *str, char *tbuf, char *endp);
-static int _strfpath(const char *format, char *str, size_t maxsize, struct t_group *group);
+static int _strfpath(const char *format, char *str, size_t maxsize, struct t_group *group, t_bool expand_all);
 static int gnksa_check_domain(char *domain);
 static int gnksa_check_domain_literal(char *domain);
 static int gnksa_check_localpart(const char *localpart);
@@ -80,7 +80,7 @@ static int gnksa_dequote_plainphrase(char *realname, char *decoded, int addrtype
 static int strfeditor(char *editor, int linenum, const char *filename, char *s, size_t maxsize, char *format);
 static void write_input_history_file(void);
 #ifdef CHARSET_CONVERSION
-	static t_bool buffer_to_local(char **line, int *max_line_len, const char *network_charset, const char *local_charset);
+	static t_bool buffer_to_local(char **line, size_t *max_line_len, const char *network_charset, const char *local_charset);
 #endif /* CHARSET_CONVERSION */
 #if 0 /* currently unused */
 	static t_bool stat_article(long art, const char *group_path);
@@ -166,7 +166,7 @@ asfail(
 
 /*
  * Quick copying of files
- * Returns FALSE if copy failed. Caller may wish to check for SIGPIPE
+ * Returns FALSE if copy failed. Caller may wish to check for errno == EPIPE.
  */
 t_bool
 copy_fp(
@@ -176,11 +176,13 @@ copy_fp(
 	char buf[8192];
 	size_t have, sent;
 
+	errno = 0;	/* To check errno after write, clear it here */
+
 	while ((have = fread(buf, 1, sizeof(buf), fp_ip)) != 0) {
 		sent = fwrite(buf, 1, have, fp_op);
 		if (sent != have) {
 			TRACE(("copy_fp wrote %d of %d:{%.*s}", sent, have, (int) sent, buf));
-			if (!got_sig_pipe) /* !SIGPIPE => more serious error */
+			if (errno && errno != EPIPE) /* not a broken pipe => more serious error */
 				perror_message(_(txt_error_copy_fp));
 			return FALSE;
 		}
@@ -269,7 +271,7 @@ copy_body(
 	if (strlen(prefix) > 240) /* truncate and terminate */
 		prefix[240] = '\0';
 
-	/* convert %S to %s, for compability reasons only */
+	/* convert %S to %s, for compatibility reasons only */
 	if (strstr(prefix, "%S")) {
 		status_char = FALSE;
 		for (i = 0; prefix[i]; i++) {
@@ -318,7 +320,7 @@ copy_body(
 					}
 					buf2[i] = '\0';
 					if (status_char)	/* already quoted */
-						retcode = fprintf(fp_op, "%s>%s", buf2, strchr(buf, '>'));
+						retcode = fprintf(fp_op, "%s>%s", buf2, BlankIfNull(strchr(buf, '>')));
 					else	/* ... to be quoted ... */
 						retcode = fprintf(fp_op, "%s%s", prefixbuf, buf);
 				} else	/* line was not already quoted (no >) */
@@ -363,7 +365,8 @@ get_val(
 t_bool
 invoke_editor(
 	const char *filename,
-	int lineno) /* return value is always ignored */
+	int lineno,
+	struct t_group *group) /* return value is always ignored */
 {
 	char buf[PATH_LEN], fnameb[PATH_LEN];
 	char editor_format[PATH_LEN];
@@ -376,7 +379,10 @@ invoke_editor(
 		first = FALSE;
 	}
 
-	my_strncpy(editor_format, (*tinrc.editor_format ? tinrc.editor_format : (tinrc.start_editor_offset ? TIN_EDITOR_FMT_ON : TIN_EDITOR_FMT_OFF)), sizeof(editor_format) - 1);
+	if (group != NULL)
+		my_strncpy(editor_format, (*group->attribute->editor_format ? group->attribute->editor_format : (group->attribute->start_editor_offset ? TIN_EDITOR_FMT_ON : TIN_EDITOR_FMT_OFF)), sizeof(editor_format) - 1);
+	else
+		my_strncpy(editor_format, (*tinrc.editor_format ? tinrc.editor_format : (tinrc.start_editor_offset ? TIN_EDITOR_FMT_ON : TIN_EDITOR_FMT_OFF)), sizeof(editor_format) - 1);
 
 	if (!strfeditor(editor, lineno, filename, buf, sizeof(buf), editor_format))
 		sh_format(buf, sizeof(buf), "%s %s", editor, filename);
@@ -411,7 +417,7 @@ invoke_ispell(
 		STRCPY(ispell, get_val("ISPELL", PATH_ISPELL));
 
 	/*
-	 * Now seperating the header and body in two different files so that
+	 * Now separating the header and body in two different files so that
 	 * the header is not checked by ispell
 	 */
 #ifdef HAVE_LONG_FILE_NAMES
@@ -570,9 +576,10 @@ tin_done(
 	if (!no_write) {
 		i = 3; /* max retries */
 		while (i--) {
-			if (((wrote_newsrc_lines = write_newsrc()) >= 0L) && (wrote_newsrc_lines >= read_newsrc_lines)) {
+			wrote_newsrc_lines = write_newsrc();
+			if ((wrote_newsrc_lines >= 0L) && (wrote_newsrc_lines >= read_newsrc_lines)) {
 				if (!batch_mode || verbose)
-					my_fputs(_(txt_newsrc_saved), stdout);
+					wait_message(0, _(txt_newsrc_saved));
 				break;
 			}
 
@@ -594,9 +601,7 @@ tin_done(
 		}
 
 		write_input_history_file();
-#if 0
 		write_attributes_file(local_attributes_file);
-#endif /* 0 */
 
 #ifdef HAVE_MH_MAIL_HANDLING
 		write_mail_active_file();
@@ -613,6 +618,7 @@ tin_done(
 
 	free_all_arrays();
 
+	/* TODO: why do we make this exception here? */
 #ifdef SIGUSR1
 	if (ret != -SIGUSR1) {
 #endif /* SIGUSR1 */
@@ -633,6 +639,11 @@ tin_done(
 	} else
 		ret = SIGUSR1;
 #endif /* SIGUSR1 */
+#ifdef HAVE_COLOR
+#	ifdef USE_CURSES
+	free_color_pair_arrays();
+#	endif /* USE_CURSES */
+#endif /* HAVE_COLOR */
 	cleanup_tmp_files();
 
 #ifdef DOALLOC
@@ -648,12 +659,6 @@ tin_done(
 	}
 #endif /* USE_DBMALLOC */
 
-#ifdef VMS
-	if (!ret)
-		ret = 1;
-	vms_close_stdin(); /* free resources used by ReadCh */
-#endif /* VMS */
-
 	exit(ret);
 }
 
@@ -745,31 +750,6 @@ rename_file(
 #endif /* M_UNIX */
 
 
-#ifdef VMS
-void
-rename_file(
-	char *old_filename,
-	char *new_filename)
-{
-	char new_filename_vms[1024];
-
-	if (!strchr(strchr(new_filename, ']') ? strchr(new_filename, ']') : new_filename, '.')) {
-		/* without final dot the new filename is not as tin expects */
-		if (strlen(new_filename) >= sizeof(new_filename_vms)) {
-			perror_message("length of %s is too large", new_filename);
-			return;
-		}
-		strcpy(new_filename_vms, new_filename);
-		strcat(new_filename_vms, ".");
-		new_filename = &new_filename_vms[0];
-	}
-
-	if (rename(old_filename, new_filename))
-		perror_message(_(txt_rename_error), old_filename, new_filename);
-}
-#endif /* VMS */
-
-
 /*
  * Note that we exit screen/curses mode when invoking
  * external commands
@@ -780,7 +760,9 @@ invoke_cmd(
 {
 	int ret;
 	t_bool save_cmd_line = cmd_line;
+#ifndef IGNORE_SYSTEM_STATUS
 	t_bool success;
+#endif /* IGNORE_SYSTEM_STATUS */
 
 	if (!save_cmd_line) {
 		EndWin();
@@ -790,13 +772,9 @@ invoke_cmd(
 
 	TRACE(("called system(%s)", _nc_visbuf(nam)));
 	ret = system(nam);
-#ifdef VMS
-	system_status = 0;
-#else
-#	ifndef USE_SYSTEM_STATUS
+#ifndef USE_SYSTEM_STATUS
 	system_status = (ret >= 0 && WIFEXITED(ret)) ? WEXITSTATUS(ret) : 0;
-#	endif /* !USE_SYSTEM_STATUS */
-#endif /* VMS */
+#endif /* !USE_SYSTEM_STATUS */
 	TRACE(("return %d (%d)", ret, system_status));
 
 	set_signal_catcher(TRUE);
@@ -810,14 +788,10 @@ invoke_cmd(
 	return TRUE;
 #else
 
-#	ifdef VMS
-		success = (ret != 0);
-#	else
-		success = (ret == 0);
-#	endif /* VMS */
+	success = (ret == 0);
 
 	if (!success || system_status != 0)
-		error_message(_(txt_command_failed), nam);
+		error_message(2, _(txt_command_failed), nam);
 
 	return success;
 #endif /* IGNORE_SYSTEM_STATUS */
@@ -831,9 +805,6 @@ draw_percent_mark(
 {
 	char buf[32]; /* should be big enough */
 	int len;
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *wbuf;
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	if (NOTESLINES <= 0)
 		return;
@@ -843,18 +814,12 @@ draw_percent_mark(
 
 	clear_message();
 	snprintf(buf, sizeof(buf), "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	if ((wbuf = char2wchar_t(buf)) != NULL) {
-		len = wcswidth(wbuf, wcslen(wbuf) + 1);
-		free(wbuf);
-	} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		len = (int) strlen(buf);
+	len = strwidth(buf);
 	MoveCursor(cLINES, cCOLS - len - (1 + BLANK_PAGE_COLS));
 	StartInverse();
 	my_fputs(buf, stdout);
-	my_flush();
 	EndInverse();
+	my_flush();
 }
 
 
@@ -867,9 +832,6 @@ base_name(
 	char *file)				/* baz */
 {
 	size_t i;
-#ifdef VMS
-	char *cp;
-#endif /* VMS */
 
 	strcpy(file, fullpath);
 
@@ -879,10 +841,6 @@ base_name(
 			break;
 		}
 	}
-#ifdef VMS
-	if ((cp = strrchr(file, '.')) != 0)
-		*cp = '\0';
-#endif /* VMS */
 }
 
 
@@ -926,14 +884,12 @@ mail_check(
 		if ((int) (buf.st_mode & S_IFMT) == (int) S_IFDIR) { /* maildir setup */
 			DIR *dirp;
 			char *maildir_box;
+			size_t maildir_box_len = strlen(mailbox_name) + strlen(MAILDIR_NEW) + 2;
 			struct dirent *dp;
 
-			maildir_box = my_malloc(strlen(mailbox_name) + strlen(MAILDIR_NEW) + 2);
-#ifdef VMS
-			joindir(maildir_box, mailbox_name, MAILDIR_NEW);
-#else
-			joinpath(maildir_box, mailbox_name, MAILDIR_NEW);
-#endif /* VMS */
+			maildir_box = my_malloc(maildir_box_len);
+			joinpath(maildir_box, maildir_box_len, mailbox_name, MAILDIR_NEW);
+
 			if (!(dirp = opendir(maildir_box))) {
 				free(maildir_box);
 				return FALSE;
@@ -980,26 +936,27 @@ eat_re(
 		data = pcre_exec(strip_re_regex.re, strip_re_regex.extra, s, strlen(s), 0, 0, offsets, size_offsets);
 		if (offsets[0] == 0)
 			s += offsets[1];
-	} while (data > 0);
+	} while (data >= 0);
 
 	if (eat_was) do {
 		data = pcre_exec(strip_was_regex.re, strip_was_regex.extra, s, strlen(s), 0, 0, offsets, size_offsets);
 		if (offsets[0] > 0)
 			s[offsets[0]] = '\0';
-	} while (data > 0);
+	} while (data >= 0);
 
 	return s;
 }
 
 
+#if defined(NO_LOCALE) || !defined(MULTIBYTE_ABLE)
 int
 my_isprint(
 	int c)
 {
-#ifndef NO_LOCALE
+#	ifndef NO_LOCALE
 	/* use locale */
 	return isprint(c);
-#else
+#	else
 	if (IS_LOCAL_CHARSET("ISO-8859"))
 		return (isprint(c) || (c >= 0xa0 && c <= 0xff));
 	else if (IS_LOCAL_CHARSET("ISO-2022"))
@@ -1010,8 +967,9 @@ my_isprint(
 		return 1;
 	else /* KOI8-* and UTF-8 */
 		return (isprint(c) || (c >= 0x80 && c <= 0xff));
-#endif /* !NO_LOCALE */
+#	endif /* !NO_LOCALE */
 }
+#endif /* NO_LOCALE || !MULTIBYTE_ABLE */
 
 
 /*
@@ -1100,6 +1058,14 @@ toggle_color(
 #	endif /* USE_CURSES */
 		use_color = bool_not(use_color);
 
+#	ifndef USE_CURSES
+	if (use_color) {
+		fcol(tinrc.col_normal);
+		bcol(tinrc.col_normal);
+	} else
+		reset_screen_attr();
+#	endif /* !USE_CURSES */
+
 	return TRUE;
 }
 
@@ -1116,6 +1082,8 @@ show_color_status(
 /*
  * Check for lock file to stop multiple copies of tin -u running and if it
  * does not exist create it so this is the only copy running
+ *
+ * FIXME: get ridd of hardcoded pid-length as pid_t might be long
  */
 void
 create_index_lock_file(
@@ -1125,12 +1093,13 @@ create_index_lock_file(
 	char buf[64];
 	struct stat sb;
 	time_t epoch;
+	int err;
 
 	if (stat(the_lock_file, &sb) == 0) {
 		if ((fp = fopen(the_lock_file, "r")) != NULL) {
 			fgets(buf, (int) sizeof(buf), fp);
 			fclose(fp);
-			error_message("\n%s: Already started pid=[%d] on %s", tin_progname, atoi(buf), buf + 8);
+			error_message(2, "\n%s: Already started pid=[%d] on %s", tin_progname, atoi(buf), buf + 8);
 			giveup();
 		}
 	} else {
@@ -1138,8 +1107,13 @@ create_index_lock_file(
 			fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
 			(void) time(&epoch);
 			fprintf(fp, "%6d  %s\n", (int) process_id, ctime(&epoch));
-			if (ferror(fp) || fclose(fp))
-				error_message(_(txt_filesystem_full), the_lock_file);
+			if ((err = ferror(fp)) || fclose(fp)) {
+				error_message(2, _(txt_filesystem_full), the_lock_file);
+				if (err) {
+					clearerr(fp);
+					fclose(fp);
+				}
+			}
 		}
 	}
 }
@@ -1148,7 +1122,7 @@ create_index_lock_file(
 /*
  * strfquote() - produce formatted quote string
  *   %A  Articles Email address
- *   %D  Articles Date
+ *   %D  Articles Date (uses tinrc.date_format)
  *   %F  Articles Address+Name
  *   %G  Groupname of Article
  *   %M  Articles MessageId
@@ -1243,8 +1217,10 @@ strfquote(
 					}
 					break;
 
-				case 'D':	/* Articles Date */
-					STRCPY(tbuf, BlankIfNull(pgart.hdr.date));
+				case 'D':	/* Articles Date (reformatted as specified in attributes->date_format) */
+					if (!my_strftime(tbuf, LEN - 1, curr_group->attribute->date_format, localtime(&arts[this_resp].date))) {
+						STRCPY(tbuf, BlankIfNull(pgart.hdr.date));
+					}
 					break;
 
 				case 'F':	/* Articles Address+Name */
@@ -1450,7 +1426,7 @@ strfpath_cp(
  *   $var/News -> /env/var/News
  *   =file     -> $HOME/Mail/file
  *   =         -> $HOME/Mail/group.name
- *   +file     -> tinrc.savedir/group.name/file
+ *   +file     -> savedir/group.name/file
  *
  * Interestingly, %G is not documented as such and apparently unused
  *   ~/News/%G -> $HOME/News/group.name
@@ -1460,6 +1436,7 @@ strfpath_cp(
  *   str		Return buffer
  *   maxsize	Size of str
  *   group		ptr to current group
+ *   expand_all	true if '+' and '=' should be expanded
  * Returns:
  *   0			on error
  *   1			if generated pathname is a mailbox
@@ -1470,7 +1447,8 @@ _strfpath(
 	const char *format,
 	char *str,
 	size_t maxsize,
-	struct t_group *group)
+	struct t_group *group,
+	t_bool expand_all)
 {
 	char *endp = str + maxsize;
 	const char *startp = format;
@@ -1493,12 +1471,7 @@ _strfpath(
 		/*
 		 * If just a normal part of the pathname copy it
 		 */
-#ifdef VMS
-		if (!strchr("~=+%", *format))
-#else
-		if (!strchr("~$=+%", *format))
-#endif /* VMS */
-		{
+		if (!strchr("~$=+%", *format)) {
 			*str++ = *format;
 			continue;
 		}
@@ -1507,7 +1480,7 @@ _strfpath(
 			case '~':			/* Users or another users homedir */
 				switch (*++format) {
 					case '/':	/* users homedir */
-						joinpath(tbuf, homedir, "");
+						joinpath(tbuf, sizeof(tbuf), homedir, "");
 						break;
 
 					default:	/* some other users homedir */
@@ -1529,7 +1502,6 @@ _strfpath(
 					return 0;
 				break;
 
-#ifndef VMS
 			case '$':	/* Read the envvar and use its value */
 				i = 0;
 				format++;
@@ -1567,7 +1539,6 @@ _strfpath(
 					return 0;
 				}
 				break;
-#endif /* !VMS */
 
 			case '=':
 				/*
@@ -1576,16 +1547,15 @@ _strfpath(
 				 * =dir expands to maildir/dir
 				 * =    expands to maildir/groupname
 				 */
-				is_mailbox = TRUE;
-
-				if (startp == format && group != NULL) {
+				if (startp == format && group != NULL && expand_all) {
 					char buf[PATH_LEN];
 
-					if (strfpath(group->attribute->maildir, buf, sizeof(buf), group)) {
+					is_mailbox = TRUE;
+					if (strfpath(cmdline.args & CMDLINE_MAILDIR ? cmdline.maildir : group->attribute->maildir, buf, sizeof(buf), group, FALSE)) {
 						if (*(format + 1) == '\0')				/* Just an = */
-							joinpath(tbuf, buf, group->name);
+							joinpath(tbuf, sizeof(tbuf), buf, group->name);
 						else
-							joinpath(tbuf, buf, "");
+							joinpath(tbuf, sizeof(tbuf), buf, "");
 						if ((str = strfpath_cp(str, tbuf, endp)) == NULL)
 							return 0;
 					} else {
@@ -1600,22 +1570,24 @@ _strfpath(
 				/*
 				 * Group name expansion
 				 * Only convert if 1st char in format
+				 * +file expands to savedir/group.name/file
 				 */
-				if (startp == format && group != NULL) {
+
+				if (startp == format && group != NULL && expand_all) {
 					char buf[PATH_LEN];
 
 					/*
 					 * Start with the savedir name
 					 */
-					if (strfpath(group->attribute->savedir, buf, sizeof(buf), group)) {
+					if (strfpath(cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : group->attribute->savedir, buf, sizeof(buf), group, FALSE)) {
 						char tmp[PATH_LEN];
 #ifdef HAVE_LONG_FILE_NAMES
 						my_strncpy(tmp, group->name, sizeof(tmp) - 1);
 #else
 						my_strncpy(tmp, group->name, 14);
 #endif /* HAVE_LONG_FILE_NAMES */
-						JOINPATH(tbuf, buf, tmp);	/* Add the group name */
-						joinpath(tmp, tbuf, "");
+						joinpath(tbuf, sizeof(tbuf), buf, tmp);	/* Add the group name */
+						joinpath(tmp, sizeof(tmp), tbuf, "");
 						if ((str = strfpath_cp(str, tmp, endp)) == NULL)
 							return 0;
 					} else {
@@ -1669,7 +1641,8 @@ strfpath(
 	const char *format,
 	char *str,
 	size_t maxsize,
-	struct t_group *group)
+	struct t_group *group,
+	t_bool expand_all)
 {
 	/*
 	 * Expand any leading env vars first in case they themselves contain
@@ -1678,11 +1651,11 @@ strfpath(
 	if (format[0] == '$') {
 		char buf[PATH_LEN];
 
-		if (_strfpath(format, buf, sizeof(buf), group))
-			return (_strfpath(buf, str, maxsize, group));
+		if (_strfpath(format, buf, sizeof(buf), group, expand_all))
+			return (_strfpath(buf, str, maxsize, group, expand_all));
 	}
 
-	return (_strfpath(format, str, maxsize, group));
+	return (_strfpath(format, str, maxsize, group, expand_all));
 }
 
 
@@ -2010,9 +1983,6 @@ make_group_path(
 	const char *name,
 	char *path)
 {
-#ifdef VMS
-	sprintf(path, "[%s]", name);
-#else
 	while (*name) {
 		*path = ((*name == '.') ? '/' : *name);
 		name++;
@@ -2020,7 +1990,6 @@ make_group_path(
 	}
 	*path++ = '/';
 	*path = '\0';
-#endif /* VMS */
 }
 
 
@@ -2034,12 +2003,13 @@ void
 make_base_group_path(
 	const char *base_dir,
 	const char *group_name,
-	char *group_path)
+	char *group_path,
+	size_t group_path_len)
 {
 	char buf[LEN];
 
 	make_group_path(group_name, buf);
-	joinpath(group_path, base_dir, buf);
+	joinpath(group_path, group_path_len, base_dir, buf);
 }
 
 
@@ -2054,7 +2024,7 @@ cleanup_tmp_files(
 	char acNovFile[PATH_LEN];
 
 	if (nntp_caps.over_cmd && !tinrc.cache_overview_files) {
-		snprintf(acNovFile, sizeof(acNovFile), "%s%d.idx", TMPDIR, (int) process_id);
+		snprintf(acNovFile, sizeof(acNovFile), "%s%ld.idx", TMPDIR, (long) process_id);
 		unlink(acNovFile);
 	}
 #endif /* 0 */
@@ -2228,7 +2198,7 @@ write_input_history_file(
 	file_tmp = get_tmpfilename(local_input_history_file);
 
 	if ((fp = fopen(file_tmp, "w")) == NULL) {
-		error_message(_(txt_filesystem_full_backup), local_input_history_file);
+		error_message(2, _(txt_filesystem_full_backup), local_input_history_file);
 		/* free memory for tmp-filename */
 		free(file_tmp);
 		umask(mask);
@@ -2250,10 +2220,14 @@ write_input_history_file(
 
 	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR)); /* rename_file() preserves mode */
 
-	if (ferror(fp) || fclose(fp)) {
-		error_message(_(txt_filesystem_full), local_input_history_file);
+	if ((his_w = ferror(fp)) || fclose(fp)) {
+		error_message(2, _(txt_filesystem_full), local_input_history_file);
 		/* fix modes for all pre 1.4.1 local_input_history_file files */
 		chmod(local_input_history_file, (mode_t) (S_IRUSR|S_IWUSR));
+		if (his_w) {
+			clearerr(fp);
+			fclose(fp);
+		}
 	} else
 		rename_file(file_tmp, local_input_history_file);
 
@@ -2344,7 +2318,7 @@ strip_name(
 static t_bool
 buffer_to_local(
 	char **line,
-	int *max_line_len,
+	size_t *max_line_len,
 	const char *network_charset,
 	const char *local_charset)
 {
@@ -2483,7 +2457,7 @@ buffer_to_local(
 				} while (inbytesleft > 0);
 
 				**&outbuf = '\0';
-				if (*max_line_len < (int) strlen(obuf) + 1) {
+				if (*max_line_len < strlen(obuf) + 1) {
 					*max_line_len = strlen(obuf) + 1;
 					*line = my_realloc(*line, *max_line_len);
 				}
@@ -2593,7 +2567,7 @@ buffer_to_ascii(
 void
 process_charsets(
 	char **line,
-	int *max_line_len,
+	size_t *max_line_len,
 	const char *network_charset,
 	const char *local_charset,
 	t_bool conv_tex2iso)
@@ -2645,7 +2619,7 @@ process_charsets(
 
 
 /*
- * checking of mail adresses for GNKSA compliance
+ * checking of mail addresses for GNKSA compliance
  *
  * son of RFC 1036:
  *   article         = 1*header separator body
@@ -3161,13 +3135,16 @@ gnksa_check_domain_literal(
 	if ((255 < x1) || (255 < x2) || (255 < x3) || (255 < x4))
 		return GNKSA_BAD_DOMAIN_LITERAL;
 
-	/* check for private ip or localhost */
+	/* check for private ip or localhost - see RFC 5735, RFC 5737 */
 	if ((!disable_gnksa_domain_check)
 	    && ((0 == x1)				/* local network */
 		|| (10 == x1)				/* private class A */
-		|| ((172 == x1) && (16 == (x2 & 0xf0)))	/* private class B */
-		|| ((192 == x1) && (168 == x2))		/* private class C */
-		|| (127 == x1)))			/* localhost */
+		|| ((172 == x1) && (16 == (x2 & 0xf0)))	/* private /12 */
+		|| ((192 == x1) && (168 == x2))		/* private class B */
+		|| ((192 == x1) && (0 == x2) && (2 == x3)) /* TEST NET-1 */
+		|| ((198 == x1) && (51 == x2) && (100 == x3)) /* TEST NET-2 */
+		|| ((203 == x1) && (0 == x2) && (113 == x3)) /* TEST NET-3 */
+		|| (127 == x1)))			/* loopback */
 		return GNKSA_LOCAL_DOMAIN_LITERAL;
 
 	return GNKSA_OK;
@@ -3241,8 +3218,14 @@ gnksa_check_domain(
 			}
 			if (disable_gnksa_domain_check)
 				result = GNKSA_OK;
-			if (GNKSA_OK != result)
-				return result;
+			if (GNKSA_OK != result) {
+#if 0 /* valid IDN ccTLDs are checked via gnksa_domain_list[] */
+				if (strlen(aux) >= 8 && !strncasecmp(aux, "xn--", 4)) /* hack for IDN ccTLDs like xn--wgbh1c (Egypt), xn--mgbaam7a8h (Emarat) or xn--mgberp4a5d4ar (AlSaudiah) */
+					result = GNKSA_OK;
+				else
+#endif /* 0 */
+					return result;
+			}
 			break;
 	}
 
@@ -3453,7 +3436,7 @@ gnksa_do_check_from(
 	decoded[0] = '\0';
 
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		wait_message(0, "From:=[%s]", from);
 #endif /* DEBUG */
 
@@ -3463,7 +3446,7 @@ gnksa_do_check_from(
 		return code;
 
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		wait_message(0, "address=[%s]", address);
 #endif /* DEBUG */
 
@@ -3477,7 +3460,7 @@ gnksa_do_check_from(
 		*addr_begin++ = '\0';
 
 #ifdef DEBUG
-		if (debug == 2)
+		if (debug & DEBUG_MISC)
 			wait_message(0, "FQDN=[%s]", addr_begin);
 #endif /* DEBUG */
 
@@ -3497,7 +3480,7 @@ gnksa_do_check_from(
 	}
 
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		wait_message(0, "realname=[%s]", realname);
 #endif /* DEBUG */
 
@@ -3509,9 +3492,9 @@ gnksa_do_check_from(
 		strcpy(realname, decoded);
 
 #ifdef DEBUG
-	if (debug == 2) {
+	if (debug & DEBUG_MISC) { /* TODO: dump to a file instead of wait_message() */
 		if (GNKSA_OK != code)
-			wait_message(3, "From:=[%s], GNKSA=[%d]", from, code);
+			wait_message(2, "From:=[%s], GNKSA=[%d]", from, code);
 		else
 			wait_message(0, "GNKSA=[%d]", code);
 	}
@@ -3607,11 +3590,9 @@ utf8_valid(
 			numc++;
 		} while ((d <<= 1) & 0x80);	/* get sequence length */
 
-		d = 1;
-		while (d < numc) {
-			if (*(c + d) == '\0' || *(c + d) == '\n')
-				illegal = TRUE;
-			d--;
+		if (c + numc > line + strlen(line)) { /* sequence runs past end of string */
+			illegal = TRUE;
+			numc = line + strlen(line) - c;
 		}
 
 		if (!illegal) {
@@ -3797,6 +3778,11 @@ tin_version_info(
 
 	fprintf(fp, "Characteristics:\n\t"
 /* TODO: complete list and do some useful grouping */
+#ifdef DEBUG
+			"+DEBUG "
+#else
+			"-DEBUG "
+#endif /* DEBUG */
 #ifdef NNTP_ONLY
 			"+NNTP_ONLY "
 #else
@@ -4004,24 +3990,26 @@ stat_article(
 	long art,
 	const char *group_path)
 {
-	char buf[NNTP_STRLEN];
 	struct t_group currgrp;
 
 	currgrp = CURR_GROUP;
 
 #	ifdef NNTP_ABLE
 	if (read_news_via_nntp && currgrp.type == GROUP_TYPE_NEWS) {
+		char buf[NNTP_STRLEN];
+
 		snprintf(buf, sizeof(buf), "STAT %ld", art);
 		return (nntp_command(buf, OK_NOTEXT, NULL, 0) != NULL);
 	} else
 #	endif /* NNTP_ABLE */
 	{
+		char filename[PATH_LEN];
 		struct stat sb;
 
-		joinpath(buf, currgrp.spooldir, group_path);
-		snprintf(&buf[strlen(buf)], sizeof(buf), "/%ld", art);
+		joinpath(filename, sizeof(filename), currgrp.spooldir, group_path);
+		snprintf(&filename[strlen(filename)], sizeof(filename), "/%ld", art);
 
-		return (stat(buf, &sb) != -1);
+		return (stat(filename, &sb) != -1);
 	}
 }
 #endif /* 0 */
