@@ -3,10 +3,10 @@
  *  Module    : newsrc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2005-05-04
+ *  Updated   : 2010-07-14
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -109,7 +109,7 @@ read_newsrc(
 			line_count++;
 
 			if (sub == SUBSCRIBED) {
-				if ((i = my_group_add(grp)) >= 0) {
+				if ((i = my_group_add(grp, FALSE)) >= 0) {
 					if (!active[my_group[i]].bogus) {
 						active[my_group[i]].subscribed = SUB_BOOL(sub);
 						parse_bitmap_seq(&active[my_group[i]], seq);
@@ -121,10 +121,11 @@ read_newsrc(
 		fclose(fp);
 		/* If you aborted with 'q', then you get what you get. */
 
-		if (cmd_line) {
+		if (!batch_mode || verbose)
 			my_fputc('\n', stdout);
-			my_flush();
-		}
+
+		if (!cmd_line && !batch_mode)
+			clear_message();
 	}
 	return line_count;
 }
@@ -159,10 +160,10 @@ write_newsrc_line(
 	 * are set to auto removal. Also check for bogus flag just in case
 	 * strip_bogus was changed since tin started
 	 */
-	group = group_find(line);
+	group = group_find(line, FALSE);
 
 	if (tinrc.strip_bogus == BOGUS_REMOVE) {
-		if (group == NULL || group->bogus) { /* group dosen't exist */
+		if (group == NULL || group->bogus) { /* group doesn't exist */
 			wait_message(2, _(txt_remove_bogus), line);
 			return 0;
 		}
@@ -199,6 +200,7 @@ write_newsrc(
 	signed long int tot = 0L;
 	struct stat note_stat_newsrc;
 	t_bool write_ok = FALSE;
+	int err;
 
 	if (no_write)
 		return 0L;
@@ -214,7 +216,7 @@ write_newsrc(
 		return 0L; /* newsrc is empty */
 	}
 
-	if ((fp_op = fopen(newnewsrc, "w" FOPEN_OPTS)) != NULL) {
+	if ((fp_op = fopen(newnewsrc, "w")) != NULL) {
 		if (newsrc_mode)
 			fchmod(fileno(fp_op), newsrc_mode);
 
@@ -224,9 +226,13 @@ write_newsrc(
 		/*
 		 * Don't rename if either fclose() fails or ferror() is set
 		 */
-		if (ferror(fp_op) || fclose(fp_op)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		if ((err = ferror(fp_op)) || fclose(fp_op)) {
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
 			unlink(newnewsrc);
+			if (err) {
+				clearerr(fp_op);
+				fclose(fp_op);
+			}
 		} else
 			write_ok = TRUE;
 	}
@@ -234,7 +240,8 @@ write_newsrc(
 	fclose(fp_ip);
 
 	if (tot < 1) {
-		error_message(_(txt_newsrc_nogroups));
+		error_message(2, _(txt_newsrc_nogroups));
+		unlink(newnewsrc);
 		return 0L;		/* So we don't get prompted to try again */
 	}
 
@@ -261,8 +268,12 @@ create_newsrc(
 		for_each_group(i)
 			fprintf(fp, "%s!\n", active[i].name);
 
-		if (ferror(fp) || fclose(fp)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		if ((i = ferror(fp)) || fclose(fp)) {
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			if (i) {
+				clearerr(fp);
+				fclose(fp);
+			}
 			return FALSE;
 		}
 		return TRUE; /* newsrc created */
@@ -281,9 +292,12 @@ open_subscription_fp(
 	if (!read_saved_news) {
 #ifdef NNTP_ABLE
 		if (read_news_via_nntp) {
-			/* if (nntp_caps.type = 2 && !nntp_caps.list_subscriptions)
+			/*
+			 * draft-elie-nntp-list-additions-00.txt
+			 */
+			if (nntp_caps.type == CAPABILITIES && !nntp_caps.list_subscriptions)
 				return NULL;
-			else */
+			else
 				return (nntp_command("LIST SUBSCRIPTIONS", OK_GROUPS, NULL, 0));
 		} else
 #endif /* NNTP_ABLE */
@@ -305,6 +319,7 @@ auto_subscribe_groups(
 	FILE *fp_newsrc;
 	FILE *fp_subs;
 	char *ptr;
+	int err;
 
 	/*
 	 * If subscription file exists then first unsubscribe to all groups
@@ -316,8 +331,10 @@ auto_subscribe_groups(
 	if (!batch_mode)
 		wait_message(0, _(txt_autosubscribing_groups));
 
-	if ((fp_newsrc = fopen(newsrc_file, "w" FOPEN_OPTS)) == NULL)
+	if ((fp_newsrc = fopen(newsrc_file, "w")) == NULL) {
+		TIN_FCLOSE(fp_subs);
 		return;
+	}
 
 	if (newsrc_mode)
 		fchmod(fileno(fp_newsrc), newsrc_mode);
@@ -325,15 +342,20 @@ auto_subscribe_groups(
 	/* TODO: test me! */
 	while ((ptr = tin_fgets(fp_subs, FALSE)) != NULL) {
 		if (ptr[0] != '#') {
-			if (group_find(ptr) != 0)
+			if (group_find(ptr, FALSE) != 0)
 				fprintf(fp_newsrc, "%s:\n", ptr);
 		}
 	}
 
 	/* We ignore user 'q'uits here. They will get them next time in any case */
 
-	if (ferror(fp_newsrc) || fclose(fp_newsrc))
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
+	if ((err = ferror(fp_newsrc)) || fclose(fp_newsrc)) {
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		if (err) {
+			clearerr(fp_newsrc);
+			fclose(fp_newsrc);
+		}
+	}
 
 	TIN_FCLOSE(fp_subs);
 }
@@ -352,23 +374,23 @@ backup_newsrc(
 
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && !read_saved_news && nntp_tcp_port != IPPORT_NNTP)
-		snprintf(filebuf, sizeof(filebuf), "%s:%d", nntp_server, nntp_tcp_port);
+		snprintf(filebuf, sizeof(filebuf), "%s:%u", nntp_server, nntp_tcp_port);
 	else
 #endif /* NNTP_ABLE */
 	{
 		STRCPY(filebuf, quote_space_to_dash(nntp_server));
 	}
-	JOINPATH(dirbuf, rcdir, filebuf);
-	joinpath(filebuf, dirbuf, OLDNEWSRC_FILE);
+	joinpath(dirbuf, sizeof(dirbuf), rcdir, filebuf);
+	joinpath(filebuf, sizeof(filebuf), dirbuf, OLDNEWSRC_FILE);
 
 	if (-1 == stat(dirbuf, &statbuf)) {
 		if (-1 == my_mkdir(dirbuf, (mode_t) (S_IRWXU)))
 			/* Can't create directory: Fall back on Homedir */
-			joinpath(filebuf, homedir, OLDNEWSRC_FILE);
+			joinpath(filebuf, sizeof(filebuf), homedir, OLDNEWSRC_FILE);
 	}
 
 	if (!backup_file(newsrc, filebuf))
-		error_message(_(txt_filesystem_full_backup), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full_backup), NEWSRC_FILE);
 }
 
 
@@ -388,33 +410,33 @@ group_get_art_info(
 {
 	DIR *dir;
 	DIR_BUF *direntry;
-	char buf[NNTP_STRLEN];
 	long artnum;
 
 	if (read_news_via_nntp && grouptype == GROUP_TYPE_NEWS) {
 #ifdef NNTP_ABLE
 		char line[NNTP_STRLEN];
 
-		snprintf(buf, sizeof(buf), "GROUP %s", groupname);
+		snprintf(line, sizeof(line), "GROUP %s", groupname);
 #	ifdef DEBUG
-		debug_nntp("group_get_art_info", buf);
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "group_get_art_info %s", line);
 #	endif /* DEBUG */
-		put_server(buf);
+		put_server(line);
 
 		switch (get_respcode(line, sizeof(line))) {
 			case OK_GROUP:
 				if (sscanf(line, "%ld %ld %ld", art_count, art_min, art_max) != 3)
-					error_message(_(txt_error_invalid_response_to_group), line);
+					error_message(2, _(txt_error_invalid_response_to_group), line);
 				break;
 
 			case ERR_NOGROUP:
-				*art_count = 0;
-				*art_min = 1;
-				*art_max = 0;
+				*art_count = 0L;
+				*art_min = 1L;
+				*art_max = 0L;
 				return -ERR_NOGROUP;
 
 			case ERR_ACCESS:
-				error_message("%s%s", cCRLF, line);
+				error_message(2, "%s%s", cCRLF, line);
 				tin_done(NNTP_ERROR_EXIT);
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -422,7 +444,8 @@ group_get_art_info(
 
 			default:
 #	ifdef DEBUG
-				debug_nntp("NOT_OK", line);
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "NOT_OK %s", line);
 #	endif /* DEBUG */
 				return -1;
 		}
@@ -431,19 +454,20 @@ group_get_art_info(
 		return 0;
 #endif /* NNTP_ABLE */
 	} else {
-		*art_count = 0;
-		*art_min = 1;
-		*art_max = 0;
+		char group_path[PATH_LEN];
+		*art_count = 0L;
+		*art_min = 0L;
+		*art_max = 0L;
 
-		make_base_group_path(tin_spooldir, groupname, buf);
+		make_base_group_path(tin_spooldir, groupname, group_path, sizeof(group_path));
 
-		if ((dir = opendir(buf)) != NULL) {
+		if ((dir = opendir(group_path)) != NULL) {
 			while ((direntry = readdir(dir)) != NULL) {
 				artnum = atol(direntry->d_name); /* should be '\0' terminated... */
-				if (artnum >= 1) {
+				if (artnum >= 1L) {
 					if (artnum > *art_max) {
 						*art_max = artnum;
-						if (*art_min == 0)
+						if (*art_min == 0L)
 							*art_min = artnum;
 					} else if (artnum < *art_min)
 						*art_min = artnum;
@@ -451,8 +475,12 @@ group_get_art_info(
 				}
 			}
 			CLOSEDIR(dir);
-		} else
+			if (*art_min == 0L)
+				*art_min = 1L;
+		} else {
+			*art_min = 1L;
 			return -1;
+		}
 	}
 
 	return 0;
@@ -473,8 +501,10 @@ get_subscribe_info(
 
 	if (grp->newsrc.num_unread > grp->count) {
 #ifdef DEBUG
-		my_printf(cCRLF "Unread WRONG %s unread=[%ld] count=[%ld]", grp->name, grp->newsrc.num_unread, grp->count);
-		my_flush();
+		if (debug & DEBUG_NEWSRC) { /* TODO: is this the right debug-level? */
+			my_printf(cCRLF "Unread WRONG %s unread=[%ld] count=[%ld]", grp->name, grp->newsrc.num_unread, grp->count);
+			my_flush();
+		}
 #endif /* DEBUG */
 		grp->newsrc.num_unread = grp->count;
 	}
@@ -482,8 +512,10 @@ get_subscribe_info(
 	if (grp->xmin != oldmin || grp->xmax != oldmax) {
 		expand_bitmap(grp, 0);
 #ifdef DEBUG
-		my_printf(cCRLF "Min/Max DIFF %s old=[%ld-%ld] new=[%ld-%ld]", grp->name, oldmin, oldmax, grp->xmin, grp->xmax);
-		my_flush();
+		if (debug & DEBUG_NEWSRC) { /* TODO: is this the right debug-level? */
+			my_printf(cCRLF "Min/Max DIFF %s old=[%ld-%ld] new=[%ld-%ld]", grp->name, oldmin, oldmax, grp->xmin, grp->xmax);
+			my_flush();
+		}
 #endif /* DEBUG */
 	}
 }
@@ -510,7 +542,7 @@ subscribe(
 	if (no_write)
 		return;
 
-	if ((newfp = fopen(newnewsrc, "w" FOPEN_OPTS)) == NULL)
+	if ((newfp = fopen(newnewsrc, "w")) == NULL)
 		return;
 
 	if (newsrc_mode)
@@ -518,18 +550,19 @@ subscribe(
 
 	if ((fp = fopen(newsrc, "r")) != NULL) {
 		while ((line = tin_fgets(fp, FALSE)) != NULL) {
-			seq = parse_newsrc_line(line, &sub);
-			if (STRCMPEQ(line, group->name)) {
-				fprintf(newfp, "%s%c %s\n", line, sub_state, seq);
-				group->subscribed = SUB_BOOL(sub_state);
+			if ((seq = parse_newsrc_line(line, &sub))) {
+				if (STRCMPEQ(line, group->name)) {
+					fprintf(newfp, "%s%c %s\n", line, sub_state, seq);
+					group->subscribed = SUB_BOOL(sub_state);
 
-				/* If previously subscribed to in .newsrc, load up any existing information */
-				if (sub_state == SUBSCRIBED)
-					parse_bitmap_seq(group, seq);
+					/* If previously subscribed to in .newsrc, load up any existing information */
+					if (sub_state == SUBSCRIBED)
+						parse_bitmap_seq(group, seq);
 
-				found = TRUE;
-			} else
-				fprintf(newfp, "%s%c %s\n", line, sub, seq);
+					found = TRUE;
+				} else
+					fprintf(newfp, "%s%c %s\n", line, sub, seq);
+			}
 		}
 
 		fclose(fp);
@@ -549,8 +582,12 @@ subscribe(
 		}
 	}
 
-	if (ferror(newfp) || fclose(newfp)) {
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
+	if ((sub = ferror(newfp)) || fclose(newfp)) {
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		if (sub) {
+			clearerr(newfp);
+			fclose(newfp);
+		}
 		unlink(newnewsrc);
 	} else
 		rename_file(newnewsrc, newsrc);
@@ -567,7 +604,7 @@ reset_newsrc(
 	int sub;
 	long i;
 
-	if (!no_write && (newfp = fopen(newnewsrc, "w" FOPEN_OPTS)) != NULL) {
+	if (!no_write && (newfp = fopen(newnewsrc, "w")) != NULL) {
 		if (newsrc_mode)
 			fchmod(fileno(newfp), newsrc_mode);
 
@@ -578,8 +615,12 @@ reset_newsrc(
 			}
 			fclose(fp);
 		}
-		if (ferror(newfp) || fclose(newfp)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		if ((sub = ferror(newfp)) || fclose(newfp)) {
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			if (sub) {
+				clearerr(newfp);
+				fclose(newfp);
+			}
 			unlink(newnewsrc);
 		} else
 			rename_file(newnewsrc, newsrc);
@@ -606,23 +647,26 @@ delete_group(
 	if (no_write)
 		return;
 
-	if ((newfp = fopen(newnewsrc, "w" FOPEN_OPTS)) != NULL) {
+	if ((newfp = fopen(newnewsrc, "w")) != NULL) {
 		if (newsrc_mode)
 			fchmod(fileno(newfp), newsrc_mode);
 
 		if ((fp = fopen(newsrc, "r")) != NULL) {
 			while ((line = tin_fgets(fp, FALSE)) != NULL) {
-				seq = parse_newsrc_line(line, &sub);
-
-				if (!STRCMPEQ(line, group))
-					fprintf(newfp, "%s%c %s\n", line, sub, seq);
-
+				if ((seq = parse_newsrc_line(line, &sub))) {
+					if (!STRCMPEQ(line, group))
+						fprintf(newfp, "%s%c %s\n", line, sub, seq);
+				}
 			}
 			fclose(fp);
 		}
 
-		if (ferror(newfp) || fclose(newfp)) {
-			error_message(_(txt_filesystem_full), NEWSRC_FILE);
+		if ((sub = ferror(newfp)) || fclose(newfp)) {
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			if (sub) {
+				clearerr(newfp);
+				fclose(newfp);
+			}
 			unlink(newnewsrc);
 		} else
 			rename_file(newnewsrc, newsrc);
@@ -643,21 +687,22 @@ grp_mark_read(
 {
 	int i;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("c/C command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("c/C command");
+#endif /* DEBUG */
 
 	if (art != NULL) {
 		for_each_art(i)
 			art_mark(group, &art[i], ART_READ);
+	} else {
+		FreeAndNull(group->newsrc.xbitmap);
+		group->newsrc.xbitlen = 0;
+		if (group->xmax > group->newsrc.xmax)
+			group->newsrc.xmax = group->xmax;
+		group->newsrc.xmin = group->newsrc.xmax + 1;
+		group->newsrc.num_unread = 0;
 	}
-
-	FreeAndNull(group->newsrc.xbitmap);
-	group->newsrc.xbitlen = 0;
-	if (group->xmax > group->newsrc.xmax)
-		group->newsrc.xmax = group->xmax;
-	group->newsrc.xmin = group->newsrc.xmax + 1;
-	group->newsrc.num_unread = 0;
 }
 
 
@@ -666,11 +711,12 @@ grp_mark_unread(
 	struct t_group *group)
 {
 	int bitlength;
-	t_bitmap *newbitmap = (t_bitmap *)0;
+	t_bitmap *newbitmap = (t_bitmap *) 0;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Z command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("Z command");
+#endif /* DEBUG */
 
 	group_get_art_info(group->spooldir, group->name, group->type, &group->count, &group->xmax, &group->xmin);
 
@@ -692,9 +738,10 @@ grp_mark_unread(
 	if (bitlength)
 		NSETRNG1(group->newsrc.xbitmap, 0L, bitlength - 1L);
 
-#ifdef DEBUG_NEWSRC
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_bitmap(group, NULL);
+#endif /* DEBUG */
 }
 
 
@@ -705,9 +752,10 @@ thd_mark_read(
 {
 	int i;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Mark thread read K command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("Mark thread read K command");
+#endif /* DEBUG */
 
 	for (i = (int) thread; i >= 0; i = arts[i].thread)
 		art_mark(group, &arts[i], ART_READ);
@@ -721,9 +769,10 @@ thd_mark_unread(
 {
 	int i;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Mark thread unread Z command");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_comment("Mark thread unread Z command");
+#endif /* DEBUG */
 
 	for (i = (int) thread; i >= 0; i = arts[i].thread)
 		art_mark(group, &arts[i], ART_WILL_RETURN);
@@ -752,15 +801,15 @@ parse_bitmap_seq(
 	while (ptr && *ptr && (*ptr < '0' || *ptr > '9'))
 		ptr++;
 
-#ifdef DEBUG_NEWSRC
-	{
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
 		char buf[NEWSRC_LINE];
 
 		snprintf(buf, sizeof(buf), "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 20), ptr);
 		debug_print_comment(buf);
 		debug_print_bitmap(group, NULL);
 	}
-#endif /* DEBUG_NEWSRC */
+#endif /* DEBUG */
 
 	if (ptr) {
 		gotseq = TRUE;
@@ -828,9 +877,10 @@ wait_message(2, "BITMAP Grp=[%s] MinMax=[%ld-%ld] Len=[%ld]\n",
 		sum = (int) ((group->count >= 0) ? group->count : ((group->newsrc.xmax - group->newsrc.xmin) + 1));
 
 	group->newsrc.num_unread = sum;
-#ifdef DEBUG_NEWSRC
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC)
+		debug_print_bitmap(group, NULL);
+#endif /* DEBUG */
 }
 
 
@@ -952,11 +1002,12 @@ parse_get_seq(
 
 
 /*
- * Loop thru arts[] array marking state of each article READ/UNREAD
+ * Loop through arts[] array marking state of each article READ/UNREAD
  */
 void
 parse_unread_arts(
-	struct t_group *group)
+	struct t_group *group,
+	long min)
 {
 	int i;
 	long unread = 0;
@@ -981,6 +1032,24 @@ parse_unread_arts(
 		NSETRNG0(newbitmap, 0L, group->newsrc.xmax - bitmin);
 	}
 
+	/*
+	 * if getart_limit > 0 preserve read/unread state
+	 * of all articles below the new minimum
+	 */
+	if (min > 0 && newbitmap) {
+		long j, tmp_bitmax;
+
+		tmp_bitmax = (bitmax < min) ? bitmax : min;
+		for (j = bitmin; j < tmp_bitmax; j++) {
+			if (NTEST(group->newsrc.xbitmap, j - bitmin) != ART_READ)
+				NSET1(newbitmap, j - bitmin);
+		}
+		while (j < min) {
+			NSET1(newbitmap, j - bitmin);
+			j++;
+		}
+	}
+
 	for_each_art(i) {
 		if (arts[i].artnum < bitmin)
 			arts[i].status = ART_READ;
@@ -1024,13 +1093,15 @@ print_bitmap_seq(
 	struct t_group *group)
 {
 	long artnum;
-	long i, last;
+	long i;
 	t_bool flag = FALSE;
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("print_bitmap_seq()");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("print_bitmap_seq()");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 
 	if (group->count == 0 || group->xmin > group->xmax) {
 		if (group->newsrc.xmax > 1)
@@ -1038,41 +1109,45 @@ print_bitmap_seq(
 
 		fprintf(fp, "\n");
 		fflush(fp);
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("print_bitmap_seq(): group->count == 0");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("print_bitmap_seq(): group->count == 0");
+#endif /* DEBUG */
 		return;
 	}
 
-	for (i = group->newsrc.xmin; i <= group->newsrc.xmax; i++) {
-		if (group->newsrc.xbitmap && NTEST(group->newsrc.xbitmap, i - group->newsrc.xmin) == ART_READ) {
-			if (flag) {
-				artnum = i;
-				fprintf(fp, ",%ld", i);
-			} else {
-				artnum = 1;
+	i = group->newsrc.xmin;
+	if (i <= group->newsrc.xmax) {
+		forever {
+			if (group->newsrc.xbitmap && NTEST(group->newsrc.xbitmap, i - group->newsrc.xmin) == ART_READ) {
+				if (flag) {
+					artnum = i;
+					fprintf(fp, ",%ld", i);
+				} else {
+					artnum = 1;
+					flag = TRUE;
+					fprintf(fp, "1");
+				}
+				while (i < group->newsrc.xmax && NTEST(group->newsrc.xbitmap, (i + 1) - group->newsrc.xmin) == ART_READ)
+					i++;
+
+				if (artnum != i)
+					fprintf(fp, "-%ld", i);
+
+			} else if (!flag) {
 				flag = TRUE;
-				fprintf(fp, "1");
+				if (group->newsrc.xmin > 1) {
+					fprintf(fp, "1");
+
+					if (group->newsrc.xmin > 2)
+						fprintf(fp, "-%ld", group->newsrc.xmin - 1);
+
+				}
 			}
+			if (group->newsrc.xmax == i)
+				break;
+
 			i++;
-
-			while (i <= group->newsrc.xmax && NTEST(group->newsrc.xbitmap, i - group->newsrc.xmin) == ART_READ)
-				i++;
-
-			last = i - 1;
-
-			if (artnum != last)
-				fprintf(fp, "-%ld", last);
-
-		} else if (!flag) {
-			flag = TRUE;
-			if (group->newsrc.xmin > 1) {
-				fprintf(fp, "1");
-
-				if (group->newsrc.xmin > 2)
-					fprintf(fp, "-%ld", group->newsrc.xmin - 1);
-
-			}
 		}
 	}
 
@@ -1082,9 +1157,10 @@ print_bitmap_seq(
 		if (group->newsrc.xmin > 2)
 			fprintf(fp, "-%ld", group->newsrc.xmin - 1);
 
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("print_bitmap_seq(): !flag && group->newsrc.xmin > 1");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("print_bitmap_seq(): !flag && group->newsrc.xmin > 1");
+#endif /* DEBUG */
 	}
 
 	fprintf(fp, "\n");
@@ -1093,7 +1169,7 @@ print_bitmap_seq(
 
 
 /*
- * rewrite .newsrc and position group at specifed position
+ * rewrite .newsrc and position group at specified position
  */
 t_bool
 pos_group_in_newsrc(
@@ -1104,10 +1180,11 @@ pos_group_in_newsrc(
 	FILE *fp_sub = NULL, *fp_unsub = NULL;
 	char *newsgroup = NULL;
 	char *line;
-	char buf[HEADER_LEN];
-	char sub[HEADER_LEN];
-	char unsub[HEADER_LEN];
+	char filename[PATH_LEN];
+	char sub[PATH_LEN];
+	char unsub[PATH_LEN];
 	int subscribed_pos = 1;
+	int err;
 	size_t group_len;
 	t_bool found = FALSE;
 	t_bool newnewsrc_created = FALSE;
@@ -1123,7 +1200,7 @@ pos_group_in_newsrc(
 	if ((fp_in = fopen(newsrc, "r")) == NULL)
 		goto rewrite_group_done;
 
-	if ((fp_out = fopen(newnewsrc, "w" FOPEN_OPTS)) == NULL)
+	if ((fp_out = fopen(newnewsrc, "w")) == NULL)
 		goto rewrite_group_done;
 
 	newnewsrc_created = TRUE;
@@ -1131,26 +1208,18 @@ pos_group_in_newsrc(
 	if (newsrc_mode)
 		fchmod(fileno(fp_out), newsrc_mode);
 
-#ifdef VMS
-	joinpath(buf, TMPDIR, "subrc");
-	snprintf(sub, sizeof(sub), "%s.%d", buf, (int) process_id);
+	joinpath(filename, sizeof(filename), TMPDIR, ".subrc");
+	snprintf(sub, sizeof(sub), "%s.%ld", filename, (long) process_id);
 
-	joinpath(buf, TMPDIR, "unsubrc");
-	snprintf(unsub, sizeof(unsub), "%s.%d", buf, (int) process_id);
-#else
-	joinpath(buf, TMPDIR, ".subrc");
-	snprintf(sub, sizeof(sub), "%s.%d", buf, (int) process_id);
+	joinpath(filename, sizeof(filename), TMPDIR, ".unsubrc");
+	snprintf(unsub, sizeof(unsub), "%s.%ld", filename, (long) process_id);
 
-	joinpath(buf, TMPDIR, ".unsubrc");
-	snprintf(unsub, sizeof(unsub), "%s.%d", buf, (int) process_id);
-#endif /* !VMS */
-
-	if ((fp_sub = fopen(sub, "w" FOPEN_OPTS)) == NULL)
+	if ((fp_sub = fopen(sub, "w")) == NULL)
 		goto rewrite_group_done;
 
 	sub_created = TRUE;
 
-	if ((fp_unsub = fopen(unsub, "w" FOPEN_OPTS)) == NULL)
+	if ((fp_unsub = fopen(unsub, "w")) == NULL)
 		goto rewrite_group_done;
 
 	unsub_created = TRUE;
@@ -1175,11 +1244,25 @@ pos_group_in_newsrc(
 		}
 	}
 
-	if (ferror(fp_sub) || fclose(fp_sub) || ferror(fp_unsub) || fclose(fp_unsub)) {
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
-		fp_sub = fp_unsub = NULL;
-		goto rewrite_group_done;
+	if ((err = ferror(fp_sub)) || fclose(fp_sub)) {
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		if (err) {
+			clearerr(fp_sub);
+			fclose(fp_sub);
+		}
+		fp_sub = NULL;
 	}
+	if ((err = ferror(fp_unsub)) || fclose(fp_unsub)) {
+		if (fp_sub) /* didn't see and error above */
+			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		if (err) {
+			clearerr(fp_unsub);
+			fclose(fp_unsub);
+		}
+		fp_unsub = NULL;
+	}
+	if (fp_sub == NULL || fp_unsub == NULL)
+		goto rewrite_group_done;
 
 	fp_sub = fp_unsub = NULL;
 	fclose(fp_in);
@@ -1233,9 +1316,13 @@ pos_group_in_newsrc(
 	/*
 	 * Try and cleanly close out the newnewsrc file
 	 */
-	if (ferror(fp_out) || fclose(fp_out))
-		error_message(_(txt_filesystem_full), NEWSRC_FILE);
-	else {
+	if ((err = ferror(fp_out)) || fclose(fp_out)) {
+		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		if (err) {
+			clearerr(fp_out);
+			fclose(fp_out);
+		}
+	} else {
 		if (repositioned) {
 			rename_file(newnewsrc, newsrc);
 			ret_code = TRUE;
@@ -1375,18 +1462,20 @@ expand_bitmap(
 		bitlen = 0;
 		FreeIfNeeded(group->newsrc.xbitmap);
 		group->newsrc.xbitmap = (t_bitmap *) 0;
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.bitlen == 0");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.bitlen == 0");
+#endif /* DEBUG */
 	} else if (group->newsrc.xbitmap == NULL) {
 		group->newsrc.xbitmap = my_malloc(BITS_TO_BYTES(bitlen));
 		if (group->newsrc.xmin > first)
 			NSETRNG0(group->newsrc.xbitmap, 0L, group->newsrc.xmin - first - 1L);
 		if (bitlen > group->newsrc.xmin - first)
 			NSETRNG1(group->newsrc.xbitmap, group->newsrc.xmin - first, bitlen - 1);
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.xbitmap == NULL");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.xbitmap == NULL");
+#endif /* DEBUG */
 	} else if (need_full_copy) {
 		t_bitmap *newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
 
@@ -1404,15 +1493,12 @@ expand_bitmap(
 		if (first < group->newsrc.xmin) {
 			NSETRNG0(newbitmap, 0L, group->newsrc.xmin - first - 1L);
 		}
-		{
-			long i;
 
-			for (i = group->newsrc.xmin; i < min; i++) {
-				if (NTEST(newbitmap, i - first) != ART_READ) {
-					NSET0(newbitmap, i - first);
-					if (group->newsrc.num_unread)
-						group->newsrc.num_unread--;
-				}
+		for (tmp = group->newsrc.xmin; tmp < min; tmp++) {
+			if (NTEST(newbitmap, tmp - first) != ART_READ) {
+				NSET0(newbitmap, tmp - first);
+				if (group->newsrc.num_unread)
+					group->newsrc.num_unread--;
 			}
 		}
 
@@ -1425,9 +1511,10 @@ expand_bitmap(
 
 		free(group->newsrc.xbitmap);
 		group->newsrc.xbitmap = newbitmap;
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1 and need full copy");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1 and need full copy");
+#endif /* DEBUG */
 	} else if (max != group->newsrc.xmax || first != group->newsrc.xmin) {
 		t_bitmap *newbitmap;
 		newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
@@ -1443,15 +1530,12 @@ expand_bitmap(
 		if (first < group->newsrc.xmin) {
 			NSETRNG0(newbitmap, 0L, group->newsrc.xmin - first - 1L);
 		}
-		{
-			long i;
 
-			for (i = group->newsrc.xmin; i < min; i++) {
-				if (NTEST(newbitmap, i - first) != ART_READ) {
-					NSET0(newbitmap, i - first);
-					if (group->newsrc.num_unread)
-						group->newsrc.num_unread--;
-				}
+		for (tmp = group->newsrc.xmin; tmp < min; tmp++) {
+			if (NTEST(newbitmap, tmp - first) != ART_READ) {
+				NSET0(newbitmap, tmp - first);
+				if (group->newsrc.num_unread)
+					group->newsrc.num_unread--;
 			}
 		}
 
@@ -1464,9 +1548,10 @@ expand_bitmap(
 
 		free(group->newsrc.xbitmap);
 		group->newsrc.xbitmap = newbitmap;
-#ifdef DEBUG_NEWSRC
-		debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+		if (debug & DEBUG_NEWSRC)
+			debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1");
+#endif /* DEBUG */
 	}
 	group->newsrc.xmin = first;
 	if (group->newsrc.xmax < max)
@@ -1491,15 +1576,21 @@ art_mark(
 			if (group != NULL) {
 				if (art->artnum >= group->newsrc.xmin && art->artnum <= group->newsrc.xmax)
 					NSET0(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
-#ifdef DEBUG_NEWSRC
-				debug_print_bitmap(group, art);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+				if (debug & DEBUG_NEWSRC)
+					debug_print_bitmap(group, art);
+#endif /* DEBUG */
 			}
 			if ((art->status == ART_UNREAD) || (art->status == ART_WILL_RETURN)) {
 				art_mark_xref_read(art);
 
-				if (group != NULL && group->newsrc.num_unread)
-					group->newsrc.num_unread--;
+				if (group != NULL) {
+					if (group->newsrc.num_unread)
+						group->newsrc.num_unread--;
+
+					if (group->attribute->show_only_unread_arts)
+						art->keep_in_base = TRUE;
+				}
 
 				art->status = ART_READ;
 			}
@@ -1508,9 +1599,13 @@ art_mark(
 		case ART_UNREAD:
 		case ART_WILL_RETURN:
 			if (art->status == ART_READ) {
-				if (group != NULL)
+				if (group != NULL) {
 					group->newsrc.num_unread++;
 
+					if (group->attribute->show_only_unread_arts)
+						art->keep_in_base = FALSE;
+				}
+
 				art->status = flag;
 			}
 			if (group != NULL) {
@@ -1518,9 +1613,10 @@ art_mark(
 					expand_bitmap(group, art->artnum);
 				else {
 					NSET1(group->newsrc.xbitmap, art->artnum - group->newsrc.xmin);
-#ifdef DEBUG_NEWSRC
-					debug_print_bitmap(group, art);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+					if (debug & DEBUG_NEWSRC)
+						debug_print_bitmap(group, art);
+#endif /* DEBUG */
 				}
 			}
 			break;
@@ -1550,144 +1646,3 @@ set_default_bitmap(
 		group->newsrc.xmax = group->newsrc.xmin - 1;
 	}
 }
-
-
-/* TEST harness */
-#ifdef DEBUG_NEWSRC
-#	if 0
-static void set_bitmap_range_read(struct t_newsrc *my_newsrc, long beg, long end);
-static void set_bitmap_range_unread(struct t_newsrc *my_newsrc, long beg, long end);
-
-void
-newsrc_test_harness(
-	void)
-{
-	FILE *fp = NULL;
-	char seq[20000];
-	int i;
-	int retry = 10; /* max. retrys */
-	long rng_min, rng_max;
-	struct t_group group;
-
-	/*
-	 * those env.-vars are not documented:
-	 * $TIN_RNG_MIN, $TIN_RNG_MAX, $TIN_MIN, $TIN_MAX, $TIN_COUNT
-	 * $TIN_SEQ
-	 */
-	rng_min = atoi(get_val("TIN_RNG_MIN", "1"));
-	rng_max = atoi(get_val("TIN_RNG_MAX", "1"));
-
-	group.name = my_strdup("test.newsrc");
-	group.xmin = atoi(get_val("TIN_MIN", "1"));
-	group.xmax = atoi(get_val("TIN_MAX", "0"));
-	group.count = atoi(get_val("TIN_COUNT", "-1"));
-	group.type = GROUP_TYPE_NEWS;
-	group.subscribed = TRUE;
-	group.newsrc.xbitmap = (t_bitmap *) 0;
-	set_default_bitmap(&group);
-
-	strcpy(seq, get_val("TIN_SEQ", ""));
-
-	fprintf(stderr, "\nENV Min=[%ld] Max=[%ld] Rng=[%ld-%ld] Count=[%ld] Seq=[%s]\n",
-		group.xmin, group.xmax, rng_min, rng_max, group.count, seq);
-
-	for (i = 0; i < 3; i++) {
-		if (group.newsrc.xbitmap != NULL) {
-			free(group.newsrc.xbitmap);
-			group.newsrc.xbitmap = (t_bitmap *) 0;
-			group.newsrc.xbitlen = 0;
-			group.newsrc.num_unread = 0;
-			group.newsrc.xmin = 1;
-			group.newsrc.xmax = 0;
-		}
-		while (retry) {
-		/* FIXME - this is secure now, but doesn't write any debug output */
-		/* (it didn't before too) */
-			if ((fp = tmpfile()) != NULL) {
-				fprintf(stderr, "\n%d. PARSE Seq=[%s]\n", i + 1, seq);
-				parse_bitmap_seq(&group, seq);
-				debug_print_newsrc(&group.newsrc, stderr);
-				print_bitmap_seq(fp, &group);
-				fprintf(stderr, "   PRINT Seq=");
-				print_bitmap_seq(stderr, &group);
-				rewind(fp);
-				break;
-			} else
-				retry--;
-		}
-
-		debug_print_newsrc(&group.newsrc, stderr);
-
-		if (!retry || !fp)
-			error_message(_(txt_cannot_create_uniq_name));
-		else {
-			fgets(seq, (int) sizeof(seq), fp);
-			seq[strlen(seq) - 1] = '\0';
-			fclose(fp);
-		}
-	}
-	set_bitmap_range_read(&group.newsrc, rng_min, rng_max);
-	debug_print_newsrc(&group.newsrc, stderr);
-
-	set_bitmap_range_unread(&group.newsrc, rng_min, rng_max);
-	debug_print_newsrc(&group.newsrc, stderr);
-
-	if (group.newsrc.xbitmap != NULL)
-		NSETBLK0(group.newsrc.xbitmap, group.newsrc.xbitlen);
-	debug_print_newsrc(&group.newsrc, stderr);
-
-	if (group.newsrc.xbitmap != NULL)
-		NSETBLK1(group.newsrc.xbitmap, group.newsrc.xbitlen);
-	debug_print_newsrc(&group.newsrc, stderr);
-	my_printf("\n");
-}
-
-
-static void
-set_bitmap_range_read(
-	struct t_newsrc *my_newsrc,
-	long beg,
-	long end)
-{
-	long length, offset;
-
-	if (beg >= my_newsrc->xmin && end <= my_newsrc->xmax) {
-		offset = beg - my_newsrc->xmin;
-		length = end - my_newsrc->xmin;
-
-fprintf(stderr, "\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
-my_newsrc->xmin, my_newsrc->xmax, beg, end, offset, length);
-
-		if (beg == end) {
-			NSET0(my_newsrc->xbitmap, offset);
-		} else {
-			NSETRNG0(my_newsrc->xbitmap, offset, length);
-		}
-	}
-}
-
-
-static void
-set_bitmap_range_unread(
-	struct t_newsrc *my_newsrc,
-	long beg,
-	long end)
-{
-	long length, offset;
-
-	if (beg >= my_newsrc->xmin && end <= my_newsrc->xmax) {
-		offset = beg - my_newsrc->xmin;
-		length = end - my_newsrc->xmin;
-
-fprintf(stderr, "\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
-my_newsrc->xmin, my_newsrc->xmax, beg, end, offset, length);
-
-		if (beg == end) {
-			NSET1(my_newsrc->xbitmap, offset);
-		} else {
-			NSETRNG1(my_newsrc->xbitmap, offset, length);
-		}
-	}
-}
-#	endif /* 0 */
-#endif /* DEBUG_NEWSRC */
