@@ -3,10 +3,10 @@
  *  Module    : prompt.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2005-10-19
+ *  Updated   : 2009-10-22
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,6 +44,8 @@
 
 
 static char *prompt_slk_message;	/* prompt message for prompt_slk_redraw */
+static char *prompt_yn_message;
+static char *prompt_yn_choice;
 
 /*
  * Local prototypes
@@ -139,7 +141,7 @@ prompt_menu_string(
 	 * connection right before a resync_active() call
 	 * would lead to a 'n' answer to the reconnect prompt
 	 */
-	fflush(stdin);
+	/* fflush(stdin); */
 	MoveCursor(line, 0);
 	if ((p = tin_getline(prompt, FALSE, var, 0, FALSE, HIST_OTHER)) == NULL)
 		return FALSE;
@@ -163,11 +165,10 @@ prompt_yn(
 {
 	char *keyprompt;
 	char keyno[MAXKEYLEN], keyyes[MAXKEYLEN];
-	int keyyes_len = 0, keyno_len = 0, maxlen;
+	int keyyes_len = 0, keyno_len = 0, maxlen, prompt_len;
 	t_function func;
-#if defined (MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	wint_t yes, no, prompt_ch, ch;
-	wchar_t *wtmp;
 #else
 	char yes, no, prompt_ch;
 	int ch;
@@ -178,41 +179,30 @@ prompt_yn(
 	yes = func_to_key(PROMPT_YES, prompt_keys);
 	no = func_to_key(PROMPT_NO, prompt_keys);
 
-#if defined (MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	printascii(keyyes, (default_answer ? towupper(yes) : yes));
-	if ((wtmp = char2wchar_t(keyyes))) {
-		keyyes_len = wcswidth(wtmp, wcslen(wtmp));
-		free(wtmp);
-	}
 	printascii(keyno, (!default_answer ? towupper(no) : no));
-	if ((wtmp = char2wchar_t(keyno))) {
-		keyno_len = wcswidth(wtmp, wcslen(wtmp));
-		free(wtmp);
-	}
 #else
 	printascii(keyyes, (default_answer ? toupper(yes) : yes));
 	printascii(keyno, (!default_answer ? toupper(no) : no));
-	keyyes_len = (int) strlen(keyyes);
-	keyno_len = (int) strlen(keyno);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	keyyes_len = strwidth(keyyes);
+	keyno_len = strwidth(keyno);
 	maxlen = MAX(keyyes_len, keyno_len);
+	prompt_len = keyyes_len + keyno_len + maxlen + 6;
+	prompt_yn_message = my_strdup(prompt);
+	prompt_yn_choice = my_malloc(prompt_len + 1);
+
+	input_context = cPromptYN;
 
 	do {
 		prompt_ch = (default_answer ? yes : no);
 		keyprompt = (default_answer ? keyyes : keyno);
 
-		if (!cmd_line) {
-			MoveCursor(cLINES, 0);
-			CleartoEOLN();
-		}
-		my_printf("%s (%s/%s) %-*s", prompt, keyyes, keyno, maxlen, keyprompt);
-		if (!cmd_line)
-			cursoron();
-		my_flush();
-		if (!cmd_line)
-			MoveCursor(cLINES, (int) strlen(prompt) + keyyes_len + keyno_len + 5);
+		snprintf(prompt_yn_choice, prompt_len, " (%s/%s) %-*s", keyyes, keyno, maxlen, keyprompt);
+		prompt_yn_redraw();
 
-#if defined (MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if (((ch = ReadWch()) == '\n') || (ch == '\r'))
 #else
 		if (((ch = (char) ReadCh()) == '\n') || (ch == '\r'))
@@ -250,6 +240,10 @@ prompt_yn(
 		func = key_to_func(ch, prompt_keys);
 	} while (func == NOT_ASSIGNED);
 
+	input_context = cNone;
+	FreeAndNull(prompt_yn_message);
+	FreeAndNull(prompt_yn_choice);
+
 	if (!cmd_line) {
 		clear_message();
 		my_flush();
@@ -258,6 +252,37 @@ prompt_yn(
 }
 
 
+/*
+ * (Re)draws and resize the prompt message for prompt_yn()
+ */
+void
+prompt_yn_redraw(
+	void)
+{
+	char *buf;
+	int choice_len = strwidth(prompt_yn_choice);
+	int message_len = strwidth(prompt_yn_message);
+
+	if (!cmd_line) {
+		MoveCursor(cLINES, 0);
+		CleartoEOLN();
+	}
+	if (message_len + choice_len > cCOLS - 1) {
+		buf = strunc(prompt_yn_message, cCOLS - choice_len - 1);
+		message_len = strwidth(buf);
+		my_printf("%s%s", buf, prompt_yn_choice);
+		free(buf);
+	} else
+		my_printf("%s%s", prompt_yn_message, prompt_yn_choice);
+
+	if (!cmd_line)
+		cursoron();
+	my_flush();
+	if (!cmd_line)
+		MoveCursor(cLINES, (message_len + choice_len) - 1);
+}
+
+
 /*
  * help_text is displayed near the bottom of the screen.
  * var is an index into a list containing size elements.
@@ -277,12 +302,11 @@ prompt_list(
 	int size)
 {
 	int ch, var_orig;
-	int i, offset;
+	int i, offset, width = 0;
 	int change;
 	int adjust = (strcasecmp(_(list[0]), _(txt_default)) == 0);
-	size_t width = 0;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *wbuf;
+	char *buf;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	var += adjust;
@@ -293,19 +317,18 @@ prompt_list(
 	 * Find the length of longest printable text
 	 */
 	for (i = 0; i < size; i++)
-		width = MAX(width, strlen(_(list[i])));
+		width = MAX(width, strwidth(_(list[i])));
 
 	show_menu_help(help_text);
 	cursoron();
 
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	if ((wbuf = char2wchar_t(_(prompt_text))) != NULL) {
-		if ((offset = wcswidth(wbuf, wcslen(wbuf) + 1)) == -1) /* something went wrong, use wcslen as fallback */
-			offset = (int) wcslen(wbuf);
-		free(wbuf);
-	} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		offset = (int) strlen(_(prompt_text));
+	offset = strwidth(_(prompt_text));
+
+	/*
+	 * Make sure to not exceed cCOLS
+	 */
+	if (offset + width >= cCOLS)
+		width = cCOLS - offset - 1;
 
 	do {
 		MoveCursor(row, col + offset);
@@ -317,7 +340,7 @@ prompt_list(
 		 *   0 = do nothing
 		 *  -1 = move to the previous list element
 		 *
-		 *  if an arrow key was pressed change ch to an other value
+		 *  if an arrow key was pressed change ch to another value
 		 *  otherwise we will exit the while loop
 		 */
 		switch (ch) {
@@ -361,14 +384,26 @@ prompt_list(
 			else
 				var %= size;
 
-			my_printf("%-*s", (int) width, _(list[var]));
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			if ((buf = spart(_(list[var]), width, TRUE)) != NULL) {
+				my_printf("%s", buf);
+				free(buf);
+			} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+				my_printf("%-*s", width, _(list[var]));
 			my_flush();
 		}
 	} while (ch != '\r' && ch != '\n' && ch != ESC);
 
 	if (ch == ESC) {
 		var = var_orig;
-		my_printf("%-*s", (int) width, _(list[var]));
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		if ((buf = spart(_(list[var]), width, TRUE)) != NULL) {
+			my_printf("%s", buf);
+			free(buf);
+		} else
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+			my_printf("%-*s", width, _(list[var]));
 		my_flush();
 	}
 
@@ -426,10 +461,15 @@ prompt_option_string(
 {
 	char *variable = OPT_STRING_list[option_table[option].var_index];
 	char prompt[LEN];
+	char old_value[LEN];
 
+	STRCPY(old_value, variable);
 	show_menu_help(option_table[option].txt->help);
 	fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
-	return prompt_menu_string(option_row(option), prompt, variable);
+	if (prompt_menu_string(option_row(option), prompt, variable))
+		return strcmp(old_value, variable) ? TRUE : FALSE;
+	else
+		return FALSE;
 }
 
 
@@ -529,7 +569,7 @@ prompt_string_default(
 		my_strncpy(def, pattern, LEN);
 	else {
 		if (def[0] == '\0') {		/* no default - give up */
-			error_message(failtext);
+			error_message(2, failtext);
 			return NULL;
 		}
 	}
@@ -605,16 +645,8 @@ sized_message(
 {
 	char *buf;
 	int max_len;
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *wformat;
 
-	if ((wformat = char2wchar_t(format)) != NULL) {
-		/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
-		max_len = cCOLS - wcswidth(wformat, wcslen(wformat) + 1) + 2 - 1;
-		free(wformat);
-	} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		max_len = cCOLS - strlen(format) + 2 - 1;	/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
+	max_len = cCOLS - strwidth(format) + 2 - 1;	/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
 
 	buf = strunc(subject, max_len);
 
@@ -678,12 +710,11 @@ prompt_slk_response(
 		else
 			func = key_to_func(ch, keys);
 
+#if 1
 		/*
-		 * TODO: ignore special-keys which are represented as a
-		 *       multibyte ESC-seq to avoid interpreting them as 'ESC' only
-		 *       like it's done in the ugly code below.
+		 * ignore special-keys which are represented as a multibyte ESC-seq
+		 * to avoid interpreting them as 'ESC' only
 		 */
-#if 0
 		if (ch == ESC) {
 			switch (get_arrow_key(ch)) {
 				case KEYMAP_UP:
@@ -695,13 +726,14 @@ prompt_slk_response(
 				case KEYMAP_HOME:
 				case KEYMAP_END:
 					ch = '\0';
+					func = NOT_ASSIGNED;
 					break;
 
 				default:
 					break;
 			}
 		}
-#endif /* 0 */
+#endif /* 1 */
 	} while (func == NOT_ASSIGNED);
 
 	input_context = cNone;
@@ -718,20 +750,11 @@ prompt_slk_redraw(
 	void)
 {
 	int column;
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *wtmp;
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	wait_message(0, "%s", prompt_slk_message);
 
 	/* get the cursor _just_ right */
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	if ((wtmp = char2wchar_t(prompt_slk_message)) != NULL) {
-		column = wcswidth(wtmp, wcslen(wtmp) + 1) - 1;
-		free(wtmp);
-	} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		column = (int) strlen(prompt_slk_message) - 1;
+	column = strwidth(prompt_slk_message) - 1;
 	MoveCursor(cLINES, column);
 }
 
@@ -747,11 +770,14 @@ prompt_continue(
 	void)
 {
 	int ch;
+	int save_signal_context = signal_context;
 
 #ifdef USE_CURSES
 	cmd_line = TRUE;
 #endif /* USE_CURSES */
 	info_message(_(txt_return_key));
+	signal_context = cMain;
+	input_context = cPromptCONT;
 
 	switch ((ch = ReadCh())) {
 		case ESC:
@@ -765,6 +791,10 @@ prompt_continue(
 			break;
 	}
 
+	input_context = cNone;
+	signal_context = save_signal_context;
+	my_fputc('\n', stdout);
+
 #ifdef USE_CURSES
 	cmd_line = FALSE;
 	my_retouch();
