@@ -3,9 +3,9 @@
  *  Module    : read.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1997-04-10
- *  Updated   : 2005-07-02
+ *  Updated   : 2006-09-02
  *
- * Copyright (c) 1997-2006 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1997-2010 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -76,20 +76,6 @@ static t_bool
 wait_for_input(
 	void)
 {
-#	ifndef HAVE_SELECT
-#		ifdef VMS
-	int ch = ReadChNowait();
-
-	if (ch == iKeyQuit || ch == 'z' || ch == iKeyAbort) {
-		if (prompt_yn(cLINES, _(txt_read_abort), FALSE) == 1)
-			return TRUE;
-	}
-	if (ch == iKeyQuitTin) {
-		if (prompt_yn(cLINES, _(txt_read_exit), FALSE) == 1)
-			tin_done(EXIT_SUCCESS);
-	}
-#		endif /* VMS */
-#	else
 	int nfds, ch;
 	fd_set readfds;
 	struct timeval tv;
@@ -105,7 +91,7 @@ wait_for_input(
 		tv.tv_sec = 0;		/* NNTP_READ_TIMEOUT; */
 		tv.tv_usec = 0;
 
-/*DEBUG_IO((stderr, "waiting on %d and %d...", STDIN_FILENO, fileno(fd)));*/
+/* DEBUG_IO((stderr, "waiting on %d and %d...", STDIN_FILENO, fileno(fd))); */
 		if ((nfds = select(STDIN_FILENO + 1, &readfds, NULL, NULL, &tv)) == -1) {
 			if (errno != EINTR) {
 				perror_message("select() failed");
@@ -122,7 +108,6 @@ wait_for_input(
 		 * Something is waiting. See what's cooking...
 		 */
 		if (nfds > 0) {
-
 			/*
 			 * User pressed something. If 'q'uit, then handle this. Process
 			 * user input 1st so they get chance to quit on busy (or stalled)
@@ -157,22 +142,16 @@ wait_for_input(
 
 			}
 
-#		if 0
+#	if 0
 			/*
 			 * Our file has something for us to read
 			 */
 			if (FD_ISSET(fileno(NEED_NNTP_FD_HERE), &readfds))
 				return TRUE;
-#		endif /* 0 */
+#	endif /* 0 */
 		}
 
 	}
-#	endif /* !HAVE_SELECT */
-/*
- * FIXME: insert !HAVE_SELECT code here - NB: we already have VMS specific
- *        code at the top of this function
- */
-	return FALSE;
 }
 #endif /* NNTP_ABLE */
 
@@ -212,7 +191,7 @@ tin_read(
 		drain_buffer(fp);
 		clear_message();
 		tin_errno = TIN_ABORT;
-		fflush(stdin);
+		/* fflush(stdin); */
 		return NULL;
 	}
 
@@ -233,8 +212,8 @@ tin_read(
 
 /* TODO: develop this next line? */
 #ifdef DEBUG
-	if (errno)
-		fprintf(stderr, "errno in tin_read %d\n", errno);
+	if (errno && (debug & DEBUG_MISC))
+		fprintf(stderr, "tin_read(%s)", strerror(errno));
 #endif /* DEBUG */
 
 	if (ptr == 0)	/* End of data? */
