@@ -3,12 +3,12 @@
  *  Module    : refs.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 2005-07-20
+ *  Updated   : 2010-01-10
  *  Notes     : Cacheing of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2006 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1996-2010 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,15 +44,10 @@
 #define MAX_REFS	100			/* Limit recursion depth */
 #define REF_SEP	" \t"			/* Separator chars in ref headers */
 
-/* Produce disgusting amounts of output to help me tame this thing */
-#undef DEBUG_REFS
-
-#ifdef DEBUG_REFS
+#ifdef DEBUG
 #	define DEBUG_PRINT(x)	fprintf x
-FILE *dbgfd;
-#else
-#	define DEBUG_PRINT(x)
-#endif /* DEBUG_REFS */
+static FILE *dbgfd;
+#endif /* DEBUG */
 
 /*
  * local prototypes
@@ -61,24 +56,20 @@ static char *_get_references(struct t_msgid *refptr, int depth);
 static struct t_msgid *add_msgid(int key, const char *msgid, struct t_msgid *newparent);
 static struct t_msgid *find_next(struct t_msgid *ptr);
 static struct t_msgid *parse_references(char *r);
-static t_bool valid_msgid(const char *msgid);
+static t_bool valid_msgid(char *msgid);
 static unsigned int hash_msgid(const char *key);
 static void add_to_parent(struct t_msgid *ptr);
 static void build_thread(struct t_msgid *ptr);
-#ifdef DEBUG_REFS
-	static void dump_thread(FILE *fp, struct t_msgid *msgid, int level);
+static void rearrange_siblings(void);
+#ifdef DEBUG
 	static void dump_msgid_thread(struct t_msgid *ptr, int level);
 	static void dump_msgid_threads(void);
-#endif /* DEBUG_REFS */
+#endif /* DEBUG */
 #if 0
 	static void dump_msgids(void);
+	static void dump_thread(FILE *fp, struct t_msgid *msgid, int level);
 #endif /* 0 */
 
-/*
- * Set if the sorting algorithm goes 'upwards'
- */
-static t_bool sort_ascend;
-
 /*
  * The msgids are all hashed into a big array, with overspill
  */
@@ -153,27 +144,14 @@ add_to_parent(
 
 	/*
 	 * Add this followup to the sibling chain of our parent.
-	 * arts[] has been sorted by build_references and we add at the start or end
-	 * of the chain depending on whether the sort method is ASCEND or DESCEND
-	 * Unavailable articles go at the start of the chain if ASCEND (because
-	 * we presume unavailable arts (ie REF_REF links) have expired), otherwise at the end.
-	 * ie: if ASCEND && REF
-	 *        add_to_start
-	 *     else
-	 *        add_to_end
+	 * We add at the end of the chain, rearrange_siblings() will
+	 * sort it later.
 	 */
-	if (sort_ascend && (ptr->article == ART_UNAVAILABLE)) {
-		/* Add to start */
-		ptr->sibling = ptr->parent->child;
-		ptr->parent->child = ptr;
-	} else {
-		/* Add to end */
-		for (p = ptr->parent->child; p->sibling != NULL; p = p->sibling)
-			;
+	for (p = ptr->parent->child; p->sibling != NULL; p = p->sibling)
+		;
 
-/*		ptr->sibling is already NULL */
-		p->sibling = ptr;
-	}
+/*	ptr->sibling is already NULL */
+	p->sibling = ptr;
 }
 
 
@@ -181,15 +159,16 @@ add_to_parent(
  * Checks if Message-ID has valid format
  * Returns TRUE if it does, FALSE if it does not
  *
- * TODO: combine with post.c:damaged_id()?
+ * TODO: combine with post.c:damaged_id()
  */
 static t_bool
 valid_msgid(
-	const char *msgid)
+	char *msgid)
 {
 	size_t mlen = 0;
-	t_bool at_present = 0;
+	t_bool at_present = FALSE;
 
+	str_trim(msgid);
 	if (!msgid || *msgid != '<')
 		return FALSE;
 
@@ -242,13 +221,16 @@ add_msgid(
 	unsigned int h;
 
 	if (!msgid) {
-		error_message("add_msgid: NULL msgid\n");
+		error_message(2, "add_msgid: NULL msgid\n");
 		giveup();
 	}
 
 	h = hash_msgid(msgid + 1);				/* Don't hash the initial '<' */
 
-	DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _("unchanged") : newparent->txt));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _("unchanged") : newparent->txt));
+#endif /* DEBUG */
 
 	/*
 	 * Look for this message id in the cache.
@@ -262,7 +244,10 @@ add_msgid(
 		 * CASE 1a - No parent specified, do nothing
 		 */
 		if (newparent == NULL) {
-			DEBUG_PRINT((dbgfd, "nop: %s No parent specified\n", i->txt));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "nop: %s No parent specified\n", i->txt));
+#endif /* DEBUG */
 			return i;
 		}
 
@@ -270,7 +255,10 @@ add_msgid(
 		 * CASE 1b - Parent not changed, do nothing
 		 */
 		if (newparent == i->parent) {
-			DEBUG_PRINT((dbgfd, "dup: %s -> %s (no change)\n", i->txt, i->parent ? i->parent->txt : "NULL"));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "dup: %s -> %s (no change)\n", i->txt, i->parent ? i->parent->txt : "NULL"));
+#endif /* DEBUG */
 			return i;
 		}
 
@@ -285,15 +273,20 @@ add_msgid(
 			 */
 			for (ptr = newparent; ptr != NULL; ptr = ptr->parent) {
 				if (ptr == i) {
-					DEBUG_PRINT((dbgfd, "Avoiding circular reference! (%s)\n", (key == MSGID_REF) ? "MSG" : "REF"));
+#ifdef DEBUG
+					if (debug & DEBUG_REFS)
+						DEBUG_PRINT((dbgfd, "Avoiding circular reference! (%s)\n", (key == MSGID_REF) ? "MSG" : "REF"));
+#endif /* DEBUG */
 					return i;
 				}
 			}
 
 			i->parent = newparent;
 			add_to_parent(i);
-
-			DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _("None")));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _("None")));
+#endif /* DEBUG */
 			return i;
 		}
 
@@ -305,18 +298,24 @@ add_msgid(
 		 * All we can do is ignore the error
 		 */
 		if (i->parent != newparent) {
-			DEBUG_PRINT((dbgfd, "Warning: (%s) Ignoring %s -> %s (already %s)\n",
-				(key == MSGID_REF) ? "MSG" : "REF", i->txt,
-				newparent ? newparent->txt : "None", i->parent->txt));
+#ifdef DEBUG
+			if (debug & DEBUG_REFS)
+				DEBUG_PRINT((dbgfd, "Warning: (%s) Ignoring %s -> %s (already %s)\n",
+					(key == MSGID_REF) ? "MSG" : "REF", i->txt,
+					newparent ? newparent->txt : "None", i->parent->txt));
+#endif /* DEBUG */
 
 			return i;
 		}
 
-		error_message("Error: Impossible combination of conditions !\n");
+		error_message(2, "Error: Impossible combination of conditions !\n");
 		return i;
 	}
 
-	DEBUG_PRINT((dbgfd, "new: %s -> %s\n", msgid, (newparent)?newparent->txt:"None"));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "new: %s -> %s\n", msgid, (newparent)?newparent->txt:"None"));
+#endif /* DEBUG */
 
 	/*
 	 * This is a new node, so build a structure for it
@@ -385,7 +384,10 @@ parse_references(
 	if (!r)
 		return NULL;
 
-	DEBUG_PRINT((dbgfd, "parse_references: %s\n", r));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "parse_references: %s\n", r));
+#endif /* DEBUG */
 
 	/*
 	 * Break the refs down, using REF_SEP as delimiters
@@ -393,14 +395,26 @@ parse_references(
 	if ((ptr = strtok(r, REF_SEP)) == NULL)
 		return NULL;
 
+	/*
+	 * As per RFC 5536, a leading comment is allowed -> skip unknown
+	 * token until we find a valid msgid
+	 *
+	 * TODO: parse these tokens to be sure it is a comment and not
+	 *       a damaged header
+	 */
+	if (!valid_msgid(ptr)) {
+		while ((ptr = strtok(NULL, REF_SEP)) != NULL && !valid_msgid(ptr))
+			;
+	}
+
+	if (ptr == NULL)
+		return NULL;
+
 	/*
 	 * By definition, the head of the thread has no parent
 	 */
 	parent = NULL;
 
-	if (!valid_msgid(ptr))
-		return NULL;
-
 	current = add_msgid(REF_REF, ptr, parent);
 
 	while ((ptr = strtok(NULL, REF_SEP)) != NULL) {
@@ -414,6 +428,60 @@ parse_references(
 }
 
 
+static void
+rearrange_siblings(
+	void)
+{
+	int i;
+	struct t_msgid *current, *p1, *p2;
+
+	for_each_art(i) {
+		current = arts[i].refptr;
+
+		if (!current)
+		continue;
+
+		for (; current->sibling == NULL && current->parent != NULL; current = current->parent)
+			;
+
+		if (current->sibling != NULL) {
+			for (p1 = current; p1->article == ART_UNAVAILABLE && p1->child != NULL; p1 = p1->child)
+				;
+
+			for (p2 = current->sibling; p2->article == ART_UNAVAILABLE && p2->child != NULL; p2 = p2->child)
+				;
+
+			if (p1->article != ART_UNAVAILABLE && p2->article != ART_UNAVAILABLE && p1->article > p2->article) {
+				if (current->parent->child == current) {
+					/*
+					 * current is the first followup
+					 *  adjust parent->child pointer
+					 */
+					current->parent->child = current->sibling;
+					p1 = current->parent->child;
+				} else {
+					/*
+					 * current is not the first followup
+					 *  find the sibling above current
+					 *  adjust the sibling pointer there
+					 */
+					for (p1 = current->parent->child; p1->sibling != current; p1 = p1->sibling)
+						;
+
+					p1->sibling = current->sibling;
+					p1 = p1->sibling;
+				}
+				/*
+				 * swap current <-> sibling
+				 */
+				current->sibling = p1->sibling;
+				p1->sibling = current;
+			}
+		}
+	}
+}
+
+
 /*
  * Reconstruct the References: field from the parent pointers
  * NB: In deep threads this can lead to a very long line. If you want to use
@@ -435,10 +503,12 @@ _get_references(
 	len += strlen(refptr->txt) + 1;	/* msgid + space */
 	if (refptr->parent == NULL || depth > MAX_REFS) {
 
-#ifdef DEBUG_REFS
-		if (depth > MAX_REFS)
-			error_message("Warning: Too many refs near to %s. Truncated\n", refptr->txt);
-#endif /* DEBUG_REFS */
+#ifdef DEBUG
+		if (debug & DEBUG_REFS) {
+			if (depth > MAX_REFS)
+				error_message(2, "Warning: Too many refs near to %s. Truncated\n", refptr->txt);
+		}
+#endif /* DEBUG */
 		refs = my_malloc(len + 1);	/* total length + nullbyte */
 		pos = 0;
 	} else
@@ -501,6 +571,45 @@ free_msgids(
 
 
 #if 0	/* But please don't remove it */
+/*
+ * Function to dump an ASCII tree map of a thread rooted at msgid.
+ * Output goes to fp, level is the current depth of the tree.
+ */
+static void
+dump_thread(
+	FILE *fp,
+	struct t_msgid *msgid,
+	int level)
+{
+	char buff[120];		/* This is _probably_ enough */
+	char *ptr = buff;
+	int i, len;
+
+	/*
+	 * Dump the current article
+	 */
+	sprintf(ptr, "%3d %*s", msgid->article, 2*level, "  ");
+
+	len = strlen(ptr);
+	i = cCOLS - len - 20;
+
+	if (msgid->article >= 0)
+		sprintf(ptr + len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
+	else
+		sprintf(ptr + len, "%-*.*s", i, i, _("[- Unavailable -]"));
+
+	fprintf(fp, "%s\n", ptr);
+
+	if (msgid->child != NULL)
+		dump_thread(fp, msgid->child, level + 1);
+
+	if (msgid->sibling != NULL)
+		dump_thread(fp, msgid->sibling, level);
+
+	return;
+}
+
+
 static void
 dump_msgids(
 	void)
@@ -580,46 +689,7 @@ clear_art_ptrs(
 }
 
 
-/*
- * Function to dump an ASCII tree map of a thread rooted at msgid.
- * Output goes to fp, level is the current depth of the tree.
- */
-#ifdef DEBUG_REFS
-static void
-dump_thread(
-	FILE *fp,
-	struct t_msgid *msgid,
-	int level)
-{
-	char buff[120];		/* This is _probably_ enough */
-	char *ptr = buff;
-	int i, len;
-
-	/*
-	 * Dump the current article
-	 */
-	sprintf(ptr, "%3d %*s", msgid->article, 2*level, "  ");
-
-	len = strlen(ptr);
-	i = cCOLS - len - 20;
-
-	if (msgid->article >= 0)
-		sprintf(ptr + len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
-	else
-		sprintf(ptr + len, "%-*.*s", i, i, _("[- Unavailable -]"));
-
-	fprintf(fp, "%s\n", ptr);
-
-	if (msgid->child != NULL)
-		dump_thread(fp, msgid->child, level + 1);
-
-	if (msgid->sibling != NULL)
-		dump_thread(fp, msgid->sibling, level);
-
-	return;
-}
-
-
+#ifdef DEBUG
 /*
  * Dump out all the threads from the msgid point of view, show the
  * related article index in arts[] where possible
@@ -666,7 +736,7 @@ dump_msgid_threads(
 
 	fprintf(dbgfd, "Dump complete.\n\n");
 }
-#endif /* DEBUG_REFS */
+#endif /* DEBUG */
 
 
 /*
@@ -787,10 +857,15 @@ thread_by_reference(
 	int i;
 	struct t_msgid *ptr;
 
-#ifdef DEBUG_REFS
-	dbgfd = fopen("Refs.info", "w");
-	dump_msgid_threads();
-#endif /* DEBUG_REFS */
+#ifdef DEBUG
+	if (debug & DEBUG_REFS) {
+		char file[PATH_LEN];
+
+		joinpath(file, sizeof(file), TMPDIR, "REFS.info");
+		dbgfd = fopen(file, "w");
+		dump_msgid_threads();
+	}
+#endif /* DEBUG */
 
 	/*
 	 * Build threads starting from root msgids (ie without parent)
@@ -804,20 +879,22 @@ thread_by_reference(
 		}
 	}
 
-#ifdef DEBUG_REFS
-	fprintf(dbgfd, "Full dump of threading info...\n");
-	fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
+#ifdef DEBUG
+	if (debug & DEBUG_REFS) {
+		fprintf(dbgfd, "Full dump of threading info...\n");
+		fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
 
-	for_each_art(i) {
-		fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
-			(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
-			(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
-			(arts[i].refptr->child) ? arts[i].refptr->child->article : -2,
-			arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
+		for_each_art(i) {
+			fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
+				(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
+				(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
+				(arts[i].refptr->child) ? arts[i].refptr->child->article : -2,
+				arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
+		}
+
+		fclose(dbgfd);
 	}
-
-	fclose(dbgfd);
-#endif /* DEBUG_REFS */
+#endif /* DEBUG */
 
 	return;
 }
@@ -914,20 +991,19 @@ build_references(
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
-	if (group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
-		sort_arts(group->attribute->sort_art_type);
+	if (group->attribute->sort_article_type != SORT_ARTICLES_BY_NOTHING)
+		sort_arts(group->attribute->sort_article_type);
 
-	sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
-	               group->attribute->sort_art_type == SORT_ARTICLES_BY_LINES_ASCEND);
+#ifdef DEBUG
+	if (debug & DEBUG_REFS) {
+		char file[PATH_LEN];
 
-#ifdef DEBUG_REFS
-	dbgfd = fopen("Refs.dump", "w");
-	SETVBUF(dbgfd, NULL, _IONBF, 0);
-	fprintf(dbgfd, "MSGID phase\n");
-#endif /* DEBUG_REFS */
+		joinpath(file, sizeof(file), TMPDIR, "REFS.dump");
+		dbgfd = fopen(file, "w");
+		SETVBUF(dbgfd, NULL, _IONBF, 0);
+		fprintf(dbgfd, "MSGID phase\n");
+	}
+#endif /* DEBUG */
 
 	/*
 	 * Add the Message-ID headers to the cache, using the last Reference
@@ -936,6 +1012,7 @@ build_references(
 	for_each_art(i) {
 		art = &arts[i];
 
+		art->refptr = add_msgid(MSGID_REF, art->msgid, NULL); /* preset art->refptr */
 		if (art->refs) {
 			strip_line(art->refs);
 
@@ -947,30 +1024,46 @@ build_references(
 			 * TODO: do this in a single pass
 			 */
 			if ((s = strrchr(art->refs, '<')) != NULL) {
-				if (!strcmp(art->msgid, s)) {
+				char *ptr;
+
+				/*
+				 * A comment can occur after another REF_SEP, remove it
+				 *
+				 * TODO: parse it to be sure it is a comment
+				 */
+				if ((ptr = strpbrk(s, REF_SEP)) != NULL)
+					*ptr = '\0';
+
+				if (valid_msgid(s) && !strcmp(art->msgid, s)) {
 					/*
 					 * Remove circular reference to current article
 					 */
-					DEBUG_PRINT((dbgfd, "removing circular reference to: %s\n", s));
+#ifdef DEBUG
+					if (debug & DEBUG_REFS)
+						DEBUG_PRINT((dbgfd, "removing circular reference to: %s\n", s));
+#endif /* DEBUG */
 					*s = '\0';
 				}
-			}
-			if (s != NULL) {
-				if (valid_msgid(art->msgid))
+				if (valid_msgid(art->msgid) && valid_msgid(s))
 					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s, NULL));
 				*s = '\0';
-			} else {
-				if (valid_msgid(art->msgid))
-					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
+			} else
 				FreeAndNull(art->refs);
-			}
-		} else
-			if (valid_msgid(art->msgid))
-				art->refptr = add_msgid(MSGID_REF, art->msgid, NULL);
+		}
+
+		/*
+		 * set art->refptr->article - rearrange_siblings() needs this
+		 */
+		if (art->refptr != NULL)
+			art->refptr->article = i;
+
 		FreeAndNull(art->msgid);	/* Now cached - discard this */
 	}
 
-	DEBUG_PRINT((dbgfd, "REFS phase\n"));
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		DEBUG_PRINT((dbgfd, "REFS phase\n"));
+#endif /* DEBUG */
 	/*
 	 * Add the References data to the cache
 	 */
@@ -993,7 +1086,14 @@ build_references(
 		FreeAndNull(art->refs);
 	}
 
-#ifdef DEBUG_REFS
-	fclose(dbgfd);
-#endif /* DEBUG_REFS */
+#ifdef DEBUG
+	if (debug & DEBUG_REFS)
+		fclose(dbgfd);
+#endif /* DEBUG */
+
+	/*
+	 * all msgids are cached now
+	 * change order of siblings if needed
+	 */
+	rearrange_siblings();
 }
