@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2005-07-02
+ *  Updated   : 2009-07-17
  *  Notes     :
  *
- * Copyright (c) 1991-2005 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,9 +48,10 @@
 #	endif /* !__UUDEVIEW_H__ */
 #endif /* HAVE_UUDEVIEW_H */
 
+#ifndef HAVE_LIBUU
 #undef OFF
-
 enum state { INITIAL, MIDDLE, OFF, END };
+#endif /* !HAVE_LIBUU */
 
 /*
  * Local prototypes
@@ -59,6 +60,7 @@ static FILE *open_save_filename(const char *path, t_bool mbox);
 static int match_content_type(t_part *part, char *type);
 static t_bool check_save_mime_type(t_part *part, const char *mime_types);
 static t_bool decode_save_one(t_part *part, FILE *rawfp, t_bool postproc);
+static t_bool expand_save_filename(char *outpath, size_t outpath_len, const char *path);
 static void generate_filename(char *buf, int buflen, const char *suffix);
 static void post_process_uud(void);
 static void post_process_sh(void);
@@ -117,16 +119,16 @@ check_start_save_any_news(
 			break;
 
 		case MAIL_ANY_NEWS:
-			joinpath(savefile, TMPDIR, "tin");
+			joinpath(savefile, sizeof(savefile), TMPDIR, "tin");
 #ifdef APPEND_PID
-			snprintf(savefile + strlen(savefile), sizeof(savefile) - strlen(savefile), ".%d", (int) process_id);
+			snprintf(savefile + strlen(savefile), sizeof(savefile) - strlen(savefile), ".%ld", (long) process_id);
 #endif /* APPEND_PID */
 			/* FALLTHROUGH */
 
 		case SAVE_ANY_NEWS:
-			joinpath(logfile, rcdir, "log");
+			joinpath(logfile, sizeof(logfile), rcdir, "log");
 
-			if (no_write || (fp_log = fopen(logfile, "w" FOPEN_OPTS)) == NULL) {
+			if (no_write || (fp_log = fopen(logfile, "w")) == NULL) {
 				perror_message(_(txt_cannot_open), logfile);
 				fp_log = stdout;
 				verbose = FALSE;
@@ -159,9 +161,6 @@ check_start_save_any_news(
 		if (group->bogus || !group->subscribed)
 			continue;
 
-		if (!index_group(group))
-			continue;
-
 		if (function == MAIL_ANY_NEWS || function == SAVE_ANY_NEWS) {
 			if (!group->attribute->batch_save)
 				continue;
@@ -175,15 +174,18 @@ check_start_save_any_news(
 			if (function == SAVE_ANY_NEWS) {
 				char tmp[PATH_LEN];
 
-				if (!strfpath(group->attribute->savedir, tmp, sizeof(tmp), group))
-					joinpath(tmp, homedir, DEFAULT_SAVEDIR);
+				if (!strfpath(cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : tinrc.savedir, tmp, sizeof(tmp), group, FALSE))
+					joinpath(tmp, sizeof(tmp), homedir, DEFAULT_SAVEDIR);
 
 				make_group_path(group->name, group_path);
-				joinpath(path, tmp, group_path);
+				joinpath(path, sizeof(path), tmp, group_path);
 				create_path(path);	/* TODO error handling */
 			}
 		}
 
+		if (!index_group(group))
+			continue;
+
 		/*
 		 * For each article in this group...
 		 */
@@ -198,7 +200,7 @@ check_start_save_any_news(
 						print_first = FALSE;
 					}
 					if (!verbose && !catchup) /* we don't need details */
-						return 2;
+						return NEWS_AVAIL_EXIT;
 					art_count++;
 					if (arts[j].score >= tinrc.score_select)
 						hot_count++;
@@ -217,10 +219,10 @@ check_start_save_any_news(
 
 					if (function == SAVE_ANY_NEWS) {
 						snprintf(buf, sizeof(buf), "%ld", arts[j].artnum);
-						joinpath(savefile, path, buf);
+						joinpath(savefile, sizeof(savefile), path, buf);
 					}
 
-					if ((savefp = fopen(savefile, "w" FOPEN_OPTS)) == NULL) {
+					if ((savefp = fopen(savefile, "w")) == NULL) {
 						fprintf(fp_log, _(txt_cannot_open), savefile);
 						if (verbose)
 							perror_message(_(txt_cannot_open), savefile);
@@ -250,7 +252,7 @@ check_start_save_any_news(
 					}
 
 					snprintf(buf, sizeof(buf), "[%5ld]  %s\n", arts[j].artnum, arts[j].subject);
-					fprintf(fp_log, "%s", buf);		/* buf may contain % */
+					fprintf(fp_log, "%s", buf);	/* buf may contain % */
 					if (verbose)
 						wait_message(0, buf);
 
@@ -289,11 +291,11 @@ check_start_save_any_news(
 			 * TODO: shall we return 2 or 0 in the -cZ case?
 			 */
 			if (unread_news && !catchup)
-				return 2;
+				return NEWS_AVAIL_EXIT;
 			else {
 				if (verbose)
 					wait_message(1, _(txt_there_is_no_news));
-				return 0;
+				return EXIT_SUCCESS;
 			}
 			/* NOTREACHED */
 
@@ -436,7 +438,7 @@ save_and_process_art(
 
 	/* The first task is to fixup the filename to be saved too. This is context dependent */
 	strncpy(path, inpath, sizeof(path) - 1);
-/*fprintf(stderr, "save_and_process_art max=%d num_save=%d starting path=(%s) postproc=%s\n", max, num_save, path, bool_unparse(post_process));*/
+/* fprintf(stderr, "save_and_process_art max=%d num_save=%d starting path=(%s) postproc=%s\n", max, num_save, path, bool_unparse(post_process)); */
 
 	/*
 	 * If using the auto-save feature on an article with Archive-Name,
@@ -464,11 +466,11 @@ save_and_process_art(
 
 		/* Add on the archive name as a directory */
 		/* TODO: maybe a s!/!.! on archive-name would be better */
-		joinpath(archpath, path, artptr->archive->name);
+		joinpath(archpath, sizeof(archpath), path, artptr->archive->name);
 
 		/* Generate the filename part and append it */
 		snprintf(filename, sizeof(filename), "%s%s", partprefix, artptr->archive->partnum);
-		joinpath(path, archpath, filename);
+		joinpath(path, sizeof(path), archpath, filename);
 /*fprintf(stderr, "save_and_process_art archive-name mangled path=(%s)\n", path);*/
 		if (!create_path(path))
 			return FALSE;
@@ -479,17 +481,13 @@ save_and_process_art(
 		 * This is backward-contemptibility with older versions of tin
 		 */
 		if (!is_mailbox && max > 1) {
-#ifdef VMS
-			const char suffixsep = '-';		/* Suffix seperator for .001 type extensions */
-#else
 			const char suffixsep = '.';
-#endif /* VMS */
 
 			sprintf(&path[strlen(path)], "%c%03d", suffixsep, num_save + 1);
 		}
 	}
 
-/*fprintf(stderr, "save_and_process_art expanded path now=(%s)\n", path);*/
+/* fprintf(stderr, "save_and_process_art expanded path now=(%s)\n", path); */
 
 	if ((fp = open_save_filename(path, is_mailbox)) == NULL)
 		return FALSE;
@@ -507,8 +505,8 @@ save_and_process_art(
 		 */
 	}
 
-	if (copy_fp(artinfo->raw, fp)) /* Write tailing newline or MMDF-mailbox seperator */
-		print_art_seperator_line(fp, is_mailbox);
+	if (copy_fp(artinfo->raw, fp)) /* Write tailing newline or MMDF-mailbox separator */
+		print_art_separator_line(fp, is_mailbox);
 	else {
 		fclose(fp);
 		unlink(path);
@@ -525,7 +523,7 @@ save_and_process_art(
 	save[num_save].path = my_strdup(path);
 	save[num_save].file = strrchr(save[num_save].path, DIRSEP) + 1;	/* ptr to filename portion */
 	save[num_save].mailbox = is_mailbox;
-/*fprintf(stderr, "SAPA (%s) (%s) mbox=%s\n", save[num_save].path, save[num_save].file, bool_unparse(save[num_save].mailbox));*/
+/* fprintf(stderr, "SAPA (%s) (%s) mbox=%s\n", save[num_save].path, save[num_save].file, bool_unparse(save[num_save].mailbox)); */
 	num_save++;			/* NB: num_save is bumped here only */
 
 	/*
@@ -558,7 +556,6 @@ t_bool
 create_path(
 	const char *path)
 {
-#ifndef VMS
 	char buf[PATH_LEN];
 	int i, j, len;
 	struct stat st;
@@ -579,15 +576,6 @@ create_path(
 			}
 		}
 	}
-#else
-	if (my_mkdir(path, (mode_t) (S_IRWXU|S_IRUGO|S_IXUGO)) == -1) {
-		if (errno != EEXIST) {
-			perror_message(_(txt_cannot_create), buf);
-			return FALSE;
-		}
-	}
-#endif /* !VMS */
-
 	return TRUE;
 }
 
@@ -614,26 +602,32 @@ generate_filename(
  * Expand metacharacters and use defaults as needed.
  * Return TRUE if the path is a mailbox, or FALSE otherwise.
  */
-t_bool
+static t_bool
 expand_save_filename(
 	char *outpath,
+	size_t outpath_len,
 	const char *path)
 {
-	int ret = strfpath(path, outpath, PATH_LEN, curr_group);
+	char base_filename[PATH_LEN];
+	char buf[PATH_LEN];
+	char buf_path[PATH_LEN];
+	int ret;
 
 	/*
-	 * If no path exists or the above failed in some way, use sensible defaults
-	 * Put the generic path into 'outpath'
+	 * Make sure that externally supplied filename is a filename only and fits
+	 * into buffer
 	 */
-	if ((ret == 0) || !(strrchr(outpath, DIRSEP))) {
-		char buf[PATH_LEN];
+	STRCPY(buf_path, path);
+	base_name(buf_path, base_filename);
 
-		if (!strfpath(curr_group->attribute->savedir, buf, sizeof(buf), curr_group))
-			joinpath(buf, homedir, DEFAULT_SAVEDIR);
-		joinpath(outpath, buf, path);
-		return FALSE;
-	} else
-		return (ret == 1);
+	/* Build default path to save to */
+	if (!(ret = strfpath(cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : curr_group->attribute->savedir, buf, sizeof(buf), curr_group, FALSE)))
+		joinpath(buf, sizeof(buf), homedir, DEFAULT_SAVEDIR);
+
+	/* Join path and filename */
+	joinpath(outpath, outpath_len, buf, base_filename);
+
+	return (ret == 1);	/* should now always evaluate to FALSE */
 }
 
 
@@ -760,8 +754,8 @@ post_process_uud(
 			my_printf(cCRLF);
 
 			/* item->mimetype seems not to be available for uudecoded files etc */
-			if (tinrc.post_process_view) {
-				joinpath(path, file_out_dir, item->filename);
+			if (curr_group->attribute->post_process_view) {
+				joinpath(path, sizeof(path), file_out_dir, item->filename);
 				view_file(path, strrchr(path, DIRSEP) + 1);
 			}
 		} else {
@@ -852,7 +846,7 @@ post_process_uud(
 							generate_filename(name, sizeof(name), "uue");
 
 						filename = name;
-						expand_save_filename(path, filename);
+						expand_save_filename(path, sizeof(path), filename);
 						filename = strrchr(path, DIRSEP) + 1;	/* ptr to filename portion */
 						if ((fp_out = fopen(path, "w")) == NULL) {
 							perror_message(_(txt_cannot_open), path);
@@ -876,8 +870,8 @@ post_process_uud(
 							uudecode_line(u, fp_out);
 						if (t[0] != 'M')
 							uudecode_line(t, fp_out);
-					} else                                  /* end */
-						state = OFF;            /* OFF => a break in the uuencoded data */
+					} else	/* end */
+						state = OFF;	/* OFF => a break in the uuencoded data */
 					break;
 
 				case OFF:
@@ -885,7 +879,7 @@ post_process_uud(
 						uudecode_line(u, fp_out);
 						uudecode_line(t, fp_out);
 						uudecode_line(s, fp_out);
-						state = MIDDLE;         /* Continue output of previously suspended data */
+						state = MIDDLE;	/* Continue output of previously suspended data */
 					} else if (STRNCMPEQ("end", s, 3)) {
 						state = END;
 						if (u[0] != 'M')
@@ -913,7 +907,7 @@ post_process_uud(
 				my_printf(_(txt_uu_success), filename);
 				my_printf(cCRLF);
 				sum_file(path, filename);
-				if (tinrc.post_process_view)
+				if (curr_group->attribute->post_process_view)
 					view_file(path, filename);
 				state = INITIAL;
 				continue;
@@ -1056,7 +1050,7 @@ uudecode_line(
 /*
  * Unpack /bin/sh archives
  * There is no end-of-shar marker so the code reads everything after
- * the start marker. This is why shar is handled seperately.
+ * the start marker. This is why shar is handled separately.
  * The code assumes shar archives do not span articles
  */
 static void
@@ -1073,12 +1067,7 @@ post_process_sh(
 	 * Grab the dirname portion
 	 */
 	my_strncpy(file_out_dir, save[0].path, save[0].file - save[0].path);
-
-#ifdef VMS
-	snprintf(file_out, sizeof(file_out), "%ssh.%05d", file_out_dir, (int) process_id);
-#else
-	snprintf(file_out, sizeof(file_out), "%ssh%05d", file_out_dir, (int) process_id);
-#endif /* VMS */
+	snprintf(file_out, sizeof(file_out), "%ssh%ld", file_out_dir, (long) process_id);
 
 	for (i = 0; i < num_save; i++) {
 		if ((fp_in = fopen(save[i].path, "r")) == NULL)
@@ -1089,7 +1078,7 @@ post_process_sh(
 		while (fgets(buf, (int) sizeof(buf), fp_in) != NULL) {
 			/* find #!/bin/sh style patterns */
 			if ((fp_out == NULL) && pcre_exec(shar_regex.re, shar_regex.extra, buf, strlen(buf), 0, 0, NULL, 0) >= 0)
-				fp_out = fopen(file_out, "w" FOPEN_OPTS);
+				fp_out = fopen(file_out, "w");
 
 			/* write to temp file */
 			if (fp_out != NULL)
@@ -1117,16 +1106,16 @@ post_process_sh(
 
 
 /*
- * write tailing (MMDF)-mailbox seperator
+ * write tailing (MMDF)-mailbox separator
  */
 void
-print_art_seperator_line(
+print_art_separator_line(
 	FILE *fp,
 	t_bool is_mailbox)
 {
 #ifdef DEBUG
-	if (debug == 2)
-		error_message("Mailbox=[%d], mailbox_format=[%s]", is_mailbox, txt_mailbox_formats[tinrc.mailbox_format]);
+	if (debug & DEBUG_MISC)
+		error_message(2, "Mailbox=[%d], mailbox_format=[%s]", is_mailbox, txt_mailbox_formats[tinrc.mailbox_format]);
 #endif /* DEBUG */
 
 	fprintf(fp, "%s", (is_mailbox && !strcasecmp(txt_mailbox_formats[tinrc.mailbox_format], "MMDF")) ? MMDFHDRTXT : "\n");
@@ -1145,15 +1134,10 @@ start_viewer(
 	t_mailcap *foo;
 
 	if ((foo = get_mailcap_entry(part, path)) != NULL) {
-		char buff[LEN];
-
 		if (foo->nametemplate)	/* honor nametemplate */
 			rename_file(path, foo->nametemplate);
 
 		wait_message(0, _(txt_starting_command), foo->command);
-
-		/* are the () needed if foo->command holds more than one cmd? */
-		snprintf(buff, sizeof(buff), "(%s)", foo->command);
 		if (foo->needsterminal) {
 			set_xclick_off();
 			EndWin();
@@ -1213,9 +1197,9 @@ decode_save_one(
 
 		lookup_extension(extension, sizeof(extension), content_types[part->type], part->subtype);
 		generate_filename(buf, sizeof(buf), extension);
-		mbox = expand_save_filename(savepath, buf);
+		mbox = expand_save_filename(savepath, sizeof(savepath), buf);
 	} else
-		mbox = expand_save_filename(savepath, name);
+		mbox = expand_save_filename(savepath, sizeof(savepath), name);
 
 	/*
 	 * Not a good idea to dump attachments over a mailbox
@@ -1226,7 +1210,7 @@ decode_save_one(
 	}
 
 	if (!(create_path(savepath))) {
-		error_message(_(txt_cannot_open_for_saving), savepath);
+		error_message(2, _(txt_cannot_open_for_saving), savepath);
 		return FALSE;
 	}
 
@@ -1234,7 +1218,7 @@ decode_save_one(
 	 * Decode/save the attachment
 	 */
 	if ((fp = open_save_filename(savepath, FALSE)) == NULL) {
-		error_message(_(txt_cannot_open_for_saving), savepath);
+		error_message(2, _(txt_cannot_open_for_saving), savepath);
 		return FALSE;
 	}
 
@@ -1279,7 +1263,7 @@ decode_save_one(
 	 * View the attachment
 	 */
 	if (postproc) {
-		if (tinrc.post_process_view) {
+		if (curr_group->attribute->post_process_view) {
 			start_viewer(part, savepath);
 			my_printf(cCRLF);
 		}
@@ -1296,7 +1280,7 @@ decode_save_one(
 	/*
 	 * Save the attachment
 	 */
-	if (postproc && tinrc.post_process_view) {
+	if (postproc && curr_group->attribute->post_process_view) {
 		my_printf(_(txt_uu_success), savepath);
 		my_printf(cCRLF);
 	}
@@ -1451,7 +1435,8 @@ decode_save_mime(
 		 * the role of a multipart part. Check to see if we want to
 		 * save text and if not, skip this part.
 		 */
-		if ((ptr->type == TYPE_MULTIPART || ((NULL != ptr->uue) && (!check_save_mime_type(ptr, curr_group->attribute->mime_types_to_save)))))
+		 /* check_save_mime_type() is done in decode_save_one() and the check for ptr->uue must be done unconditionally */
+		if (ptr->type == TYPE_MULTIPART || (NULL != ptr->uue /* && !check_save_mime_type(ptr, curr_group->attribute->mime_types_to_save) */ ))
 			continue;
 
 		if (!(decode_save_one(ptr, art->raw, postproc)))
