@@ -3,10 +3,10 @@
  *  Module    : search.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2005-07-02
+ *  Updated   : 2010-10-01
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -94,7 +94,7 @@ get_search_pattern(
 		last_forward = *forward;
 		my_strncpy(last_pattern, def, LEN);
 
-		/* HIST_BODY_SEARCH doesn't exist, hence i_key_search_last is set directly in search_body() */
+		/* HIST_BODY_SEARCH doesn't exist, hence last_search is set directly in search_body() */
 		if (which_hist == HIST_AUTHOR_SEARCH)
 			last_search = *forward ? GLOBAL_SEARCH_AUTHOR_FORWARD : GLOBAL_SEARCH_AUTHOR_BACKWARD;
 		else
@@ -186,6 +186,80 @@ search_config(
 }
 
 
+/*
+ * called by save.c (search for attachment) and page.c (search for URL)
+ */
+int
+generic_search(
+	t_bool forward,
+	t_bool repeat,
+	int current,
+	int last,
+	int level)
+{
+	char *pattern;
+	char buf[BUFSIZ];
+	const char *name, *charset;
+	int n = current;
+	int result = current;
+	t_bool found = FALSE;
+	t_part *part;
+	t_url *urlptr;
+
+	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_config, HIST_CONFIG_SEARCH)))
+		return result;
+
+	if (tinrc.wildcard && !(compile_regex(pattern, &search_regex, PCRE_CASELESS)))
+		return result;
+
+	do {
+		if (n == 0 && !forward)
+			n = last;
+		else {
+			if (n == last && forward)
+				n = 0;
+			else
+				n += forward ? 1 : -1;
+		}
+		switch (level) {
+			case ATTACHMENT_LEVEL:
+				part = get_part(n);
+				if (!(name = get_filename(part->params))) {
+					if (!(name = part->description))
+						name = _(txt_attachment_no_name);
+				}
+				charset = get_param(part->params, "charset");
+				snprintf(buf, sizeof(buf), "%s %s/%s %s, %s", name, content_types[part->type], part->subtype, content_encodings[part->encoding], charset ? charset : "");
+				break;
+
+			case URL_LEVEL:
+				urlptr = find_url(n);
+				snprintf(buf, sizeof(buf), "%s", urlptr->url);
+				break;
+
+			default:
+				buf[0] = '\0';
+				break;
+		}
+		if (match_regex(buf, pattern, &search_regex, TRUE)) {
+			result = n;
+			found = TRUE;
+			break;
+		}
+	} while (n != current);
+
+	clear_message();
+	if (tinrc.wildcard) {
+		FreeAndNull(search_regex.re);
+		FreeAndNull(search_regex.extra);
+	}
+	if (!found)
+		info_message(_(txt_no_match));
+
+	return result;
+}
+
+
 /*
  * Search active[] looking for a groupname
  * Called by select.c
@@ -303,7 +377,7 @@ body_search(
 				art_close(&pgart);		/* Switch the pager over to matched art */
 				pgart = artinfo;
 #ifdef DEBUG
-				if (debug == 2)
+				if (debug & DEBUG_MISC)
 					fprintf(stderr, "art_switch(%p = %p)\n", (void *) &pgart, (void *) &artinfo);
 #endif /* DEBUG */
 				free(line);
@@ -435,7 +509,7 @@ search_group(
 		}
 
 		/* Only search displayed articles */
-		if (curr_group->attribute->show_only_unread && arts[i].status != ART_UNREAD)
+		if (curr_group->attribute->show_only_unread_arts && arts[i].status != ART_UNREAD)
 			continue;
 
 		ret = search_func(i, searchbuff);
@@ -519,7 +593,7 @@ search_article(
 {
 	char *pattern, *ptr, *tmp;
 	int i = start_line;
-	int tmp_srch_offsets[2];
+	int tmp_srch_offsets[2] = {0, 0};
 	t_bool wrap = FALSE;
 	t_bool match = FALSE;
 
@@ -648,7 +722,7 @@ search_body(
 	/*
 	 * Count up the articles to be processed for the progress meter
 	 */
-	if (group->attribute->show_only_unread) {
+	if (group->attribute->show_only_unread_arts) {
 		for (i = 0; i < grpmenu.max; i++)
 			total_cnt += new_responses(i);
 	} else {
