@@ -3,10 +3,10 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2005-07-02
+ *  Updated   : 2010-10-07
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -205,7 +205,7 @@ selection_page(
 			case GLOBAL_SEARCH_SUBJECT_BACKWARD:
 			case GLOBAL_SEARCH_REPEAT:
 				if (func == GLOBAL_SEARCH_REPEAT && last_search != GLOBAL_SEARCH_SUBJECT_FORWARD && last_search != GLOBAL_SEARCH_SUBJECT_BACKWARD)
-					info_message(_(txt_bad_command), printascii(key, key_to_func(GLOBAL_HELP, select_keys)));
+					info_message(_(txt_no_prev_search));
 				else {
 					if ((i = search_active((func == GLOBAL_SEARCH_SUBJECT_FORWARD), (func == GLOBAL_SEARCH_REPEAT))) != -1) {
 						move_to_item(i);
@@ -251,7 +251,7 @@ selection_page(
 				break;
 
 			case GLOBAL_EDIT_FILTER:
-				if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
+				if (!invoke_editor(filter_file, filter_file_offset, NULL))
 					break;
 				(void) read_filter_file(filter_file);
 				break;
@@ -342,8 +342,7 @@ selection_page(
 				break;
 
 			case GLOBAL_OPTION_MENU:
-				(void) change_config_file(NULL);
-				read_attributes_files();
+				config_page(CURR_GROUP.name);
 				show_selection_page();
 				break;
 
@@ -382,6 +381,8 @@ selection_page(
 				show_selection_page();
 				if (tinrc.show_only_unread_groups)
 					info_message(_(txt_show_unread));
+				else
+					clear_message();
 				break;
 
 			case GLOBAL_BUGREPORT:
@@ -455,11 +456,15 @@ selection_page(
 
 			case GLOBAL_POST:			/* post a basenote */
 				if (!selmenu.max) {
+					if (!can_post) {
+						info_message(_(txt_cannot_post));
+						break;
+					}
 					snprintf(buf, sizeof(buf), _(txt_post_newsgroups), tinrc.default_post_newsgroups);
 					if (!prompt_string_default(buf, tinrc.default_post_newsgroups, _(txt_no_newsgroups), HIST_POST_NEWSGROUPS))
 						break;
-					if (group_find(tinrc.default_post_newsgroups) == NULL) {
-						error_message(_(txt_not_in_active_file), tinrc.default_post_newsgroups);
+					if (group_find(tinrc.default_post_newsgroups, FALSE) == NULL) {
+						error_message(2, _(txt_not_in_active_file), tinrc.default_post_newsgroups);
 						break;
 					} else {
 						strcpy(buf, tinrc.default_post_newsgroups);
@@ -469,7 +474,7 @@ selection_page(
 						 * CHARSET_CONVERSION conversion case in new_part()
 						 * which relies currently relies on CURR_GROUP
 						 */
-						selmenu.curr = my_group_add(buf);
+						selmenu.curr = my_group_add(buf, FALSE);
 						/*
 						 * and the next hack to avoid a grabbled selection
 						 * screen after the posting
@@ -564,7 +569,7 @@ show_selection_page(
 	 */
 	if (yanked_out) {
 		for (i = 0; i < selmenu.max; i++) {
-			if ((len = strlen(active[my_group[i]].name)) > groupname_len)
+			if ((len = strwidth(active[my_group[i]].name)) > groupname_len)
 				groupname_len = len;
 			if (show_description && groupname_len > tinrc.groupname_max_length) {
 				/* no need to search further, we have reached max length */
@@ -574,7 +579,7 @@ show_selection_page(
 		}
 	} else {
 		for_each_group(i) {
-			if ((len = strlen(active[i].name)) > groupname_len)
+			if ((len = strwidth(active[i].name)) > groupname_len)
 				groupname_len = len;
 			if (show_description && groupname_len > tinrc.groupname_max_length) {
 				/* no need to search further, we have reached max length */
@@ -606,23 +611,41 @@ static void
 build_gline(
 	int i)
 {
-	char tmp[10];
-	char active_name[255];
-	char group_descript[255];
 	char subs;
+	char tmp[10];
+	int n, blank_len;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	char *name_buf = NULL;
+	char *desc_buf = NULL;
+	int name_len = groupname_len;
+	wchar_t *active_name = NULL;
+	wchar_t *active_name2 = NULL;
+	wchar_t *active_desc = NULL;
+	wchar_t *active_desc2 = NULL;
+#else
+	char *active_name;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #ifdef USE_CURSES
 	char sptr[BUFSIZ];
 #else
 	char *sptr = screen[INDEX2SNUM(i)].col;
 #endif /* USE_CURSES */
-	int n, blank_len;
 
-	blank_len = (MIN(cCOLS, (int) sizeof(group_descript)) - (groupname_len + SELECT_MISC_COLS)) + (show_description ? 2 : 4);
+#define DESCRIPTION_LENGTH 255
+
+	blank_len = (MIN(cCOLS, DESCRIPTION_LENGTH) - (groupname_len + SELECT_MISC_COLS)) + (show_description ? 2 : 4);
 
 	if (active[my_group[i]].inrange)
 		strcpy(tmp, "    #");
 	else if (active[my_group[i]].newsrc.num_unread) {
-		strcpy(tmp, tin_ltoa(active[my_group[i]].newsrc.num_unread, 5));
+		int getart_limit;
+		long num_unread;
+
+		getart_limit = cmdline.args & CMDLINE_GETART_LIMIT ? cmdline.getart_limit : tinrc.getart_limit;
+		num_unread = active[my_group[i]].newsrc.num_unread;
+		if (getart_limit > 0 && getart_limit < num_unread)
+			num_unread = getart_limit;
+		strcpy(tmp, tin_ltoa(num_unread, 5));
 	} else
 		strcpy(tmp, "     ");
 
@@ -638,33 +661,94 @@ build_gline(
 	 */
 	if (active[n].bogus)					/* Group is not in active list */
 		subs = 'D';
-	else if (active[n].subscribed)			/* Important that this preceeds Newgroup */
+	else if (active[n].subscribed)			/* Important that this precedes Newgroup */
 		subs = group_flag(active[n].moderated);
 	else
 		subs = ((active[n].newgroup) ? 'N' : 'u'); /* New (but unsubscribed) group or unsubscribed group */
 
-	strncpy(active_name, active[n].name, groupname_len);
-	active_name[groupname_len] = '\0';
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	if ((active_name = char2wchar_t(active[n].name)) == NULL) /* If char2wchar_t() fails try again after replacing unprintable characters */
+		active_name = char2wchar_t(convert_to_printable(active[n].name));
+
+	if (show_description && active[n].description)
+		active_desc = char2wchar_t(active[n].description);
+
+	if (active_name && tinrc.abbreviate_groupname) {
+		if (show_description && !active_desc)
+			active_name2 = abbr_wcsgroupname(active_name, (size_t) (groupname_len + blank_len));
+		else
+			active_name2 = abbr_wcsgroupname(active_name, (size_t) groupname_len);
+
+		free(active_name);
+	} else
+		active_name2 = active_name;
+
+	if (active_name2 && (active_name = wcspart(active_name2, groupname_len, TRUE)) != NULL) {
+		free(active_name2);
+		if ((name_buf = wchar_t2char(active_name)) != NULL) {
+			free(active_name);
+			name_len = (int) strlen(name_buf);
+		}
+	}
 
 	if (show_description) {
-		if (active[n].description) {
-			strncpy(group_descript, active[n].description, blank_len);
-			group_descript[blank_len] = '\0';
+		if (active_desc) {
+			if ((active_desc2 = wcspart(active_desc, blank_len, TRUE)) != NULL) {
+				if ((desc_buf = wchar_t2char(active_desc2)) != NULL)
+					blank_len = strlen(desc_buf);
+				free(active_desc);
+				free(active_desc2);
+			}
+		}
+		if (desc_buf) {
+			sprintf(sptr, "  %c %s %s  %-*.*s  %-*.*s%s",
+				subs, tin_ltoa(i + 1, 4), tmp,
+				name_len, name_len, BlankIfNull(name_buf),
+				blank_len, blank_len, desc_buf, cCRLF);
+			free(desc_buf);
+		} else
+			sprintf(sptr, "  %c %s %s  %-*.*s  %s",
+				subs, tin_ltoa(i + 1, 4), tmp,
+				(name_len + blank_len),
+				(name_len + blank_len), BlankIfNull(name_buf), cCRLF);
+	} else {
+		if (tinrc.draw_arrow)
+			sprintf(sptr, "  %c %s %s  %-*.*s%s", subs, tin_ltoa(i + 1, 4), tmp, name_len, name_len, BlankIfNull(name_buf), cCRLF);
+		else
+			sprintf(sptr, "  %c %s %s  %-*.*s%*s%s", subs, tin_ltoa(i + 1, 4), tmp, name_len, name_len, BlankIfNull(name_buf), blank_len, " ", cCRLF);
+	}
+
+	FreeIfNeeded(name_buf);
+#else
+	if (tinrc.abbreviate_groupname) {
+		if (show_description && !active[n].description)
+			active_name = abbr_groupname(active[n].name, (size_t) (groupname_len + blank_len));
+		else
+			active_name = abbr_groupname(active[n].name, (size_t) groupname_len);
+	} else
+		active_name = my_strdup(active[n].name);
+
+	if (show_description) {
+		if (active[n].description)
 			sprintf(sptr, "  %c %s %s  %-*.*s  %-*.*s%s",
 				 subs, tin_ltoa(i + 1, 4), tmp,
 				 groupname_len, groupname_len, active_name,
-				 blank_len, blank_len, group_descript, cCRLF);
-		} else
+				 blank_len, blank_len, active[n].description, cCRLF);
+		else
 			sprintf(sptr, "  %c %s %s  %-*.*s  %s",
 				 subs, tin_ltoa(i + 1, 4), tmp,
 				 (groupname_len + blank_len),
-				 (groupname_len + blank_len), active[n].name, cCRLF);
+				 (groupname_len + blank_len), active_name, cCRLF);
 	} else {
 		if (tinrc.draw_arrow)
 			sprintf(sptr, "  %c %s %s  %-*.*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, cCRLF);
 		else
 			sprintf(sptr, "  %c %s %s  %-*.*s%*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, blank_len, " ", cCRLF);
 	}
+
+	free(active_name);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
 	if (tinrc.strip_blanks)
 		strcat(strip_line(sptr), cCRLF);
 
@@ -743,8 +827,8 @@ active_comp(
 	t_comptype p1,
 	t_comptype p2)
 {
-	const struct t_group *s1 = (const struct t_group *)p1;
-	const struct t_group *s2 = (const struct t_group *)p2;
+	const struct t_group *s1 = (const struct t_group *) p1;
+	const struct t_group *s2 = (const struct t_group *) p2;
 
 	return strcasecmp(s1->name, s2->name);
 }
@@ -760,7 +844,7 @@ save_restore_curr_group(
 	t_bool saving)
 {
 	static char *oldgroup;
-	static int oldmax;
+	static int oldmax = 0;
 	int ret;
 
 	/*
@@ -830,7 +914,7 @@ choose_new_group(
 
 	clear_message();
 
-	if ((idx = my_group_add(tinrc.default_goto_group)) == -1)
+	if ((idx = my_group_add(tinrc.default_goto_group, TRUE)) == -1)
 		info_message(_(txt_not_in_active_file), tinrc.default_goto_group);
 
 	return idx;
@@ -867,11 +951,12 @@ skip_newgroups(
 int
 add_my_group(
 	const char *group,
-	t_bool add)
+	t_bool add,
+	t_bool ignore_case)
 {
 	int i, j;
 
-	if ((i = find_group_index(group)) < 0)
+	if ((i = find_group_index(group, ignore_case)) < 0)
 		return -1;
 
 	for (j = 0; j < selmenu.max; j++) {
@@ -925,7 +1010,7 @@ reposition_group(
 	 * Can't move into newgroups, they aren't in .newsrc
 	 */
 	if (pos_num <= newgroups) {
-		error_message(_(txt_skipping_newgroups));
+		error_message(2, _(txt_skipping_newgroups));
 		return default_num;
 	}
 
@@ -1122,6 +1207,7 @@ toggle_my_groups(
 		}
 	}
 #else
+	/* preserv group ordering based on newsrc */
 	if ((fp = fopen(newsrc, "r")) == NULL)
 		return;
 
@@ -1129,14 +1215,14 @@ toggle_my_groups(
 		if ((ptr = strchr(buf, SUBSCRIBED)) != NULL) {
 			*ptr = '\0';
 
-			if ((i = find_group_index(buf)) < 0)
+			if ((i = find_group_index(buf, FALSE)) < 0)
 				continue;
 
 			if (tinrc.show_only_unread_groups) {
 				if (active[i].newsrc.num_unread || (active[i].bogus && tinrc.strip_bogus == BOGUS_SHOW))
-					my_group_add(buf);
+					my_group_add(buf, FALSE);
 			} else
-				my_group_add(buf);
+				my_group_add(buf, FALSE);
 		}
 	}
 	fclose(fp);
@@ -1146,8 +1232,8 @@ toggle_my_groups(
 
 
 /*
- * Subscribe or unsubscribe from a list of groups. List can be full list as supported
- * by match_group_list()
+ * Subscribe or unsubscribe from a list of groups. List can be full list as
+ * supported by match_group_list()
  */
 static void
 subscribe_pattern(
@@ -1169,37 +1255,17 @@ subscribe_pattern(
 
 	wait_message(0, message);
 
-	/*
-	 * TODO: why do we do a pass over my_group[] before another one
-	 *       over active[]? AFAICS the first loop can go away.
-	 */
-	for (i = 0; i < selmenu.max; i++) {
-		if (match_group_list(active[my_group[i]].name, buf)) {
-			if (active[my_group[i]].subscribed != (state != FALSE)) {
+	for_each_group(i) {
+		if (match_group_list(active[i].name, buf)) {
+			if (active[i].subscribed != (state != FALSE)) {
 				spin_cursor();
-				subscribe(&active[my_group[i]], SUB_CHAR(state), TRUE);
-				subscribe_num++;
-			}
-		}
-	}
-
-	if (num_active > selmenu.max) {			/* ie, there are groups yanked out */
-		for_each_group(i) {
-			if (match_group_list(active[i].name, buf)) {
-				if (active[i].subscribed != (state != FALSE)) {
-					spin_cursor();
-					/* If found and group is not subscribed add it to end of my_group[]. */
-					subscribe(&active[i], SUB_CHAR(state), TRUE);
-					if (state) {
-						my_group_add(active[i].name);
-						/*
-						 * TODO: grp_mark_unread() or something needs to do a
-						 *       group_get_art_info() to get initial count right
-						 */
-						grp_mark_unread(&active[i]);
-					}
-					subscribe_num++;
+				/* If found and group is not subscribed add it to end of my_group[]. */
+				subscribe(&active[i], SUB_CHAR(state), TRUE);
+				if (state) {
+					my_group_add(active[i].name, FALSE);
+					grp_mark_unread(&active[i]);
 				}
+				subscribe_num++;
 			}
 		}
 	}
@@ -1221,6 +1287,7 @@ select_quit(
 	void)
 {
 	write_config_file(local_config_file);
+	ClearScreen();
 	tin_done(EXIT_SUCCESS);	/* Tin END */
 }
 
