@@ -3,10 +3,10 @@
  *  Module    : sigfile.c
  *  Author    : M. Gleason & I. Lea
  *  Created   : 1992-10-17
- *  Updated   : 2003-09-19
+ *  Updated   : 2010-11-13
  *  Notes     : Generate random signature for posting/mailing etc.
  *
- * Copyright (c) 1992-2006 Mike Gleason
+ * Copyright (c) 1992-2010 Mike Gleason
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -77,7 +77,7 @@ msg_write_signature(
 			char *sigcmd;
 			char cmd[PATH_LEN];
 
-			fprintf(fp, "\n%s", tinrc.sigdashes ? SIGDASHES : "\n");
+			fprintf(fp, "\n%s", thisgroup->attribute->sigdashes ? SIGDASHES : "\n");
 			sigcmd = my_malloc(strlen(thisgroup->attribute->sigfile + 1) + strlen(thisgroup->name) + 4);
 			sprintf(sigcmd, "%s \"%s\"", thisgroup->attribute->sigfile + 1, thisgroup->name);
 
@@ -93,9 +93,9 @@ msg_write_signature(
 #endif /* !DONT_HAVE_PIPING */
 		get_cwd(cwd);
 
-		if (!strfpath(thisgroup->attribute->sigfile, path, sizeof(path), thisgroup)) {
-			if (!strfpath(tinrc.sigfile, path, sizeof(path), thisgroup))
-				joinpath(path, homedir, ".Sig");
+		if (!strfpath(thisgroup->attribute->sigfile, path, sizeof(path), thisgroup, FALSE)) {
+			if (!strfpath(tinrc.sigfile, path, sizeof(path), thisgroup, FALSE))
+				joinpath(path, sizeof(path), homedir, ".Sig");
 		}
 
 		/*
@@ -107,23 +107,23 @@ msg_write_signature(
 		 */
 		if ((sigfp = open_random_sig(path)) != NULL) {
 #ifdef DEBUG
-			if (debug == 2)
-				error_message("USING random sig=[%s]", sigfile);
+			if (debug & DEBUG_MISC)
+				error_message(2, "USING random sig=[%s]", sigfile);
 #endif /* DEBUG */
-			fprintf(fp, "\n%s", tinrc.sigdashes ? SIGDASHES : "\n");
-			joinpath(pathfixed, path, ".sigfixed");
+			fprintf(fp, "\n%s", thisgroup->attribute->sigdashes ? SIGDASHES : "\n");
+			joinpath(pathfixed, sizeof(pathfixed), path, ".sigfixed");
 #ifdef DEBUG
-			if (debug == 2)
-				error_message("TRYING fixed sig=[%s]", pathfixed);
+			if (debug & DEBUG_MISC)
+				error_message(2, "TRYING fixed sig=[%s]", pathfixed);
 #endif /* DEBUG */
 			if ((fixfp = fopen(pathfixed, "r")) != NULL) {
 				copy_fp(fixfp, fp);
 				fclose(fixfp);
 			} else {
-				joinpath(pathfixed, homedir, ".sigfixed");
+				joinpath(pathfixed, sizeof(pathfixed), homedir, ".sigfixed");
 #ifdef DEBUG
-				if (debug == 2)
-					error_message("TRYING fixed sig=[%s]", pathfixed);
+				if (debug & DEBUG_MISC)
+					error_message(2, "TRYING fixed sig=[%s]", pathfixed);
 #endif /* DEBUG */
 				if ((fixfp = fopen(pathfixed, "r")) != NULL) {
 					copy_fp(fixfp, fp);
@@ -135,25 +135,25 @@ msg_write_signature(
 			my_chdir(cwd);
 			return;
 		}
-	}
 
-	if ((sigfp = fopen(path, "r")) != NULL) {
-		fprintf(fp, "\n%s", tinrc.sigdashes ? SIGDASHES : "\n");
-		copy_fp(sigfp, fp);
-		fclose(sigfp);
-		return;
-	}
-
-	/*
-	 * Use ~/.signature as a last resort, but only if mailing or
-	 * using internal inews (external inews appends it automagically).
-	 */
-	if ((sigfp = fopen(default_signature, "r")) != NULL) {
-		if (include_dot_signature) {
-			fprintf(fp, "\n%s", tinrc.sigdashes ? SIGDASHES : "\n");
+		if ((sigfp = fopen(path, "r")) != NULL) {
+			fprintf(fp, "\n%s", thisgroup->attribute->sigdashes ? SIGDASHES : "\n");
 			copy_fp(sigfp, fp);
+			fclose(sigfp);
+			return;
+		}
+
+		/*
+		 * Use ~/.signature as a last resort, but only if mailing or
+		 * using internal inews (external inews appends it automagically).
+		 */
+		if ((sigfp = fopen(default_signature, "r")) != NULL) {
+			if (include_dot_signature) {
+				fprintf(fp, "\n%s", thisgroup->attribute->sigdashes ? SIGDASHES : "\n");
+				copy_fp(sigfp, fp);
+			}
+			fclose(sigfp);
 		}
-		fclose(sigfp);
 	}
 }
 
@@ -171,14 +171,14 @@ open_random_sig(
 
 			if (thrashdir(sigdir) || !*sigfile) {
 #ifdef DEBUG
-				if (debug == 2)
-					error_message("NO sigfile=[%s]", sigfile);
+				if (debug & DEBUG_MISC)
+					error_message(2, "NO sigfile=[%s]", sigfile);
 #endif /* DEBUG */
 				return (FILE *) 0;
 			} else {
 #ifdef DEBUG
-				if (debug == 2)
-					error_message("sigfile=[%s]", sigfile);
+				if (debug & DEBUG_MISC)
+					error_message(2, "sigfile=[%s]", sigfile);
 #endif /* DEBUG */
 				return fopen(sigfile, "r");
 			}
@@ -197,7 +197,7 @@ thrashdir(
 	DIR_BUF *dp;
 	char *cwd;
 	int safeguard, recurse;
-	int c = 0, numentries, pick;
+	int c = 0, numentries = 0, pick;
 	struct stat st;
 
 	sigfile[0] = '\0';
@@ -205,8 +205,7 @@ thrashdir(
 	if ((dirp = opendir(CURRENTDIR)) == NULL)
 		return 1;
 
-	numentries = 0;
-	while ((dp = readdir(dirp)) != NULL)
+	while (readdir(dirp) != NULL)
 		numentries++;
 
 	/*
@@ -229,8 +228,8 @@ thrashdir(
 	 */
 	for (safeguard = 0, dp = NULL; safeguard < MAXLOOPS && dp == NULL; safeguard++) {
 #ifdef DEBUG
-		if (debug == 2)
-			error_message("sig loop=[%d] recurse=[%d]", safeguard, recurse);
+		if (debug & DEBUG_MISC)
+			error_message(2, "sig loop=[%d] recurse=[%d]", safeguard, recurse);
 #endif /* DEBUG */
 #ifdef HAVE_REWINDDIR
 		rewinddir(dirp);
@@ -276,8 +275,8 @@ thrashdir(
 					strcat(sigfile, "/");
 					strcat(sigfile, dp->d_name);
 #ifdef DEBUG
-					if (debug == 2)
-						error_message("Found a file=[%s]", sigfile);
+					if (debug & DEBUG_MISC)
+						error_message(2, "Found a file=[%s]", sigfile);
 #endif /* DEBUG */
 				}
 			}
@@ -285,8 +284,8 @@ thrashdir(
 	}
 	free(cwd);
 #ifdef DEBUG
-	if (debug == 2)
-		error_message("return 0: sigfile=[%s]", sigfile);
+	if (debug & DEBUG_MISC)
+		error_message(2, "return 0: sigfile=[%s]", sigfile);
 #endif /* DEBUG */
 	CLOSEDIR(dirp);
 
