@@ -3,10 +3,10 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2004-06-21
+ *  Updated   : 2010-01-01
  *  Notes     : signal handlers for different modes and window resizing
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,15 +48,6 @@
 #	include "version.h"
 #endif /* !VERSION_H */
 
-
-#if defined(VMS) && defined(SIGBUS)
-	/*
-	 * SIGBUS works on VMS, but many useful information will be hidden
-	 * if catched by signal handler.
-	 */
-#	undef SIGBUS
-#endif /* VMS && SIGBUS */
-
 /*
  * Needed for resizing under an xterm
  */
@@ -104,6 +95,9 @@ static void _CDECL signal_handler(SIG_ARGS);
 #ifdef SIGTSTP
 	static t_bool do_sigtstp = FALSE;
 #endif /* SIGTSTP */
+#if defined(SIGWINCH) || defined(SIGTSTP)
+	static t_bool redraw_after_suspend;
+#endif /* SIGWINCH || SIGTSTP */
 
 int signal_context = cMain;
 int input_context = cNone;
@@ -123,7 +117,7 @@ static const struct {
 	{ SIGINT,	"SIGINT" },	/* ctrl-C */
 #	endif /* SIGINT */
 #	ifdef SIGQUIT
-	{ SIGQUIT,	"SIGQUIT " },	/* ctrl-\ */
+	{ SIGQUIT,	"SIGQUIT" },	/* ctrl-\ */
 #	endif /* SIGQUIT */
 #	ifdef SIGILL
 	{ SIGILL,	"SIGILL" },	/* illegal instruction */
@@ -140,6 +134,9 @@ static const struct {
 #	ifdef SIGPIPE
 	{ SIGPIPE,	"SIGPIPE" },	/* broken pipe */
 #	endif /* SIGPIPE */
+#	ifdef SIGALRM
+	{ SIGALRM,	"SIGALRM" },	/* real-time timer expired */
+#	endif /* SIGALRM */
 #	ifdef SIGCHLD
 	{ SIGCHLD,	"SIGCHLD" },	/* death of a child process */
 #	endif /* SIGCHLD */
@@ -155,6 +152,9 @@ static const struct {
 #	ifdef SIGUSR1
 	{ SIGUSR1,	"SIGUSR1" },	/* User-defined signal 1 */
 #	endif /* SIGUSR1 */
+#	ifdef SIGUSR2
+	{ SIGUSR2,	"SIGUSR2" },	/* User-defined signal 2 */
+#	endif /* SIGUSR2 */
 #	ifdef SIGTERM
 	{ SIGTERM,	"SIGTERM" },	/* termination */
 #	endif /* SIGTERM */
@@ -274,9 +274,10 @@ handle_resize(
 	switch (signal_context) {
 		case cArt:
 			ClearScreen();
-			show_art_msg(curr_group->name);
+			show_art_msg(CURR_GROUP.name);
 			break;
 
+		case cAttrib:
 		case cConfig:
 			refresh_config_page(SIGNAL_HANDLER);
 			break;
@@ -290,6 +291,7 @@ handle_resize(
 			break;
 
 		case cGroup:
+		case cScope:
 		case cSelect:
 		case cThread:
 			ClearScreen();
@@ -301,6 +303,11 @@ handle_resize(
 			draw_page(curr_group->name, 0);
 			break;
 
+		case cReconnect:
+			ClearScreen();
+			show_title(tin_progname);
+			break;
+
 		case cMain:
 			break;
 	}
@@ -309,14 +316,24 @@ handle_resize(
 			gl_redraw();
 			break;
 
+		case cPromptCONT:
+			if (redraw_after_suspend)
+				info_message(_(txt_return_key));
+			break;
+
 		case cPromptSLK:
 			prompt_slk_redraw();
 			break;
 
+		case cPromptYN:
+			prompt_yn_redraw();
+			break;
+
 		default:
 			break;
 	}
 	my_fflush(stdout);
+	redraw_after_suspend = FALSE;
 #endif /* SIGWINCH || SIGTSTP */
 }
 
@@ -326,22 +343,34 @@ static void
 handle_suspend(
 	void)
 {
+	t_bool save_cmd_line = cmd_line;
+	t_bool save_state = (!batch_mode || !cmd_line);
+
 	TRACE(("handle_suspend(%d)", signal_context));
 
 	set_keypad_off();
 	if (!cmd_line)
 		set_xclick_off();
 
-	Raw(FALSE);
+	if (save_state) {
+		EndWin();
+		Raw(FALSE);
+	}
+
 	wait_message(0, _(txt_suspended_message), tin_progname);
 
 	kill(0, SIGSTOP);				/* Put ourselves to sleep */
 
 	RESTORE_HANDLER(SIGTSTP, signal_handler);
 
-	if (!batch_mode) {
+	if (save_state) {
 		Raw(TRUE);
+		InitWin();
+		cmd_line = save_cmd_line;
+		if (!cmd_line)
+			my_retouch();
 		need_resize = cRedraw;		/* Flag a redraw */
+		redraw_after_suspend = TRUE;
 	}
 	set_keypad_on();
 	if (!cmd_line)
@@ -366,13 +395,21 @@ signal_handler(
 	switch (sig) {
 #ifdef SIGINT
 		case SIGINT:
-			if (!batch_mode) {
-				RESTORE_HANDLER(sig, signal_handler);
-				return;
-			}
-			break;
+			RESTORE_HANDLER(sig, signal_handler);
+			return;
 #endif /* SIGINT */
 
+/*
+ * fatal error but we don't want the "signal handler caught signal"
+ * message here
+ */
+#ifdef SIGALRM
+		case SIGALRM:
+			error_message(2, "NNTP connection error. Exiting...");
+			tin_done(NNTP_ERROR_EXIT);
+			return;
+#endif /* SIGALRM */
+
 #ifdef SIGCHLD
 		case SIGCHLD:
 			wait(&wait_status);
@@ -381,13 +418,6 @@ signal_handler(
 			return;
 #endif /* SIGCHLD */
 
-#ifdef SIGPIPE
-		case SIGPIPE:
-			got_sig_pipe = TRUE;
-			RESTORE_HANDLER(sig, signal_handler);
-			return;
-#endif /* SIGPIPE */
-
 #ifdef SIGTSTP
 		case SIGTSTP:
 			handle_suspend();
@@ -397,10 +427,18 @@ signal_handler(
 #ifdef SIGWINCH
 		case SIGWINCH:
 			need_resize = cYes;
-			RESTORE_HANDLER(SIGWINCH, signal_handler);
+			RESTORE_HANDLER(sig, signal_handler);
 			return;
 #endif /* SIGWINCH */
 
+#ifdef SIGUSR2
+		case SIGUSR2:
+			if (!no_write) /* TODO: add more config-files to be saved */
+				write_newsrc();
+			RESTORE_HANDLER(sig, signal_handler);
+			return;
+#endif /* SIGUSR2 */
+
 		default:
 			break;
 	}
@@ -424,12 +462,17 @@ signal_handler(
 			break;
 #endif /* SIGHUP || SIGUSR1 || SIGTERM */
 
-#ifdef SIGBUS
-		case SIGBUS:
-#endif /* SIGBUS */
 #ifdef SIGSEGV
 		case SIGSEGV:
+#	if defined(SIGBUS) && (SIGSEGV != SIGBUS) /* on Haiku SIGSEGV == SIGBUS */
+		case SIGBUS:
+#	endif /* SIGBUS && SIGSEGV != SIGBUS */
+#else
+#	ifdef SIGBUS
+		case SIGBUS:
+#	endif /* SIGBUS */
 #endif /* SIGSEGV */
+
 #if defined(SIGBUS) || defined(SIGSEGV)
 			my_fprintf(stderr, _(txt_send_bugreport), tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME, bug_addr);
 			my_fflush(stderr);
@@ -483,6 +526,11 @@ set_signal_handlers(
 
 	for (n = 0; n < ARRAY_SIZE(signal_list); n++) {
 		switch ((code = signal_list[n].code)) {
+#ifdef SIGPIPE
+		case SIGPIPE:
+			sigdisp(code, SIG_IGN);
+			break;
+#endif /* SIGPIPE */
 #ifdef SIGTSTP
 		case SIGTSTP:
 			ptr = sigdisp(code, SIG_DFL);
