@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2005-11-04
+ *  Updated   : 2010-09-10
  *  Notes     :
  *
- * Copyright (c) 1997-2006 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2010 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,15 +65,18 @@
 /*
  * local prototypes
  */
-#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static UChar *char2UChar(const char *str);
-	static char *UChar2char(const UChar *ustr);
-#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && !NO_LOCALE */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	static wchar_t *my_wcsdup(const wchar_t *wstr);
+#	if defined(HAVE_LIBICUUC)
+		static UChar *char2UChar(const char *str);
+		static char *UChar2char(const UChar *ustr);
+#	endif /* HAVE_LIBICUUC */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 
 /*
  * special ltoa()
- * converts value into a string with a maxlen of digits (usualy should be
+ * converts value into a string with a maxlen of digits (usually should be
  * >=4), last char may be one of the following:
  * 'K'ilo, 'M'ega, 'G'iga, 'T'erra, 'P'eta, 'E'xa, 'Z'etta, 'Y'otta,
  * 'X'ona, 'W'eka, 'V'unda, 'U'da (these last 4 are no official SI-prefixes)
@@ -125,6 +128,7 @@ tin_ltoa(
 }
 
 
+#if !defined(USE_DMALLOC) || (defined(USE_DMALLOC) && !defined(HAVE_STRDUP))
 /*
  * Handrolled version of strdup(), presumably to take advantage of
  * the enhanced error detection in my_malloc
@@ -138,15 +142,15 @@ my_strdup(
 	size_t len = strlen(str) + 1;
 	void *ptr = my_malloc(len);
 
-#if 0 /* as my_malloc exits on error, ptr can't be NULL */
+#	if 0 /* as my_malloc exits on error, ptr can't be NULL */
 	if (ptr == NULL)
 		return NULL;
-#endif /* 0 */
+#	endif /* 0 */
 
 	memcpy(ptr, str, len);
 	return (char *) ptr;
 }
-
+#endif /* !USE_DMALLOC || (USE_DMALLOC && !HAVE_STRDUP) */
 
 /*
  * strtok that understands empty tokens
@@ -422,7 +426,7 @@ OUT:
 
 	return (sign * (-val));
 }
-#	undef DIGIT(x)
+#	undef DIGIT
 #	undef MBASE
 #endif /* !HAVE_STRTOL */
 
@@ -670,6 +674,9 @@ sh_format(
 #	ifdef HAVE_SYS_ERRLIST
 		extern int sys_nerr;
 #	endif /* HAVE_SYS_ERRLIST */
+#	ifdef DECL_SYS_ERRLIST
+		extern char *sys_errlist[];
+#	endif /* DECL_SYS_ERRLIST */
 char *
 my_strerror(
 	int n)
@@ -755,6 +762,29 @@ wchar_t2char(
 }
 
 
+/*
+ * Interface to wcspart for non wide character strings
+ */
+char *
+spart(
+	const char *str,
+	int columns,
+	t_bool pad)
+{
+	char *buf = NULL;
+	wchar_t *wbuf, *wbuf2;
+
+	if ((wbuf = char2wchar_t(str)) != NULL) {
+		wbuf2 = wcspart(wbuf, columns, pad);
+		free(wbuf);
+		buf = wchar_t2char(wbuf2);
+		FreeIfNeeded(wbuf2);
+	}
+
+	return buf;
+}
+
+
 /*
  * returns a new string fitting into 'columns' columns
  * if pad is TRUE the resulting string will be filled with spaces if necessary
@@ -795,9 +825,141 @@ wcspart(
 
 	return wbuf;
 }
+
+
+/*
+ * wcs version of abbr_groupname()
+ */
+wchar_t *
+abbr_wcsgroupname(
+	const wchar_t *grpname,
+	int len)
+{
+	wchar_t *src, *dest, *tail, *new_grpname;
+	int tmplen, newlen;
+
+	dest = new_grpname = my_wcsdup(grpname);
+
+	if (wcswidth(grpname, wcslen(grpname)) > len) {
+		if ((src = wcschr(grpname, (wchar_t) '.')) != NULL) {
+			tmplen = wcwidth(*dest++);
+
+			do  {
+				*dest++ = *src;
+				tmplen += wcwidth(*src++);
+				*dest++ = *src;
+				tmplen += wcwidth(*src++);
+				tail = src;
+				newlen = wcswidth(tail, wcslen(tail)) + tmplen;
+			} while ((src = wcschr(src, (wchar_t) '.')) != NULL && newlen > len);
+
+			if (newlen > len)
+				*dest++ = (wchar_t) '.';
+			else {
+				while (*tail)
+					*dest++ = *tail++;
+			}
+
+			*dest = (wchar_t) '\0';
+			new_grpname = my_realloc(new_grpname, sizeof(wchar_t) * (wcslen(new_grpname) + 1));
+
+			if (wcswidth(new_grpname, wcslen(new_grpname)) > len) {
+				dest = wstrunc(new_grpname, len);
+				free(new_grpname);
+				new_grpname = dest;
+			}
+		} else {
+			dest = wstrunc(new_grpname, len);
+			free(new_grpname);
+			new_grpname = dest;
+		}
+	}
+
+	return new_grpname;
+}
+
+#else /* !MULTIBYTE_ABLE || NO_LOCALE */
+
+/*
+ * Abbreviate a groupname like this:
+ * 	foo.bar.baz
+ * 	f.bar.baz
+ * 	f.b.baz
+ * 	f.b.b.
+ * depending on the given length
+ */
+char *
+abbr_groupname(
+	const char *grpname,
+	size_t len)
+{
+	char *src, *dest, *tail, *new_grpname;
+	size_t tmplen, newlen;
+
+	dest = new_grpname = my_strdup(grpname);
+
+	if (strlen(grpname) > len) {
+		if ((src = strchr(grpname, '.')) != NULL) {
+			dest++;
+			tmplen = 1;
+
+			do  {
+				*dest++ = *src++;
+				*dest++ = *src++;
+				tmplen += 2;
+				tail = src;
+				newlen = strlen(tail) + tmplen;
+			} while ((src = strchr(src, '.')) != NULL && newlen > len);
+
+			if (newlen > len)
+				*dest++ = '.';
+			else {
+				while (*tail)
+					*dest++ = *tail++;
+			}
+
+			*dest = '\0';
+			new_grpname = my_realloc(new_grpname, strlen(new_grpname) + 1);
+
+			if (strlen(new_grpname) > len) {
+				dest = strunc(new_grpname, len);
+				free(new_grpname);
+				new_grpname = dest;
+			}
+		} else {
+			dest = strunc(new_grpname, len);
+			free(new_grpname);
+			new_grpname = dest;
+		}
+	}
+
+	return new_grpname;
+}
 #endif /* MULTIBYTE_ABLE && !NOLOCALE */
 
 
+/*
+ * Returns the number of screen positions a string occupies
+ */
+int
+strwidth(
+	const char *str)
+{
+	int columns = (int) strlen(str);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	int width;
+	wchar_t *wbuf;
+
+	if ((wbuf = char2wchar_t(str)) != NULL) {
+		if ((width = wcswidth(wbuf, wcslen(wbuf) + 1)) > 0)
+			columns = width;
+		free(wbuf);
+	}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	return columns;
+}
+
+
 #define TRUNC_TAIL	"..."
 /*
  * shortens 'mesg' that it occupies at most 'len' screen positions.
@@ -890,7 +1052,7 @@ wstrunc(
 /*
  * duplicates a wide-char string
  */
-wchar_t *
+static wchar_t *
 my_wcsdup(
 	const wchar_t *wstr)
 {
@@ -1008,11 +1170,11 @@ normalize(
 		if ((ustr = char2UChar(tmp)) == NULL) /* something went wrong, return the original string (as valid UTF8) */
 			return tmp;
 
-		needed = unorm_normalize(ustr, -1, mode, 0 , NULL, 0, &status);
+		needed = unorm_normalize(ustr, -1, mode, 0, NULL, 0, &status);
 		status = U_ZERO_ERROR;		/* reset status */
 		norm_len = needed + 1;
 		norm = my_malloc(sizeof(UChar) * norm_len);
-		needed = unorm_normalize(ustr, -1, mode, 0 , norm, norm_len, &status);
+		needed = unorm_normalize(ustr, -1, mode, 0, norm, norm_len, &status);
 		if (U_FAILURE(status)) {
 			/* something went wrong, return the original string (as valid UTF8) */
 			free(ustr);
@@ -1023,6 +1185,8 @@ normalize(
 		/* convert back to UTF-8 */
 		if ((buf = UChar2char(norm)) == NULL) /* something went wrong, return the original string (as valid UTF8) */
 			buf = tmp;
+		else
+			free(tmp);
 
 		free(ustr);
 		free(norm);
@@ -1044,7 +1208,7 @@ normalize(
 
 
 /*
- * returns a pointer to allocated buffer containing the formated string
+ * returns a pointer to allocated buffer containing the formatted string
  * must be freed if not needed anymore
  */
 char *
