@@ -3,10 +3,10 @@
  *  Module    : tags.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1999-12-06
- *  Updated   : 2005-05-10
+ *  Updated   : 2010-04-02
  *  Notes     : Split out from other modules
  *
- * Copyright (c) 1999-2006 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2010 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -136,7 +136,7 @@ get_multiparts(
 	MultiPartInfo **malloc_and_setme_info)
 {
 	MultiPartInfo tmp, tmp2;
-	MultiPartInfo *info = 0;
+	MultiPartInfo *info = NULL;
 	int i;
 	int part_index;
 
@@ -207,7 +207,7 @@ int
 tag_multipart(
 	int base_index)
 {
-	MultiPartInfo *info = 0;
+	MultiPartInfo *info = NULL;
 	int i;
 	const int qty = get_multiparts(base_index, &info);
 
@@ -253,7 +253,7 @@ line_is_tagged(
 {
 	int code = 0;
 
-	if (curr_group->attribute->thread_arts) {
+	if (curr_group->attribute->thread_articles) {
 		int i;
 		for (i = n; i >= 0; i = arts[i].thread) {
 			if (arts[i].tagged > code)
@@ -341,10 +341,10 @@ untag_all_articles(
  * Return TRUE if a range was successfully read, parsed and set
  *
  * Allowed syntax is 0123456789-.$ (blanks are ignored):
- *   1-23    mark grp/art 1 thru 23
- *   1-.     mark grp/art 1 thru current
- *   1-$     mark grp/art 1 thru last
- *   .-$     mark grp/art current thru last
+ *   1-23    mark grp/art 1 through 23
+ *   1-.     mark grp/art 1 through current
+ *   1-$     mark grp/art 1 through last
+ *   .-$     mark grp/art current through last
  */
 t_bool
 set_range(
@@ -374,12 +374,12 @@ set_range(
 			range = tinrc.default_range_thread;
 			break;
 
-		default:
+		default:	/* should no happen */
 			return FALSE;
 	}
 
 #if 0
-	error_message("Min=[%d] Max=[%d] Cur=[%d] DefRng=[%s]", min, max, curr, range);
+	error_message(2, "Min=[%d] Max=[%d] Cur=[%d] DefRng=[%s]", min, max, curr, range);
 #endif /* 0 */
 	prompt = fmt_string(_(txt_enter_range), range);
 
@@ -523,14 +523,17 @@ do_auto_select_arts(
 
 	for_each_art(i) {
 		if (arts[i].status == ART_UNREAD && !arts[i].selected) {
-#	ifdef DEBUG_NEWSRC
-			debug_print_comment("group.c: X command");
-#	endif /* DEBUG_NEWSRC */
+#	ifdef DEBUG
+			if (debug & DEBUG_NEWSRC)
+				debug_print_comment("group.c: X command");
+#	endif /* DEBUG */
 			art_mark(curr_group, &arts[i], ART_READ);
 			arts[i].zombie = TRUE;
 		}
+		if (curr_group->attribute->show_only_unread_arts)
+			arts[i].keep_in_base = FALSE;
 	}
-	if (curr_group->attribute->show_only_unread)
+	if (curr_group->attribute->show_only_unread_arts)
 		find_base(curr_group);
 
 	grpmenu.curr = 0;
@@ -547,14 +550,15 @@ undo_auto_select_arts(
 
 	for_each_art(i) {
 		if (arts[i].status == ART_READ && arts[i].zombie) {
-#	ifdef DEBUG_NEWSRC
-			debug_print_comment("group.c: + command");
-#	endif /* DEBUG_NEWSRC */
+#	ifdef DEBUG
+			if (debug & DEBUG_NEWSRC)
+				debug_print_comment("group.c: + command");
+#	endif /* DEBUG */
 			art_mark(curr_group, &arts[i], ART_UNREAD);
 			arts[i].zombie = FALSE;
 		}
 	}
-	if (curr_group->attribute->show_only_unread)
+	if (curr_group->attribute->show_only_unread_arts)
 		find_base(curr_group);
 
 	grpmenu.curr = 0;	/* do we want this? */
@@ -591,45 +595,3 @@ arts_selected(
 
 	return FALSE;
 }
-
-
-/*
- * Mark all tagged articles as read.
- * Return number of articles that have been marked.
- */
-int
-mark_tagged_read(
-	struct t_group *group)
-{
-	int i, cnt = 0;
-
-	for_each_art(i) {
-		if (arts[i].tagged && (arts[i].status == ART_UNREAD || arts[i].status == ART_WILL_RETURN)) {
-			art_mark(group, &arts[i], ART_READ);
-			cnt++;
-		}
-	}
-	untag_all_articles();
-
-	return cnt;
-}
-
-
-/*
- * Check if there is at least one tagged _and_ unread article.
- */
-t_bool
-got_tagged_unread_arts(
-	void)
-{
-	int i;
-
-	if (num_of_tagged_arts > 0) {
-		for_each_art(i) {
-			if (arts[i].tagged && (arts[i].status == ART_UNREAD || arts[i].status == ART_WILL_RETURN))
-				return TRUE;
-		}
-	}
-
-	return FALSE;
-}
