@@ -3,10 +3,10 @@
  *  Module    : thread.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2006-07-20
+ *  Updated   : 2010-10-07
  *  Notes     :
  *
- * Copyright (c) 1991-2006 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,8 +54,7 @@ t_bool show_subject;
  */
 static char get_art_mark(struct t_article *art);
 static int enter_pager(int art, t_bool ignore_unavail, int level);
-static int mark_art_read(struct t_group *group);
-static int thread_catchup(t_function func);
+static int thread_catchup(t_function func, struct t_group *group);
 static int thread_tab_pressed(void);
 static t_bool find_unexpired(struct t_msgid *ptr);
 static t_bool has_sibling(struct t_msgid *ptr);
@@ -89,7 +88,7 @@ get_art_mark(
 	if (art->inrange) {
 		return tinrc.art_marked_inrange;
 	} else if (art->status == ART_UNREAD) {
-		return (art->selected ? tinrc.art_marked_selected : (tinrc.recent_time && ((time((time_t) 0) - art->date) < (tinrc.recent_time * DAY))) ? tinrc.art_marked_recent : tinrc.art_marked_unread);
+		return (art->selected ? tinrc.art_marked_selected : (tinrc.recent_time && ((time((time_t *) 0) - art->date) < (tinrc.recent_time * DAY))) ? tinrc.art_marked_recent : tinrc.art_marked_unread);
 	} else if (art->status == ART_WILL_RETURN) {
 		return tinrc.art_marked_return;
 	} else if (art->killed && tinrc.kill_level != KILL_NOTHREAD) {
@@ -147,7 +146,7 @@ build_tline(
 	 * Start with 2 spaces for ->
 	 * then index number of the message and whitespace (2+4+1 chars)
 	 */
-#if 0 /* usefull? see also group.c:build_sline() */
+#if 0 /* useful? see also group.c:build_sline() */
 	if (!tinrc.draw_arrow)
 		sprintf(buffer, "%s ", tin_ltoa(l + 1, 6));
 	else
@@ -247,6 +246,7 @@ build_tline(
 
 			for (ptr = art->refptr->parent; ptr && IS_EXPIRED(ptr); ptr = ptr->parent)
 				;
+
 			if (!(ptr && arts[ptr->article].subject == art->subject)) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				if ((wtmp = char2wchar_t(art->subject)) != NULL) {
@@ -269,14 +269,7 @@ build_tline(
 		 * If we need to show the author, pad out to the start of the author field,
 		 */
 		if (len_from) {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-			if ((wtmp = char2wchar_t(buffer)) != NULL) {
-				fill = cCOLS - len_from - wcswidth(wtmp, wcslen(wtmp) + 1);
-				free(wtmp);
-			} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-				fill = cCOLS - len_from - strlen(buffer);
-
+			fill = cCOLS - len_from - strwidth(buffer);
 			gap = strlen(buffer);
 			for (i = 0; i < fill; i++)
 				buffer[gap + i] = ' ';
@@ -322,17 +315,8 @@ build_tline(
 	convert_to_printable(buffer);
 
 	if (!tinrc.strip_blanks) {
-		/*
-		 * Pad to end of line so that inverse bar looks 'good'
-		 */
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		if ((wtmp = char2wchar_t(buffer)) != NULL) {
-			fill = cCOLS - wcswidth(wtmp, wcslen(wtmp) + 1);
-			free(wtmp);
-		} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-			fill = cCOLS - strlen(buffer);
-
+		/* Pad to end of line so that inverse bar looks 'good' */
+		fill = cCOLS - strwidth(buffer);
 		gap = strlen(buffer);
 		for (i = 0; i < fill; i++)
 			buffer[gap + i] = ' ';
@@ -364,7 +348,7 @@ static t_function
 thread_left(
 	void)
 {
-	if (tinrc.thread_catchup_on_exit)
+	if (curr_group->attribute->thread_catchup_on_exit)
 		return SPECIAL_CATCHUP_LEFT;			/* ie, not via 'c' or 'C' */
 	else
 		return GLOBAL_QUIT;
@@ -403,6 +387,7 @@ thread_page(
 	char key[MAXKEYLEN];
 	char mark[] = { '\0', '\0' };
 	int i, n;
+	long old_artnum = 0L;
 	t_bool repeat_search;
 	t_function func;
 
@@ -426,8 +411,8 @@ thread_page(
 	if (thread_depth)
 		thdmenu.curr = thread_depth;
 	else {
-		if (tinrc.pos_first_unread) {
-			if ((i = new_responses(thread_basenote))) {
+		if (group->attribute->pos_first_unread) {
+			if (new_responses(thread_basenote)) {
 				for (n = 0, i = (int) base[thread_basenote]; i >= 0; i = arts[i].thread, n++) {
 					if (arts[i].status == ART_UNREAD || arts[i].status == ART_WILL_RETURN) {
 						if (arts[i].thread == ART_EXPIRED)
@@ -508,49 +493,59 @@ thread_page(
 				break;
 
 			case GLOBAL_SET_RANGE:		/* set range */
-				if (set_range(THREAD_LEVEL, 1, thdmenu.max, thdmenu.curr + 1))
+				if (set_range(THREAD_LEVEL, 1, thdmenu.max, thdmenu.curr + 1)) {
+					range_active = TRUE;
 					show_thread_page();
+				}
 				break;
 
 			case GLOBAL_PIPE:			/* pipe article(s) to command */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_PIPE, THREAD_LEVEL, group, find_response(thread_basenote, thdmenu.curr));
+					feed_articles(FEED_PIPE, THREAD_LEVEL, NOT_ASSIGNED, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 
 #ifndef DISABLE_PRINTING
 			case GLOBAL_PRINT:			/* print article(s) */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_PRINT, THREAD_LEVEL, group, find_response(thread_basenote, thdmenu.curr));
+					feed_articles(FEED_PRINT, THREAD_LEVEL, NOT_ASSIGNED, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 #endif /* !DISABLE_PRINTING */
 
 			case THREAD_MAIL:	/* mail article(s) to somebody */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_MAIL, THREAD_LEVEL, group, find_response(thread_basenote, thdmenu.curr));
+					feed_articles(FEED_MAIL, THREAD_LEVEL, NOT_ASSIGNED, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 
 			case THREAD_SAVE:	/* save articles with prompting */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_SAVE, THREAD_LEVEL, group, find_response(thread_basenote, thdmenu.curr));
+					feed_articles(FEED_SAVE, THREAD_LEVEL, NOT_ASSIGNED, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 
 			case THREAD_AUTOSAVE:	/* Auto-save articles without prompting */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_AUTOSAVE, THREAD_LEVEL, group, (int) base[grpmenu.curr]);
+					feed_articles(FEED_AUTOSAVE, THREAD_LEVEL, NOT_ASSIGNED, group, (int) base[grpmenu.curr]);
+				break;
+
+			case MARK_FEED_READ:	/* mark selected articles as read */
+				if (thread_basenote >= 0)
+					ret_code = feed_articles(FEED_MARK_READ, THREAD_LEVEL, NOT_ASSIGNED, group, find_response(thread_basenote, thdmenu.curr));
+				break;
+
+			case MARK_FEED_UNREAD:	/* mark selected articles as unread */
+				if (thread_basenote >= 0)
+					feed_articles(FEED_MARK_UNREAD, THREAD_LEVEL, NOT_ASSIGNED, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 
 			case GLOBAL_MENU_FILTER_SELECT:
 			case GLOBAL_MENU_FILTER_KILL:
-				/*
-				 * FIXME: a filter which kills all remaining messages in a thread
-				 *        is 'troublesome'
-				 */
 				n = find_response(thread_basenote, thdmenu.curr);
-				filter_menu(func, group, &arts[n]);
-				if (filter_articles(group)) {
+				if (filter_menu(func, group, &arts[n])) {
+					old_artnum = arts[n].artnum;
+					unfilter_articles(group);
+					filter_articles(group);
 					make_threads(group, FALSE);
-					if ((n = next_unread(n)) == -1) {
-						ret_code = GRP_EXIT;
+					if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) { /* We have lost the thread */
+						ret_code = GRP_KILLED;
 						break;
 					}
 					fixup_thread(n, TRUE);
@@ -559,14 +554,14 @@ thread_page(
 				break;
 
 			case GLOBAL_EDIT_FILTER:
-				if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
-					break;
-				unfilter_articles();
-				(void) read_filter_file(filter_file);
-				if (filter_articles(group)) {
+				if (invoke_editor(filter_file, filter_file_offset, NULL)) {
+					old_artnum = arts[find_response(thread_basenote, thdmenu.curr)].artnum;
+					unfilter_articles(group);
+					(void) read_filter_file(filter_file);
+					filter_articles(group);
 					make_threads(group, FALSE);
-					if ((n = next_unread(n)) == -1) {
-						ret_code = GRP_EXIT;
+					if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) { /* We have lost the thread */
+						ret_code = GRP_KILLED;
 						break;
 					}
 					fixup_thread(n, TRUE);
@@ -582,6 +577,21 @@ thread_page(
 				ret_code = thread_tab_pressed();
 				break;
 
+			case THREAD_CANCEL:		/* cancel current article */
+				if (can_post || group->attribute->mailing_list != NULL) {
+					char *progress_msg = my_strdup(_(txt_reading_article));
+					int ret;
+
+					n = find_response(thread_basenote, thdmenu.curr);
+					ret = art_open(TRUE, &arts[n], group, &pgart, TRUE, progress_msg);
+					free(progress_msg);
+					if (ret != ART_UNAVAILABLE && ret != ART_ABORT && cancel_article(group, &arts[n], n))
+						show_thread_page();
+					art_close(&pgart);
+				} else
+					info_message(_(txt_cannot_post));
+				break;
+
 			case GLOBAL_POST:		/* post a basenote */
 				if (post_article(group->name))
 					show_thread_page();
@@ -620,11 +630,19 @@ thread_page(
 			case SPECIAL_CATCHUP_LEFT:				/* come here when exiting thread via <- */
 			case CATCHUP:				/* catchup thread, move to next one */
 			case CATCHUP_NEXT_UNREAD:	/* -> next with unread arts */
-				ret_code = thread_catchup(func);
+				ret_code = thread_catchup(func, group);
 				break;
 
-			case THREAD_MARK_ARTICLE_READ: /* mark article as read */
-				ret_code = mark_art_read(group);
+			case THREAD_MARK_ARTICLE_READ:	/* mark current article/range/tagged articles as read */
+			case MARK_ARTICLE_UNREAD:		/* or unread */
+				if (thread_basenote >= 0) {
+					t_function function, type;
+
+					function = func == THREAD_MARK_ARTICLE_READ ? (t_function) FEED_MARK_READ : (t_function) FEED_MARK_UNREAD;
+					type = range_active ? FEED_RANGE : (num_of_tagged_arts && !group->attribute->mark_ignore_tags) ? NOT_ASSIGNED : FEED_ARTICLE;
+					if (feed_articles(function, THREAD_LEVEL, type, group, find_response(thread_basenote, thdmenu.curr)) == 1)
+						ret_code = GRP_EXIT;
+				}
 				break;
 
 			case THREAD_TOGGLE_SUBJECT_DISPLAY:	/* toggle display of subject & subj/author */
@@ -636,11 +654,13 @@ thread_page(
 				break;
 
 			case GLOBAL_OPTION_MENU:
-				change_config_file(group);
 				n = find_response(thread_basenote, thdmenu.curr);
-				if (which_thread(n) == -1) /* We have lost the thread */
+				old_artnum = arts[n].artnum;
+				config_page(group->name);
+				if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) { /* We have lost the thread */
+					pos_first_unread_thread();
 					ret_code = GRP_EXIT;
-				else {
+				} else {
 					fixup_thread(n, FALSE);
 					thdmenu.curr = which_response(n);
 					show_thread_page();
@@ -657,6 +677,10 @@ thread_page(
 					ret_code = enter_pager(n, FALSE, THREAD_LEVEL);
 				break;
 
+			case GLOBAL_SEARCH_REPEAT:
+				info_message(_(txt_no_prev_search));
+				break;
+
 			case GLOBAL_SEARCH_BODY:			/* search article body */
 				if ((n = search_body(group, find_response(thread_basenote, thdmenu.curr), repeat_search)) != -1) {
 					fixup_thread(n, FALSE);
@@ -735,15 +759,6 @@ thread_page(
 				info_message(cvers);
 				break;
 
-			case MARK_ARTICLE_UNREAD:	/* mark article as unread */
-				n = find_response(thread_basenote, thdmenu.curr);
-				art_mark(group, &arts[n], ART_WILL_RETURN);
-				mark[0] = get_art_mark(&arts[n]);
-				mark_screen(thdmenu.curr, MARK_OFFSET, mark);
-				draw_thread_arrow();
-				info_message(_(txt_marked_as_unread), _(txt_article_upper));
-				break;
-
 			case MARK_THREAD_UNREAD:		/* mark thread as unread */
 				thd_mark_unread(group, base[thread_basenote]);
 				update_thread_page();
@@ -822,7 +837,7 @@ show_thread_page(
 	/*
 	 * If threading by Refs, it helps to see the subject line
 	 */
-	show_subject = ((arts[thread_respnum].archive != NULL) || (curr_group->attribute->thread_arts == THREAD_REFS) || (curr_group->attribute->thread_arts == THREAD_BOTH));
+	show_subject = ((arts[thread_respnum].archive != NULL) || (curr_group->attribute->thread_articles == THREAD_REFS) || (curr_group->attribute->thread_articles == THREAD_BOTH));
 
 	if (show_subject)
 		title = fmt_string(_(txt_stp_list_thread), grpmenu.curr + 1, grpmenu.max);
@@ -955,7 +970,9 @@ which_thread(
 			return j;
 	}
 
-	error_message(_(txt_cannot_find_base_art), n);
+#ifdef DEBUG
+	error_message(2, _(txt_cannot_find_base_art), n);
+#endif /* DEBUG */
 	return -1;
 }
 
@@ -1103,7 +1120,7 @@ stat_thread(
 		if (arts[i].killed)
 			++sbuf->killed;
 
-		if ((curr_group->attribute->thread_arts == THREAD_MULTI) && global_get_multipart_info(i, &minfo) && (minfo.total >= 1)) {
+		if ((curr_group->attribute->thread_articles == THREAD_MULTI) && global_get_multipart_info(i, &minfo) && (minfo.total >= 1)) {
 			sbuf->multipart_compare_len = minfo.subject_compare_len;
 			sbuf->multipart_total = minfo.total;
 			sbuf->multipart_have++;
@@ -1119,7 +1136,7 @@ stat_thread(
 	else if (sbuf->selected_unread)
 		sbuf->art_mark = tinrc.art_marked_selected;
 	else if (sbuf->unread) {
-		if (tinrc.recent_time && (time((time_t) 0) - sbuf->time) < (tinrc.recent_time * DAY))
+		if (tinrc.recent_time && (time((time_t *) 0) - sbuf->time) < (tinrc.recent_time * DAY))
 			sbuf->art_mark = tinrc.art_marked_recent;
 		else
 			sbuf->art_mark = tinrc.art_marked_unread;
@@ -1236,7 +1253,7 @@ next_unread(
 		n = next_response(n);
 	}
 
-	if (tinrc.wrap_on_next_unread) {
+	if (curr_group->attribute->wrap_on_next_unread) {
 		n = base[0];
 		while (n != cur_base_art) {
 			if (((arts[n].status == ART_UNREAD) || (arts[n].status == ART_WILL_RETURN)) && arts[n].thread != ART_EXPIRED)
@@ -1361,7 +1378,8 @@ make_prefix(
  */
 static int
 thread_catchup(
-	t_function func)
+	t_function func,
+	struct t_group *group)
 {
 	char buf[LEN];
 	int i, n;
@@ -1375,12 +1393,10 @@ thread_catchup(
 	}
 
 	if (i != -1) {				/* still unread arts in this thread */
-		/*
-		 * TODO: if (group->attribute->thread_arts == THREAD_NONE)
-		 *          snprintf(buf, sizeof(buf), _("Mark article as read%s?"), (func == CATCHUP_NEXT_UNREAD) ? _(" and enter next unread article") : "");
-		 *       else
-		 */
-		snprintf(buf, sizeof(buf), _(txt_mark_thread_read), (func == CATCHUP_NEXT_UNREAD) ? _(txt_enter_next_thread) : "");
+		if (group->attribute->thread_articles == THREAD_NONE)
+			snprintf(buf, sizeof(buf), _(txt_mark_art_read), (func == CATCHUP_NEXT_UNREAD) ? _(txt_enter_next_unread_art) : "");
+		else
+			snprintf(buf, sizeof(buf), _(txt_mark_thread_read), (func == CATCHUP_NEXT_UNREAD) ? _(txt_enter_next_thread) : "");
 		if ((!TINRC_CONFIRM_ACTION) || (pyn = prompt_yn(buf, TRUE)) == 1)
 			thd_mark_read(curr_group, base[thread_basenote]);
 	}
@@ -1442,9 +1458,11 @@ again:
 	switch ((i = show_page(curr_group, art, &thdmenu.curr))) {
 		/* These exit to previous menu level */
 		case GRP_QUIT:				/* 'Q' all the way out */
+		case GRP_EXIT:				/*     back to group menu */
 		case GRP_RETSELECT:			/* 'T' back to select menu */
 		case GRP_NEXT:				/* 'c' Move to next thread on group menu */
 		case GRP_NEXTUNREAD:		/* 'C' */
+		case GRP_KILLED:			/*     article/thread was killed at page level */
 			break;
 
 		case GRP_ARTABORT:			/* user 'q'uit load of article */
@@ -1471,8 +1489,6 @@ again:
 			return 0;
 
 		default:					/* >=0 normal exit, new basenote */
-			if (filtered_articles)
-				return GRP_KILLED; /* ? set group cursor back to 0 and do nothing */
 			fixup_thread(this_resp, FALSE);
 
 			if (currmenu != &grpmenu)	/* group menu will redraw itself */
@@ -1516,65 +1532,45 @@ thread_tab_pressed(
 
 
 /*
- * If there are any tagged and unread articles, prompt user to mark either
- * all tagged arts as read, or only current article, or cancel operation.
- * Otherwise, use current article.
- * Finally move to next unread article.
+ * Redraw all necessary parts of the screen after FEED_MARK_(UN)READ
+ * Move cursor to next unread item if needed
  *
- * Return GRP_EXIT if there are no more unread articles in this group,
- * else return 0.
+ * Returns TRUE when no next unread art, FALSE otherwise
  */
-static int
-mark_art_read(
-	struct t_group *group)
+t_bool
+thread_mark_postprocess(
+	int function,
+	t_function feed_type,
+	int respnum)
 {
-	char keytagged[MAXKEYLEN], keycurrent[MAXKEYLEN], keyquit[MAXKEYLEN];
-	int n, cnt = 0;
-	int tmp_num_of_tagged_arts = num_of_tagged_arts;
-	t_function func = MARK_READ_CURRENT;
+	char mark[] = { '\0', '\0' };
+	int n;
 
-	if (!tinrc.mark_ignore_tags && got_tagged_unread_arts()) {
-		func = prompt_slk_response(MARK_READ_TAGGED,
-				mark_read_keys,
-				_(txt_mark_art_read_tagged_current),
-				printascii(keytagged, func_to_key(MARK_READ_TAGGED, mark_read_keys)),
-				printascii(keycurrent, func_to_key(MARK_READ_CURRENT, mark_read_keys)),
-				printascii(keyquit, func_to_key(GLOBAL_QUIT, mark_read_keys)));
-	}
-
-	switch (func) {
-		case MARK_READ_TAGGED: /* mark tagged unread articles as read */
-			cnt = mark_tagged_read(group);
-			show_thread_page();
-			n = find_response(thread_basenote, thdmenu.curr);
-			break;
-
-		case MARK_READ_CURRENT: /* mark current article as read */
-			n = find_response(thread_basenote, thdmenu.curr);
-			if ((arts[n].status == ART_UNREAD) || (arts[n].status == ART_WILL_RETURN)) {
-				char mark[] = { '\0', '\0' };
-
-				art_mark(group, &arts[n], ART_READ);
-				mark[0] = get_art_mark(&arts[n]);
+	switch (function) {
+		case (FEED_MARK_READ):
+			if (feed_type == FEED_ARTICLE) {
+				mark[0] = get_art_mark(&arts[respnum]);
 				mark_screen(thdmenu.curr, MARK_OFFSET, mark);
-			}
+			} else
+				show_thread_page();
+
+			if ((n = next_unread(respnum)) == -1)	/* no more unread articles */
+				return TRUE;
+			else
+				fixup_thread(n, TRUE);	/* We may be in the next thread now */
+			break;
+
+		case (FEED_MARK_UNREAD):
+			if (feed_type == FEED_ARTICLE) {
+				mark[0] = get_art_mark(&arts[respnum]);
+				mark_screen(thdmenu.curr, MARK_OFFSET, mark);
+				draw_thread_arrow();
+			} else
+				show_thread_page();
 			break;
 
-		case GLOBAL_QUIT: /* cancel operation */
-		case GLOBAL_ABORT:
 		default:
-			return 0;
-			/* NOTREACHED */
 			break;
 	}
-
-	if ((n = next_unread(n)) == -1)	/* no more unread articles */
-		return GRP_EXIT;
-	else
-		fixup_thread(n, TRUE);	/* We may be in the next thread now */
-
-	if (func == MARK_READ_TAGGED)
-		info_message(_(txt_marked_tagged_arts_as_read), cnt, tmp_num_of_tagged_arts, PLURAL(tmp_num_of_tagged_arts, txt_article));
-
-	return 0;
+	return FALSE;
 }
